/*
 * Allele.java
 *
 * Created on den 13 juli 2006, 13:06
 *
 * To change this template, choose Tools | Template Manager
 * and open the template in the editor.
 */

package phenobase.data;

import java.io.*;
import java.util.regex.*;
import phenobase.handling.*;

/**
 * This is a class for representing the Allele data.
 * It is able to contain all information stored in the HGVbaseG2P database for alleles
 * 
 * @version 0.1
 * @author Fredrik Sjholm
 */
public class Allele implements DataElement {
    
    /**
     * Used to define the case when te sequence type is not known
     */
    public static final short UNKNOWN = 0;
    
    /**
     * Used to define the sequence type 'genomic'.
     */
    public static final short GENOMIC = 1;
    
    /**
     * Used to define the sequence type 'cDNA'.
     */
    public static final short CDNA = 2;
    
    /** 
     * Creates a new instance of <code>Allele</code>.
     * All values are originally set to <code>null</code> except for sequence type which is set
     * to UNKNOWN. A unique ID is also created for this new instance of the class <code>Allele</code>.  
     */
    public Allele() {
        alleleID = null;
        alleleSequence = null;
        upstream30bp = null;
        downstream30bp = null;
        sequenceType = UNKNOWN;
        id = COUNT;
        COUNT++;
    }
    
    /**
     * Gets the ID used for this allele within the HGVbaseG2P database. 
     * 
     * @return      HGVbaseG2P ID for this allele or <code>null</code> if this has not been set.
     */
    public String getAlleleID() {
        return alleleID;
    }
    
    /**
     * Sets the HGVbaseG2P alleleID for this this allele.
     *
     * @param s     The HGVbaseG2P alleleID for this allele.
     * @throws InputDataException   if the given <code>String</code> does not have the correct 
     *                              alleleID syntax.
     */
    public void setAlleleID(String s) throws InputDataException {
        RegularExpressionHandler handler = new RegularExpressionHandler();
        if(s != null && !handler.isAlleleID(s)) {
            throw new InputDataException("The alleleID " + s + " given was not in the correct syntax.");
        }
        else {
            alleleID = s;
        }
    }
    
    /**
     * Gets the allele sequence for this allele. 
     * 
     * @return      the allele sequence for this allele or <code>null</code> if this has not been set
     */
    public String getAlleleSequence() {
        return alleleSequence;
    }
    
    /**
     * Sets the allele sequence for this allele.
     *
     * @param s     the allele sequence for this allele. 
     * @throws InputDataException   if the given sequence is not made up by a,c,g,t and u only.
     */
    public void setAlleleSequence(String s) throws InputDataException {
        if(!Pattern.matches("[acgtunACGTUN]*|-|\\(\\w+\\)\\d+|\\(.+\\)", s)) {
            throw new InputDataException("The allele sequence given was not in the correct syntax.");
        }
        else {
            alleleSequence = s;
        }
    }
    
    /**
     * Sets the 30 bases flanking this allele on the 5' end.
     *
     * @param s     the 30 bases flanking this allele on the 5' end.
     * @throws InputDataException   if the given sequence is not made up by exactly 30 a,c,g,t and u only.  
     */
    public void setUpstream30bp(String s) throws InputDataException {
        if(!Pattern.matches("[acgtunbdhkmrsvwyACGTUNBDHKMRSVWY]*", s)) {
            throw new InputDataException("The upstream sequence given was not in the correct syntax.");
        }
        else {
            upstream30bp = s;
        }
    }
    
    /**
     * Gets the 30 bases flanking this allele on the 5' end. 
     * 
     * @return      the 30 bases flanking this allele on the 5' end or null if this has not been set
     */
    public String getUpstream30bp() {
        return upstream30bp;
    }
    
    /**
     * Sets the 30 bases flanking this allele on the 3' end.
     *
     * @param s     the 30 bases flanking this allele on the 3' end.
     * @throws InputDataException   if the given sequence is not made up by exactly 30 a,c,g,t and u only.
     */
    public void setDownstream30bp(String s) throws InputDataException {
        if(!Pattern.matches("[acgtunbdhkmrsvwyACGTUNBDHKMRSVWY]*", s)) {
            throw new InputDataException("The downstream sequence given was not in the correct syntax.");
        }
        else {
            downstream30bp = s;
        }
    }
    
    /**
     * Gets the 30 bases flanking this allele on the 3' end. 
     * 
     * @return      the 30 bases flanking this allele on the 3' end or null if this has not been set
     */
    public String getDownstream30bp() {
        return downstream30bp;
    }
    
    /**
     * Sets the sequence type for this allele using the predefined constant values. 
     *
     * @param s     the predefined value for the type of sequence.
     * @throws InputDataException   if the value sent is not a predefined sequence type value.
     */
    public void setSequenceType(short s) throws InputDataException {
        if(s < 0 || s > 2) {
            throw new InputDataException("The sequence type given was unknown or not allowed.");
        }
        else {
            sequenceType = s;
        }
    }
    
    /**
     * Used to set the sequence type using a text string representation of the value. 
     * Accepts the values 'genomic', 'cDNA' and 'unknown' which are used in the XML file.
     *
     * @param s     The string representation of a sequence type value.
     * @throws InputDataException   if the value sent is not a valid sequence type.
     */
    public void setSequenceType(String s) throws InputDataException {
        if(s.equals("genomic")) {
            setSequenceType(GENOMIC);
        }
        else if(s.equals("cDNA")) {
            setSequenceType("CDNA");
        }
        else if(s.equals("unknown")) {
            setSequenceType(UNKNOWN);
        }
        else {
            throw new InputDataException("The sequence type given was unknown or not allowed.");
        }
    }
    
    /**
     * Defines how a <code>Allele</code> is written out when used in a list or similar structures.
     * 
     * @return      returns the alleleID used in HGVbaseG2P which will therefore be used whenever this
     *              class needs to be represented as a <code>String</code>. 
     */
    public String toString() {
        return alleleSequence;
    }
    
    /**
     * Gets the internal ID used for identification within the software.
     *
     * @return      internal ID representing this instance of <code>Allele</code>.
     * @see         DataElement
     */
    public int getID() {
        return id;
    }
    
    /**
     * Gets the identifying nr for the Allele Data Element. 
     * Enables distinction from other subclasses of <code>DataElement</code>. The nr is defined 
     * in the <code>DataElement</code> interface.
     *
     * @return      the number that identifies the <code>Allele</code> in <code>DataElement</code>.
     * @see         DataElement
     */
    public int getType() {
        return DataElement.ALLELE;
    }

    
    /**
     * Writes the values of this instance of <code>Allele</code> in XML to the stream given. 
     * 
     * @param out       the stream the XML is written on.
     * @param offset    all lines in the XML is pushed <code>offset</code> nr of steps to the right 
     *                  to make it easier to read.
     */
    public void writeXML(PrintStream out, int offset) {
        //Creates the offset string that is pasted to the beginning of each line of XML
        String offsetString = "";
        for(int i = 0; i < offset; i++) {
            offsetString = offsetString + " ";
        }
        //Writes the XML
        out.println(offsetString + "<allele>");
        if(alleleID != null) {
            out.println(offsetString + "  <alleleID>" + alleleID + "</alleleID>");
        }
        if(alleleSequence != null) {
            out.println(offsetString + "  <alleleSequence>" + alleleSequence + "</alleleSequence>");
        }
        if(downstream30bp != null) {
            out.println(offsetString + "  <downstream30bp>" + downstream30bp + "</downstream30bp>");
        }    
        if(upstream30bp != null) {
            out.println(offsetString + "  <upstream30bp>" + upstream30bp + "</upstream30bp>");
        }
        out.println(offsetString + "  <sequenceType>" + getSequenceTypeString() + "</sequenceType>");
        out.println(offsetString + "</allele>");
    }
    
    //Start of private methods
    
    //This method is used internally to translate between the numeric representation of a sequence type
    //as defined in this class and the textual representation of the same sequence type.
    private String getSequenceTypeString() {
        String res = "";
        if(sequenceType == UNKNOWN) {
            res = "unknown";
        }
        else if(sequenceType == GENOMIC) {
            res = "genomic";
        }
        else if(sequenceType == CDNA) {
            res = "cDNA";
        }
        return res;
    }
    
    //Start of private variable declaration
    
    private static int COUNT;    
    private String alleleID;
    private String alleleSequence;
    private String upstream30bp;
    private String downstream30bp;
    private short sequenceType;
    private int id;
}