/*
 * Assay.java
 *
 * Created on den 17 juli 2006, 15:44
 *
 * To change this template, choose Tools | Template Manager
 * and open the template in the editor.
 */

package phenobase.data;

import java.io.*;
import phenobase.storage.*;

/**
 * This is a class for representing Assay data. 
 * It is able to contain all information stored about assays in the HGVbaseG2P database.
 *
 * @author Fredrik Sjholm
 */
public class Assay implements DataElement {
    
    /** 
     * Creates a new instance of <code>Assay</code>. 
     * All values are originally set to <code>null</code>. 
     */
    public Assay() {
        name = null;
        type = null;
        method = null;
        conditions = null;
        markers = new DataArray();
        oligos = new DataArray();
        id = COUNT;
        COUNT = COUNT + 1;
    }
    
    /**
     * Sets the name for this assay.
     *
     * @param s     The name of the assay.
     */
    public void setName(String s) {
        name = s;
    }
    
    /** 
     * Gets the name of this assay.
     *
     * @return      The name of the assay or <code>null</null> if no name has been set.
     */
    public String getName() {
        return name;
    }
    
    /**
     * Sets the type of method used for this assay.
     *
     * @param s     The type of method used for this assay.
     */
    public void setMethodType(String s) {
        type = s;
    }
    
    /**
     * Gets the type of method used for this assay.
     *
     * @return      The type of method for this assay or <code>null</code> if no method type has been set.
     */
    public String getMethodType() {
        return type;
    }
    
    /**
     * Set the genotyping method used for this assay.
     *
     * @param s     The genotyping method used for this assay.
     */
    public void setMethod(String s) {
        method = s;
    }
    
    /**
     * Gets the genotyping method used for this assay. 
     *
     * @return      The genotyping method used for this assay or <code>null</code> if no method has been set.
     */
    public String getMethod() {
        return method;
    }
    
    /** 
     * Sets the conditions under which this assay was performed.
     *
     * @param s     The conditions under which this assay wasa performed.
     */
    public void setConditions(String s) {
        conditions = s;
    }
    
    /**
     * Gets the conditions under which this assay was performed.
     *
     * @return      The conditions under which this assay was performed or <code>null</code> if no conditions has been set.
     */
    public String getConditions() {
        return conditions;
    }
    
    /**
     * Adds a marker to the list of markers this assay was performed on.
     *
     * @param m     The marker to add to the list of markers this assay was performed on.
     */
    public void addMarker(Marker m) {
        markers.add(m);
    }
    
    /**
     * Gets all markers added to this assay.
     * These are the markers that the assay was performed on.
     *
     * @return      an array of <code>Marker</code> which contains all markers that this assay was performed on.
     */
    public Object[] getMarkers() {
        return markers.getArray();
    }
    
    /**
     * Add a oligo to the list of oligos used in this assay.
     *
     * @param o     The oligo to add to the lisyt of oligos used in this assay.
     */
    public void addOligo(Oligo o) {
        oligos.add(o);
    }
    
    /**
     * Gets all oligos used in this assay. 
     *
     * @return      an array of <code>Oligo</code> which contains all oligos added to this assay.
     */
    public Object[] getOligos() {
        return oligos.getArray();
    }
    
    /**
     * Add a citation to the list of citations used in this assay.
     *
     * @param c     The citation to add to the list of citations used in this assay.
     */
    public void addCitation(Citation c) {
        citations.add(c);
    }
    
    /**
     * Gets all citations used in this assay. 
     *
     * @return      an array of <code>Citation</code> which contains all citations added to this assay.
     */
    public Object[] getCitations() {
        return citations.getArray();
    }
    
    /**
     * Defines how a <code>Assay</code> is written out when used in a list or similar structures.
     * 
     * @return      returns the labname used in HGVbaseG2P which will therefore be used whenever this
     *              class needs to be represented as a <code>String</code>. 
     */
    public String toString() {
        return name;
    }
    
   /**
     * Gets the internal ID used for identification within the software.
     *
     * @return      internal ID representing this instance of <code>Assay</code>.
     * @see         DataElement
     */
    public int getID() {
        return id;
    }
    
    /**
     * Gets the identifying nr for the Assay Data Element. 
     * Enables distinction from other subclasses of <code>DataElement</code>. The nr is defined 
     * in the <code>DataElement</code> interface.
     *
     * @return      the number that identifies the <code>Assay</code> in <code>DataElement</code>.
     * @see         DataElement
     */
    public int getType() {
        return DataElement.ASSAY;
    }
    
    /**
     * Writes the values of this instance of <code>Assay</code> in XML to the stream given. 
     * 
     * @param out       the stream the XML is written on.
     * @param offset    all lines in the XML is pushed <code>offset</code> nr of steps to the right 
     *                  to make it easier to read.
     */
    public void writeXML(PrintStream out, int offset) {
        String os = "";
        for(int i = 0; i < offset; i++) {
            os = os + " ";
        }
        out.println(os + "<assay>");
        if(name != null) {
            out.println(os + "  <name>" + name + "</name>");
        }
        if(type != null) {
            out.println(os + "  <type>" + type + "</type>");
        }
        if(method != null) {
            out.println(os + "  <method>" + method + "</method>");
        }
        if(conditions != null) {
            out.println(os + "  <conditions>" + conditions + "</conditions>");
        }            
        for(int i = 0; i < markers.nrOfElements(); i++) {
            Marker m = (Marker)markers.get(i);
            out.println(os + "  <markerID>" + m.getID() + "</markerID>");
        }
        for(int i = 0; i < oligos.nrOfElements(); i++) {
            Oligo o = (Oligo)oligos.get(i);
            out.println(os + "  <oligoID>" + o.getID() + "</oligoID>");
        }
        for(int i = 0; i < citations.nrOfElements(); i++) {
            Citation c = (Citation)citations.get(i);
            out.println(os + "  <citationID>" + c.getID() + "</citationID>");
        }
        out.println(os + "</assay>");
    }
    
    //Start of private variable declaration
    
    private static int COUNT;
    private String name;
    private String type;
    private String method;
    private String conditions;
    private DataArray markers;
    private DataArray oligos;
    private DataArray citations;
    private int id;
    
}
