/*
 * CitationsPanel.java
 *
 * Created on den 2 december 2005, 13:19
 */

package phenobase.gui;

import phenobase.data.*;
import phenobase.handling.*;
import javax.swing.*;
import javax.swing.event.*;
import java.util.*;
import java.net.*;
import java.io.*;

/**
 * The panel used for getting citations from PubMed and adding them to the <code>Study</code>.
 * Like every panel displaying data this panel is a <code>Observer</code> that listens 
 * to the <code>Observable</code> model represented by the <code>Study</code> object. 
 * This means that the data in the panel will be automatically updated as soon as 
 * the current study data is updated. For more information on this, please refer
 * to the MVC (ModelViewController) design pattern. 
 *
 * @author  fresjo
 */
public class CitationPanel extends javax.swing.JPanel implements Observer {
    
    /** 
     * Creates a new instance of <code>CitationsPanel</code>.
     *
     * @param mf    the submission tool that this panel belong to. 
     */
    public CitationPanel(HGVbaseSubmissionTool mf) {
        mainFrame = mf;
        mainFrame.getCurrentStudy().addObserver(this);
        initComponents();
    }
    
    /**
     * This method is invoked when the observed <code>Study</code> changes. For more
     * information, refer to the MVC (ModelViewController) design pattern and the
     * <code>Observer</code> class in the java standard library. 
     */
    public void update(Observable o, Object arg) {
        Study s = (Study)o;
        addedCitationsList.setListData(s.getCitationArray());
        validate();
        repaint();
    }
    
    //Gets the mesh terms from the list and returns them as an array of <code>String</code>s
    private String[] getAddedMeshTerms() {
        ListModel lm = meshTermList.getModel();
        String[] meshTermArray = new String[lm.getSize()];
        for(int i = 0; i < lm.getSize(); i++) {
            meshTermArray[i] = (String)lm.getElementAt(i);
        }
        return meshTermArray;
    }
  
    //Gets the authors from the list and returns them as an array of <code>String</code>s
    private String[] getAddedAuthors() {
        ListModel lm = authorsList.getModel();
        String[] authorArray = new String[lm.getSize()];
        for(int i = 0; i < lm.getSize(); i++) {
            authorArray[i] = (String)lm.getElementAt(i);
        }
        return authorArray;
    }
    
    //Clears the form and empties all fields
    private void clearForm() {
        addedCitationsList.clearSelection();
        authorsList.setListData(new String[0]);
        pubMedIDField.setText("");
        referenceField.setText("");
        citationDetailsField.setText("");
        meshTermList.setListData(new String[0]);
        validate();
        repaint();
    }
    
    //Fills the form using them given <code>Citation</code>
    private void fillForm(Citation c) {
        pubMedIDField.setText("" + c.getPubMedID());
        citationDetailsField.setText(c.getCitationDetail());
        referenceField.setText(c.toString());
        Object[] authors = c.getAuthors();
        authorsList.setListData(authors);
        Object[] meshTerms = c.getMeshTerms();
        meshTermList.setListData(meshTerms);
        validate();
        repaint();
    }
    
    //Deletes the <code>Citation</code> currently selected in the list
    private void deleteCitation() {
        Citation c = (Citation)addedCitationsList.getSelectedValue();
        if(c == null) {
            JOptionPane.showMessageDialog(this,
                "You have to select a citation to be able to delete it.",
                "Error",
                JOptionPane.ERROR_MESSAGE);
        } else {
            int answer = JOptionPane.showConfirmDialog(this,
                    "Do you wish to permanently delete the selected citation from the submission?",
                    "Warning",
                    JOptionPane.WARNING_MESSAGE,
                    JOptionPane.YES_NO_OPTION);
            if(answer == JOptionPane.YES_OPTION) {
                mainFrame.getCurrentStudy().deleteCitation(c.getID());
                clearForm();    
            }
        }
    }
    
    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    // <editor-fold defaultstate="collapsed" desc=" Generated Code ">//GEN-BEGIN:initComponents
    private void initComponents() {
        java.awt.GridBagConstraints gridBagConstraints;

        addedCitationsPanel = new javax.swing.JPanel();
        addedCitationsScrollPane = new javax.swing.JScrollPane();
        addedCitationsList = new javax.swing.JList();
        jLabel2 = new javax.swing.JLabel();
        citationPanel = new javax.swing.JPanel();
        jLabel6 = new javax.swing.JLabel();
        pubMedIDField = new javax.swing.JTextField();
        jLabel7 = new javax.swing.JLabel();
        jLabel12 = new javax.swing.JLabel();
        referenceField = new javax.swing.JTextField();
        jLabel13 = new javax.swing.JLabel();
        jLabel14 = new javax.swing.JLabel();
        citationDetailsField = new javax.swing.JTextField();
        jLabel15 = new javax.swing.JLabel();
        authorsPanel = new javax.swing.JPanel();
        authorsScrollPane = new javax.swing.JScrollPane();
        authorsList = new javax.swing.JList();
        jLabel4 = new javax.swing.JLabel();
        meshTermPanel = new javax.swing.JPanel();
        meshTermScrollPane = new javax.swing.JScrollPane();
        meshTermList = new javax.swing.JList();
        jLabel5 = new javax.swing.JLabel();
        jPanel1 = new javax.swing.JPanel();
        autoEnterInfoButton = new javax.swing.JButton();
        buttonPanel = new javax.swing.JPanel();
        buttons = new javax.swing.JPanel();
        newButton = new javax.swing.JButton();
        deleteCitationButton = new javax.swing.JButton();
        addCitationButton = new javax.swing.JButton();

        setLayout(new java.awt.BorderLayout());

        setBorder(javax.swing.BorderFactory.createTitledBorder("Citations: enter all the publications you will refer to in this submission"));
        addedCitationsPanel.setLayout(new java.awt.GridBagLayout());

        addedCitationsPanel.setBorder(javax.swing.BorderFactory.createTitledBorder("Added Citations"));
        addedCitationsScrollPane.setPreferredSize(new java.awt.Dimension(810, 80));
        addedCitationsList.setBackground(new java.awt.Color(255, 255, 204));
        addedCitationsList.addListSelectionListener(new javax.swing.event.ListSelectionListener() {
            public void valueChanged(javax.swing.event.ListSelectionEvent evt) {
                addedCitationsListValueChanged(evt);
            }
        });

        addedCitationsScrollPane.setViewportView(addedCitationsList);

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.fill = java.awt.GridBagConstraints.BOTH;
        gridBagConstraints.insets = new java.awt.Insets(3, 3, 1, 3);
        addedCitationsPanel.add(addedCitationsScrollPane, gridBagConstraints);

        jLabel2.setFont(new java.awt.Font("Microsoft Sans Serif", 0, 9));
        jLabel2.setForeground(new java.awt.Color(255, 51, 51));
        jLabel2.setText("Citations already entered (click entries to view and edit)");
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 1;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.insets = new java.awt.Insets(1, 3, 3, 3);
        addedCitationsPanel.add(jLabel2, gridBagConstraints);

        add(addedCitationsPanel, java.awt.BorderLayout.NORTH);

        citationPanel.setLayout(new java.awt.GridBagLayout());

        citationPanel.setBorder(javax.swing.BorderFactory.createTitledBorder("Citation Details"));
        jLabel6.setForeground(java.awt.SystemColor.textHighlight);
        jLabel6.setText("PubMed ID");
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.insets = new java.awt.Insets(3, 3, 3, 3);
        citationPanel.add(jLabel6, gridBagConstraints);

        pubMedIDField.setBorder(javax.swing.BorderFactory.createLineBorder(java.awt.Color.blue, 2));
        pubMedIDField.setPreferredSize(new java.awt.Dimension(400, 19));
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 1;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.insets = new java.awt.Insets(3, 3, 1, 3);
        citationPanel.add(pubMedIDField, gridBagConstraints);

        jLabel7.setFont(new java.awt.Font("Microsoft Sans Serif", 0, 9));
        jLabel7.setForeground(new java.awt.Color(255, 51, 51));
        jLabel7.setText("PubMed ID for the citation you wish to add, enter one PubMed ID and click the \u2018Find Citation\u2019 button");
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 2;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.insets = new java.awt.Insets(1, 3, 3, 3);
        citationPanel.add(jLabel7, gridBagConstraints);

        jLabel12.setForeground(java.awt.SystemColor.textHighlight);
        jLabel12.setText("Title");
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 3;
        gridBagConstraints.gridwidth = 2;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.insets = new java.awt.Insets(3, 3, 3, 3);
        citationPanel.add(jLabel12, gridBagConstraints);

        referenceField.setEditable(false);
        referenceField.setPreferredSize(new java.awt.Dimension(810, 19));
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 7;
        gridBagConstraints.gridwidth = 2;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.insets = new java.awt.Insets(3, 3, 1, 3);
        citationPanel.add(referenceField, gridBagConstraints);

        jLabel13.setFont(new java.awt.Font("Microsoft Sans Serif", 0, 9));
        jLabel13.setForeground(new java.awt.Color(255, 51, 51));
        jLabel13.setText("Title of the citation as given in the PubMed database");
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 5;
        gridBagConstraints.gridwidth = 2;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.insets = new java.awt.Insets(1, 3, 3, 3);
        citationPanel.add(jLabel13, gridBagConstraints);

        jLabel14.setForeground(java.awt.SystemColor.textHighlight);
        jLabel14.setText("Reference");
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 6;
        gridBagConstraints.gridwidth = 2;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.insets = new java.awt.Insets(3, 3, 3, 3);
        citationPanel.add(jLabel14, gridBagConstraints);

        citationDetailsField.setEditable(false);
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 4;
        gridBagConstraints.gridwidth = 2;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.insets = new java.awt.Insets(3, 3, 1, 3);
        citationPanel.add(citationDetailsField, gridBagConstraints);

        jLabel15.setFont(new java.awt.Font("Microsoft Sans Serif", 0, 9));
        jLabel15.setForeground(new java.awt.Color(255, 51, 51));
        jLabel15.setText("Details for where and when the citation was published given in the format used in the PubMed database");
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 8;
        gridBagConstraints.gridwidth = 2;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.insets = new java.awt.Insets(1, 3, 3, 3);
        citationPanel.add(jLabel15, gridBagConstraints);

        authorsPanel.setBorder(javax.swing.BorderFactory.createTitledBorder("Author(s)"));
        authorsScrollPane.setPreferredSize(new java.awt.Dimension(390, 80));
        authorsList.setBackground(new java.awt.Color(255, 255, 204));
        authorsScrollPane.setViewportView(authorsList);

        jLabel4.setFont(new java.awt.Font("Microsoft Sans Serif", 0, 10));
        jLabel4.setForeground(new java.awt.Color(102, 102, 255));
        jLabel4.setText("The author(s) of this citation");

        org.jdesktop.layout.GroupLayout authorsPanelLayout = new org.jdesktop.layout.GroupLayout(authorsPanel);
        authorsPanel.setLayout(authorsPanelLayout);
        authorsPanelLayout.setHorizontalGroup(
            authorsPanelLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(authorsPanelLayout.createSequentialGroup()
                .add(jLabel4)
                .addContainerGap(260, Short.MAX_VALUE))
            .add(authorsScrollPane, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 388, Short.MAX_VALUE)
        );
        authorsPanelLayout.setVerticalGroup(
            authorsPanelLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(authorsPanelLayout.createSequentialGroup()
                .add(jLabel4)
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                .add(authorsScrollPane, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 72, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                .addContainerGap())
        );
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 9;
        gridBagConstraints.fill = java.awt.GridBagConstraints.BOTH;
        gridBagConstraints.insets = new java.awt.Insets(3, 3, 3, 3);
        citationPanel.add(authorsPanel, gridBagConstraints);

        meshTermPanel.setBorder(javax.swing.BorderFactory.createTitledBorder("Mesh term(s)"));
        meshTermScrollPane.setPreferredSize(new java.awt.Dimension(390, 80));
        meshTermList.setBackground(new java.awt.Color(255, 255, 204));
        meshTermScrollPane.setViewportView(meshTermList);

        jLabel5.setFont(new java.awt.Font("Microsoft Sans Serif", 0, 10));
        jLabel5.setForeground(new java.awt.Color(102, 102, 255));
        jLabel5.setText("Mesh term(s) associated with this citation");

        org.jdesktop.layout.GroupLayout meshTermPanelLayout = new org.jdesktop.layout.GroupLayout(meshTermPanel);
        meshTermPanel.setLayout(meshTermPanelLayout);
        meshTermPanelLayout.setHorizontalGroup(
            meshTermPanelLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(meshTermPanelLayout.createSequentialGroup()
                .add(meshTermPanelLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                    .add(jLabel5)
                    .add(meshTermScrollPane, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 190, Short.MAX_VALUE))
                .addContainerGap())
        );
        meshTermPanelLayout.setVerticalGroup(
            meshTermPanelLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(meshTermPanelLayout.createSequentialGroup()
                .add(jLabel5)
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(meshTermScrollPane, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 72, Short.MAX_VALUE)
                .addContainerGap())
        );
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 9;
        gridBagConstraints.fill = java.awt.GridBagConstraints.BOTH;
        gridBagConstraints.insets = new java.awt.Insets(3, 3, 3, 3);
        citationPanel.add(meshTermPanel, gridBagConstraints);

        jPanel1.setLayout(new java.awt.BorderLayout());

        autoEnterInfoButton.setText("Find Citation");
        autoEnterInfoButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                autoEnterInfoButtonActionPerformed(evt);
            }
        });

        jPanel1.add(autoEnterInfoButton, java.awt.BorderLayout.WEST);

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 1;
        gridBagConstraints.fill = java.awt.GridBagConstraints.BOTH;
        gridBagConstraints.insets = new java.awt.Insets(3, 3, 3, 3);
        citationPanel.add(jPanel1, gridBagConstraints);

        add(citationPanel, java.awt.BorderLayout.CENTER);

        buttonPanel.setLayout(new java.awt.BorderLayout());

        newButton.setText("New Citation");
        newButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                newButtonActionPerformed(evt);
            }
        });

        buttons.add(newButton);

        deleteCitationButton.setText("Delete Citation");
        deleteCitationButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                deleteCitationButtonActionPerformed(evt);
            }
        });

        buttons.add(deleteCitationButton);

        addCitationButton.setText("Add Citation");
        addCitationButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                addCitationButtonActionPerformed(evt);
            }
        });

        buttons.add(addCitationButton);

        buttonPanel.add(buttons, java.awt.BorderLayout.EAST);

        add(buttonPanel, java.awt.BorderLayout.SOUTH);

    }// </editor-fold>//GEN-END:initComponents

    //Invoked when the auto enter button is clicked
    private void autoEnterInfoButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_autoEnterInfoButtonActionPerformed
        try {
            int pubMedID = Integer.parseInt(pubMedIDField.getText());           
            PubMedAdapter adapter = new PubMedAdapter();
            Citation c = adapter.getCitation(pubMedID);
            if(c.getPubMedID() == 0) {
                JOptionPane.showMessageDialog(this,
                    "No citation was found with the given PubmedID",
                    "Error",
                    JOptionPane.ERROR_MESSAGE);
                clearForm();
            }
            else {
                fillForm(c);
            }
        }
        catch(NumberFormatException nfe) {
            JOptionPane.showMessageDialog(this,
                "The PubMedID must be a number",
                "Error",
                JOptionPane.ERROR_MESSAGE);
        }
    }//GEN-LAST:event_autoEnterInfoButtonActionPerformed

    //Invoked when the clear button is clicked
    private void newButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_newButtonActionPerformed
        clearForm();
    }//GEN-LAST:event_newButtonActionPerformed

    //Invoked when the selection in the list of citations changes
    private void addedCitationsListValueChanged(javax.swing.event.ListSelectionEvent evt) {//GEN-FIRST:event_addedCitationsListValueChanged
        Citation c = (Citation)addedCitationsList.getSelectedValue();
        if(c != null) {
            fillForm(c);
        }
    }//GEN-LAST:event_addedCitationsListValueChanged

    //Invoked when the delete button is clicked
    private void deleteCitationButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_deleteCitationButtonActionPerformed
        deleteCitation();
    }//GEN-LAST:event_deleteCitationButtonActionPerformed

    //Invoked when the add button is clicked
    private void addCitationButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_addCitationButtonActionPerformed
        try {
            Citation c = new Citation();
            if(!pubMedIDField.getText().equals("")) {
                c.setPubMedID(Integer.parseInt(pubMedIDField.getText()));
            }
            c.setCitationDetail(citationDetailsField.getText());
            c.setTitle(referenceField.getText());
            String[] authors = getAddedAuthors();
            for(int i = 0; i < authors.length; i++) {
                c.addAuthor(authors[i]);
            }
            String[] meshTerms = getAddedMeshTerms();
            for(int i = 0; i < meshTerms.length; i++) {
                c.addMeshTerm(meshTerms[i]);
            }
            if(addedCitationsList.getSelectedValue() == null) {
                mainFrame.getCurrentStudy().addCitation(c);
                clearForm();
            }
            else {
                int answer = JOptionPane.showConfirmDialog(this,
                        "Do you wish to update the selected citation?",
                        "Warning",
                        JOptionPane.WARNING_MESSAGE,
                        JOptionPane.YES_NO_OPTION);
                if(answer == JOptionPane.YES_OPTION) {
                    Citation selectedCitation = (Citation)addedCitationsList.getSelectedValue();
                    int id = selectedCitation.getID();
                    mainFrame.getCurrentStudy().updateCitation(c, id);
                    clearForm();    
                }
            }
        }
        catch(NumberFormatException nfe) {
            JOptionPane.showMessageDialog(this,
                "All ID:s must be numeric values.",
                "Error",
                JOptionPane.ERROR_MESSAGE);
        }
    }//GEN-LAST:event_addCitationButtonActionPerformed
    
    private HGVbaseSubmissionTool mainFrame;
    
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JButton addCitationButton;
    private javax.swing.JList addedCitationsList;
    private javax.swing.JPanel addedCitationsPanel;
    private javax.swing.JScrollPane addedCitationsScrollPane;
    private javax.swing.JList authorsList;
    private javax.swing.JPanel authorsPanel;
    private javax.swing.JScrollPane authorsScrollPane;
    private javax.swing.JButton autoEnterInfoButton;
    private javax.swing.JPanel buttonPanel;
    private javax.swing.JPanel buttons;
    private javax.swing.JTextField citationDetailsField;
    private javax.swing.JPanel citationPanel;
    private javax.swing.JButton deleteCitationButton;
    private javax.swing.JLabel jLabel12;
    private javax.swing.JLabel jLabel13;
    private javax.swing.JLabel jLabel14;
    private javax.swing.JLabel jLabel15;
    private javax.swing.JLabel jLabel2;
    private javax.swing.JLabel jLabel4;
    private javax.swing.JLabel jLabel5;
    private javax.swing.JLabel jLabel6;
    private javax.swing.JLabel jLabel7;
    private javax.swing.JPanel jPanel1;
    private javax.swing.JList meshTermList;
    private javax.swing.JPanel meshTermPanel;
    private javax.swing.JScrollPane meshTermScrollPane;
    private javax.swing.JButton newButton;
    private javax.swing.JTextField pubMedIDField;
    private javax.swing.JTextField referenceField;
    // End of variables declaration//GEN-END:variables
    
}
