/*
 * PhenotypePropertyPanel.java
 *
 * Created on den 28 februari 2006, 16:53
 */

package phenobase.gui;

import phenobase.data.*;
import java.awt.*;
import javax.swing.*;
import java.util.*;

/**
 * This panel is used to display, enter and modify phenotype property data. It contains
 * two subpanels, one for general information and one for other information. 
 * Like every panel displaying data this panel is a <code>Observer</code> that listens 
 * to the <code>Observable</code> model represented by the <code>Study</code> object. 
 * This means that the data in the panel will be automatically updated as soon as 
 * the current study data is updated. For more information on this, please refer
 * to the MVC (ModelViewController) design pattern. 
 *
 * @author  fresjo
 */
public class PhenotypePropertyPanel extends javax.swing.JPanel implements Observer {
    
    protected static final int GENERAL = 0;
    
    protected static final int OTHER = 1;
    
    /**
     * Creates a new instance of <code>PhenotypePropertyPanel</code>.
     *
     * @param t     The <code>HGVbaseSubmissionTool</code> that this panel
     *              belong to.  
     */
    public PhenotypePropertyPanel(HGVbaseSubmissionTool t) {
        initComponents();
        mainFrame = t;
        mainFrame.getCurrentStudy().addObserver(this);
        generalInfoPanel = new PhenotypeProperty_GeneralInformationPanel(t, this);
        otherInfoPanel = new PhenotypeProperty_OtherInformationPanel(t, this);
        contentPanel.add(generalInfoPanel, BorderLayout.NORTH);
        validate();
        repaint();
    }
    
    /**
     * This method is invoked when the observed <code>Study</code> changes. For more
     * information, refer to the MVC (ModelViewController) design pattern and the
     * <code>Observer</code> class in the java standard library. 
     */
    public void update(Observable o, Object arg) {
        Study s = (Study)o;
        phenotypePropertyList.setListData(s.getPhenotypePropertyArray());
        validate();
        repaint();
    }
    
    public PhenotypeProperty getSelectedPhenotype() {
        return (PhenotypeProperty)phenotypePropertyList.getSelectedValue();
    }
    
    protected void switchPanel(int panel) {
        contentPanel.removeAll();
        if(panel == GENERAL) {
            contentPanel.add(generalInfoPanel, BorderLayout.NORTH);
        }
        else if(panel == OTHER) {
            contentPanel.add(otherInfoPanel, BorderLayout.NORTH);
        }
        validate();
        repaint();
    }
    
    private void fillForm(PhenotypeProperty pg) {
        generalInfoPanel.fillForm(pg);
        otherInfoPanel.fillForm(pg);
    }
    
    private void clearForm() {
        phenotypePropertyList.clearSelection();
        generalInfoPanel.clearForm();
        otherInfoPanel.clearForm();
    }
    
    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    // <editor-fold defaultstate="collapsed" desc=" Generated Code ">//GEN-BEGIN:initComponents
    private void initComponents() {
        java.awt.GridBagConstraints gridBagConstraints;

        addedPhenotypePropertiesPanel = new javax.swing.JPanel();
        phenotypePropertyScrollPane = new javax.swing.JScrollPane();
        phenotypePropertyList = new javax.swing.JList();
        jLabel2 = new javax.swing.JLabel();
        contentPanel = new javax.swing.JPanel();
        buttonPanel = new javax.swing.JPanel();
        eastPanel = new javax.swing.JPanel();
        clearFormsButton = new javax.swing.JButton();
        deleteButton = new javax.swing.JButton();
        addModifyButton = new javax.swing.JButton();

        setLayout(new java.awt.BorderLayout());

        setBorder(javax.swing.BorderFactory.createTitledBorder("Phenotype Entities: enter all phenotype characteristics/properties (BUT NOT THEIR VALUES) examined in your study"));
        addedPhenotypePropertiesPanel.setLayout(new java.awt.GridBagLayout());

        addedPhenotypePropertiesPanel.setBorder(javax.swing.BorderFactory.createTitledBorder("Added Phenotype Entities"));
        phenotypePropertyScrollPane.setPreferredSize(new java.awt.Dimension(810, 80));
        phenotypePropertyList.setBackground(new java.awt.Color(255, 255, 204));
        phenotypePropertyList.addListSelectionListener(new javax.swing.event.ListSelectionListener() {
            public void valueChanged(javax.swing.event.ListSelectionEvent evt) {
                phenotypePropertyListValueChanged(evt);
            }
        });

        phenotypePropertyScrollPane.setViewportView(phenotypePropertyList);

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 1;
        gridBagConstraints.fill = java.awt.GridBagConstraints.BOTH;
        gridBagConstraints.insets = new java.awt.Insets(3, 3, 1, 3);
        addedPhenotypePropertiesPanel.add(phenotypePropertyScrollPane, gridBagConstraints);

        jLabel2.setFont(new java.awt.Font("Microsoft Sans Serif", 0, 9));
        jLabel2.setForeground(new java.awt.Color(255, 51, 51));
        jLabel2.setText("Phenotype Entities already entered (click entries to view and edit)");
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 2;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.NORTHWEST;
        gridBagConstraints.insets = new java.awt.Insets(1, 3, 3, 3);
        addedPhenotypePropertiesPanel.add(jLabel2, gridBagConstraints);

        add(addedPhenotypePropertiesPanel, java.awt.BorderLayout.NORTH);

        contentPanel.setLayout(new java.awt.BorderLayout());

        contentPanel.setBorder(javax.swing.BorderFactory.createTitledBorder("Phenotype Entity Details"));
        add(contentPanel, java.awt.BorderLayout.CENTER);

        buttonPanel.setLayout(new java.awt.BorderLayout());

        clearFormsButton.setText("New Phenotype Entity");
        clearFormsButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                clearFormsButtonActionPerformed(evt);
            }
        });

        eastPanel.add(clearFormsButton);

        deleteButton.setText("Delete Phenotype Entity");
        eastPanel.add(deleteButton);

        addModifyButton.setText("Save Phenotype Entity");
        addModifyButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                addModifyButtonActionPerformed(evt);
            }
        });

        eastPanel.add(addModifyButton);

        buttonPanel.add(eastPanel, java.awt.BorderLayout.EAST);

        add(buttonPanel, java.awt.BorderLayout.SOUTH);

    }// </editor-fold>//GEN-END:initComponents

    private void clearFormsButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_clearFormsButtonActionPerformed
        clearForm();
    }//GEN-LAST:event_clearFormsButtonActionPerformed

    private void phenotypePropertyListValueChanged(javax.swing.event.ListSelectionEvent evt) {//GEN-FIRST:event_phenotypePropertyListValueChanged
        PhenotypeProperty pg = (PhenotypeProperty)phenotypePropertyList.getSelectedValue();
        if(pg != null) {
            fillForm(pg);
        }
    }//GEN-LAST:event_phenotypePropertyListValueChanged

    private void addModifyButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_addModifyButtonActionPerformed
        boolean incomplete = false;
        PhenotypeProperty pp = new PhenotypeProperty();
        if(generalInfoPanel.getPropertyName().equals("")) {
            incomplete = true;
        }
        else {
            pp.setPropertyName(generalInfoPanel.getPropertyName());
        }
        pp.setComment(generalInfoPanel.getComment());
        String[] subphenotypes = generalInfoPanel.getAddedSubphenotypes();
        for(int i = 0; i < subphenotypes.length; i++) {
            pp.addSubphenotype(subphenotypes[i]);
        }
        String[] keywords = otherInfoPanel.getAddedKeywords();
        for(int i = 0; i < keywords.length; i++) {
            pp.addKeyword(keywords[i]);
        }
        String[] diseaseAreas = otherInfoPanel.getAddedDiseaseAreas();
        for(int i = 0; i < diseaseAreas.length; i++) {
            pp.addDiseaseArea(diseaseAreas[i]);
        }
        String[] categories = otherInfoPanel.getAddedAnatomicScales();
        for(int i = 0; i < categories.length; i++) {
            pp.addAnatomicScale(categories[i]);
        }
        Citation[] citations = otherInfoPanel.getAddedCitations();
        for(int i = 0; i < citations.length; i++) {
            pp.addCitation(citations[i]);
        }
        if(incomplete) {
            JOptionPane.showMessageDialog(this,
                "All of the required information has not been entered",
                "Error",
                JOptionPane.ERROR_MESSAGE);
        }
        else {
            if(phenotypePropertyList.getSelectedValue() == null) {
                if(!mainFrame.getCurrentStudy().isAddedProperty(pp.getPropertyName())) {
                    mainFrame.getCurrentStudy().addPhenotypeProperty(pp);
                    clearForm();
                }
                else {
                    JOptionPane.showMessageDialog(this,
                        "A Phenotype with this name has already been added to the current Study",
                        "Error",
                        JOptionPane.ERROR_MESSAGE);
                }
            }
            else {
                int answer = JOptionPane.showConfirmDialog(this,
                        "Do you wish to update the selected phenotype general?",
                        "Warning",
                        JOptionPane.WARNING_MESSAGE,
                        JOptionPane.YES_NO_OPTION);
                if(answer == JOptionPane.YES_OPTION) {
                    PhenotypeProperty selectedPhenotypeProperty = (PhenotypeProperty)phenotypePropertyList.getSelectedValue();
                    int id = selectedPhenotypeProperty.getID();
                    mainFrame.getCurrentStudy().updatePhenotypeProperty(pp, id);
                    clearForm(); 
                }
            }
        }
    }//GEN-LAST:event_addModifyButtonActionPerformed
    
    private HGVbaseSubmissionTool mainFrame;
    private PhenotypeProperty_GeneralInformationPanel generalInfoPanel;
    private PhenotypeProperty_OtherInformationPanel otherInfoPanel;
    
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JButton addModifyButton;
    private javax.swing.JPanel addedPhenotypePropertiesPanel;
    private javax.swing.JPanel buttonPanel;
    private javax.swing.JButton clearFormsButton;
    private javax.swing.JPanel contentPanel;
    private javax.swing.JButton deleteButton;
    private javax.swing.JPanel eastPanel;
    private javax.swing.JLabel jLabel2;
    private javax.swing.JList phenotypePropertyList;
    private javax.swing.JScrollPane phenotypePropertyScrollPane;
    // End of variables declaration//GEN-END:variables
    
}
