#!perl

# $Id: ConfigUploader.pm 782 2008-06-26 13:39:21Z rcf8 $

=head1 NAME

HGVbaseG2Pmart::ConfigUploader - Methods for uploading saved XML Biomart config files
to the database 


=head1 SYNOPSIS

    use HGVbaseG2Pmart::ConfigUploader;
    update_config_xml ($xml_directory, $dataset);

=head1 DESCRIPTION

This class contains methods to help with uploading XML Biomart config files to a
a database for a particular dataset.  

=head1 SUBROUTINES/METHODS 

=cut

package HGVbaseG2Pmart::ConfigUploader;

use strict;
use warnings;
use Compress::Zlib;
use Digest::MD5;

use FindBin;
use lib "$FindBin::Bin/../";
use HGVbaseG2Pmart::Database;
use HGVbaseG2Pmart::Exception;
use HGVbaseG2P::Util qw(openInfile);
use Exporter qw(import);
use Log::Log4perl qw( :easy );

our @EXPORT_OK = qw(update_config_xml);

=head2 update_config_xml

  Usage      :  update_dataset_config_xml($location, $dataset, $martdb);

  Description:  Updates dataset config XML in database from XML files at location.

  Return type:  None

  Exceptions :

  Caller     :

=cut

sub update_config_xml {
    my ($location,$dataset, $martdb,$config_file)=@_;
	#retrieve and update data using HGVbaseG2Pmart::Database object to 
	my $dataset_id=$martdb->get_dataset_id($dataset);
    
	#retrieve uncompressed XML from file and compress 
	my $xml_template_file=$location.$dataset."_template.template.xml";
	my $xml_file=$location.$dataset.".xml";
	DEBUG("XML file name:$xml_file, dataset ID:$dataset_id");
	my $config_xml = _get_config_file($xml_file,$dataset_id,$config_file);
	
	DEBUG("XML file name:$xml_template_file, dataset ID:$dataset_id");
	my $template_xml = _get_config_file($xml_template_file,$dataset_id,$config_file);
	
	my $compressed_xml= _compress_data($config_xml,$dataset_id);
	my $compressed_template_xml = _compress_data($template_xml,$dataset_id);

	
	#update config and template with compressed XML  	
	$martdb->update_conf_xml($compressed_xml,$dataset_id);
	$martdb->update_template_xml($compressed_template_xml,$dataset);
}

=head2 _get_config_file

  Usage      :  $temp = _get_config_file ($filename, $dataset_id)

  Description:  Retrieves data from a file, and replaces any dataset ID with the new dataset ID.

  Return type:  A scalar containing the contents of the config file.

  Exceptions :  

  Caller     :

=cut

sub _get_config_file {
	my ($file, $dataset_id,$config_file) = @_;
	
	my $fh = openInfile($file);
	
	my $config="";
	
	while (<$fh>) {
		my $line = $_;
		if ($line =~ /\<Replacer module=\"(.+)\"\/>/) {

			my $replacer_mod = "HGVbaseG2Pmart::Replacer::$1";
			INFO("Replacing <Replacer /> in $file using $replacer_mod");
			eval "require $replacer_mod";
			my $replacer = $replacer_mod->new($config_file);
			$line = $replacer->replace_text;
		}
		$config.=$line;
		
	}

	$fh->close;
	
	$config =~ s/(datasetID=".+?")/datasetID=\"$dataset_id\"/;

	return $config;
}
1;

=head2 _compress_data

  Usage      :  $compressref = _compress_data ($data)

  Description:  Compresses data in a scalar it using Gzip and creates an MD5
  digest of the uncompressed data.

  Return type:  A hash-ref containing uncompressed data, compressed data and the digest

  Exceptions :  

  Caller     :

=cut

sub _compress_data {
	my ($data) = shift;
	
	my $digest = Digest::MD5->new;
  	$digest->md5($data);
	
	my $ret=();
	$ret->{uncompressed}=$data;
	$ret->{compressed}=Compress::Zlib::memGzip($data);
	$ret->{digest}=$digest;
	return $ret;
}
1;

=head1 SEE ALSO

=head1 BUGS AND LIMITATIONS

There are no known bugs in this module.
Please report problems to Rob Free <rcfree@gmail.com>

=head1 CONTACT

This module is part of the HGVbase-G2P project

=head1 AUTHOR

Rob Free <rcfree@gmail.com>

=head1 LICENCE AND COPYRIGHT

Copyright (c) <2007> University of Leicester, Leicester, UK. All rights reserved.

This module is free software; you can redistribute it and/or
modify it under the same terms as Perl itself.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 

=head1 CVSINFO

$Id: ConfigUploader.pm 782 2008-06-26 13:39:21Z rcf8 $

=cut