#!/usr/bin/perl

# $Id: DataGenerator.pm 909 2008-08-18 17:10:55Z rcf8 $

=head1 NAME

HGVbaseG2Pmart::DataGenerator - Bulk-data file generator for HGVbaseG2P from mart database

=head1 SYNOPSIS

    use HGVbaseG2Pmart::DataGenerator;
    my $dg = HGVbaseG2Pmart::DataGenerator->new(conf_file => 'hgvmart.conf', log4perl_conf => 'log4perl.conf');
    

=head1 DESCRIPTION

  This class provides functions for generation of bulk-data files using the BioMart API or a BioMart web service (mart service) 
  The functions are generic, with all specific fields/values etc provided by the config file.
  
  For each query specified in the config file, DataGenerator retrieves a list of IDs from the HGVbaseG2P database (though DBStag),
  OR from a list in the config file.
  
  For each ID it replaces the {replaceid} placeholder in the XML query with this ID, and uses the specified method
  to submit the query and outputs each file named according to the ID (this will be automagically compressed if .gz
  extension is used). Currently the methods available include a MartService and the Mart API although additional methods
  can be added easily.

=head1 SUBROUTINES/METHODS 
 TODO
=cut

package HGVbaseG2Pmart::DataGenerator; {

	use strict;
	use warnings;
	use FindBin;
	use lib "$FindBin::Bin/../";
	# Use OIO inside-out object framework for our OO hierarachy
	use Object::InsideOut;
	use Object::InsideOut::Metadata;
	use Log::Log4perl::Level;

	use English qw( -no_match_vars );
	use Data::Dumper qw(Dumper);
	use HGVbaseG2P::Database;
	use HGVbaseG2P::Util qw(loadConfig initLogger openInfile openOutfile);
	use HTTP::Request;
	use HTTP::Headers;
	use LWP::UserAgent;
	use File::Basename;
	use XML::Simple qw(:strict);
	use BioMart::Query;
	use BioMart::QueryRunner;
	use BioMart::Initializer;
	
	# Private variables & constants
	my $logger;     # Reference to master logger
	my $dbh;
	my %config;     # Master configuration settings from config-file
	my $registry;
	
	# Non-private fields with accessor methods
	my @argv       : Field : Acc(argv) :Arg('Name' => 'argv','Mandatory' => 1); # arguments from the commandline
	
	# Initialization arguments
	my %init_args :InitArgs = ('conf_file'     => '',
				    'log4perl_conf' => '',
				    'argv'          => '',
				    );
	
	# Initialization method (see OIO docs for details)
	sub _init : Init {
	    my ( $self, $args ) = @_;
	    
	    # Load master config IF config file is provided
	    $args->{conf_file} and %config = loadConfig($args->{conf_file});
	    
	    # Initialize logger
	    my $log4perl_conf = $args->{log4perl_conf} || $config{log4perl_conf};
	    $logger = initLogger( $log4perl_conf, __PACKAGE__ );
	    
	    # Adjust logging level to verbose if requested in args
	    $self->argv->{'verbose'} and $logger->level($DEBUG);
	    
	    #load hgvmart config file
	    $logger->info("Using hgvmart config file '".$args->{conf_file}."'");
	    
	    if (!%config) {
	    	$logger->logdie("There was an error while loading the config file '$args->{conf_file}'");
	    }
	    
	    !$config{bulkdata_method} and $config{bulkdata_method}="martservice";
	    
	    if ($config{bulkdata_method} eq 'api') {
			my $initializer = BioMart::Initializer->new('registryFile'=>$config{mart_config}, 'action'=>$config{registry_action});
			$registry = $initializer->getRegistry;
	    }
	    
	}
	
=head2 generate_data

  Usage      : $markerImport->process_file('/path/to/markerfile.xml','dbSNP b128')
  Purpose    : Retrieves data related to one or more BioMart XML query for the specified set of IDs.
  Returns    : Nothing
  Arguments  : None
  Throws     : 
  Status     : Public
  Comments   : This is the core method used by the DataGenerator module.

=cut
	
	sub generate_data :Public {
		my ($self) = shift;
		my $bdconfig = $config{BulkData};

		if ($config{hgvbase_config_file}) {
			$dbh = HGVbaseG2P::Database->new(conf_file => $config{hgvbase_config_file});
		}
		else {
			$logger->info("An 'hgvbase_config_file' was not supplied in the config file");
		}
		
		my $get_method = "get_data_using_".$config{bulkdata_method};
		$logger->info("Using '$get_method' method to get bulkdata");
		
		#do retrieval of each bulkdata XML query		
		foreach my $bulkdata_id(keys %{$bdconfig}) {
			my $itemconfig = $bdconfig->{$bulkdata_id};

			my $xml = $self->retrieve_xml_file($bulkdata_id);
			
			#create output directories if not present
			my $path = $config{bulkdata_basedir};
			mkdir $path if ! -e $path;
			
			$path.="/".$itemconfig->{bulkdata_dir};
			mkdir $path if ! -e $path;
			
			#get id list from DBStag or the config file (determined by config)
			my @id_list= $self->get_id_list($bulkdata_id);
			$logger->debug("The following IDs were retrieved:".Dumper(\@id_list));
			
			#for each ID run the get method to retrieve query data for current query and ID into a file
			foreach my $curr_id(@id_list) {
				my $subbed_xml = $xml;
				$subbed_xml =~ s/{replaceid}/$curr_id/;
				
				my $output_filename = $path."/".$itemconfig->{file_prefix}.$curr_id.".".$itemconfig->{file_ext};
				my $output_fh = openOutfile($output_filename);
				
				eval {
					$self->$get_method($subbed_xml,$output_fh);
				};
				$logger->logdie($@) if $@;
					
				$output_fh->close;
				$logger->info("Retrieved data for id $curr_id as $output_filename");
			}
		}
		
		#finish off
		if ($config{hgvbase_config_file}) {
			$dbh->disconnect();
			$logger->info("Disconnected from database");
		}
		$logger->info("Completed bulk-data generation");
	}
	
=head2 retrieve_xml_file

  Usage      : my $xml = $self->retrieve_xml_file($bulkdata_id);
  Purpose    : Gets XML query file as specified in config file by the bulkdata_id
  Returns    : String containing XML query
  Arguments  : Bulkdata ID as defined in config file
  Throws     : openInfile dies if file not found
  Status     : Private
  Comments   : 

=cut

	sub retrieve_xml_file :Private {
		my ($self, $bulkdata_id) = @_;
		my $query_xml_file=$config{'query_basedir'}."/".$config{BulkData}->{$bulkdata_id}->{'biomart_query_xml'};

		my $xml;
		my $xml_fh = openInfile($query_xml_file);
		
		while (<$xml_fh>) {
			$xml.=$_;
		}
		$xml_fh->close;
		
		$logger->info("Using XML query file:$query_xml_file");
		return $xml;
	}

=head2 get_id_list

  Usage      : my @id_list= $self->get_id_list($bulkdata_id);
  Purpose    : Gets ID list from the config file or using DBStag. The method is specified in config file by the bulkdata_id
  Returns    : Array containing list of IDs
  Arguments  : Bulkdata ID as defined in config file
  Throws     : 
  Status     : Private
  Comments   : 

=cut

	sub get_id_list :Private {
		my ($self, $bulkdata_id) = @_;

		my $bd_config = $config{BulkData}{$bulkdata_id}; 
		if ($bd_config->{id_list}) {
			$logger->debug("id_list found in config");
			my $separator = $bd_config->{id_separator} || ",";
			return split($separator, $bd_config->{id_list});
		}
		else {
			!$config{hgvbase_config_file} and $logger->logdie("Need an 'hgvbase_config_file' to retrieve id list from DBStag");
			$logger->debug("Using id list obtained through DBStag");
			my $stag_set = $dbh->selectall_stag(-template => $bd_config->{stag_template});
	
			my @id_list=();
			
			foreach($stag_set->find($bd_config->{stag_nesting})) {
				push @id_list, $_->get($bd_config->{stag_id_field});
			}
			return @id_list;
		}
	}

=head2 get_data_using_...

  Usage      : $self->get_data_using_api($subbed_xml,$output_fh);
               $self->get_data_using_martservice($subbed_xml,$output_fh);
               #
               # or alternatively a generic approach (the method name is supplied by the config file)
               my $get_method = "get_data_using_".$config{bulkdata_method};
               $self->$get_method($subbed_xml,$output_fh);
               
  Purpose    : Gets data using the supplied XML query and ID. It does this using the Mart API or the martservice
  Returns    : Undef
  Arguments  : XML with ID included (ie. substituted placeholder)
               Output file handle
  Throws     : 
  Status     : Private
  Comments   : This approach allows the creation of new methods which require different processing.

=cut
	
	sub get_data_using_api : Private {
		my ($self, $xml, $output_fh) = @_;
		
		my $query = BioMart::Query->new(
			 'registry' => $registry,
			 'virtualSchemaName' => 'default',
			 'xml'      => $xml
		);
    
    	my $runner = BioMart::QueryRunner->new();
    		
    	$runner->execute($query);
		$logger->info("Executed query");
        if ($query->header && $query->header eq '1') {
		    $runner->printHeader($output_fh);
		}
		
		my $formatter = $runner->get('formatter');

		my $counter;
		no warnings 'uninitialized';
		while (my $row = $formatter->nextRow)
		{
		    next if ($row eq "\n");
			$counter++;
			$counter%1000==0 && $logger->info("Read $counter rows");
			print $output_fh $row;
		}
		$logger->info("Got $counter rows");
		$runner->printFooter($output_fh);
	}
	
	sub get_data_using_martservice :Private {
		my ($self, $xml, $output_fh) = @_;
		
		my $url=$config{martservice_url} || $logger->logdie("No 'martservice_url' value was provided!");
			
		my $request = HTTP::Request->new("POST",$url,HTTP::Headers->new,'query='.$xml."\n");
		my $ua = LWP::UserAgent->new;
		
		my $response;
		
		$ua->request($request, 
		     sub{   
				 my($data, $response) = @_;
				 if ($response->is_success) {
					$output_fh->print($data);
				 }
				 else {
				    $logger->logwarn("Problems with the web server: ".$response->status_line);
				 }
				 
		     },1000
		 );
	}
}