

# $Id: XMLdb.pm 591 2008-05-06 11:12:19Z rcf8 $

=head1 NAME

HGVbaseG2Pmart::DbCreator - Module for creating/destroying and populating a test db etc


=head1 SYNOPSIS

	
=head1 DESCRIPTION

This class inherits from Test::Class and adds features for testing the main relational
database central to the HGVbase-G2P system. The main feature is the startup-method 
which initializes the database and connects to it via HGVbaseG2P::XMLStore, a common
procedure for each of the derived classes that inherit this functionality.


=head1 SUBROUTINES/METHODS 

=cut

package HGVbaseG2Pmart::DbCreator; {
	use strict;
	use warnings;
	
	use Data::Dumper;
	
	use Env qw($DBI_DSN $DBI_USER $DBI_PASS $DBSTAG_TEMPLATE_DIRS);
	use HGVbaseG2P::Util qw(loadConfig);
	use HGVbaseG2P::XMLStore;
	use HGVbaseG2P::Util qw(loadConfig initLogger openOutfile);
	
	use FindBin;
	use lib "$FindBin::Bin/../lib";
	
	# Use OIO inside-out object framework for our OO hierarachy
	use Object::InsideOut;
	
	my %config=(); 
	my $logger;
	
# Initialization arguments
	my %init_args : InitArgs = (
		'conf_file'     => '',
		'log4perl_conf' => '',
	);
	
	my @dsn : Field : Acc(dsn) : Arg( 'Name' => 'dsn' );
	my @user : Field : Acc(user) : Arg( 'Name' => 'user' );
	my @password : Field : Acc(password) : Arg( 'Name' => 'password' );
	my @xmldb : Field : Acc(xmldb) : Arg( 'Name' => 'xmldb' ) :Type(HGVbaseG2P::XMLStore);
	my @args : Field : Acc(args) : Arg('Name' => 'args');
	
	# Initialization method (see OIO docs for details)
	sub _init : Init {
		my ( $self, $args ) = @_;

		# Load master config IF config file is provided
		$args->{conf_file} and %config = loadConfig( $args->{conf_file} );

		# Initialize logger
		my $log4perl_conf = $args->{log4perl_conf} || $config{log4perl_conf};
		$logger = initLogger( $log4perl_conf, __PACKAGE__ );
		$self->args($args);
	}

	# Initialize database with schema definition, dropping the test-db if it already exists, and
	# connecting to it with the XML middleware.
	sub create_db {
	    my $self = shift;
	    my $dbname = "test999_".$config{dbuser};
	    my $dbuser = $config{dbuser};
	    my $ddl_file = $config{schema_file};
	    -f $ddl_file or die "Provided schema file $ddl_file does not exist";
	    my $credentials = "-u ".$config{dbuser}. " -p".$config{dbpassw}." -h " . $config{dbhost}.($config{dbport} ? " -P " . $config{dbport} : "");
	    system("echo This will drop $dbname on ". ($config{dbhost} || " localhost" ));
	    system("echo Are you sure you want to do this?");
	    my $yn=<STDIN>;
	    if ($yn =~ /^n/) { exit; };
	    $logger->info("mysql $credentials -e 'drop database if exists $dbname'");
	    system("mysql $credentials -e 'drop database if exists $dbname'") == 0 or die "Problem dropping test database '$dbname' if it exists";
	    system("mysqladmin $credentials create $dbname") == 0 or die "Problem creating test database '$dbname'";
	    system("mysql $credentials $dbname < $ddl_file") == 0 or die "Problem loading schema into test database '$dbname'";
	    
	    my $dsn = $config{dsn};
	    $self->dsn($dsn);
	    $self->user($config{dbuser});
	    $self->password($config{dbpassw});
	    
	    # Set environmental variable to point DBStag to the SQL-templates
	    $DBSTAG_TEMPLATE_DIRS = $config{sqltemplates};
	}
	
	sub store_xml {
		my ($self, $file, $description) = @_;
		if (!defined($self->xmldb)) {
			my $xmldb = HGVbaseG2P::XMLStore->new(conf_file => $self->args->{conf_file},               
                              log4perl_conf => $config{log4perl_conf},);
            defined($xmldb) or die "Not able to define XML-db object";
            $self->xmldb($xmldb);
		}
		$self->xmldb->storeFile($file) or die "Cannot store $description XML-file";
	    $self->xmldb->commit() or die "Cannot commit $description transaction";
	}
	
	sub generate_xml {
		my ($self, $datatype, $source, $studyid, $infile, $replacecase) = @_;
		
		my $importclass = "HGVbaseG2P::DataImport::".$datatype."::".$source;
	
		eval "require $importclass;";
	    $@ and die "DataImport class '$importclass' not loaded ok $@";
		my $dataimport = $importclass->new(
		   conf_file => $config{hgvbase_config_file},
		   source => $source,
		   argv   => {studyid => $studyid, assayedpanels=>'CONTROL,CASE', replacecase=>$replacecase},
		   dsn    => $self->dsn) or die "Unable to create DataImport class";
		
		$dataimport->process_file($infile);
	}
	
	sub db_cleanup {
	    my $self = shift;
	    $self->xmldb->disconnect() or die "Cannot disconnect from database";
	}
}

1;

=head1 SEE ALSO

=head1 BUGS AND LIMITATIONS

There are no known bugs in this module.
Please report problems to Gudmundur Thorisson <gthorisson@gmail.com>

=head1 CONTACT

This module is part of the HGVbase-G2P project

=head1 AUTHOR

Gudmundur Thorisson <gthorisson@gmail.com>

=head1 LICENCE AND COPYRIGHT

Copyright (c) <2007> University of Leicester, Leicester, UK. All rights reserved.

This module is free software; you can redistribute it and/or
modify it under the same terms as Perl itself.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 

=head1 CVSINFO

$Id: XMLdb.pm 591 2008-05-06 11:12:19Z rcf8 $

=cut
