#!perl

# $Id: MartCreator.pm 782 2008-06-26 13:39:21Z rcf8 $

=head1 NAME

HGVbaseG2Pmart::MartCreator - Class for creating a mart based using
details in a supplied config file


=head1 SYNOPSIS

    use HGVbaseG2Pmart::MartCreator;
    my $creator=MartCreator->new($config_filename, $verbose_flag);
    $creator->create_mart;

=head1 DESCRIPTION

=head1 SUBROUTINES/METHODS 

=cut

package HGVbaseG2Pmart::MartCreator; {

	use strict;
	use warnings;
	use FindBin;
	use lib "$FindBin::Bin/../";
	use HGVbaseG2P::Util qw(loadConfig initLogger);
	use HGVbaseG2Pmart::ConfigUploader qw(update_config_xml);
	use HGVbaseG2Pmart::Database;
	use Data::Dumper qw(Dumper);
	use Net::FTP;
	use Log::Log4perl::Level;

	# Use OIO inside-out object framework for our OO hierarachy
	use Object::InsideOut;
	
	my %config=(); 
	my $logger;
	
# Initialization arguments
	my %init_args : InitArgs = (
		'conf_file'     => '',
		'log4perl_conf' => '',
		'update' => 0,
		'scripts' => '',
		'verbose' => 0,
		'append' => 0,
		'dataset' => '',
		'replaceid' => '',
		'noconfirm' => 0,
		'ignorebase' => 0,
		'killbase' => 0,
	);
	
	my @sourcedb : Field : Acc(sourcedb) : Arg( 'Name' => 'sourcedb' );
	my @martdb : Field : Acc(martdb) : Arg( 'Name' => 'martdb' );
	my @user : Field : Acc(user) : Arg( 'Name' => 'user' );
	my @password : Field : Acc(password) : Arg( 'Name' => 'password' );
	my @xmldb : Field : Acc(xmldb) : Arg( 'Name' => 'xmldb' ) :Type(HGVbaseG2P::XMLStore);

=head2 _init
	
	  Usage      : $creator = MartCreator->new($configfile,$verbose) #calls _init following initialisation
	  Purpose    : Internally called following creation of MartCreator OIO object
	  Returns    : MartCreator object ref
	  Arguments  : Config file name (see conf/g2pmart.conf)
	               Verbose (debug) switch
	  Throws     : 
	  Status     : 
	  Comments   : 
	
=cut
	
	# Initialization method (see OIO docs for details)
	sub _init : Init {
		my ( $self, $args ) = @_;

		# Load master config IF config file is provided
		$args->{conf_file} and %config = loadConfig( $args->{conf_file} );

		# Initialize logger
		my $log4perl_conf = $args->{log4perl_conf} || $config{log4perl_conf};
		$logger = initLogger( $log4perl_conf, __PACKAGE__ );
		
		if ($args->{verbose}) {
			$logger->level($DEBUG)
		}
		
		#set command line parameters in config
		$config{update}=$args->{update};
		$config{scripts}=$args->{scripts};
		$config{dataset_to_use}=$args->{dataset};
		$config{replaceid}=$args->{replaceid};
		$config{noconfirm}=$args->{noconfirm};
		$config{append}=$args->{append};
		$config{ignorebase}=$args->{ignorebase};
		$config{killbase}=$args->{killbase};
	}
	
=head2 create_mart
	
	  Usage      : $creator->create_mart;
	  Purpose    : Creates the mart using the properties of the MartCreator object
	  Returns    : Nothing
	  Arguments  : None
	  Throws     : 
	  Status     : 
	  Comments   : 
	
=cut
	
	sub create_mart {
		my ($self)=@_;
	
		#setup variables with config info
		my $sql_location=$config{sql_location};
		
		#set up source DB object as this is used by both update and install
		my $sourcedb=HGVbaseG2Pmart::Database->new($config{SourceDatabase});
		$self->sourcedb($sourcedb);
		
		my $martdb;
		
		if ($config{update} || $config{append}) {
			$martdb = $self->use_mart;
			$logger->info("Using current mart database ".$martdb->get_name);
			$martdb->clear_configs;
			$logger->info("Cleared current mart config");
	
		}
		else {
			if (!$config{noconfirm}) {
				system("echo This will drop the following database: ". $config{MartDatabase}->{dsn});
	            system("echo Are you sure you want to do this?");
	            my $yn=<STDIN>;
	            if ($yn =~ /^n/) { exit; };
			}
            
			$martdb = $sourcedb->drop_and_create($config{MartDatabase});
			$self->martdb($martdb);
			 
			$logger->info("Dropped and created mart database ".$martdb->get_name);
			$sourcedb->DESTROY;
			
			#create dataset config tables
			my $script = $martdb->get_script("$sql_location/create_config.sql",$martdb->get_name,$sourcedb->get_name);
			$martdb->execute_script($script);
			$logger->info("Created config tables");
		}
		
		if ($config{scripts}) {
			#for each SQL script generate script ref and execute each one
			foreach my $scriptfile(split(',',$self->{scripts})) {
				my $script = $martdb->get_script("$sql_location/$scriptfile",$martdb->get_name,$sourcedb->get_name);
				$logger->info("Executing individual '$scriptfile' SQL file");
				$martdb->execute_script($script);
			}
		}
		
		$self->martdb($martdb);
		
		#add datasets

		my $dataset_cfg = $config{Dataset};
		my $use_dataset=0;
		foreach my $dataset_name(keys %{$dataset_cfg}) {
			if ($config{dataset_to_use}) {
				$logger->debug("dataset to use:".$config{dataset_to_use}, " vs $dataset_name");
                if ($config{dataset_to_use} eq $dataset_name) {
                	$use_dataset=1;
                }
                else {
                	$use_dataset=0;
                }
			}
			else {
				$use_dataset=1;
			}
			
			if ($use_dataset) {
				my $dataset = $dataset_cfg->{$dataset_name};
				$self->_add_dataset({
					dataset=>$dataset_name,
					description=>$dataset->{desc},
					visible=>$dataset->{visible},
					importtable=>$dataset->{importtable},
				});	
			}
		}
	}

=head2 use_mart
	
	  Usage      : $creator->use_mart;
	  Purpose    : Uses the mart database specified in the MartCreator config file as MartDatabase
	  Returns    : Nothing
	  Arguments  : None
	  Throws     : 
	  Status     : 
	  Comments   : 
	
=cut
	
	sub use_mart {
		my ($self)=@_;
	
		#setup variables with config info
		my $martdb_cfg=$config{MartDatabase};
	
		#need to use existing db (source db) as 'handle' in order to create mart db
		my $martdb=HGVbaseG2Pmart::Database->new($martdb_cfg);
		$self->martdb($martdb);
	}
	
=head2 _add_dataset
	
	  Usage      : $self->_add_dataset({
	  				dataset=>$dataset_name,
					description=>$description,
					visible=>$visible,
					importtable=>$importtable
	  			   });
	  Purpose    : Private method used to add a dataset to the mart including data and structure
	  Returns    : Nothing
	  Arguments  : Hash reference containing:
	                dataset => dataset name
	                description => description displayed for dataset
	                visible => dataset visible in martview?
	                importtable => import table using FTP?
	  Throws     : 
	  Status     : 
	  Comments   : 
	
=cut
	
	sub _add_dataset {
		my ($self,$attrs) = @_;
		
		my $dataset = $attrs->{dataset};
		my $importtable = $attrs->{importtable};
		my $martdb = $self->martdb;
		my $sourcedb = $self->sourcedb;
		
		my $sql_location = $config{sql_location};
		my $xml_location=$config{xml_location};
	
		
		#obtain list of dataset SQL files
		my $suffix = "_*.sql";
		my @scriptfiles = <$sql_location/$dataset$suffix>;

		unshift @scriptfiles, "$sql_location/$dataset.sql" unless $config{ignorebase};
		
		my $type="Imported ";
		if (!$importtable) {
			if (!$config{append} && !$config{update}) {
				push @scriptfiles, "$sql_location/create_$dataset.sql";
				$type="Created";
			}
			
			if ($config{append} ) {
				push @scriptfiles, "$sql_location/append_$dataset.sql";
				$type="Appended to";
			}
		}
		
		if (!$config{update}) {
			#for each SQL script generate script ref and execute each one
			foreach my $scriptfile(@scriptfiles) {
				my $script = $martdb->get_script("$scriptfile",$martdb->get_name,$sourcedb->get_name,$config{replaceid});
				$logger->info("Executing '$scriptfile' SQL file");
				$martdb->execute_script($script);
			}
			
			$logger->info("$type the '$dataset' dataset");
			
			if ($importtable) {
				$self->_get_table_via_ftp($importtable);
				$martdb->import_tsv_file($config{temp_dir}."/".$importtable.".txt", $importtable);
			}
		}
			
		#add config records for dataset and insert XML
		$logger->info("Adding '$dataset' to config tables with xml config");
		$martdb->add_dataset_config($dataset,$attrs->{description},$attrs->{visible});
		update_config_xml("$xml_location/",$dataset,$martdb,$config{hgvbase_config_file});
	}
	
=head2 _add_dataset
	
	  Usage      : $self->_add_dataset({
	  				dataset=>$dataset_name,
					description=>$description,
					visible=>$visible,
					importtable=>$importtable
	  			   });
	  Purpose    : Private method used to add a dataset to the mart including data and structure
	  Returns    : Nothing
	  Arguments  : Hash reference containing:
	                dataset => dataset name
	                description => description displayed for dataset
	                visible => dataset visible in martview?
	                importtable => import table using FTP?
	  Throws     : 
	  Status     : 
	  Comments   : 
	
=cut
	
	sub update_dataset {
		my ($self,$attrs) = @_;
		
		my $dataset = $attrs->{dataset};
		my $xml_location = $attrs->{xmllocation};
		my $martdb = $self->martdb;
		
		#add config records for dataset and insert XML
		$logger->info("Updating '$dataset' to config tables with xml config at '$xml_location'");
		$logger->debug(Dumper($martdb));
		update_config_xml("$xml_location/",$dataset,$martdb);
	}
	
=head2 _get_table_via_ftp
	
	  Usage      : $self->_get_table_via_ftp($tablename)
	  Purpose    : Private method used to get text file containing table from FTP site
	  Returns    : Nothing
	  Arguments  : Table name (file downloaded is <tablename>.txt.gz
	  Throws     : 
	  Status     : 
	  Comments   : 
	
=cut
	
	sub _get_table_via_ftp {
		my ($self, $tablename) = @_;
		
		my $ftp_host = $config{ftp_host};
		my $temp_dir = $config{temp_dir};
		my $file = "$tablename.txt";
	 
	#    my $ftp = Net::FTP->new($ftp_host, Debug => 0)
	#      or die "Unable to connect to $ftp_host: $@";
	#
	#    $ftp->login("anonymous",'-anonymous@')
	#      or die "Unable to login to $ftp_host: ", $ftp->message;
	#
	#	$ftp->cwd($ftp_dir);
	#	$ftp->get($file,"$temp_dir/$file") or warn $ftp->message;
	#
	#    $ftp->quit;
	    
	    if (!$config{use_local}) {
	    	my $success = system "curl $ftp_host/$file.gz > /$temp_dir/$file.gz";
	    	!$success ? $logger->info("File '$file' obtained from $ftp_host successfully") : $logger->logdie("Unable to obtain '$file' from $ftp_host!");
	    }
	    else {
	        if (-e "$temp_dir/$file.gz") {
	                $logger->info("Using local file '$temp_dir/$file.gz'");
	        }
	        else {
	                die "DIED - Local file '$temp_dir/$file.gz' not found!";
	        }
	    }
	    system "gunzip -c $temp_dir/$file.gz > $temp_dir/$file";
	
	}
}
1;
