#!perl

# $Id: Test.pm 772 2008-06-23 17:37:28Z rcf8 $

=head1 NAME

HGVbaseG2Pmart::Test - Mart test base features


=head1 SYNOPSIS

	use base qw(HGVbaseG2Pmart::Test);
	use Test::More;
	
	sub before_test:Test(setup) {
		my $self = shift;
		my %testdata = $self->get_data;
		my $query = $self->create_query;
		
		$query->setDataset("study");
		$query->addFilter("title",[$testdata{studytitle1}]);
		$query->addAttribute("title");
		$query->addAttribute("submissiondate");
		$self->set_query($query);
		$self->set_description("specific study");
	}
	
	sub test_sub:Test(1) {
		my $self = shift;
		
		my $data = $self->get_data;
		my $query = $self->get_query;
		
		#add/remove from query here
		...
		
		my $count = $self->get_executed_count;
		
		#run test based on count here
		...
	}
	
=head1 DESCRIPTION

This class inherits from Test::Class, adds features specific to BioMart including simple accessors for
the BioMart::Query object and for running a count query with BioMart::QueryRunner.

It also includes accessors for the hash-based test data (get_data) which is obtained from the
test mart config file (../t/data/g2pmart.conf) and the description property (get_description/set_description).

It or a descendent is used as the base class for all BioMart test modules. 



=head1 SUBROUTINES/METHODS 

=cut

package HGVbaseG2Pmart::Test;

use strict;
use warnings;

use FindBin;

use lib "$FindBin::Bin/../../lib";
use BioMart::Query;
use BioMart::Initializer;
use BioMart::QueryRunner;


use HGVbaseG2P::Util qw(loadConfig initLogger);
use HGVbaseG2Pmart::Constants;
use Data::Dumper qw(Dumper);
use base qw(Test::Class);
use Test::More;

my $logger;

sub init : Test(startup) {
	my $self = shift;
	
	my %config=(); 
	
	eval {%config = loadConfig($HGVbaseG2Pmart::Constants::TEST_CONFIG_FILE);};
	die "Unable to load config file:".$@ if $@;
	my $confFile = $config{mart_config};
	
	my $action=$config{registry_action};
	
	# Initialize logger
    my $log4perl_conf = $config{log4perl_conf};
    $logger = initLogger( $log4perl_conf, __PACKAGE__ );
        
	#my $action="clean";
	my $initializer = BioMart::Initializer->new('registryFile'=>$confFile, 'action'=>$action);
	$self->{reg} = $initializer->getRegistry;
	
	my $expected=();
	my $data=();
	if ($config{TestData}) {
		$data = $config{TestData};
	}
	else {
		die "Unable to find test data element in config file";
	}
	
	foreach my $module(keys %{$data}) {
		foreach my $line(keys %{$data->{$module}}) {
			my $item = $data->{$module}->{$line};
			my @temp = split($config{testdata_expected_separator} || ',',$item);
			$data->{$module}->{$line}=$temp[0];
			my $exp_value = $temp[1] || '0.0';
			$expected->{$module}->{$line}=$exp_value;
		} 
	}
	$self->{data}=$data;
	$self->{expected}=$expected;
	$self->{separator}=$config{testdata_value_separator};
}

sub add_attribute_page {
	my ($self, $dataset, $page)=@_;
	my $query = $self->{query};
	my $registry = $query->getRegistry;
	
	$query->setDataset($dataset);
	
	my $config_tree = $registry->getConfigTreeForDataset($dataset);
	my $attT = $config_tree->getAttributeTreeByName($page);
	
	foreach my $attG(@{$attT->getAllAttributeGroups}) {
		foreach my $attC(@{$attG->getAllCollections}) {
			foreach my $att(@{$attC->getAllAttributes}) {
				$query->addAttribute($att->name);
			}
		}
	}
}

sub get_executed_count {
	#N.B. This count is a hack to compensate for an error which occurs when the QueryRunner count
	#method is used.
	
	my ($self,$no_count,$limit) = @_;
	my $runner = BioMart::QueryRunner->new;
	my $query = $self->{query};
	$query->count(0);
	$runner->execute($query, $self->get_attribute_page);
	
#	#output formatted (ie. TSV) query results to a scalar
#	my ($qryout,$qryresult)=undef;
#	open $qryout, '>', \$qryresult;
#	$runner->printResults($qryout,$limit ? $limit : 0);
#	
#	#split the scalar by \n to retrieve the number of records
#	if (!$qryresult) {
#		return 0;	
#	}
#	else {
#		my @count = split("\n",$qryresult);
#		return scalar(@count);
#	} 
	
	my $formatter = $runner->get('formatter');
	
	my $counter=0;
	return $formatter->nextRow ? 1 : 0;
}

sub get_actual_count {
	#N.B. This count is a hack to compensate for an error which occurs when the QueryRunner count
	#method is used.
	
	my ($self,$no_count,$limit) = @_;
	my $runner = BioMart::QueryRunner->new;
	my $query = $self->{query};
	$query->count(0);
	$runner->execute($query, $self->get_attribute_page);
	
	my $formatter = $runner->get('formatter');
	my $counter=0;
		while (my $row = $formatter->nextRow)
	    {
			$counter++;
		}
#	#output formatted (ie. TSV) query results to a scalar
#	my ($qryout,$qryresult)=undef;
#	open $qryout, '>', \$qryresult;
#	$runner->printResults($qryout,$limit ? $limit : 0);
#	
#	#split the scalar by \n to retrieve the number of records
#	if (!$qryresult) {
#		return 0;	
#	}
#	else {
#		my @count = split("\n",$qryresult);
#		return scalar(@count);
#	} 

	return $counter;
}

sub get_normal_count {
	
	my ($self,$no_count,$limit) = @_;
	my $runner = BioMart::QueryRunner->new;
	my $query = $self->{query};
	$query->count(1);
	$runner->execute($query, $self->get_attribute_page);
	return $runner->getCount;
}

sub execute_query {
	my ($self) = @_;
	my $runner = BioMart::QueryRunner->new;
	my $query = $self->{query};
	$query->count(0);
	$runner->execute($query,$self->get_attribute_page);
	return $runner;
}

sub create_query {
	my $self =shift;
	my $registry = $self->{reg};
	my $query = BioMart::Query->new('registry'=>$registry,'virtualSchemaName'=>'default');
	$self->{query}=$query;
	return $query;
}

sub get_data {
	my ($self,$key) = @_;
	my $caller = ref($self);
	!$key and die "Null key passed to get_data (caller: $caller)";
	my $callerdata = $self->{data}->{$caller};
	!$callerdata and die "Data values for caller module $caller not found!";
	
	!$callerdata->{$key} and die "Data values for caller module $caller and key $key undefined!";

	return $callerdata->{$key};
}

sub get_expected {
	my ($self,$key)=@_;
	my $caller = ref($self);
	!$self->{expected}->{$caller} and die "Expected values for caller $caller not found!";
	!$self->{expected}->{$caller}->{$key} and die "Expected values for caller $caller and key $key not found!";
	return $self->{expected}->{$caller}->{$key};
}

sub set_query {
	my ($self,$query)=@_;
	$self->{query}=$query;
}

sub set_description {
	my ($self,$description)=@_;
	$self->{description}=$description;
}

sub get_description {
	my $self=shift;
	return $self->{description};
}

sub get_separator {
	my $self=shift;
	return $self->{separator};
}

sub get_query {
	my $self=shift;
	return $self->{query};
}

sub get_attribute_page {
	return undef;
}

sub filter_count_test {
	my ($self, $filtername, $testid, $description)=@_;
	my $query = $self->get_query;
	my $separator = $self->get_separator;
	my $filtervalue=[split("$separator",$self->get_data($testid))];
	$query->addFilter($filtername,$filtervalue);
	my $count = $self->get_actual_count;
	is($count,int($self->get_expected($testid)),"$description ".join(",",@{$filtervalue})." (attr page:".$self->get_description.") - expected ".int($self->get_expected($testid)));
}

sub multi_filter_count_test {
	my ($self, $filternames, $testids, $description)=@_;
	my $query = $self->get_query;
	my $separator = $self->get_separator;
	my $counter = 0;
	foreach my $filter(@{$filternames}) {
		$query->addFilter($filter,[split("$separator",$self->get_data($testids->[$counter]))]);
		$counter++;
	}
	my $count = $self->get_actual_count;
	is($count,int($self->get_expected($testids->[0])),"$description (attr page:".$self->get_description.") - expected ".int($self->get_expected($testids->[0])));
}
1;

=head1 SEE ALSO

=head1 BUGS AND LIMITATIONS

There are no known bugs in this module.
Please report problems to Rob Free <rcfree@gmail.com>

=head1 CONTACT

This module is part of the HGVbase-G2P project

=head1 AUTHOR

Rob Free <rcfree@gmail.com>

=head1 LICENCE AND COPYRIGHT

Copyright (c) <2007> University of Leicester, Leicester, UK. All rights reserved.

This module is free software; you can redistribute it and/or
modify it under the same terms as Perl itself.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 

=head1 CVSINFO

$Id: Test.pm 772 2008-06-23 17:37:28Z rcf8 $

=cut


