# $Id: Analyse.pm 1090 2008-11-26 11:14:36Z rcf8 $

=head1 NAME

HGVbaseG2P::Browser::Analyse - Base class for analysis of data in the form of an array of hashes or arrays.


=head1 SYNOPSIS
    
   package HGVbaseG2P::Browser::Analyse::*; {
    
   use strict;
   use warnings;

   # Use OIO inside-out object framework for our OO hierarachy
   use Object::InsideOut qw( HGVbaseG2P::Analyse );
   
=head1 DESCRIPTION

Data is generally obtained from a DB (using HGVbaseG2P::Database::*), indirectly as files (using HGVbaseG2P::FileReader) 
or can be provided in the constructor in an array of hashes/arrays. This module provides general utility methods for
handling any form of analysis, including simple statistical analysis.

=head1 SUBROUTINES/METHODS 

		
=cut

package HGVbaseG2P::Browser::Analyse;
{

    use strict;
    use warnings;

    # Use OIO inside-out object framework for our OO hierarachy
    use Object::InsideOut;
    use Object::InsideOut::Metadata;
    use Carp qw(cluck confess);
    use Data::Dumper;

    use HGVbaseG2P::Util qw(load_config init_logger load_module);
    use HGVbaseG2P::Browser::Util qw(poslog);
    
    use POSIX qw(strftime);
    use File::Basename;
    
    
    # Private variables & constants
    my @log : Field :All(log); #contains Log4perl Logger
    my @config : Field :All(config); #contains loaded config
    my @data : Field :All(data);
    
    # Initialization arguments
    my %init_args : InitArgs = (
        'conf_file'     => '',
        'log4perl_conf' => '',
        'file' => '',
        'no_headings' => '',
        'file_options' => '',
        'line_options' => '',
    );
    
    sub has_config {
        return 0;
    }
    
=head2 _init

	  Usage      : Internally called by new method
	  Purpose    : Initialise config, logger and read data from file or use data structure
	  Returns    : Nothing
	  Arguments  : Hashref of:
	                conf_file - config file name or hash containing config
	                log4perl_conf - log4perl file (if not supplied use from config)
	                
	                optional:
	                file - name of file to read data from
	                no_headings - read into array of arrays
	                file_options - file-based options passed to HGVbaseG2P::FileReader module
	                line_options - line-based options passed to HGVbaseG2P::FileReader module
	  Throws     : 
	  Status     : Public
	  Comments   : 
=cut

    sub _init : Init
    {
        my ( $self, $args ) = @_;

        # Load supplied config file, if non provided use default
        my $conf_file = $args->{conf_file} || "../conf/hgvbase.conf";
        $self->has_config ? $self->config({load_config( $conf_file )}) : $self->config({});

        # Initialize logger
        my $log4perl_conf = $args->{log4perl_conf} || $self->config->{log4perl_conf};
        $self->log(init_logger( $log4perl_conf, __PACKAGE__ ));
        
        #if raw data supplied, store this in the object
        if ($args->{data}) {
        	$self->log->info("Using raw data");
        	$self->data($args->{data});
        }
    }


=head2 process_values

	  Usage      : $analyse->process_values(
					{
						process_field   => '4',
						process_handler => sub {
							my $val = shift;
							$val ? sprintf( "%.2f", -log($val) / log(10) ) : '0.00';
				  		}
					}
					);
	  Purpose    : Processes values in an Analyse dataset using the field provided and the process handler.
	  Returns    : Nothing
	  Arguments  : Hashref containing:
	  				process_field -  identifies field to use for process 
	  								number if data is array of arrays, field (ie. key) if data is an array of hashes
	  			    process handler - reference to sub for processing the value to process is provided as the first argument in the routine
	  Throws     : 
	  Status     : Public
	  Comments   : 
=cut
  
    sub process_values {
    	my ($self, $attrs) = @_;
    	my $process_field = $attrs->{process_field};
    	my $process_handler = $attrs->{process_handler};
    	
        my $data = $self->data;
        
        #do different things for hash and array based data structure
        if (ref($data->[0]) eq 'HASH') {
	        foreach my $item(@{$data}) {
	        	my $unprocessed = $item->{$process_field};
	        	my $processed = $process_handler->($unprocessed);
	        	$item->{$process_field}=$processed;
	        }
        }
        elsif (ref($data->[0]) eq 'ARRAY') {
        	foreach my $item(@{$data}) {
                my $unprocessed = $item->[$process_field];
                my $processed = $process_handler->($unprocessed);
                $item->[$process_field]=$processed;
            }
        }
    }

=head2 combine_data

	  Usage      :  my @data = ($data1, $data2);
	  				my $combi_data = HGVbaseG2P::Analyse::combine_data(\@data);
					
	  Purpose    : Combines data from multiple data sets (ie. arrays of hashes) and renumbers keys if they are the same
	               eg. combining results of three binning experiments will result in bin_content, bin_content1 and bin_content2.
	               NOTE: The datasets must represent the same points (on a graph for example)
	  Returns    : Array of hashes with the combined data
	  Arguments  : Arrayref containing datasets (ie. arrayref of hashes)
	  Throws     : 
	  Status     : Public
	  Comments   : 
=cut
  
    sub combine_data {
    	my ($data) = @_;
    	
    	my @newdata=();
    	my $set_no=0;
    	foreach my $set(@{$data}) {
    		
    	   my $item_no=0;
    	   foreach my $item(@{$set}) {
    	       foreach my $key(keys %{$item}) {
    	       	   if ($newdata[$item_no]->{$key}) {
    	       	       $newdata[$item_no]->{$key.$set_no}=$item->{$key};
    	       	   }
    	           else {
    	               $newdata[$item_no]->{$key}=$item->{$key};
    	           }      
    	       }
    	       $item_no++; 
    	   }
    	   $set_no++;    
    	}
        return \@newdata;
    }
    
=head2 use_threshold

	  Usage      :  $analyse->use_threshold(
					{
						value => 4,
						field => '4',
					}
					);
					
	  Purpose    : Processes data in an Analyse object to remove data below a threshold.
	  Returns    : Nothing
	  Arguments  : Hashref containing:
	  				value - value of threshold
	  				field - field (key) name or number of array position
	  				type - for future expansion - will default to above but may add below etc.
	  Throws     : 
	  Status     : Public
	  Comments   : 
=cut    

    sub use_threshold {
        my ($self, $attrs) = @_;
        my $type = $attrs->{type};
        my $field = $attrs->{field};
        my $value = $attrs->{value};
        my $old_data = $self->data;
        my @new_data = ();
        my $counter=0;
        
        my $is_array=0;
    	ref($old_data->[0]) eq 'ARRAY' and $is_array = 1;
    	
        #$self->log->info("Use threshold options:" . Dumper($attrs));
        if ($is_array) {
	        foreach my $item(@{$old_data}) {
	            if ($item->[$field] < $value) {
	            }
	            else {
	                #$self->log->debug("ADDED item ".$item->{$field}. "< $value -> $counter");
	                push @new_data, $item;
	            }
	        }
        }
        else {
        	foreach my $item(@{$old_data}) {
	            if ($item->{$field} < $value) {
	            }
	            else {
	                #$self->log->debug("ADDED item ".$item->{$field}. "< $value -> $counter");
	                push @new_data, $item;
	            }
	        }
        }
        	
        $self->data(\@new_data);
    }

=head2 statistic

	  Usage      :  $analyse->statistic(
					{
						type => 'Maximum',
						field => 'PValue',
					}
					);
					
	  Purpose    : Returns a statistic for the data in the Analyse object. Wraps the Statistic::Basic::* classes to do most of these
	               except Minimum and Maxumum
	  Returns    : The statistic based on the type and field supplied
	  Arguments  : Hashref containing:
	  				type - Type of Statistic. The options are Minimum, Maximum plus those available in CPAN including 
	  				       Mean, Mode, Median, StdDev and Variance
	  				field - field (key) name to use
	  Throws     : 
	  Status     : Public
	  Comments   : 
=cut    
 
    sub statistic {
        my ($self, $attrs) = @_;
        my $vector = $self->get_values_as_array($attrs->{field},$attrs->{sum}, $attrs->{logged});
        if ($attrs->{type} eq 'Maximum') {
        	my @sorted = reverse sort { $a <=> $b } @{$vector};
        	return $sorted[0];
        }
        if ($attrs->{type} eq 'Minimum') {
        	my @sorted = sort  { $a <=> $b } @{$vector};
        	return $sorted[0];
        }
        my $stats_mod = "Statistics::Basic::".$attrs->{type};
        $self->log->info("Using $stats_mod to retrieve statistic");
        $self->log->info("Passing $vector");
        my $stat = load_module($stats_mod,$vector);
        return $stat->query;
    }

=head2 get_values_as_array

	  Usage      :  $analyse->get_values_as_array('PValue');

	  Purpose    : Get an arrayref of values from the data part of an Analyse object using the field provided.
	  Returns    : An arrayref of values from an Analyse object.
	  Arguments  : Field name - key in hash of Analyse dataset
	               
	               optional:
	               Raw data - use data from outside the Analyse object
	  Throws     : 
	  Status     : Public
	  Comments   : 
=cut    
     
    sub get_values_as_array {
    	my ($self, $field, $sum)=@_;
        my @vector = ();
        
        use List::Util qw(sum);
        
        my $data = $self->data;
        
        if ($sum) {
        	foreach my $item(@{$data}) {
        		my $value=0;
        		foreach(split(/\|/, $item->{$field})) {
        			if ($_>0) { $value += poslog($_) };
        		}
	            push @vector, $value
	        }
        }
        else {
	        foreach my $item(@{$data}) {
	            push @vector, $item->{$field};
	        }
        }
        
        return \@vector;
    }
    
    
    
}    #package
1;

=head1 SEE ALSO



=head1 BUGS AND LIMITATIONS

There are no known bugs in this module.
Please report problems to Rob Free <rcfree@gmail.com>

=head1 CONTACT

This module is part of the HGVbase-G2P project

=head1 AUTHOR

Rob Free <rcfree@gmail.com>


=head1 LICENCE AND COPYRIGHT

Copyright (c) <2008> University of Leicester, Leicester, UK. All rights reserved.

This module is free software; you can redistribute it and/or
modify it under the same terms as Perl itself.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 

=head1 CVSINFO

$Id: Analyse.pm 1090 2008-11-26 11:14:36Z rcf8 $ 

=cut
 
 
 