# $Id: Bin.pm 1160 2009-04-20 14:04:10Z rcf8 $

=head1 NAME

HGVbaseG2P::Browser::Analyse::Bin - Specialised class for creation of binned values including generation of bin-based stats


=head1 SYNOPSIS
    
   package HGVbaseG2P::Browser::Analyse::*; {
    
   use strict;
   use warnings;

   # Use OIO inside-out object framework for our OO hierarachy
   use Object::InsideOut qw( HGVbaseG2P::Browser::Analyse );
   
=head1 DESCRIPTION

Uses the HGVbaseG2P::Browser::Analyse base module to provide basic functions for data retrieval and then adds additional binning features.
The commonly used methods are bin and bin_with_threshold. These output binned data as an array of hashes or arrays. The latter processes
the data so only values above a threshold are included.

=head1 SUBROUTINES/METHODS 

		
=cut

package HGVbaseG2P::Browser::Analyse::Bin; {
    
use strict;
use warnings;

# Use OIO inside-out object framework for our OO hierarachy
use Object::InsideOut qw( HGVbaseG2P::Browser::Analyse );
use Object::InsideOut::Metadata;
use HGVbaseG2P::Util qw(load_module);

use English qw( -no_match_vars );
use Data::Dumper;
use Math::Round qw(nlowmult);

my @bin_data : Field :All(bin_data); #binned form of data
my @output_to : Field :All(output_to); #output target eg. hash, array

=head2 latest_max

	  Usage      : my $max = $bin_analyse->latest_max;
	  Purpose    : Get maximum value of binned data once it has been processed (object stores the latest_data in $self->bin_data
	  Returns    : Value of maximum
	  Arguments  : None
	  Throws     : 
	  Status     : Public
	  Comments   : 
=cut

sub latest_max {
	my ($self) = @_;
	my $bin_data = $self->bin_data;

    if ($self->output_to eq 'array') {
	    my @sorted_items = reverse sort { $a->[1] <=> $b->[1] } @{$bin_data};
	    warn "latest max:".$sorted_items[0]->[1];
	    return $sorted_items[0]->[1];
    }
    else {
    	my @sorted_items = reverse sort { $a->{bin_content} <=> $b->{bin_content} } @{$bin_data};
	    warn "latest max:".$sorted_items[0]->{bin_content};
	    return $sorted_items[0]->{bin_content};
    }
}

=head2 clear_latest

	  Usage      : Usually internally used
	  Purpose    : Clear the binned data store
	  Returns    : Nothing
	  Arguments  : None
	  Throws     : 
	  Status     : Public
	  Comments   : 
=cut

sub clear_latest {
	my ($self) = @_;
	$self->bin_data([]);
}

=head2 bin

	  Usage      : my $binned = $analyse->bin(
						{
							position_field => '2',
							bin_size       => $attrs->{bin_size},
							total_length   => $self->chr_length,
							value_field    => '4',
							bin_content    => 'Count',
							calc_ranges    => 1,
							start_position      => $self->round_start,
							stop_position      => $self->round_stop,
							output_to      => 'array'
						}
					);
	  Purpose    : Perform binning algorithm.
	  Returns    : Binned data (in an arrayref of hashes)
	               Hashes contain the following info:
	                 bin_no - number of bin
	                 bin_content - depends on bin_content specified can be a value or an arrayref
	                 bin_start - depends on whether this is asked for (calculated from the start and bin_size)
	                 bin_stop - depends on whether this is asked for (calculated from the start and bin_size)
	  Arguments  : Hashref containing:
	  						position_field - hash key or array position specifying the field containing the position
							bin_size       - size of bins to create
							total_length   - total length of zone to create bins for
							value_field    - hash key or array position specifying the field containing the value
							bin_content    - output type of binning - available options are array, values, count         
							calc_ranges    - generate bin_start and bin_stop in hash as well as bin_content
							start_position      - start point for calc_ranges
							stop_position      - stop point for calc_ranges
							output_to      - output to array of arrays (array) or hashes (hash)
	  Throws     : 
	  Status     : Public
	  Comments   : Calls the bin_to_array or bin_to_hash method depending on the value of output_to
=cut

sub bin {
	my ($self, $attrs)=@_;
	$self->log->debug(Dumper($attrs));
	my $output_to = $attrs->{output_to} || 'hash';
	$self->output_to($output_to);
	
	my $method = "bin_to_".$output_to;
	return $self->$method($attrs);
}

#internal method to bin data to array of arrays (faster)
sub bin_to_array {
    my ($self, $attrs)=@_;

    my $position_field = $attrs->{position_field};
    my $bin_size = $attrs->{bin_size};
    my $total_length = $attrs->{total_length};
    my $bin_content = $attrs->{bin_content};
    my $value_field = $attrs->{value_field};
    my $process_handler = $attrs->{process_handler};
    my $threshold = $attrs->{threshold};
    
    my $bin_start = $attrs->{start_position} || 1;
    my $bin_stop = $attrs->{stop_position} || $total_length;
    $self->log->trace("bin_start:$bin_start and bin_stop:$bin_stop");
    my $output_to = 'array';
    
    my $data = $self->data;
    $self->log->trace("Total length:$total_length and bin size:$bin_size");
    
    my $number_of_bins = int(($bin_stop-$bin_start)/$bin_size)+1;
    $self->log->trace("No of bins $number_of_bins");
    my @bins = ();
    
    #set empty bin values (arrayrefs)
    	for(0..$number_of_bins) { 
        	$bins[$_]->[0]="$_";
        	$bins[$_]->[1]=[];
    	}
	
	#if input is an array of arrays then set flag
    my $is_array=0;
    ref($data->[0]) eq 'ARRAY' and $is_array = 1;
	
	#loop through each item in the data and generate bins with arrays of values
    foreach my $item(@{$data}) {
    	
    	#get position and value from item
        my ($pos,$value); 
        if ($is_array) {
            $pos = $item->[$position_field]-$bin_start;
            $value = $item->[$value_field];
        }
        else {
            $pos = $item->{$position_field}-$bin_start;
            $value = $item->{$value_field};
        }
        
        #set the bin position value based on item pos / bin size 
        #go the next item if the pos is greater than the expected number of bins
        my $bin_pos = nlowmult(1,$pos/$bin_size);
        next if $bin_pos > $number_of_bins;
        	
        	#add the value to the array in the bin
        	if ($bin_content eq 'array') {
	            push @{$bins[$bin_pos]->[1]},$item;
	        }
	        else {
	            push @{$bins[$bin_pos]->[1]},$value;
	        }
    } 
    
    #calculate stats for the contents of all bins
    if ($bin_content ne 'array' and $bin_content ne 'values') {
    	@bins = @{$self->calculate_bin_stats(\@bins,$bin_content,$output_to)};
    }
    
    #process binned stats or values if a process handler is provided
    if ($process_handler) {
         	foreach my $bin(@bins) {
            	$bin->[1]=$process_handler->($bin->[1]);
        	}
    } 
    
    #calculate bin ranges (start and stop) based on provided start and stop
    if ($attrs->{calc_ranges}) {
        @bins = @{$self->calculate_ranges(\@bins,$bin_start,$attrs->{bin_size},$output_to)};
    }
    
    #return final binned data
    $self->bin_data(\@bins);
    return \@bins;
}

#internal method to bin data to array of hashes (slower but useful for development)
sub bin_to_hash {
    my ($self, $attrs)=@_;

    my $position_field = $attrs->{position_field};
    my $bin_size = $attrs->{bin_size};
    my $total_length = $attrs->{total_length};
    my $bin_content = $attrs->{bin_content};
    my $value_field = $attrs->{value_field};
    my $process_handler = $attrs->{process_handler};
    my $threshold = $attrs->{threshold};
    
    my $bin_start = $attrs->{start_position} || 1;
    my $bin_stop = $attrs->{stop_position} || $total_length;
    
    my $output_to = 'hash';
    
    my $data = $self->data;
    $self->log->trace("Total length:$total_length and bin size:$bin_size");
    
    my $number_of_bins = int(($bin_stop-$bin_start)/$bin_size);
    $self->log->trace("No of bins $number_of_bins");
    
    #set empty bin values (arrayrefs)
    my @bins = ();

	    for(0..$number_of_bins) { 
	        $bins[$_]->{bin_no}="$_";
	        $bins[$_]->{bin_content}=[];
	    }
    
    #if input is an array of arrays then set flag
    my $is_array=0;
	
    ref($data->[0]) eq 'ARRAY' and $is_array = 1;
	
	#loop through each item in the data and generate bins with arrays of values
    foreach my $item(@{$data}) {
        my ($pos,$value); 
 		
 		#get position and value from item
        if ($is_array) {
            $pos = $item->[$position_field]-$bin_start;
            $value = $item->[$value_field];
        }
        else {
            $pos = $item->{$position_field}-$bin_start;
            $value = $item->{$value_field};
        }
        
        #set the bin position value based on item pos / bin size 
        #go the next item if the pos is greater than the expected number of bins
        
        my $bin_pos = nlowmult(1,$pos/$bin_size);
        
        #goto next item if outside binning area
        next if $bin_pos > $number_of_bins || $bin_pos < 0;

        #$self->log->debug("bin_pos: $bin_pos, number_of_bins:$number_of_bins, value:".$item->{$value_field});
        
        	#add the value to the array in the bin
	        if ($bin_content eq 'array') {
	            push @{$bins[$bin_pos]->{bin_content}},$item;
	        }
	        else {
	            push @{$bins[$bin_pos]->{bin_content}},$value;
	        }
    } 
    
    #calculate stats for the contents of all bins
    if ($bin_content ne 'array' and $bin_content ne 'values') {
    	@bins = @{$self->calculate_bin_stats(\@bins,$bin_content,$output_to)};
    }
    
    #process binned stats or values if a process handler is provided
    if ($process_handler) {
	        foreach my $bin(@bins) {
	            $bin->{bin_content}=$process_handler->($bin->{bin_content});
	        }
    } 
    
    #calculate bin ranges (start and stop) based on provided start and stop
    if ($attrs->{calc_ranges}) {
        @bins = @{$self->calculate_ranges(\@bins,$bin_start,$attrs->{bin_size},$output_to)};
    }
    
    #return final binned data
    $self->bin_data(\@bins);
    return \@bins;
}

=head2 calculate_bin_stats

	  Usage      : @bins = @{$self->calculate_bin_stats(\@bins,"Max",'array')};
	  Purpose    : Calculate statistics for data binned into arrays
	  Returns    : New binned data object containing either array or hash of binned data
	  Arguments  : Binned data - data binned into arrays
	               Statistic - value to generate for each bin. Options include Max, Min, Count and Statistics::Basic::* modules
	                           eg. StDev, Variance etc.
	               Output to - array of arrays or hashes
	  Throws     : 
	  Status     : Public
	  Comments   : 
=cut

sub calculate_bin_stats {
	my ($self, $bins, $bin_stat, $output_to) = @_;
	
    foreach my $item(@{$bins}) {
    	my @items = ();
    	
    	if ($output_to eq 'array') {
    		@items = @{$item->[1]};
    	}
    	else {
    		@items = @{$item->{bin_content}};
    	}
    	
        if ($bin_stat eq 'Max') {
        	#warn "item:".$item->{bin_no}."-".Dumper($item);
            if (@items) {
            	my @sorted_items = reverse sort { $a <=> $b } @items;
            #$self->log->info("reverse items:".Dumper(\@sorted_items));
            	$output_to eq 'array' ? $item->[1]=$sorted_items[0] : $item->{bin_content}=$sorted_items[0];
            }
            else {
            	$output_to eq 'array' ? $item->[1]='0.00' : $item->{bin_content}='0.00';
            }
            #warn "content:".$item->{bin_content};
        }
        elsif ($bin_stat eq 'Min') {
        	if (@items) {
            	my @sorted_items = sort { $a <=> $b } @items;
            #$self->log->info("reverse items:".Dumper(\@sorted_items));
            	$output_to eq 'array' ? $item->[1]=$sorted_items[0] : $item->{bin_content}=$sorted_items[0];
            }
            else {
            	$output_to eq 'array' ? $item->[1]='0.00' : $item->{bin_content}='0.00';
            }
        }
        elsif ($bin_stat eq 'Count') {
        	$output_to eq 'array' ? $item->[1]=scalar(@items) : $item->{bin_content}=scalar(@items);
        }
        else {
            my $stats_mod = "Statistics::Basic::".$bin_stat;
            my $stat = load_module($stats_mod,\@items);
            $output_to eq 'array' ? $item->[1]=$stat->query : $item->{bin_content} = $stat->query;
        }
    }
    return $bins;
}

=head2 bin_with_threshold

	  Usage      : my $binned = $analyse->bin_with_threshold(
						{
							position_field => '2',
							bin_size       => $attrs->{bin_size},
							total_length   => $self->chr_length,
							value_field    => '4',
							bin_content    => 'Count',
							calc_ranges    => 1,
							bin_start      => $self->round_start,
							bin_stop       => $self->round_stop,
							output_to      => 'array',
							threshold_value => 4,
						    threshold_type => 'number'
						}
					);
	  Purpose    : Perform binning algorithm with a threshold on resulting bins
	  Returns    : Binned data passing a threshold - see 'bin'
	  Arguments  : Hashref containing the same as 'bin':
	  						position_field - hash key or array position specifying the field containing the position
							bin_size       - size of bins to create
							total_length   - total length of zone to create bins for
							value_field    - hash key or array position specifying the field containing the value
							bin_content    - output type of binning - available options are array, values, count         
							calc_ranges    - generate bin_start and bin_stop in hash as well as bin_content
							bin_start      - start point for calc_ranges
							bin_stop       - stop point for calc_ranges
							output_to      - output to array of arrays (array) or hashes (hash)
				   Also:
				            threshold_value - value to use as threshold
				            threshold_type - type of threshold which can be number (only data returned are top X values)
				                             or percent (only the top percentage of values are returned eg. 20% of the top values)
	  Throws     : 
	  Status     : Public
	  Comments   : Using the standard binning algorithm to bin data into arrays initially then uses this data to remove values not passing
	               the threshold. Finally calculates statistics for these eg. mean of top 20% of values
=cut


sub bin_with_threshold {
    my ($self, $attrs) = @_;
    my $content_type = $attrs->{bin_content};
    my $output_to = $attrs->{output_to};
    
    #get bins with values in and then use these to build means etc. - use standard bin with attributes passed in
    $attrs->{bin_content} = 'values';
    my @bins = @{$self->bin($attrs)};
    
    my $t_value = $attrs->{threshold_value};
    my $t_type = $attrs->{threshold_type};
    
    my $item_no=0;
    
    #loop through each data item
    foreach my $item (@bins) {
    	
    	my $no_in_bin = scalar(@{$item->{bin_content}});
	    my $bin_content = $output_to eq 'array' ? $item->[1] :$item->{bin_content};
	    
	    #for threshold type 'number' reverse sort the data (ie. descending) and use the top X values  
	    if ($t_type eq 'number') {
	       my $threshold = $t_value;
	       my @sorted = reverse sort { $a <=> $b } @{$bin_content};
	       #$self->log->info("Sorting count: no in bin:$no_in_bin, threshold value:$t_value");
		       
	       $threshold>$no_in_bin and $threshold = $no_in_bin;
	       
           my @slice = @sorted[0..$threshold-1];
           $output_to eq 'array' ? $item->[1] = \@slice : $item->{bin_content} = \@slice;
 
	    }
	    #for threshold type 'percent' reverse sort the data (ie. descending) and use the top X% of values  
	    elsif ($t_type eq 'percent') {
	       my $threshold = $no_in_bin * ($t_value/100);
           my @sorted = reverse sort { $a <=> $b } @{$bin_content};
           #$self->log->info("Sorting percent: no in bin:$no_in_bin, threshold value:$t_value");
           
           my @slice = @sorted[0..$threshold-1];
           $output_to eq 'array' ? $item->[1] = \@slice : $item->{bin_content} = \@slice;
           #$self->log->info("Sorted: new threshold: $threshold, content:".Dumper($bin_content));

        }    
        $item_no++;
    }
    
    #calculate the stats on the thresholded data
    @bins = @{$self->calculate_bin_stats(\@bins,$content_type)};
    
    #return the thresholded data
    $self->bin_data(\@bins);
    return \@bins;
}

=head2 calculate_ranges

	  Usage      : @bins = @{$self->calculate_ranges(\@bins,$bin_start,$bin_size,$output_to)};
	  Purpose    : Calculate ranges for each for item binned into arrays
	  Returns    : New binned data object containing bin_start and bin_stop keys in each hash
	  Arguments  : Binned data - data binned into arrays
	               Start - start point of zone
	               Bin size - size of bins to generate ranges for
	               Output to - Specify if array of arrays (array) or hashes (hash)
	  Throws     : 
	  Status     : Public
	  Comments   : 
=cut

sub calculate_ranges {
    my ($self, $bins, $bin_start, $bin_size, $output_to) = @_;
    
    if ($output_to eq 'array') {
	    foreach my $item(@{$bins}) {
	        $item->[2] = $bin_start;
	        $bin_start+=$bin_size-1;
	        $item->[3] = $bin_start;
	        $bin_start++;
	    }
	    
    }
    else {
    	foreach my $item(@{$bins}) {
	        $item->{bin_start} = $bin_start;
	        $bin_start+=$bin_size-1;
	        $item->{bin_stop} = $bin_start;
	        $bin_start++;
	    }
    }
    
    return $bins;
}

}
1;

=head1 SEE ALSO

L<HGVbaseG2P::FileParser>, L<HGVbaseG2P::Database> and related classes

=head1 BUGS AND LIMITATIONS

There are no known bugs in this module.
Please report problems to Rob Free <rcfree@gmail.com>

=head1 CONTACT

This module is part of the HGVbase-G2P project

=head1 AUTHOR

Rob Free <rcfree@gmail.com>

=head1 LICENCE AND COPYRIGHT

Copyright (c) <2008> University of Leicester, Leicester, UK. All rights reserved.

This module is free software; you can redistribute it and/or
modify it under the same terms as Perl itself.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 

=head1 CVSINFO

$Id: Bin.pm 1160 2009-04-20 14:04:10Z rcf8 $ 

=cut

