# $Id: Core.pm 1258 2009-08-03 11:13:08Z rcf8 $

=head1 NAME

HGVbaseG2P::Browser::Core - High level logic for G2P genome and region view browsers


=head1 SYNOPSIS

e.g. for region view:

	use HGVbaseG2P::Browser::Core;

	my $browser = HGVbaseG2P::Browser::Core->new({
				  conf_file =>
					"/Users/robf/Projects/hgvbaseG2P/perl/conf/hgvbase.conf",
					feature_list=>$self->new_feature_list,
					autoconnect=>1
	  });
	  
	  my %attrs = %{$self->configuration};
	  $attrs{ref}=$ref;
	  $attrs{seg_start}=$abs_start;
	  $attrs{seg_stop}=$abs_stop;
	  
	  my $feature_list = $browser->generate_region_features(\%attrs);
	  
=head1 DESCRIPTION

The Browser::Core module makes use of the Database::Browser and Browser::Analyse modules to generate feature data for GBrowse plugins.

The separation of core logic makes it easier to test the module by use of regression/unit testing. It also reduces
the amount of code in the GBrowse config file which is useful for error checking and debugging etc.

=head1 SUBROUTINES/METHODS 

=cut

package HGVbaseG2P::Browser::Core;
{

	use strict;
	use warnings;

	# Use OIO inside-out object framework for our OO hierarachy
	use Object::InsideOut;
	use Object::InsideOut::Metadata;

	use English qw( -no_match_vars );
	use Data::Dumper;
	use Math::Round qw(:all);
	use Bio::Graphics::Feature;
	use Bio::Graphics::FeatureFile;
	use HGVbaseG2P::Database::Browser;
	use HGVbaseG2P::Browser::Analyse::Bin;
	use HGVbaseG2P::Util qw(load_config init_logger);
	use HGVbaseG2P::Browser::Util
	  qw(chr_list chr_length split_data_by_field format_bases poslog);
	
	use List::Util qw(sum max);
	use List::MoreUtils qw(true firstval);

	use Carp qw(croak cluck confess);
	use constant GENOME_MB => 3; #genome bin size default (megabases)
	use constant GENOME_BP => 3000000; #genome bin size default (base pairs)

	my $VERSION = "1.00";

	# Private variables & constants
	my @log : Field : All(log);                   #contains Log4perl Logger
	my @config : Field : All(config);             #contains loaded config
	my @db : Field : All(db);                     #contains browser DB object
	my @seg_start : Field : All(seg_start);       #contains actual segment start
	my @seg_stop : Field : All(seg_stop);         #contains actual segment stop
	my @round_start : Field : All(round_start);   #contains rounded-down start
	my @round_stop : Field : All(round_stop);     #contains rounded-up stop
	my @chr : Field : All(chr);                   #contains chr no (eg. 5)
	my @ref : Field : All(ref);                   #contains full chr (eg. Chr5)
	my @features : Field : All(features)
	  ;    #reference to all features whether genome or region based

	my @_resultsetid : Field : All(_resultsetid)
	  ;    #analysis method id as array of IDs
	my @is_displayed : Field : All(is_displayed)
	  ;    #flag for whether to display tracks

	my @threshold : Field : All(threshold);    #significance threshold
	my @autodb : Field : All(autodb);          #flag for auto create DB object
	my @compare : Field : All(compare)
	  ;    #type of comparison between result sets (ie. single or combined)
	my @logscale : Field : All(logscale)
	  ;    #flag to specify if 'count' graphs have log scale
	my @viewtype : Field : All(viewtype);  #specifies genome or region view type

	my @featureno : Field : All(featureno)
	  ;    #specifies incrementable count for features
	my @summaries : Field : All(summaries); #access to resultset summary records
	my @_tracks : Field : All(_tracks);     #private _tracks property
	my @_settings : Field : All(_settings); #private _settings property
	my @messages : Field : All(messages); #contains error/warn messages
	
	# Initialization arguments
	my %init_args : InitArgs = (
		'conf_file'     => '',
		'log4perl_conf' => '',
		'browser_db'    => '',
		'feature_list'  => '',
	);

	# Initialization method (see OIO docs for details)
	sub _init : Init {
		my ( $self, $args ) = @_;

		# Load supplied config file, if non provided use default
		my $conf_file = $args->{conf_file};
		$self->config( { load_config($conf_file) } );

		# Initialize logger
		my $log4perl_conf = $args->{log4perl_conf}
		  || $self->config->{log4perl_conf};
		$self->log( init_logger( $log4perl_conf, __PACKAGE__ ) );

		#use supplie list or create new features list
		$self->features(
			$args->{feature_list} || Bio::Graphics::FeatureFile->new(
				-smart_features => 1,
				-safe           => undef
			)
		);
		$self->settings( {} );
		$self->messages([]);
	}
	
	#increments the feature number - used internally
	sub next_featureno {
		my ($self) = @_;
		my $featureno = $self->featureno;
		if ( $self->featureno ) {
			$featureno++;
		}
		else {
			$featureno = 1;
		}
		$self->featureno($featureno);
		return $featureno;
	}
	
	#populates the _resultsetid accessors (if value provided) or returns a sorted array of resultsetids
	sub resultsetid {
		my ( $self, $value ) = @_;
		if ($value) {
			$self->_resultsetid($value);
		}
		else {
			if (!$self->_resultsetid) {
				return undef;
			}
			my @resultsetid = sort { $a <=> $b } @{ $self->_resultsetid };
			return \@resultsetid;
		}
	}
	
	#internal - removes resultsets from the object which are either not to be displayed OR are not present in the Browser DB
	#returns an array of messages related to the resultsets
	sub remove_unavailable_resultsets {
		my ($self)=@_;
		my $summaries = $self->summaries;
		my @messages = ();
		my @available_rs=();
		foreach my $rsid(@{$self->resultsetid}) {
			my $summary = $summaries->{$rsid};
			if ($summary) {
				if ($summary->{IsDisplayed}) {
					push @available_rs, $rsid;
				}
				else {
					push @messages,"Data from resultset ID '". $rsid. "' is not available due to anonymity issues";
				}
			}
			else {
				push @messages,"Data from resultset ID '". $rsid. "' does not seem to be present in the Browser Database";
			}
				
		}
		$self->resultsetid(\@available_rs);
		return @messages;
	}

=head2 generate_region_header

	  Usage      : my ($html, $sess_data) = $hgvbrowser->generate_region_header({
						resultsetid => $resultsetid,
						threshold => $threshold,
						sess_data => $session->config_hash->{plugins}->{'P value Data'},
						chr=>$segment->seq_id,
						start=>$segment->start,
						length=>$segment->length
	               });
	  Purpose    : Generate specific HTML header and session data for region browser containing study info and das link etc.
	  Returns    : HTML header text and hash containing Gbrowse session data
	  Arguments  : In attributes hash:
	  			   sess_data => session data hash from GBrowse plugin
	               resultsetid => analysis method id (generally from CGI param)
	               threshold => significance threshold (generally from CGI param)
	               chr => chromosome name (ChrN)
	               start => start of GBrowse segment
	               length => length of GBrowse segment
	               
	  Throws     : 
	  Status     : Public
	  Comments   : Separates web logic from feature generation - all values required are passed in/out.
	               This means the session data hash and parameter data are passed in and returned.
	               N.B. This may change as it is not as important as in V1 of the Browser
	
=cut

	sub generate_region_header {
		my ( $self, $attrs ) = @_;

		my $sess_data       = $attrs->{sess_data};
		my $par_resultsetid = $attrs->{resultsetid};
		my $par_threshold   = $attrs->{threshold};
		my $chr             = $attrs->{chr};
		my $start           = $attrs->{start};
		my $length          = $attrs->{length};
		my $par_settings    = $attrs->{settings};
		my $par_tracks      = $attrs->{tracks};
		my $threshold;
		my $fulllabel;
		my $html = '';

		my $sess_resultsetid   = $sess_data->{resultsetid};
		my $sess_resultsetinfo = $sess_data->{resultsetinfo};
		my $sess_fulllabel     = $sess_data->{fulllabel};
		my $is_displayed       = $sess_data->{is_displayed};

		#compare session and CGI params
		my ( $resultsetid, $get_info, $resultsetinfo ) =
		  $self->compare_resultsetid( $par_resultsetid, $sess_resultsetid );

		#set values in session hash
		$par_threshold and $sess_data->{threshold} = $par_threshold;
		$sess_data->{resultsetid}  = $resultsetid;
		$sess_data->{is_displayed} = $is_displayed;
		$self->log->info( "header_settings:" . $par_settings );
		$par_tracks   and $sess_data->{tracks}   = $par_tracks;
		$par_settings and $sess_data->{settings} = $par_settings;

		#generate HTML for header
		$chr =~ /Chr(.+)/;
		return ( $html, $sess_data );
	}

=head2 get_resultset_summary

	  Usage      : my ($label, $name) = $hgvbrowser->get_resultset_summary(7);
	  Purpose    : Get specific info for a single resultset
	  Returns    : The FullLabel and Study_Name properties for the resultset
	  Arguments  : A resultset ID 
	  Throws     : 
	  Status     : Public
	  Comments   : 
	
=cut

	sub get_resultset_summary {
		my ( $self, $resultsetid ) = @_;
		my $db = $self->get_db;
		my $rs = $db->get_resultset_details( { resultsetid => $resultsetid } );
		return ( $rs->[0]->{FullLabel}, $rs->[0]->{Study_Name} );
	}

=head2 get_resultset_summaries

	  Usage      : my ($summaries) = $hgvbrowser->get_resultset_summaries;
	  Purpose    : Get summary data for all resultsets in the Browser object
	  Returns    : Hashref containing key of Resultset ID and value of Resultset description for use by the browser
	  Arguments  : None   
	  Throws     : 
	  Status     : Public
	  Comments   : 
	
=cut

	sub get_resultset_summaries {
		my ($self)    = @_;
		my $db        = $self->get_db;
		my $summaries = {};
		my $studies   = {};
		foreach my $rsid ( @{ $self->resultsetid } ) {
			my $rs = $db->get_resultset_details( { resultsetid => $rsid } );
			next if !$rs;
			warn "rs:$rsid".Dumper($rs);
			
			my $studyname     = $rs->{Study_Name};
			my $resultsetname = $rs->{Resultset_Name};

			#trim non-supported chars
			$studyname     =~ s/\:|\,|\(|\)//g;
			$resultsetname =~ s/\:|\,|\(|\)//g;
			$rs->{UniqueName} = $studyname . " - " . $resultsetname . " " . $rs->{Resultset_Identifier};
			$summaries->{$rsid} = $rs;
		}
		return $summaries;
	}

=head2 compare_resultsetid

	  Usage      : my ($resultsetid, $get_info,$resultsetinfo) = $self->compare_resultsetid($par_resultsetid, $sess_resultsetid);
	  Purpose    : Compare analysis method ID retrieved from parameter and session and returns get_info flag and other useful info
	  Returns    : $resultsetid - resultsetid to use
	               $get_info - flag to specify whether to read DB for study info 
	               $resultsetinfo - method info for empty resultsetid
	  Arguments  : $par_resultsetid - resultsetid from CGI parameter
	               $sess_resultsetid - resultsetid from session parameter
	  Throws     : 
	  Status     : Public
	  Comments   : 
	
=cut

	sub compare_resultsetid {
		my ( $self, $par_resultsetid, $sess_resultsetid ) = @_;
		my ( $resultsetid, $get_info, $rsinfo );
		if ( $par_resultsetid && $sess_resultsetid ) {
			
			if ( $par_resultsetid ne $sess_resultsetid ) {
				$sess_resultsetid = $par_resultsetid;
				$resultsetid      = $par_resultsetid;
				$get_info         = 1;
			}
			else {
				$resultsetid = $par_resultsetid;
				$get_info = 1;
			}
		}
		elsif ( $par_resultsetid && !$sess_resultsetid ) {
			$resultsetid = $par_resultsetid;
			$get_info    = 1;
		}
		elsif ( !$par_resultsetid && $sess_resultsetid ) {
			$resultsetid = $sess_resultsetid;
			$get_info    = 1;
		}
		else {
			$rsinfo = "No Result Set";
		}
		return ( $resultsetid, $get_info, $rsinfo );
	}

=head2 get_db

	  Usage      : my $db = $self->get_db;
	  Purpose    : Retrieve HGVbaseG2P::Database::Browser object. If autodb = 1 then auto create it
	  Returns    : HGVbaseG2P::Database::Browser object from $self->db
	  Arguments  : None
	  Throws     : 
	  Status     : Public
	  Comments   : 
	
=cut

	sub get_db {
		my ( $self, $db ) = @_;
		if ( $self->autodb ) {
			if ( !$self->db ) {
				my $db =
				  HGVbaseG2P::Database::Browser->new(
					{ conf_file => $self->config } );
				$self->db($db);
				return $db;
			}
			else {
				if ( $self->db->dbh ) {
				}
				else {
					$self->db->reconnect;
				}
				return $self->db;
			}
		}
		else {
			return $self->db;
		}
	}

=head2 get_chr_max_count

	  Usage      : my ($max_count) = $browser->get_chr_max_count({
    				data=>/@data,
    			   });
    			   
	  Purpose    : Determine maximum 'bin_counts' value for supplied bin data
	  Returns    : Maximum count in data
	  Arguments  : In hash ref
	  				 data - arrayref containing data
	  Throws     : 
	  Status     : Public
	  Comments   : 
	
=cut

	sub get_chr_max_count {
		my ( $self, $attrs ) = @_;
		my $data    = $attrs->{data};
		my $analyse =
		  HGVbaseG2P::Browser::Analyse->new(
			{ data => $data, conf_file => $self->config } );
		my $chr_max_count =
		  $analyse->statistic(
			{ field => 'bin_counts', type => 'Maximum', sum => 1, log => 1 } );
		return $chr_max_count;
	}

=head2 generate_genome_features

	  Usage      : my ($feature_data,$max_count) = $browser->generate_karyotype_features({
    				bin_size=>3000000,
    				threshold=>$threshold,
    				resultsetid=>$resultsetid,
    				logscale=>$logscale,

    			   });
    			   
	  Purpose    : Main method to generate feature data for use by modified hgvbrowse_karyotype CGI script. Also provides the max value for stacked plot for any chromosome
	  Returns    : $feature_data - hgvbrowse_karyotype feature data,
	  			   $max_count - max count found in data set
	  Arguments  : In hash ref
	  				 threshold - -log of P value threshold
	  				 resultsetid - comma-separated list of resultsetids to generate genome data for
	  				 logscale - flag indicating whether log (1) or linear (0) scale should be used
	  Throws     : 
	  Status     : Public
	  Comments   : 
	
=cut

	sub generate_genome_features {
		my ( $self, $attrs ) = @_;
		$self->viewtype('genome');
		my $rs_counts={};
		my @max_counts   = ();
		my $config       = $self->config;
		my @resultsetids = split( /,/, $attrs->{resultsetid} );
		my $threshold    = $attrs->{threshold};
		my $logscale     = $attrs->{logscale} || 1;
		my $chr          = $attrs->{chr};
		$self->tracks( $attrs->{tracks} );
		$self->settings( $attrs->{settings} );

		#set browser accessors with attributes
		$chr          and $self->chr($chr);
		$threshold    and $self->threshold($threshold);
		@resultsetids and $self->resultsetid( \@resultsetids );
		$logscale     and $self->logscale($logscale);
		
		foreach(@{$self->resultsetid}) {
			$rs_counts->{$_}=0;
		}
			
		$self->seg_start(1);

		#get precalculated counts for analysis and threshold
		$self->summaries( $self->get_resultset_summaries );
		my @messages = $self->remove_unavailable_resultsets;
		
		
		my $chr_data =
		  $self->get_marker_binned_data( { bin_size => GENOME_MB } );
		my $height =
		  ( $self->chr && length( $self->chr ) < 3 )
		  ? $self->setting('region_stacked_height')
		  : $self->setting('genome_stacked_height');

		#add sig counts glyph
		$self->add_sig_counts_glyph( { height => $height } );

   #if genome_present track switched on add glyph indicating presence of markers
		if ( $self->track('genome_present') ) {
			$self->add_bins_present_glyph( { height => int( $height / 10 ) } );
		}

	 #loop through each chromosome and add stacked plot data for each bin within
		foreach my $chr_name ( keys %{$chr_data} ) {
			my ($chr_max_count);
			($chr_max_count, $rs_counts) =
			  $self->add_sig_counts(
				{ data => $chr_data->{$chr_name}, chr => $chr_name, rs_counts => $rs_counts } );

			if ( $self->track('genome_present') ) {

				$self->seg_stop( chr_length($chr_name) );
				$self->add_bins_present(
					{
						data             => $chr_data->{$chr_name},
						bin_size         => GENOME_BP,
						bin_size_compact => GENOME_MB,
						chr              => $chr_name,
					}
				);
			}
			defined($chr_max_count) and push @max_counts, $chr_max_count;
		}

		#find largest count in whole genome - log it if logscale flag is 1
		my @sorted = reverse sort { $a <=> $b } @max_counts;

		#reset the max_score attribute of the type to the largest count
		my $max_count = $sorted[0];
		$self->features->set( 's', max_score => $max_count );
		
		my $html;
		if (scalar(@messages)>0) {
			$html = "<div style='background-color:#ffaaaa;border:solid 1px #ff0000;padding:0.5em;margin-right:0.5em'>". join("<br>",@messages)."</div>";
		}
		#return the feature list
		return ( $self->features, $max_count, $rs_counts, $html );
	}

=head2 add_sig_counts

	  Usage      : my ($max_count) = $browser->add_sig_counts({
    				chr=>'9',
    				bin_size=>3000000,
    				data=>\@data,
   			   });
    			   
	  Purpose    : Add binned significant count features to specific chromosome Main method to generate feature data for use by modified hgvbrowse_karyotype CGI script. Also provides the max value for stacked plot for any chromosome
	  Returns    : $max_value - highest value for bin_count found in data set
	  Arguments  : In hash ref
	  				 chr - Chromosome number (automatically adds 'Chr' prefix if region view
	  				 binsize - comma-separated list of resultsetids to generate genome data for
	  				 data - arrayref of binned data (see HGVbaseG2P::Browser::Analyse::Bin)
	  Throws     : 
	  Status     : Public
	  Comments   : 
	
=cut

	sub add_sig_counts {
		my ( $self, $attrs ) = @_;
		
		my $chr        = $attrs->{chr} || $self->chr;
		my $rs_counts = $attrs->{rs_counts} || {};
		my $logscale   = $self->logscale;
		my $chr_length = chr_length($chr);

		my $features = $self->features;
		my $data     = $attrs->{data};
		my $bin_size = $attrs->{bin_size} || 3000000;

		my $counter    = 1;
		my @sum_scores = ();
		
		#define expected value if totally empty (eg. -1|-1|...)
		my $empty_score = "-1|" x scalar(@{$self->resultsetid});
		$empty_score = substr($empty_score,0,length($empty_score)-1);
		
		#define expected value if no markers in the region (eg. 0|0|..)
		my $zero_score = "0|" x scalar(@{$self->resultsetid});
		$zero_score = substr($zero_score,0,length($zero_score)-1);
		
		my $chr_prefix = $self->viewtype eq 'region' ? 'Chr' : ''; #add Chr prefix if region view
		
		#loop through each item in data
		for my $item ( @{$data} ) {
			my $actual_score = $item->{'bin_counts'} || '0';
			my $score = 0;
			my $sum_score;
			my @temp = ();
			
			#get array of actual scores for data item and loop through them
			my @scores = split( /\|/, $actual_score );
			foreach ( @scores ) {
				
				#if score is not empty or zero process for scale-type and add to array
				#otherwise add a '0'
				if ( $_ ne -1 && $_ ne 0 ) {

					#log count if logscale flag is 1
					if ($self->setting('scale_type') eq 'Log10') {
						push @temp, sprintf( "%.3f", poslog( $_ + 1 ) );
					}
					else {
						push @temp, $_;
					}
				}
				else {
					push @temp, '0';
				}
				#join the scores together with | and get a total for the bin
				$score = join( '|', @temp );
				$sum_score = sum(@temp);
			}

			#prevent start and stop going off ends of chromosome
			my $range_start =
			    $item->{'bin_start'} >= $bin_size
			  ? $item->{'bin_start'}
			  : 1;
			my $range_stop =
			    $item->{'bin_stop'} < $chr_length
			  ? $item->{'bin_stop'}
			  : $chr_length;
			my $name = "s$chr" . "_" . $counter;

			my $short_start = sprintf( "%03d", int( $range_start / 1000000 ) );

			#add final data to list of feature data chr and position are represented as Chr.Pos (ie. 4.351)
			my $url = "javascript:submitSearch('".$chr_prefix.$chr.":"
			  . $item->{bin_start} . ".."
			  . $item->{bin_stop}."',true)";
			my @colors = split(/\ /, $self->config->{HGVBrowser}->{bgcolors});
			my $panel = Bio::Graphics::Panel->new;
			
			#deal with chrX and Y (passed in by JS as 23 and 24)
				my $js_chr = $chr eq 'X' ? '23' : $chr eq 'Y' ? '24' : $chr;
				
				#| separated score is dealt with by custom Bio::Graphics::Glyph::stackplot in GBrowse
				#include hack with onclick to call JS function 
				my $feature = Bio::Graphics::Feature->new(
					-start      => $item->{bin_start},
					-stop       => $item->{bin_stop},
					-score      => $score,
					-ref        => $chr_prefix . $chr,
					-type       => 's',
					-name       => "Count:$actual_score",
					-attributes => {
						'onClick' => ["gvToRV($js_chr.$short_start)"],
					},
					-url => ( $self->viewtype eq 'region' ? $url : '' )
				);
				
				#store actual score in feature as well (used to sort out axes in stackplot)
				$feature->{actualscore} = $actual_score;
				
				#sort out HTML popup for each bin and store this in the feature as well
				if ($actual_score eq $empty_score) {
					$feature->{popup}="No markers in bin";
					$feature->{empty}=1;
				}
				else {
					my $html = "<u>Number of significant markers in bin</u>";
					$html.="<table>";
					my $rscounter=0;
					my $rows=0;
					foreach(@scores) {
						my $rsid = $self->resultsetid->[$rscounter];
						
						if ($_>0) {
							$rs_counts->{$rsid}+=$_;
							my $summary = $self->summaries->{$rsid};
							my @rgb = $panel->color_name_to_rgb($colors[$rscounter]);
							$html.="<tr><td style='vertical-align:middle'>$_</td>";
							$html.="<td style='background-color:rgb(".join(",",@rgb)."); width:16px; height:12px;border:solid 1px black'>&nbsp;</td>";
							$self->summaries and $html.="<td><b>".$summary->{Study_Name}." (".$summary->{Study_Identifier}.")</b><br><i>".$summary->{Resultset_Name}." (".$summary->{Resultset_Identifier}.")</i></td>";
							$html.="</tr>";
							$rows++;
						}
						$rscounter++;
					}
					
					if ($rows>0) {
						$html.="</table>";
					}
					else {
						$html="No significant markers in bin";
					}
					
					$feature->{popup} = $html;
				}
				
				#add the feature to the feature list
				$features->add_feature( $feature => 's' );
				
				#add the total score to an array of total scores
				push @sum_scores, $sum_score;
				$counter++;	
		}
		
		#once finished looping through the data items get the highest score in the array of total scores
		my @sorted = reverse sort { $a <=> $b } @sum_scores;
		return ($sorted[0], $rs_counts);
	}

=head2 add_bins_present

	  Usage      : $browser->add_bins_present({
    				chr=>'9',
    				bin_size=>3000000,
    				data=>\@data,
   			   });
    			   
	  Purpose    : Add binned significant count features to specific chromosome Main method to generate feature data for use by modified hgvbrowse_karyotype CGI script. Also provides the max value for stacked plot for any chromosome
	  Returns    : Nothing
	  Arguments  : In hash ref
	  				 chr - Chromosome number (automatically adds 'Chr' prefix if region view
	  				 binsize - size of bins to add in base pairs
	  				 data - arrayref of binned data (see HGVbaseG2P::Browser::Analyse::Bin)
	  Throws     : 
	  Status     : Public
	  Comments   : 
	
=cut

		sub add_bins_present {
			my ( $self, $attrs ) = @_;
			my $chr      = $attrs->{chr} || $self->chr;
			my $data     = $attrs->{data};
			my $features = $self->features;

			my $counter   = 1;
			my %start_pos = ();
			my %end_pos   = ();
			my %prev_stop = ();
			my %within    = ();

			my $chr_prefix = $self->viewtype eq 'region' ? 'Chr' : '';

#more elegant and speeds up rendering to draw lines as required, rather than on or off feature for each bin
			foreach my $rsid ( @{ $self->resultsetid } ) {
				$start_pos{$rsid} = $self->seg_start;
				$end_pos{$rsid}   = $self->seg_start;
			}
			#uses a hash based flag system to determine whether it is 'in' or 'out' of a coloured bar
			#loop through each item in the data array
			for my $item ( @{$data} ) {
				my $rscounter  = 0;
				my $score_list = $item->{'bin_counts'};

#get scores for each resultset by splitting by | and loop through these
#draw a coloured box if score is not -1 (colouring of box is determined in 'add_bins_present_glyph')

				#get the score for this item in resultsetid order
				my @scores = split( /\|/, $score_list );
				
				#loop through each resultset id
				foreach my $rsid ( @{ $self->resultsetid } ) {
					
					#if within a coloured bar for this resultset..
					if ( $within{$rsid} ) {
						#if the bin is empty then make this bin the end the coloured bar and add the feature
						if ( $scores[$rscounter] eq '-1' ) {

							$end_pos{$rsid} = $prev_stop{$rsid};
							$within{$rsid}  = 0;
							my $feature = Bio::Graphics::Feature->new(
								-start => $start_pos{$rsid},
								-stop  => $end_pos{$rsid},
								-ref   => $chr_prefix . $chr,
								-type  => "l$rsid",
								-score => 1,
							);

							$features->add_feature( $feature => "l$rsid" );
						}
					}
					
					#if not within a coloured bar for this resultset..
					else {
						#if the bin is not empty then make this bin the end of the coloured bar with a start determined by the previous end
						if ( $scores[$rscounter] ne '-1' ) {

							my $feature = Bio::Graphics::Feature->new(
								-start => $end_pos{$rsid},
								-stop  => $item->{'bin_start'},
								-ref   => $chr_prefix . $chr,
								-type  => "l$rsid",
								-score => 0,
							);

							$features->add_feature( $feature => "l$rsid" );

							$start_pos{$rsid} = $item->{'bin_start'};
							$within{$rsid}    = 1;
						}
					}
					
					#set the previous stop to be the end of this bin
					$prev_stop{$rsid} = $item->{'bin_stop'};
					$rscounter++;
				}

				$counter++;
			}
			
			#ensure that any bars that any resultsets that were still within a bin have features created to the end of the region
			foreach my $rsid ( @{ $self->resultsetid } ) {
				if ( $within{$rsid} ) {
					my $feature = Bio::Graphics::Feature->new(
						-start => $start_pos{$rsid},
						-stop  => $self->seg_stop,
						-ref   => $chr_prefix . $chr,
						-type  => "l$rsid",
						-score => 1,
					);
					$features->add_feature($feature);
				}
				else {
					my $feature = Bio::Graphics::Feature->new(
						-start => $end_pos{$rsid},
						-stop  => $self->seg_stop,
						-ref   => $chr_prefix . $chr,
						-type  => "l$rsid",
						-score => 0,
					);
					$features->add_feature($feature);
				}
			}
		}

=head2 get_marker_data

	  Usage      : $browser->ref('chr9');
	               my $data = $browser->get_marker_data;
    			   
	  Purpose    : Wrapper method to get different aspects of marker data from the Browser database. The data returned depends on the object settings for chr, round_start and round_stop.
	               If only resultsetid is set, gets all data for the resultsets in question. If chr is also set gets just for that chromosome.
	               If a range is also set, gets just data for that range.
	  Returns    : Arrayref of data
	  Throws     : 
	  Status     : Public
	  Comments   : 
	
=cut

		sub get_marker_data {
			my ( $self, $attrs ) = @_;

			my $db = $self->get_db;
			my $data;
			if ( $self->chr ) {
				if ( $self->round_start && $self->round_stop ) {
					
					$data = $db->get_marker_data_by_range_and_resultsets(
						{
							resultsetid => $self->resultsetid,
							chr         => $self->chr,
							start       => $self->round_start,
							stop        => $self->round_stop,
							output_to   => 'array'
						}
					);
				}
				else {
					$data = $db->get_marker_data_by_chr_and_resultsets(
						{
							resultsetid => $self->resultsetid,
							chr         => $self->chr,
							output_to   => 'array'
						}
					);
				}
			}
			else {
				$data = $db->get_marker_data_by_resultsets(
					{
						resultsetid => $self->resultsetid,
						output_to   => 'array'
					}
				);
			}

			$self->autodb and $db->disconnect;
			return $data;
		}

=head2 get_marker_binned_data

	  Usage      : my $marker_data = $browser->get_marker_data;
    			   
	  Purpose    : Wrapper method which gets appropriate precalculated binned marker data from browser DB depending on browser object settings.
	               Affected by setting 'chr', 'round_start' and 'round_stop'.
	  Returns    : Appropriate binned marker data from the DB, as a hashref (using Reference as a key) containing a arrayrefs of data items
	  Arguments  : None
	  Throws     : 
	  Status     : Public
	  Comments   : If no region accessors set, get all binned marker data for all resultsets
	               If chr accessor set, but round_start and round_stop not set, gets binned marker data for that chromosome for all resultsets
	               If all region accessors set, gets binned marker data for that chromosome and region
	
=cut

		sub get_marker_binned_data {
			my ( $self, $attrs ) = @_;

			my $db = $self->get_db;
			my $data;
			
			my $resultsetids = $self->resultsetid;
			if (!$resultsetids || scalar(@{$resultsetids})==0) {
				return {};
			}
			
#get only single chr data if only one chr present (ie. if chr info length is less than 3)
			if ( $self->chr && length( $self->chr ) > 0 ) {
				$self->log->info("round_start:".$self->round_start." and round_stop:".$self->round_stop);
				if ( $self->round_start && $self->round_stop ) {
					$self->log->info("get_marker_binned_by_resultsets_range_and_threshold");
					$data =
					  $db->get_marker_binned_by_resultsets_range_and_threshold(
						{
							resultsetid => $resultsetids,
							bin_size    => $attrs->{bin_size},
							chr         => $self->chr,
							start       => $self->round_start,
							stop        => $self->round_stop,
							threshold   => $self->threshold,
						}
					  );
				}
				else {
					$self->log->info("get_marker_binned_by_resultsets_chr_and_threshold");
					$data =
					  $db->get_marker_binned_by_resultsets_chr_and_threshold(
						{
							resultsetid => $resultsetids,
							bin_size    => $attrs->{bin_size},
							chr         => $self->chr,
							threshold => $self->threshold || 'ZERO',
						}
					  );
				}
			}
			else {
				$self->log->info("get_marker_binned_by_resultsets_and_threshold");
				$data = $db->get_marker_binned_by_resultsets_and_threshold(
					{
						resultsetid => $resultsetids,
						bin_size    => $attrs->{bin_size},
						threshold => $self->threshold || 'ZERO',
					}
				);
			}

			$self->autodb and $db->disconnect;
			return split_data_by_field( $data, "Reference", 1 );
		}

=head2 calculate_binned_data

	  Usage      : my $binned_data = $browser->calculate_binned_data({
	  				data => \@data,
	  				bin_size => 3000000
	  });    			   
	  Purpose    : Calculates binned data using the data and bin_size provided
	  Returns    : An arrayref of binned data 
	  Arguments  : In hash ref
	  				 binsize - size of bins to add in bp
	  				 data - arrayref of binned data (see HGVbaseG2P::Browser::Analyse::Bin)
	  Throws     : 
	  Status     : Public
	  Comments   : Data format is an array of hashes (and the Start and NegLogPValue values act as position and value respectively)
	
=cut

		sub calculate_binned_data {
			my ( $self, $attrs ) = @_;

			my $data = $attrs->{data};

#bin marker data but provide content rather than count (ie. retain elements created above)
			my $analyse =
			  HGVbaseG2P::Browser::Analyse::Bin->new( { data => $data } );

			my $binned = $analyse->bin(
				{
					position_field => 'Start',
					bin_size       => $attrs->{bin_size},
					total_length   => chr_length( $self->chr ),
					value_field    => 'NegLogPValue',
					bin_content    => 'array',
					calc_ranges    => 1,
					start_position => $self->round_start,
					stop_position  => $self->round_stop,
				}
			);

			return $binned;
		}

=head2 is_high_resolution

	  Usage      : my $is_hr = $self->is_high_resolution;
	  Purpose    : Check if need to display low or high resolution
	  Returns    : 0 for low resolution and 1 for high resolution
	  Arguments  : None
	  Throws     : 
	  Status     : Public
	  Comments   : 
	
=cut

		sub is_high_resolution {
			my ($self)        = @_;
			if (!$self->seg_start && !$self->seg_stop) {
				return 0;
			}
			my $region_size   = $self->seg_stop - $self->seg_start;
			my $use_minor_bin =
			  $region_size >= ( $self->setting("lowres_binsize") * 8 )
			  ? 0
			  : 1;
			return $use_minor_bin;
		}

=head2 add_high_res_features

	  Usage      : $self->add_high_res_features;
	  Purpose    : Main method called to add high resolution-based features to feature list
	  Returns    : Nothing
	  Arguments  : None
	  Throws     : 
	  Status     : Public
	  Comments   : Uses 'tracks' and 'settings' accessors to determine what to display and sizes etc.
	
=cut

		sub add_high_res_features {

			my ($self) = @_;

			my $data = $self->get_marker_data;
			$self->autodb and $self->db->disconnect;

			my $major_bin = $self->setting("lowres_binsize") / 1000000;
			my $minor_bin = $self->setting("highres_binsize") / 1000;

			my ($binned_data) = $self->calculate_binned_data(
				{
					bin_size => $self->setting("highres_binsize"),
					data     => $data
				}
			);
			my (
				$max_and_counts, $sig_markers, $multi_sig_markers,
				$max_pvalue,     $max_count
			  )
			  = $self->calculate_max_and_sig_counts( { data => $binned_data } );

			my $threshold = $self->threshold;

			if ( $self->track("lowres_stacked") ) {
				$self->add_line();
				$self->add_heading(
"Stacked counts of -log P-values at least $threshold <low-resolution $major_bin"
					  . "-Mb bins> - not shown at this resolution" );
			}

			if ( $self->track("lowres_present") ) {
				$self->add_line();
				$self->add_heading(
"Marker coverage for selected studies <low-resolution $major_bin"
					  . "-Mb bins> - not shown at this resolution" );
			}

			if ( $self->track("highres_stacked") ) {
				$self->add_line();
				$self->add_heading(
"Stacked counts of -log P-values at least $threshold <high-resolution $minor_bin"
					  . "-kb bins>" );

				$self->add_sig_counts_glyph( { height => 50 } );

				my ($chr_max_count, $rs_counts) = $self->add_sig_counts(
					{
						data     => $max_and_counts,
						bin_size => 15000,
					}
				);

				$self->features->set( 's', 'max_score' => $chr_max_count );
			}

			if ( $self->track("highres_present") ) {
				$self->add_line();
				$self->add_heading(
"Marker coverage for selected studies <high-resolution $minor_bin"
					  . "-kb bins>" );

				$self->add_bins_present_glyph();
				$self->add_bins_present(
					{
						data     => $max_and_counts,
						bin_size => 15000,
					}
				);
			}

			if ( $self->track("line_trace") ) {
				$self->add_line();
				$self->add_heading(
"Line plot of maximum -log P value for selected studies <$minor_bin"
					  . "-kb bins>" );

				$self->add_pvalue_traces(
					{
						height     => 50,
						data       => $max_and_counts,
						max_pvalue => $max_pvalue
					}
				);
			}

			if ( $self->track("sig_markers_single") ) {
				$self->add_line();
				$self->add_sig_markers( { data => $sig_markers } );
			}

			if ( $self->track("sig_markers_multiple") ) {
				$self->add_line();
				$self->add_combined_sig_markers(
					{ data => $multi_sig_markers } );
			}

		}

=head2 add_low_res_features

	  Usage      : $self->add_low_res_features;
	  Purpose    : Main method called to add low resolution-based features to feature list
	  Returns    : Nothing
	  Arguments  : None
	  Throws     : 
	  Status     : Public
	  Comments   : Uses 'tracks' and 'settings' accessors to determine what to display and sizes etc.
	
=cut

		sub add_low_res_features {
			my ($self) = @_;

			my $major_bin = $self->setting("lowres_binsize") / 1000000;
			my $minor_bin = $self->setting("highres_binsize") / 1000;

			my $data =
			  $self->get_marker_binned_data( { bin_size => $major_bin } );
			$self->logscale(1);
			$self->autodb and $self->db->disconnect;
			
			my $threshold = $self->threshold;

			if ( $self->track("lowres_stacked") ) {
				$self->add_line();
				$self->add_heading(
"Stacked counts of -log P-values at least $threshold <low-resolution $major_bin"
					  . "-Mb bins> click a histogram bar to view region" );

				$self->add_sig_counts_glyph( { height => 50 } );

				my ($chr_max_count, $rs_counts) = $self->add_sig_counts(
					{
						data           => $data->{ $self->chr },
						bin_size       => $self->setting("lowres_binsize"),
						bin_size_short => $major_bin
					}
				);
				$self->features->set( 's', 'max_score' => $chr_max_count );
			}

			if ( $self->track("lowres_present") ) {
				$self->add_line();
				$self->add_heading(
"Marker coverage for selected studies <low-resolution $major_bin"
					  . "-Mb bins view>" );

				$self->add_bins_present_glyph();
				$self->add_bins_present(
					{
						data           => $data->{ $self->chr },
						bin_size       => 1000000,
						bin_size_short => 1
					}
				);
			}

			if ( $self->track("highres_stacked") ) {
				$self->add_line();
				$self->add_heading(
"Stacked counts of -log P-values at least $threshold <high-resolution $minor_bin"
					  . "-kb bins> - not shown at this resolution" );
			}

			if ( $self->track("highres_present") ) {
				$self->add_line();
				$self->add_heading(
"Marker coverage for selected studies <high-resolution $minor_bin"
					  . "-kb bins> - not shown at this resolution" );
			}

			if ( $self->track("line_trace") ) {
				$self->add_line();
				$self->add_heading(
"Line plot of maximum -log P value for selected studies <$minor_bin"
					  . "-kb bins>" );
			}

			if ( $self->track("sig_markers_single") ) {
				$self->add_line();
				$self->add_heading(
"Significant Markers for individual resultsets - not shown at this resolution"
				);
			}
			if ( $self->track("sig_markers_multiple") ) {
				$self->add_line();
				$self->add_heading(
"Significant Markers across all resultsets - not shown at this resolution"
				);
			}
		}

=head2 add_bins_present_glyph

	  Usage      : $browser->add_bins_present_glyph({
    				height=>'20'
   			   });
    			   
	  Purpose    : Add glyphs of marker coverage for each resultset in Browser object
	  Returns    : Nothing
	  Arguments  : In hash ref
	  				 height - Height of boxes to draw
	  Throws     : 
	  Status     : Public
	  Comments   : 
	
=cut

		sub add_bins_present_glyph {
			my ( $self, $attrs ) = @_;
			my $panel = Bio::Graphics::Panel->new;	
			my @colors =
			  split( ' ', $self->config->{HGVBrowser}->{bgcolors} );

			my $rscounter = 0;

			foreach my $rsid ( @{ $self->resultsetid } ) {
				my $html="<u>Marker coverage for:</u></br>";
					
				my $summary = $self->summaries->{$rsid};
				my @rgb = $panel->color_name_to_rgb($colors[$rscounter]);
				$html.="<table><tr><td style='vertical-align:middle'>$_</td>";
				$html.="<td style='background-color:rgb(".join(",",@rgb)."); width:16px; height:12px;border:solid 1px black'>&nbsp;</td>";
				$self->summaries and $html.="<td><b>".$summary->{Study_Name}." (".$summary->{Study_Identifier}.")</b><br><i>".$summary->{Resultset_Name}." (".$summary->{Resultset_Identifier}.")</i></td>";
				$html.="</tr>";
				$html.="</table>";
				
				$self->features->add_type(
					"l$rsid" => {
						glyph       => 'graded_segments',
						fgcolor     => 'lightgrey',
						bgcolor     => $colors[$rscounter],
						bump        => 0,
						label       => 0,
						description => 0,
						title       => 0,
						key         => 0,
						height      => $attrs->{height},
						method      => 'presence',
						source      => 'HGVbaseG2P',
						max_score   => 1,
						min_score   => 0,
						'balloon width' => 200,
						'balloon hover' => 'sub { return "'.$html.'";}'
					}
				);
				$rscounter++;
			}
		}

=head2 add_sig_counts_glyph

	  Usage      : $browser->add_bins_present_glyph({
    				height=>'20'
   			   });
    			   
	  Purpose    : Add binned significant count glyphs for each resultset in Browser object
	  Returns    : Nothing
	  Arguments  : In hash ref
	  				 max_count - Maximum count in stackplot graph
	  				 height - Height in pixels of stackplot graph
	  Throws     : 
	  Status     : Public
	  Comments   : 
	
=cut

		sub add_sig_counts_glyph {
			my ( $self, $attrs ) = @_;

			my $max_count = $attrs->{max_count} || 20;

			$self->features->add_type(
				's' => {
					glyph      => 'stackplot',
					graph_type => 'boxes',
					fgcolor    => sub {
						my $f = shift;
						return $f->{empty} ? "lightgrey" : "black";
					},
					bgcolors        => $self->config->{HGVBrowser}->{bgcolors},
					height          => $attrs->{height} || 20,
					min_score       => 0,
					max_score       => 10,
					scale           => 'none',
					bump            => 0,
					key             => 0,
					label           => 0,
					'balloon width' => 300,
					'balloon hover' =>
'sub { my $feature = shift; return $feature->{"popup"}; }',
					method => 'sig_counts',
					source => 'HGVbaseG2P'
				}
			);
		}

=head2 is_no_features

	  Usage      : my $no_features = $self->is_no_features;
	  Purpose    : Determines if no tracks or resultsetid available. Adds features to feature list to show this.
	  Returns    : 1 if no features
	  Arguments  : None
	  Throws     : 
	  Status     : Public
	  Comments   : 
	
=cut

		sub is_no_features {
			my ($self) = @_;
			my $features = $self->features;
			if ( $self->resultsetid eq 'NONE' ) {
				$self->add_heading(
"No Analysis Results set was specified. Please return to the linking page and try again."
				);
				$self->add_heading(
" If this does not work please report the error to the HGVbaseG2P team."
				);

				return 1;
			}

			if (    !$self->maxpval_low
				and !$self->maxpval_high
				and !$self->sigmarkers
				and !$self->sigmarkers_count )
			{
				$self->add_heading(
					"No tracks selected in the configuration settings");
				return 1;
			}
		}

=head2 format_chr

	  Usage      : $self->format_chr("ChrX");
	  Purpose    : Sets $self->ref with unformatted chr (eg. ChrX) and $self->chr with removed chr (eg. X)
	  Returns    : None
	  Arguments  : Full chromosome name (eg. ChrY)
	  Throws     : 
	  Status     : Public
	  Comments   : 
	
=cut

		sub format_chr {
			my ( $self, $ref ) = @_;
			$self->ref($ref);
			$ref =~ /^Chr(.+)/;
			my $chr = $1;
			$self->chr($chr);
		}

=head2 round_positions

	  Usage      : my ($start, $stop) = $self->round_positions
	  Purpose    : Rounds down the start and up the stop position to the nearest multiple of $self->major_bin ands sets accessors
	  Returns    : Rounded start and stop
	  Arguments  : None
	  Throws     : 
	  Status     : Public
	  Comments   : 
	
=cut

		sub round_positions {
			my ($self) = @_;
			!$self->setting("lowres_binsize")
			  and $self->log->logdie(
"You must set the 'lowres_binsize' setting before using this method"
			  );
			!$self->seg_start
			  and $self->log->logdie(
				"You must set the 'seg_start' accessor before using this method"
			  );
			!$self->seg_stop
			  and $self->log->logdie(
				"You must set the 'seg_stop' accessor before using this method"
			  );

			my $bin_start =
			  nlowmult( $self->setting("lowres_binsize"), $self->seg_start );
			if ($bin_start==0) {
				$bin_start = 1;
			}
			my $bin_stop =
			  nhimult( $self->setting("lowres_binsize"), $self->seg_stop );
			$self->round_start($bin_start);
			$self->round_stop($bin_stop);
			$self->log->info("start and stop rounded");
			return ( $bin_start, $bin_stop );
		}

=head2 tracks

	  Usage      : $browser->tracks('track1,track2');
	  Purpose    : Populates the _tracks accessor by creating a hash (with all values = 1) from a comma-separated string. Generally populated with CGI parameters
	  Returns    : Nothing
	  Arguments  : $value - contains a comma-separated string
	  Throws     : 
	  Status     : Public
	  Comments   : 
	
=cut

		sub tracks {
			my ( $self, $value ) = @_;
			if ($value) {
				my %track_hash;

				foreach ( split( ',', $value ) ) {
					$track_hash{$_} = 1;
				}
				$self->_tracks( \%track_hash );
			}
			return $self->_tracks;
		}

=head2 settings

	  Usage      : $browser->settings('setting1|test1,setting2|test2);
	  Purpose    : Populates the _settings accessor by creating a hash from a comma-separated and pipe-separated string. Generally populated with CGI parameters
	  Returns    : Nothing
	  Arguments  : $value - contains a comma/pipe-separated string
	  Throws     : 
	  Status     : Public
	  Comments   : 
	
=cut

		sub settings {
			my ( $self, $value ) = @_;
			if ($value) {
				my %settings_hash;
				foreach ( split( ',', $value ) ) {
					my @temp = split( /\|/, $_ );
					$settings_hash{ $temp[0] } = $temp[1];
				}
				$self->_settings( \%settings_hash );
			}
			return $self->_settings;
		}

=head2 setting

	  Usage      : my $value = $browser->setting('setting1');
	  Purpose    : Get the value for a setting in the browser object
	  Returns    : The value corresponding to the provided key from the _setting accessor
	  Arguments  : $key - key to look for in settings
	  Throws     : 
	  Status     : Public
	  Comments   : 
	
=cut

		sub setting {
			my ( $self, $key, $value ) = @_;
			$value and $self->_settings->{$key} = $value;
			return $self->_settings->{$key};
		}

=head2 track

	  Usage      : if ($browser->track('track1')) { #do something if present; } else { #do something else if not present;}
	  Purpose    : Determine if a track has been added in the browser object
	  Returns    : 1 if present, undef if not
	  Arguments  : $key - key to look for in tracks
	  Throws     : 
	  Status     : Public
	  Comments   : 
	
=cut

		sub track {
			my ( $self, $key ) = @_;
			my $track;
			eval {
				$track =  $self->_tracks->{$key}
			};
			$@ and return undef;
			return $track;
		}

=head2 generate_region_features

	  Usage      : my $features = $browser->generate_region_features(\%attrs);
	  Purpose    : Core method for generating features in GBrowse region view. Ties together logic from other methods in this module.
	  Returns    : List of features for use by GBrowse plugin
	  Arguments  : Hash ref containing:
					seg_start - start of segment
					seg_stop - end of segment
					ref - segment reference
					resultsetid - comma-separated list of resultset IDs
					tracks - list of tracks added to browser (comma-separated) - usually from CGI parameters
					settings - list of settings for browser (comma-separated, and key/value separated by pipe) - usually from CGI parameters
					threshold - P value threshold
	  Throws     : 
	  Status     : Public
	  Comments   : 
	
=cut

		sub generate_region_features {
			my ( $self, $attrs ) = @_;
			$self->viewtype('region');
			$self->tracks( $attrs->{tracks} || []);
			$self->settings( $attrs->{settings} || {} );
			$self->seg_start( $attrs->{seg_start} );
			$self->seg_stop( $attrs->{seg_stop} );
			$self->format_chr( $attrs->{ref} );
			return unless $attrs->{resultsetid};
			my @resultsetids = split( ',', $attrs->{resultsetid} );
			$self->resultsetid( \@resultsetids );
			
			#get precalculated counts for analysis and threshold
			#get resultset summary info and round up/down the start/stop positions
			$self->summaries( $self->get_resultset_summaries );
			my @messages = $self->remove_unavailable_resultsets;
			
			if (scalar(@messages)>0) {
				my $counter=1;
				foreach(@messages) {
					$self->add_textbox("Data Not Available - ".$counter,$_,'red','pink');
					$counter++;
				}
			}
		
			#passed in from CGI param or session data
			$self->threshold( $attrs->{'threshold'} );
			$self->is_displayed( $attrs->{'is_displayed'} );
			
			my ( $bin_start, $bin_stop ) = $self->round_positions;

			#generate features based on resolution level
			if ( $self->is_high_resolution ) {
				$self->add_high_res_features;
			}
			else {
				$self->add_low_res_features;
			}
			
			return $self->features;
		}

=head2 search_for_markers

	  Usage      : my $features = $browser->generate_region_features(\%attrs);
	  Purpose    : Core method for searching for significant markers by ID in the current resultsets - called by MarkerFinder plugin
	  Returns    : \@features - list of matching features
	  			   $ref - chromosome
	  			   $start - start position
	  			   $stop - stop position
	  Arguments  : Hash ref containing:
					searchterms - ID to search for (can be either HGVM id or rsid)
	  Throws     : 
	  Status     : Public
	  Comments   : Need to set resultsetids and threshold in the Browser object before calling this method.
	
=cut

		sub search_for_markers {
			my ( $self, $attrs ) = @_;

			my $markers = $self->get_db->get_markers_by_identifier(
				{
					identifier => $attrs->{identifier},
					resultsetid     => $self->resultsetid,
					threshold       => $self->threshold,
					chr=>$attrs->{chr}
				}
			);

			my @features = ();
			my $start;
			my $stop;
			my $ref;

			if ( scalar( @{$markers} ) > 0 ) {
				my $marker = $markers->[0];
				$start = $marker->{Start} - 1000000;
				$stop  = $marker->{Stop} + 1000000;
				$ref   = "Chr" . $marker->{Chr};
				
				my $f = Bio::Graphics::Feature->new(
					-ref   => $ref,
					-type  => 'Significant Markers',
					-name  => $marker->{Marker_Accession},
					-start => $marker->{Start},
					-score => $marker->{NegLogPValue},
					-end   => $marker->{Stop},
				);
				push @features, $f;
			}

			return ( \@features, $ref, $start, $stop );
		}

=head2 add_heading

	  Usage      : $self->add_heading("Nothing inparticular");
	  Purpose    : Add a heading 'feature' to Browser feature list
	  Returns    : Nowt
	  Arguments  : Heading text
	  Throws     : 
	  Status     : Public
	  Comments   : This is a bit of a hack. It adds a heading by creating a white line which has a label (ie. the text)
	               So far this seems to have worked ok. May need to expand on this anyway - will probably create a specific glyph
				   N.B. Don't use () or [] in the text as this breaks GBrowse!
=cut

		sub add_heading {
			my ( $self, $heading ) = @_;

			$self->features->add_type(
				$heading => {
					glyph          => 'line',
					fgcolor        => 'white',
					bgcolor        => 'white',
					width          => 1,
					label          => 0,
					padding_bottom => 0,
					description    => 0,
					height         => 1
				}
			);

			my $feature = Bio::Graphics::Feature->new(
				-start => $self->seg_start,
				-stop  => $self->seg_start,
				-ref   => $self->ref,
				-score => 0,
			);
			$self->features->add_feature( $feature => $heading );
		}
		
=head2 add_textbox

	  Usage      : $self->add_textbox("Nothing inparticular");
	  Purpose    : Add a heading 'feature' to Browser feature list
	  Returns    : Nowt
	  Arguments  : Heading text
	  Throws     : 
	  Status     : Public
	  Comments   : This is a bit of a hack. It adds a heading by creating a white line which has a label (ie. the text)
	               So far this seems to have worked ok. May need to expand on this anyway - will probably create a specific glyph
				   N.B. Don't use () or [] in the text as this breaks GBrowse!
=cut

		sub add_textbox {
			my ( $self, $title, $text, $outline, $background ) = @_;

			$self->features->add_type(
				$title => {
					glyph          => 'text_in_box',
					fgcolor        => $outline,
					bgcolor        => $background,
					fontcolor => 'black',
					label     => 0,
					text => $text,
					text_bgcolor => $background,
					text_pad => 5,
					bump => 1,
					height => 24,
				}
			);

			my $feature = Bio::Graphics::Feature->new(
				-start => $self->seg_start,
				-stop  => $self->seg_stop,
				-ref   => $self->ref,
			);

			$self->features->add_feature( $feature => $title );
		}

=head2 add_dashed_line

	  Usage      : $self->add_dashed_line;
	  Purpose    : Add a dashed line which goes from 'start of region to end' to Browser feature list
	  Returns    : Nowt
	  Arguments  : None
	  Throws     : 
	  Status     : Public
	  Comments   : 
=cut

		sub add_dashed_line {
			my ($self) = @_;

			my $type = "line" . $self->next_featureno;
			$self->features->add_type(
				$type => {
					glyph       => 'dashed_line',
					fgcolor     => 'lightgrey',
					textcolor   => 'white',
					label       => 0,
					title       => 0,
					pad_bottom  => 0,
					pad_top     => 0,
					label       => 0,
					description => 0,
					label       => 0,
					key         => 0,
					linewidth   => 0.5,
				}
			);

			my $feature = Bio::Graphics::Feature->new(
				-start => (
					$self->seg_start == $self->seg_stop ? $self->seg_start - 10
					: $self->seg_start
				),
				-stop => (
					  $self->seg_start == $self->seg_stop ? $self->seg_stop + 10
					: $self->seg_stop
				),
				-ref   => $self->ref,
				-score => 0
			);
			$self->features->add_feature( $feature => $type );
		}

=head2 add_line
	  Usage      : $self->add_line;
	  Purpose    : Add a divider line of full detail width to Browser feature list
	  Returns    : Nowt
	  Arguments  : None
	  Throws     : 
	  Status     : Public
	  Comments   : 
=cut

		sub add_line {
			my ($self) = @_;

			my $type = "line" . $self->next_featureno;
			$self->features->add_type(
				$type => {
					glyph       => 'line',
					fgcolor     => 'lightgrey',
					textcolor   => 'white',
					label       => 0,
					title       => 0,
					pad_bottom  => 0,
					pad_top     => 0,
					label       => 0,
					description => 0,
					label       => 0,
					key         => 0,
					linewidth   => 2,
				}
			);

			
			my $feature = Bio::Graphics::Feature->new(
				-start => (
					$self->seg_start == $self->seg_stop ? $self->seg_start - 10
					: $self->seg_start
				),
				-stop => (
					  $self->seg_start == $self->seg_stop ? $self->seg_stop + 10
					: $self->seg_stop
				),
				-ref   => $self->ref,
				-score => 0
			);
			$self->features->add_feature( $feature => $type );
		}

=head2 add_link

	  Usage      : $self->add_link("Nothing inparticular");
	  Purpose    : Add a link 'feature' to Browser feature list
	  Returns    : Nowt
	  Arguments  : Link text and address
	  Throws     : 
	  Status     : Public
	  Comments   : This uses text_in_box glyph coloured blue to simulate a link
=cut

		sub add_link {
			my ( $self, $link_text, $link ) = @_;

			my $heading = " ";
			$self->features->add_type(
				$heading => {
					glyph     => 'text_in_box',
					fgcolor   => 'blue',
					bgcolor   => 'black',
					fontcolor => 'blue',
					label     => 0,
				}
			);

			my $feature = Bio::Graphics::Feature->new(
				-start => $self->seg_start,
				-stop  => $self->seg_stop,
				-ref   => $self->ref,
				-score => 0,
				-type  => $heading,
				-name  => ' ',
				-url   => $link,

			);
			$feature->{text} = $link_text;
			$self->features->add_feature( $feature => $heading );
		}

=head2 calculate_max_pvalues

	  Usage      : my ($binned, $max_pval) = $self->calculate_max_pvalues({
					 bin_size => $attrs->{bin_size}
				   });
	  Purpose    : Calculate max pvalues for specified bin size (therefore used by low and high resolution views)
	  Returns    : Arrayref of hash bin data (see HGVbaseG2P::Browser::Analyse::Bin) and the max value found in the analysis
	  Arguments  : Hashref of:
	                bin_size - size of bin to use
	  Throws     : 
	  Status     : Public
	  Comments   : 
=cut

		sub calculate_max_pvalues {
			my ( $self, $attrs ) = @_;
			my $binned     = $self->all_binned;
			my @resultsets = split( ',', $self->resultsetid );
			my @tot_max    = ();

			#loop through each binned item
			foreach my $item ( @{$binned} ) {
				my @max = [];

				#loop through contents each binned item
				foreach my $content ( @{ $item->{bin_content} } ) {

#loop through each resultset of binned items and get max P value for each in a bin

					for (
						my $rscounter = 0 ;
						$rscounter < scalar(@resultsets) ;
						$rscounter++
					  )
					{

#print "content:".Dumper($content->[2])." and ".($content->[1]->[$rscounter] or 'none')."\n";
						!$max[$rscounter] and $max[$rscounter] = [];
						if ( $content->[1]->[$rscounter] ) {
							push @{ $max[$rscounter] },
							  $content->[1]->[$rscounter]->{'NegLogPValue'};
						}
					}
				}

				#add calculated counts to bin data
				foreach my $max (@max) {
					$max = max( @{$max} );
				}
				$item->{bin_max} = \@max;
				my $all_max = max(@max);
				$all_max and push @tot_max, $all_max;
			}

			my @sorted   = reverse sort { $a <=> $b } @tot_max;
			my $max_pval = $sorted[0];

			return ( $binned, $max_pval );
		}

=head2 add_pvalue_traces

	  Usage      : $self->add_max_pvalues({
					 bin_size => $attrs->{bin_size}
				   });
	  Purpose    : Get max pvalues for specified bin size and add to feature list as overlapping traces
	  Returns    : Nowt
	  Arguments  : Hashref of:
	  				max_pvalue - maximum value in data set - used as max_score for glyph
	  				data - binned data to use
	                bin_size - size of bin to use
	  Throws     : 
	  Status     : Public
	  Comments   : 
=cut

		sub add_pvalue_traces {
			my ( $self, $attrs ) = @_;

			my $max_pval = $attrs->{max_pvalue};
			my $features = $self->features;
			my $ref      = $self->ref;
			my $data     = $attrs->{data};

			$features->add_type(
				maxp => {
					glyph       => 'stackplot',
					fgcolor     => 'red',
					bgcolors    => $self->config->{HGVBrowser}->{bgcolors},
					scale_color => 'black',
					graphtype   => 'line',
					height      => $attrs->{height},
					min_score   => 0,
					max_score   => (
						  $self->threshold > $max_pval
						? $self->threshold
						: $max_pval
					),
					scale           => 'left',
					bump            => 0,
					key             => 0,
					threshold       => $self->threshold,
					'balloon hover' => 'Maximum P values across this region',
					method          => 'trace',
					source          => 'HGVbaseG2P'
				}
			);

#loop through items, create a subfeature array, finally add as a segment to a feature and add this to the feature list
			my @subfeatures;
			foreach my $item ( @{$data} ) {

				my $score = $item->{bin_max};

				my $subfeature = Bio::Graphics::Feature->new(
					-start => $item->{bin_start},
					-stop  => $item->{bin_stop},
					-score => $score,
					-ref   => $ref,
				);
				push @subfeatures, $subfeature;

			}

			my $f = Bio::Graphics::Feature->new(
				-segments => \@subfeatures,
				-type     => 'maxp'
			);
			$features->add_feature($f);

		}

=head2 calculate_max_and_sig_counts

	  Usage      : my ($data, $sig_markers, $multi_sig_markers, $max_pval, $max_counts ) = $self->calculate_max_and_sig_counts;

	  Purpose    : Core logic to calculate counts, maximums and significant markers (in single and multiple studies) for high res view
	  Returns    : 1) Processed data
	  			   2) A hashref of single significant markers with a key of resultset id (used by add_sig_markers)
	  			   3) A hashref of an array of significant markers in different resultsets with a key of marker ID (used by add_combined_sig_markers)
	  			   4) Max pvalue found across all data (used by add_pvalue_traces)
	  			   5) Max counts found across all data (used by add_sig_counts)
	  Arguments  : Hashref of:
	                data - arrayref of binned data (as hash including actual markers in 'bin_content' key)
	  Throws     : 
	  Status     : Public
	  Comments   : 
=cut

		sub calculate_max_and_sig_counts {
			my ( $self, $attrs ) = @_;

#get marker data for all resultsets as hash with key of Start - resultsets are present in an arrayref
			my $data       = $attrs->{'data'};
			my @resultsets = @{ $self->resultsetid };
			my @all_sums   = ();
			my @tot_max    = ();
			my $threshold = $self->threshold eq 'ZERO' ? '0' : $self->threshold;

			my %sig_markers       = ();
			my %multi_sig_markers = ();
			foreach (@resultsets) { $sig_markers{$_} = [] }

			#loop through each binned item
			foreach my $item ( @{$data} ) {
				my %counts;
				my %allcounts;
				my %values;
				my %bin_max;

				#reset count/allcount values for resultsets for each bin
				for my $rsid (@resultsets) {

					#set counts to 0
					$counts{$rsid}    = 0;
					$allcounts{$rsid} = 0;
					$bin_max{$rsid}   = 0;
					$values{$rsid}    = [];
				}

#loop through each marker in binned items and count number of markers for each in a bin
				foreach my $marker ( @{ $item->{bin_content} } ) {
					my $rsid     = $marker->{'ResultsetID'};
					my $pvalue   = $marker->{'NegLogPValue'};
					my $markerid = $marker->{'Marker_Identifier'};

					#add to count only if significance above threshold
					if ($marker) {
						if ( $pvalue >= $threshold ) {
							push @{ $sig_markers{$rsid} },           $marker;
							push @{ $multi_sig_markers{$markerid} }, $marker;
							$counts{$rsid}++;
						}
						$allcounts{$rsid}
						  ++;    #always add to all counts if marker in bin
						push @{ $values{$rsid} }, $pvalue ? $pvalue : '0.00';
					}
				}

#get cumulative sum of counts for bin and add to all_sums (do this before replacing 0s with -1s)
#used by stack plots (top value used as max score value)
				my $bin_sum = sum( values %counts );
				push @all_sums, $bin_sum;

			  #add bin counts to item and find maximum for each resultset in bin
			  #if bin has no markers at all then set score to -1
				my @count_sorted = ();
				my @max_sorted   = ();
				foreach my $rsid (@resultsets) {
					$counts{$rsid} = -1 if $allcounts{$rsid} == 0;
					push @count_sorted, $counts{$rsid};
					$bin_max{$rsid} = max( @{ $values{$rsid} } ) || 0;
					if ( $counts{$rsid} == -1 ) {
						push @max_sorted, -1;
					}
					else {
						push @max_sorted, $bin_max{$rsid};

					}
				}
				$item->{bin_counts} = join( "|", @count_sorted );
				$item->{bin_max}    = join( "|", @max_sorted );

#find bin maximum for all resultsets and add to array of all max values (used by P value trace)
				my $max_of_bin = max( values %bin_max );
				push @tot_max, $max_of_bin;
			}

			my $max_counts = max(@all_sums);
			my $max_pval   = max(@tot_max);

			return ( $data, \%sig_markers, \%multi_sig_markers, $max_pval,
				$max_counts );
		}

=head2 add_sig_markers

	  Usage      : $self->add_sig_markers( { data => $data } );
	  Purpose    : For each resultset, creates a marker glyph and adds supplied markers to feature list
	  Returns    : Nowt
	  Arguments  : Hashref of:
	                data - hashref of markers with resultsetid as key (see calculate_max_and_sig_counts)
	  Throws     : 
	  Status     : Public
	  Comments   : 
=cut

		sub add_sig_markers {
			my ( $self, $attrs ) = @_;

			my $data          = $attrs->{data};
			my $features      = $self->features;
			my $ref           = $self->ref;
			my $feature_count = 0;

			my @bgcolors =
			  split( /\s/, $self->config->{HGVBrowser}->{bgcolors} );

			my $rscounter = 0;

			#loop through each resultset in data and add a dot glyph
			foreach my $rsid ( @{ $self->resultsetid } ) {
				my $type =
				  "Significant Markers for "
				  . $self->summaries->{$rsid}->{UniqueName};
				$features->add_type(
					$type => {
						bgcolor         => $bgcolors[$rscounter],
						glyph           => 'dot',
						fgcolor         => 'black',
						label           => 1,
						bump            => 1,
						title           => 1,
						height          => 8,
						clip            => 'true',
						scale           => 'none',
						'balloon hover' =>
'sub { my $feature = shift; return "Click to view significance" }',
					}
				);

#loop through each significant marker for the resultset and add it to the features
				my $sig_count = 0;
				foreach my $sig ( @{ $data->{$rsid} } ) {
					if ($sig) {
						my $feature = Bio::Graphics::Feature->new(
							-start => $sig->{'Start'},
							-stop  => $sig->{'Stop'},
							-score => $sig->{'NegLogPValue'},
							-ref   => $ref,
							-type  => 'Marker' . $rsid,
							-name  => $sig->{'Marker_Accession'} . " ("
							  . sprintf( "%.2f", $sig->{'NegLogPValue'} ) . ")",
							-url => "javascript:showSigs("
							  . $sig->{SignificanceID}
							  . ",this.offsetTop, this.offsetLeft);"
						);

						$features->add_feature( $feature => $type );
						$sig_count++;
					}

				}

				#if no significant markers for a resultset - highlight this
				if ( $sig_count == 0 ) {
					$self->add_heading("$type - none present in this region");
				}
				$rscounter++;
				$rscounter != scalar( @{ $self->resultsetid } )
				  and $self->add_dashed_line;
			}
		}

=head2 add_combined_sig_markers

	  Usage      : $self->add_combined_sig_markers( { data => $data } );
	  Purpose    : For marker data provided (significant in multiple resultsets) adds markers to feature list
	  Returns    : Nowt
	  Arguments  : Hashref of:
	                data - hashref of markers with markerid as key (see calculate_max_and_sig_counts)
	  Throws     : 
	  Status     : Public
	  Comments   : 
=cut

		sub add_combined_sig_markers {
			my ( $self, $attrs ) = @_;

			my $data = $attrs->{data};

			my $features      = $self->features;
			my $ref           = $self->ref;
			my $feature_count = 0;

			my $rscounter = 0;

			#add diamond glyph to represent multi markers
			my $type = "Significant Markers across all resultsets";
			$features->add_type(
				$type => {
					bgcolor         => 'black',
					glyph           => 'diamond',
					fgcolor         => 'black',
					label           => 1,
					bump            => 1,
					title           => 1,
					height          => \&marker_size,
					width           => \&marker_size,
					clip            => 'true',
					scale           => 'none',
					'balloon hover' =>
'sub { my $feature = shift; return "Click to view significances" }',
				}
			);

			my $sig_count = 0;

			#loop through each marker
			foreach my $marker ( keys %{$data} ) {
				my @sigids = ();

				#for each marker, add significance IDs into an array
				my @multi_sigs = @{ $data->{$marker} };
				foreach (@multi_sigs) {
					push @sigids, $_->{'SignificanceID'};
				}
				my $sig      = $multi_sigs[0];
				my $rs_count = scalar(@multi_sigs);

#only add the feature if the number of resultsets they are signficant in passes the threshold set by the user
#use the array of significances in the javascript call (showSigs)
				if ( $rs_count >= $self->setting('resultsets_with_sig') ) {
					my $feature = Bio::Graphics::Feature->new(
						-start => $sig->{'Start'},
						-stop  => $sig->{'Stop'},
						-score => scalar(@multi_sigs),
						-ref   => $ref,
						-type  => 'MarkerAll',
						-name  => $sig->{'Marker_Accession'} . " ("
						  . $rs_count
						  . " resultsets)",
						-url => "javascript:var sigs = new Array("
						  . join( ",", @sigids )
						  . "); showSigs(sigs);"
					);

					$features->add_feature( $feature => $type );
					$sig_count++;
				}

			}
			
			#if none significant in multiple resultset highlight this
			if ( $sig_count == 0 ) {
				$self->add_heading("$type - none present in this region");
			}
			$rscounter++;

		}

=head2 marker_size

	  Usage      : height          => \&marker_size,
	  Purpose    : GBrowse callback to calculate size of glyph depending on feature score (used by add_combined_sig_markers)
	  Returns    : 4*score or 1 if no score
	  Arguments  : Dealt with by GBrowse
	  Throws     : 
	  Status     : Public
	  Comments   : 
=cut

		sub marker_size {
			my ($feature) = @_;
			my $val = $feature->score;
			if ( !$val ) {
				return 1;
			}
			else {
				return $val * 4;
			}
		}
		
		sub get_markers_in_resultsets {
			my ($self) = @_;
			return $self->db->get_markers_in_resultsets({
				chr=>$self->chr,
				start=>$self->round_start,
				stop=>$self->round_stop,
				threshold=>$self->threshold
			});
		}
		
		sub genomic_region {
		
			my ($self) = @_;
			if ($self->seg_start && $self->seg_stop) {
				return "Chr".$self->chr.":".$self->seg_start."..".$self->seg_stop;
			}
			else {
				return "Chr".$self->chr;
			}
		}
	}
	1;

=head1 SEE ALSO

=head1 BUGS AND LIMITATIONS

There are no known bugs in this module.
Please report problems to Rob Free <rcfree@gmail.com>

=head1 CONTACT

This module is part of the HGVbase-G2P project

=head1 AUTHOR

Rob Free <rcfree@gmail.com>


=head1 LICENCE AND COPYRIGHT

Copyright (c) <2009> University of Leicester, Leicester, UK. All rights reserved.

This module is free software; you can redistribute it and/or
modify it under the same terms as Perl itself.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.

=head1 CVSINFO

$Id: Core.pm 1258 2009-08-03 11:13:08Z rcf8 $ 

=cut

