# $Id: Core.pm 1250 2009-07-15 13:17:26Z rob $

=head1 NAME

HGVbaseG2P::DataImport::Core - High level logic for data DataImport and import pipeline


=head1 SYNOPSIS

	#for dbGaP import of a file
	use HGVbaseG2P::Browser::Core;

	my $validator = HGVbaseG2P::DataImport::Core->new({
				  conf_file =>
					"/Users/robf/Projects/hgvbaseG2P/perl/conf/hgvbase.conf",
				  argv      => \%ARGV,
				  template_file => '../conf/import/dbGaP.conf',
	  });

	  $validator->process_file($ARGV{sourcefile});
	  $logger->info("lines successful:\n".$validator->marker_success);

      my $marker_stats  = $validator->marker_failed;

=head1 DESCRIPTION

The DataImport::Core module ties together the logic of RetrieveMarker, LogicMaker, Plugin::* and Export::* modules.

The separation of core logic makes it easier to test the module by use of regression/unit testing. It also provides the option to
add additional Rule modules or a different Export module.

See HGVbaseG2P::DataImport for more information on the principles behind the import system and the helper classes involved.

=head1 SUBROUTINES/METHODS

=cut

package HGVbaseG2P::DataImport::Core;
{

	use strict;
	use warnings;

	# Use OIO inside-out object framework for our OO hierarachy
	use Object::InsideOut qw(HGVbaseG2P::DataImport);
	use Object::InsideOut::Metadata;

	use English qw( -no_match_vars );
	use Data::Dumper;
	use FindBin;
    use lib "$FindBin::Bin/../";
	use HGVbaseG2P::Database::Study;
	use HGVbaseG2P::Database::Marker;
	use HGVbaseG2P::DataImport::RetrieveMarker;
	use HGVbaseG2P::FileParser;
	use HGVbaseG2P::Util qw(load_config load_module);
	use HGVbaseG2P::DataImport::DataElement;
	use HGVbaseG2P::DataImport::LogicMaker;


	# Private variables & constants
	my @log : Field : All(log);          #contains Log4perl log
	my @config : Field : All(config);    #contains loaded config
	my @template : Field : All(template)
	  ;    #contains import config describing data file format etc.
	my @rules : Field : All(rules)
	  ; #contains arrayref of 'HGVbaseG2P::DataImport::Rule's to use on each line
	my @plugins : Field : All(plugins);
	my @error_stats : Field : All(stats);
	my @db : Field : All(db);
	my $line_count;
	my @conf_file : Field : All(conf_file);
	my @retrieve_marker : Field : All(retrieve_marker);
	my @fields : Field : All(fields);
	my @accession_db : Field : All(accession_db);
	my @comment_prefix : Field : All(comment_prefix);
	my @field_separator : Field : All(field_separator);
	my @file_parser : Field : All(file_parser);
	my @export : Field : All(export);
	my @head : Field : All(head);
	my @marker_failed : Field : All(marker_failed);
	my @marker_success : Field : All(marker_success);
	my @line_dump : Field : All(line_dump);
	my @chr_limit : Field : All(chr_limit);
	my @logic_maker : Field : All(logic_maker);
	my @argv : Field : All(argv);

	my $SUMMARY_NO = 500;
	my $BATCHING = 1;
	my $BATCH_SIZE = 500;
	my @elements=();

	# Initialization arguments
	my %init_args : InitArgs = ( 'template_file' => '', 'argv' => {}, 'sourcefile' => '');

	sub _init : Init {
		my ( $self, $args ) = @_;

		if ( $args->{template_file} ) {
			my %template = load_config( $args->{template_file}, 1 );
			$self->template( \%template );
		}

		my $study_db = HGVbaseG2P::Database::Study->new(
			{
				conf_file  => $self->config
			}
		);

		$self->db($study_db);

		my $retrieve_marker =
		  HGVbaseG2P::DataImport::RetrieveMarker->new(
			{ conf_file => $self->config } );
		$retrieve_marker->accession_db( $self->template->{'accession_db'} );

		my $marker_cred = $self->config->{MarkerDB};
		my $marker_db = HGVbaseG2P::Database::Marker->new(
			{
				conf_file  => $self->config
			}
		);
		$retrieve_marker->db($marker_db);
		$self->retrieve_marker($retrieve_marker);

		my $file_parser =
		  HGVbaseG2P::FileParser->new( { conf_file => $self->config } );
		$self->file_parser($file_parser);

		my $lmaker =
		  HGVbaseG2P::DataImport::LogicMaker->new( { template => $self->template } );

		$self->logic_maker($lmaker);
		$self->marker_failed( [] );
		$self->marker_success(0);
		$self->line_dump($args->{argv}->{l});
		$self->chr_limit($args->{argv}->{'ref'});
	}

	sub has_config { 1; }

=head2 process_file

  Usage      : $gtimport->process_file('/path/to/gtdatafile.xml')
  Purpose    : Main method for processing an input gt-data file from a particular source
  Returns    : undef if non-fatal errors are encountered during run, dies on fatal errors
  Arguments  : Path to input file.
  Throws     :
  Status     : Public
  Comments   :

=cut

	sub process_file : Public {
		my ( $self, $filename ) = @_;
		my @startTime  = localtime(time);
		my $line_count = 0;
		my $template   = $self->template;

		$self->setup;

		my $export = $self->export;

		#run 'before' method for each plugin if it exists
		if ($self->plugins) {
			foreach ( @{ $self->plugins } ) {
				$_->can("before") and $_->before($self);
			}
		}

		$export->begin;
		$self->run_codehook('before',[$self]);

		my $line_code;
		if ( $template->{line_code} ) {
			"eval '$line_code = sub {" . $template->{line_code} . "}'";
		}

		my $handler = sub {
			my ($line) = @_;

  #process line into fields hashref and pass this to preprocess and process line
			my $fields = $self->line_to_fields(@_);

			if ($self->chr_limit) {

				if ($fields->{chromosome} ne $self->chr_limit) {
					$self->log->info("ignore ".$fields->{accession});
					return;
				}

			}

			#ignore line if header or comment
			if ( $self->comment_prefix
				and substr( $line->[0], 0, 1 ) eq $self->comment_prefix )
			{
				$self->log->info(
					"Comment line - ignored:" . substr( $line->[0], 0, 30 ) );
				return;
			}

			if ( $self->head > 0 ) {
				my $head = $self->head;
				$self->head( --$head );
				return;
			}

   #run 'process' method for each plugin prior to processing each line if exists
			if ( $self->plugins ) {
				foreach my $plugin ( @{ $self->plugins } ) {
					$plugin->process_before_marker( $fields, $self );

				}
			}
			$self->run_codehook('process_before_marker',[$fields,$self]);
			$self->process_line($fields,$line_count);
			$line_count++;
			$line_count % $SUMMARY_NO == 0
			  and $self->log->info("$line_count entries processed");
		};

		$self->file_parser->parse_file(
			filename  => $filename,
			filetype  => 'delimited',
			handler   => $handler,
			field_sep => $self->field_separator,
			limit     => $template->{limit},
			line_code => $line_code,
			head      => $template->{head},
		);

		if ($BATCHING && scalar(@elements)>0) {
			$self->log->info("Storing final batch of ".scalar(@elements)." entries");
			$self->export->output(\@elements);
			@elements=();
		}

		#run 'after' method for each plugin if it exists
		if ( $self->plugins ) {
			foreach ( @{ $self->plugins } ) {
				$_->can("after") and $_->after($self);
			}
		}
		$self->run_codehook("after",[$self]);

		$export->after($self);

		if ($self->argv->{n}) {
			$self->log->info("Data NOT committed. -n flag set");
			$export->rollback;
			$export->db->disconnect;
		}
		else {
			$self->log->info("Data committed. No -n flag set");
			$export->commit;
			$export->db->disconnect;
		}

		my @endTime = localtime(time);

		$self->log->info("Disconnect from database");
		$self->db->disconnect;    # Finally explicitly close database
	}

	sub setup {
		my ($self) = @_;

		my $lmaker = $self->logic_maker;
		$self->rules( $lmaker->prepare_rules($self->template) );

		#get panels or undef if supplied as a field (panelname)
		my $panelnames = $lmaker->panelnames;

		#get resultsets or undef if supplied as a global (resultset_identifier)
		my $resultsets = $lmaker->resultsets;

		my $export = $lmaker->prepare_export;
		$export->before($self);

		$self->export($export);


		$self->plugins($self->import_plugins);

		#cycle through simple options and set parameters based on those found
		# -if freq_fields and assoc_fields set combined to 0 else 1
		$self->comment_prefix( $self->template->{comment_prefix} );

		my $sep = $self->template->{field_separator} || "\t";

		$sep =~ s/\\s/\ /;
		$sep =~ s/\\t/\t/;
		$self->field_separator($sep);
		$self->head( $self->template->{head} );
		$self->fields($lmaker->fields);
		# -any other
	}

	sub process_line {
		my ( $self, $line, $line_no ) = @_;

		$self->line_dump and $self->log->info("input line:".Dumper($line));

		my ( $marker, $status ) =
		  $self->retrieve_marker->get_validated_marker( $line->{accession} );

		my $element = HGVbaseG2P::DataImport::DataElement->new;
		$element->marker($marker);
		$element->status($status);
		$element->line($line);

		#run 'process_after_marker' method for each plugin after getting marker
		if ( $self->plugins ) {
			foreach my $plugin ( @{ $self->plugins } ) {
				$plugin->process_after_marker( $element, $self );
			}
		}

		$self->run_codehook('process_after_marker',[$element,$self]);

		if ($self->marker_error($status,$line,$line_no)) {
			$element->line_no($line_no);
			push @{ $self->marker_failed },$element;
			return;
		}

		foreach my $rule ( @{ $self->rules } ) {
			eval { $rule->run($element); };
			if ($@) {
				$self->log->logdie( "Error running 'run' rule "
					  . ref($rule)
					  . "\n$@\nsettings:"
					  . Dumper( $rule->settings )
					  . "\nline:"
					  . Dumper($line) );
			}
		}

		$self->run_codehook('process_after_rules',[$element,$self]);

		foreach my $rule ( @{ $self->rules } ) {
			eval { $rule->after_rules($element); };
			if ($@) {
				$self->log->logdie( "Error running 'after_rules' rule "
					  . ref($rule)
					  . "\n$@\nsettings:"
					  . Dumper( $rule->settings )
					  . "\nline:"
					  . Dumper($line) );
			}
		}

		$self->line_dump and $self->log->info("output line:".Dumper($line));

		if ( $status->{a} ) {
			$element->line_no($line_no);
			$self->log->error($marker->source_tag." NOT added as alleles invalid");
			push @{ $self->marker_failed },$element;
			return;
		}

		if ($BATCHING) {
			push @elements, $element;
			if (scalar(@elements)==$BATCH_SIZE) {
				$self->log->info("Storing batch of ".scalar(@elements)." entries");
				$self->export->output(\@elements);
				@elements=();
			}
		}
		else {
			$self->export->output([$element]);
		}

		my $success = $self->marker_success;
		$self->marker_success( ++$success );
	}

	sub marker_error {
		my ($self,$status,$line) = @_;
		if ( $status->{r} ) {
			$self->log->error($line->{accession}. " NOT added as replacement marker dead");
			return 1;
		}

		if ( $status->{n} ) {
			$self->log->error($line->{accession}. " NOT added as marker not found");
			return 1;
		}

		if ( $status->{s} ) {
			$self->log->error($line->{accession}. " NOT added as marker strand not present");
			return 1;
		}

		return 0;
	}
	sub line_to_fields {
		my ( $self, $line ) = @_;
		my $field_counter = 0;
		my %values;
		foreach ( @{ $self->fields } ) {
			$values{$_} = $line->[$field_counter];
			$field_counter++;
		}

		return \%values;
	}

	sub import_plugins {
		my ($self) = @_;
		my @plugin_names = split( ' ', $self->template->{plugins} );
		my @plugins = ();
		foreach my $plugin (@plugin_names) {
			$self->log->info("loading plugin $plugin");
			my $plugin_module =
			  load_module("HGVbaseG2P::DataImport::Plugin::$plugin");
			push @plugins, $plugin_module;
		}
		return \@plugins;
	}

	sub increment_stat : Public {
		my ( $self, $stat, $value ) = @_;
		$self->stats->{$stat}++;
		return $value;
	}

	sub run_codehook : Public {
		my ($self, $code_label, $args) = @_;

		my $code = $self->template->{'Code'}->{$code_label}->{sub};
		if ($code) {
			my $coderef;
			eval '$coderef = sub '.$code;

			eval {
				$coderef->(@{$args});
			};
			if ($@) {
				$self->log->logdie("Error attempting to run codehook: ".$code_label."\nOffending code:".$code."\nERROR:".$@);
			}
		}
	}
}
1;

=head1 SEE ALSO

=head1 BUGS AND LIMITATIONS

There are no known bugs in this module.
Please report problems to Rob Free <rcfree@gmail.com>

=head1 CONTACT

This module is part of the HGVbase-G2P project

=head1 AUTHOR

Rob Free <rcfree@gmail.com>


=head1 LICENCE AND COPYRIGHT

Copyright (c) <2009> University of Leicester, Leicester, UK. All rights reserved.

This module is free software; you can redistribute it and/or
modify it under the same terms as Perl itself.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.

=head1 SVNINFO

$Id: Core.pm 1250 2009-07-15 13:17:26Z rob $

=cut

