# $Id: Database.pm 1248 2009-07-15 11:55:06Z rob $

=head1 NAME

HGVbaseG2P::DataImport::Export::Database - Base database class for DataImport Export via Database Modules


=head1 SYNOPSIS
	
	package HGVbaseG2P::DataImport::Export::Database::Test
	use base qw(HGVbaseG2P::DataImport::Export::Database);
	
	#overrides base export method
	sub output {
		my ($self, $lines);
		my @lines = @{$lines};
		
		#use methods from the base class eg.
		my $db     = $self->db;

        my ( $study, $experiment ) = $self->get_study_and_experiment($line);
	}
	
=head1 DESCRIPTION

Base class for exporting to database (using HGVbaseG2P::Database modules) containing commmon methods used by MultiPanel and OnePanel modules. 
In inheriting classes the 'output' method is defined and calls these methods.

=head1 SUBROUTINES/METHODS 

=cut

package HGVbaseG2P::DataImport::Export::Database;
{

    use strict;
    use warnings;

    # Use OIO inside-out object framework for our OO hierarachy
    use Object::InsideOut qw(HGVbaseG2P::DataImport::Export);
    use Object::InsideOut::Metadata;

    use English qw( -no_match_vars );
    use Data::Dumper;
    use Carp qw(confess);

    my @study : Field : All(study); #hash of DBIx::Class study objects with definable key
    my @resultset : Field : All(resultset); #hash of DBIx::Class resultset objects with definable key
    my @experiment : Field : All(experiment); #hash of DBIx::Class experiment objects with definable key
    my @panel : Field : All(panel); #hash of hash of DBIx::Class assayedpanel objects with definable key of study then panel name
    my @multi_panel : Field : All(multi_panel); #multi panel flag (ie. all panels on one line or not)

    my @db : Field : All(db); #HGVbaseG2P::Study::Database object

    sub _init : Init {
        my ($self) = @_;
        $self->experiment( {} );
        $self->resultset(  {} );
        $self->panel(      {} );
        $self->study(      {} );
    }
	
	#sets up database for use by child modules
    sub before {
        my ( $self, $core ) = @_;

        my $db = $core->db;
        $self->db($db);
    }
	
=head2 generate_frequencies

	  Usage      : my ( $afs, $gfs ) = $self->generate_frequencies(
			            'CONTROL',         $element->line, $upstream30bp[0],
			            $downstream30bp[0], $self->multi_panel
			        );
	  Purpose    : Generate hashrefs containing allele and genotype frequency data using the assayedpanel name, flanks and line
	  Returns    : Allele frequency hashref; genotype frequency hashref
	  Arguments  : Panel name; DataElement line; upstream 30bp flank;downstream 30bp flank; multipanel flag
	  Throws     : 
	  Status     : Public
	  Comments   : Uses the multipanel flag to determine which field in the line to use to retrieve frequencies
	  
=cut

    sub generate_frequencies {
        my ( $self, $panelname, $line, $upstream30bp, $downstream30bp,
            $multi_panel )
          = @_;

        my @afs = ();

        foreach my $allele ( 1 .. 2 ) {
            my $temp_af = {
                allelecombo => substr( $upstream30bp, -10 ) . "("
                  . $line->{"allele$allele"} . ")"
                  . substr( $downstream30bp, 0, 10 ),
                frequencyasproportion => $line->{
                    "allele" . $allele . "_freq"
                      . ( $multi_panel ? ":$panelname" : "" )
                  }
            };

            push @afs, $temp_af;
        }

        my @gfs = ();
        foreach my $gt (qw(11 12 22)) {
            my $temp_gf = {
                genotypecombo => substr( $upstream30bp, -10 ) . "["
                  . $line->{"genotype$gt"} . "]"
                  . substr( $downstream30bp, 0, 10 ),
                frequencyasproportion => $line->{
                    "genotype" . $gt . "_freq"
                      . ( $multi_panel ? ":$panelname" : "" )
                  },
                numbersampleswithgenotype => $line->{
                        "genotype" . $gt
                      . "_number"
                      . ( $multi_panel ? ":$panelname" : "" )
                  }
            };

            push @gfs, $temp_gf;
        }

        return ( \@afs, \@gfs );

    }

=head2 get_data_item

	  Usage      : my $experiment = $self->get_data_item( 'experiment', $experimentid, 'identifier' );
	  Purpose    : Get database item. Builds up a method name based on the type and field.
	               e.g. the example above becomes 'get_experiment_by_identifier' and this is called against the HGVbaseG2P::Database::Study object
	               For speed, items are cached in a hash (e.g. studies, experiments etc.)
	  Returns    : HGVbaseG2P::Schema object
	  Arguments  : Type of object;key to retrieve;field name
	  Throws     : 
	  Status     : Public
	  Comments   : 
	  
=cut

    sub get_data_item {
        my ( $self, $type, $key, $field ) = @_;
        
        !$key and confess "No key provided";
        my $items = $self->$type;
        my $item  = $items->{$key};
        if ( !$item ) {
        	$self->log->debug("Getting type:$type, key:$key, field:$field");
            my $get_method = "get_$type" . "_by_$field";
            $item = $self->db->$get_method($key);
            if ( !$item ) {
                $self->log->logdie( "Unable to find $type by " . $field
                      . " '$key' - have you loaded metadata?" );
            }
            $items->{$key} = $item;
        }
        else {
            $self->log->debug( "Got cached: $type by " . $field
                      . " '$key'" );
        }
        return $item;
    }

=head2 get_data_item_from_other

	  Usage      : my $experiment = $self->get_data_item_from_other( 'study','identifier', 'experiment', $experimentid);
	  Purpose    : Get database item based on another item. Builds up a method name based on the types and fields.
	               e.g. the example above becomes 'get_experiment_by_identifier_and_study' and this is called against the HGVbaseG2P::Database::Study object
	               For speed, items are cached in a hash of hashes (e.g. experiments within studies etc.)
	  Returns    : HGVbaseG2P::Schema object
	  Arguments  : Type of object;key to retrieve;field name;other type of object;other key to retrieve
	  Throws     : 
	  Status     : Public
	  Comments   : 
	  
=cut

    sub get_data_item_from_other {
        my ( $self, $type, $key, $field, $other, $otherid ) = @_;
        my $items = $self->$type;
        !$items->{$otherid} and $items->{$otherid}={};
        my $item  = $items->{$otherid}->{$key};
        if ( !$item ) {
            my $get_method = "get_$type" . "_by_$field" . "_and_$other";
			$self->log->debug( "Getting type from other: $type by " . $field
                      . " '$key' in $other '$otherid'" );
            $item = $self->db->$get_method( $key, $otherid );
            if ( !$item ) {
                $self->log->logdie( "Unable to find $type by " . $field
                      . " '$key' in $other '$otherid' - have you loaded correct metadata?"
                );
            }
            else {
                $self->log->debug( "Got cached: $type by " . $field
                      . " '$key' in $other '$otherid'" );
            }
            $items->{$otherid}->{$key} = $item;
        }

        return $item;
    }

=head2 get_study_and_experiment

	  Usage      : my ($study,$experiment) = $self->get_study_and_experiment($element->line);
	  Purpose    : Gets study and experiment from the database or the line
	  Returns    : HGVbaseG2P::Schema::Study;HGVbaseG2P::Schema;Experiment
	  Arguments  : DataElement line
	  Throws     : 
	  Status     : Public
	  Comments   : 
	  
=cut

    sub get_study_and_experiment {
        my ( $self, $line ) = @_;

		#NOTE: may need to rewrite to deal with lookups by name/title etc (see resultsets)
		#get studyid from template or line
        my $studyid = $self->settings->{'study_identifier'}
          || $line->{study_identifier};

        #retrieve study object and cache
        my $study = $self->get_data_item( 'study', $studyid, 'identifier' );

        #get experimentid from template or line
        my $experimentid = $self->settings->{'experiment_identifier'}
          || $line->{experiment_identifier};

        #retrieve experiment object and cache it
        my $experiment =
          $self->get_data_item( 'experiment', $experimentid, 'identifier' );

        return ( $study, $experiment );
    }

=head2 get_resultset

	  Usage      : my ($resultset) = $self->get_resultset($element->line,$experiment,$rs_identifier);
	  Purpose    : Gets resultset from the database or the line
	  Returns    : HGVbaseG2P::Schema::Resultset
	  Arguments  : DataElement line; HGVbaseG2P::Schema::Experiment object; Resultset Identifier
	  Throws     : 
	  Status     : Public
	  Comments   : 
	  
=cut

    sub get_resultset {
        my ( $self, $line, $experiment, $rs_identifier ) = @_;
        
        #if multiple_resultsets then get the resultset from the experiment using the identifier
        if ($self->resultsets) {
		        my $resultset =
		          $self->get_data_item_from_other( 'resultset', $rs_identifier,
		            $self->resultset_lookup || 'identifier',
		            'experiment', $experiment->experimentid );
	        return $resultset;
        }
       
        else {
	        #get resultset lookup field and retrieve new or cached resultset object based on this
	        my $resultsetid =
	            $self->resultset_field
	          ? $line->{ $self->resultset_field }
	          : $self->settings->{'resultset_identifier'};
	
	        my $resultset =
	          $self->get_data_item_from_other( 'resultset', $resultsetid,
	            $self->resultset_lookup,
	            'experiment', $experiment->experimentid );
			
	        return $resultset;
        }
    }

=head2 store_usedmarkerset

	  Usage      : my $ums = $self->store_usedmarkerset( $experiment, $marker );
	  Purpose    : Create/find a usedmarkerset for an experiment and marker
	  Returns    : HGVbaseG2P::Schema::Usedmarkerset
	  Arguments  : HGVbaseG2P::Schema::Experiment object; DataElement marker (BioSeqFeature)
	  Throws     : 
	  Status     : Public
	  Comments   : 
	  
=cut

    sub store_usedmarkerset {
        my ( $self, $experiment, $marker ) = @_;

        my $template = $self->template;
        #check if exists already in this experiment
        my $ums = $experiment->find_or_create_related(
            'usedmarkersets',
            {
                markeridentifier => $marker->display_name
            }
        );

        return $ums;
    }

=head2 store_frequencycluster

	  Usage      : my $fc = $self->store_frequencycluster($element, $panelname, $study, $ums);
	  Purpose    : Create/find frequencycluster (including allele/genotype frequencies) for a single panel
	  Returns    : HGVbaseG2P::Schema::Frequencycluster
	  Arguments  : DataElement object; Assayedpanel name; HGVbaseG2P::Schema::Study object; HGVbaseG2P::Schema::Usedmarkerset object; 
	  Throws     : 
	  Status     : Public
	  Comments   : 
	  
=cut

    sub store_frequencycluster {
        my ( $self, $element, $panelname, $study, $ums ) = @_;

        #get flanking sequences for combos
        my @upstream30bp   = $element->marker->get_tag_values('upstream30bp');
        my @downstream30bp = $element->marker->get_tag_values('downstream30bp');
        my ( $afs, $gfs ) = $self->generate_frequencies(
            $panelname,         $element->line, $upstream30bp[0],
            $downstream30bp[0], $self->multi_panel
        );

        my $panel =
          $self->get_data_item_from_other( 'panel', $panelname, 'name', 'study',
            $study->studyid );

        my $pvaluehwe =
          $element->line->{ 'pvaluehwe:'
              . ( $self->multi_panel ? ":$panelname" : "" ) };

        my $total_field =
          "genotype_total"
          . ( $self->multi_panel ? ":$panelname" : "" );

        my $freq_cluster = {
            assayedpanelid           => $panel->assayedpanelid,
            allelefrequencies        => $afs,
            genotypefrequencies      => $gfs,
            numberofgenotypedsamples => $element->line->{$total_field},
            pvaluehwe                => $pvaluehwe,
        };
		$self->log->debug("Store frequency_cluster:".Dumper($freq_cluster));
        my $frequencyclusters =
          $ums->find_or_create_related( 'frequencyclusters',
            $freq_cluster );
        return $frequencyclusters;
    }

=head2 store_signficances

	  Usage      : $self->store_significances($element,$ums, $experiment,\@frequencyclusters)
	  Purpose    : Create/find significances (including fcs) for a single line (if multiple pvalues creates multiple significances)
	  Returns    : HGVbaseG2P::Schema::Frequencycluster
	  Arguments  : DataElement object; HGVbaseG2P::Schema::Usedmarkerset object; HGVbaseG2P::Schema::Experiment object; Arrayref of HGVbaseG2P::Schema::Frequencycluster objects or undef
	  Throws     : 
	  Status     : Public
	  Comments   : 
	  
=cut

    sub store_significances {
        my ( $self, $element, $ums, $experiment, $fcs ) = @_;
		my @significances;
		 
        #frequency cluster will be the same for each marker in an experiment
		#if no frequencyclusters passed to method look them up in the experiment
		
        #create fcs for the individual significance
        my @fcs;
        if (!$fcs) {
        		$self->log->debug("fcs not found - searching experiment for frequency cluster and creating fcs");
        		my @frequencyclusters = $ums->frequencyclusters->all;
        		@fcs = map { { frequencyclusterid => $_->frequencyclusterid } } @frequencyclusters;
        }
        else {
        	$self->log->debug("fcs found:".Dumper($fcs));
        	@fcs = @{$fcs};
        }
                
        #set up a single significance for a single resultset
        if ($self->resultsets) {
        	foreach my $resultset(@{$self->resultsets}) {
	        
		        my $significance = {
			        unadjustedpvalue => $element->line->{'pvalue:'.$resultset},
		            usedmarkersetid  => $ums->usedmarkersetid,
		        };
				
				my $info = {
					marker_identifier => $ums->markeridentifier,
					experiment => $ums->experimentid->identifier,
				};
				
				foreach my $fcluster($ums->frequencyclusters->all) {
					my ($af,$gf);
					foreach my $afrequency($fcluster->allelefrequencies->all) {
						$af->{$afrequency->allelecombo}=$afrequency->frequencyasproportion;
					}
					foreach my $gfrequency($fcluster->genotypefrequencies->all) {
						$gf->{$gfrequency->genotypecombo}=$gfrequency->frequencyasproportion;
					}
					$info->{$fcluster->assayedpanel->name}={
						frequencyclusterid=>$fcluster->frequencyclusterid,
						allelefrequencies=>$af,
						genotypefrequencies=>$gf,
					};

				}
					
				#create the significance record using the resultset
				$self->log->debug("significance for $resultset:".Dumper($significance).Dumper($info));
        		my $sig = $self->get_resultset($element->line, $experiment, $resultset)->find_or_create_related( 'significances', $significance );
        		$sig->find_or_create_related('fcs',@fcs);
        	}
        	
        }
        else {
	        my $significance = ({
	            unadjustedpvalue => $element->line->{'pvalue'},
	            usedmarkersetid  => $ums->usedmarkersetid,
	        });
	        
	        
	        #create the significance record using the above hashref
        	my $sig = $self->get_resultset($element->line, $experiment, undef)->find_or_create_related( 'significances', $significance );
       		$sig->find_or_create_related('fcs',$fcs);
        }
    }

=head2 begin

	  Usage      : $export->begin;
	  Purpose    : Begin Export database 'transaction'
	  Returns    : Nowt
	  Arguments  : None
	  Throws     : 
	  Status     : Public
	  Comments   : 
	  
=cut

    sub begin {
        my ($self)=@_;
        $self->db->dbh->txn_begin;
    }

=head2 commit

	  Usage      : $export->commit;
	  Purpose    : Commit Export database 'transaction' changes
	  Returns    : Nowt
	  Arguments  : None
	  Throws     : 
	  Status     : Public
	  Comments   : 
	  
=cut

    sub commit {
        my ($self)=@_;
        $self->log->info("commit");
        $self->db->dbh->txn_commit;
        #$self->db->dbh->disconnect;
    }
    
=head2 rollback

	  Usage      : $export->rollback;
	  Purpose    : Rollback Export database 'transaction' changes
	  Returns    : Nowt
	  Arguments  : None
	  Throws     : 
	  Status     : Public
	  Comments   : 
	  
=cut
    sub rollback {
        my ($self)=@_;
        $self->log->info("rollback");
        $self->db->dbh->txn_rollback;
    }
}
1;

=head1 SEE ALSO

=head1 BUGS AND LIMITATIONS

There are no known bugs in this module.
Please report problems to Rob Free <rcfree@gmail.com>

=head1 CONTACT

This module is part of the HGVbase-G2P project

=head1 AUTHOR

Rob Free <rcfree@gmail.com>


=head1 LICENCE AND COPYRIGHT

Copyright (c) <2009> University of Leicester, Leicester, UK. All rights reserved.

This module is free software; you can redistribute it and/or
modify it under the same terms as Perl itself.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 

=head1 SVNINFO

$Id: Database.pm 1248 2009-07-15 11:55:06Z rob $ 

=cut

