# $Id: AffymetrixLookup.pm 1248 2009-07-15 11:55:06Z rob $

=head1 NAME

HGVbaseG2P::DataImport::Plugin::AffymetrixLookup - DataImport Plugin dealing with affymetrix IDs


=head1 SYNOPSIS

	#in DataImport template file
	plugins = AffymetrixLookup
	
	#the plugin is called by the DataImport pipeline

=head1 DESCRIPTION

Plugin to deal with Affymetrix ID lookups.

Does the following:
Gets affymetrix->rsid data from CSV files provided by Affymetrix. Then for each line it:
Gets the RSid using the Affymetrix ID
Compares the strand in the DB with the strand in the Affymetrix file -> if they are different does a strand flip

=head1 SUBROUTINES/METHODS 

=cut

package HGVbaseG2P::DataImport::Plugin::AffymetrixLookup;
{
	use strict;
	use warnings;

	# Use OIO inside-out object framework for our OO hierarachy
	use Object::InsideOut qw(HGVbaseG2P::DataImport::Plugin);
	use Object::InsideOut::Metadata;
	use Data::Dumper qw(Dumper);
	
	my @affyid : Field :All(affyid);
	
	sub _init :Init {
		my ($self)=@_;
		$self->affyid({});
	}
	
	sub before {
		my ($self, $core)=@_;
		
		my @annot_files = @{$core->config->{wtccc_annot_file}};
		
		$self->log->logdie("'affymetrixid' field not set in template!") if !$core->logic_maker->field_exists('affymetrixid');
			
      	my $fileparser = HGVbaseG2P::FileParser->new();
		
		$self->log->info("before AffymetrixLookup method");
		foreach my $file ( @annot_files ) {
			$self->log->info("Plugin reading affy -> rsid lookup $file");
			$fileparser->parse_file(
				filename => $file,
				filetype => 'delimited',
				field_sep=>',',
				handler  => sub { $self->_affyid_reader(@_) },
			);
		}
	}
	
	sub process_before_marker {
		my ($self, $fields) = @_;
		my $accession = $fields->{'accession'};
		if ($accession eq ' ') {
			my $affyid = $fields->{'affymetrixid'};
			my $lookup = $self->affyid->{$affyid}->{accession};
			$fields->{'accession'}=$lookup; 
			$self->log->debug("Looked up rsid in affy hash:$lookup");
		}
	}
	
	sub process_after_marker {
		my ($self, $element, $core)=@_;
		$core->db->dbh->resultset
		return if !$element->marker;
		
		my $affyid = $element->line->{'affymetrixid'};
		
		if ($self->affyid->{$affyid}) {
			if (!$element->marker->strand) {
				$self->log->error("Strand not found in database marker");
				$element->status->{s}=1;
				return;
			}
			
			if (!$self->affyid->{$affyid}->{strand}) {
				$self->log->error("Strand not found in Affymetrix file");
				$element->status->{s}=1;
				return;
			}
			
			$self->log->debug("strands: marker = ".$element->marker->strand.", affy = ".$self->affyid->{$affyid}->{strand});
			
			if ($self->affyid->{$affyid}->{strand} ne $element->marker->strand) {
				$self->log->debug("Setting strand flip for ".$element->marker->source_tag);
				$element->flip(1);
			}
		}
		else {
			$self->log->error("AffyID $affyid not found for strand flip! Strand was:".$element->marker->strand);
		}
	}
	
	sub _affyid_reader {
		my ( $self, $line ) = @_;
		return if $line->[0] =~ /^Probe/;
		return if $line->[0] =~ /^#/;
		
		#print "affy:".$line->[0]."=> accession = ".$line->[2].", strand = ".$line->[5]."1\n";
		$self->affyid->{ $line->[0] } =  {accession=>$line->[2], strand=>($line->[5] eq "---" ? undef : int($line->[5]."1") )};
	}
	
	sub after {
		
	}
}	
1;

=head1 SEE ALSO

=head1 BUGS AND LIMITATIONS

There are no known bugs in this module.
Please report problems to Rob Free <rcfree@gmail.com>

=head1 CONTACT

This module is part of the HGVbase-G2P project

=head1 AUTHOR

Rob Free <rcfree@gmail.com>


=head1 LICENCE AND COPYRIGHT

Copyright (c) <2009> University of Leicester, Leicester, UK. All rights reserved.

This module is free software; you can redistribute it and/or
modify it under the same terms as Perl itself.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 

=head1 SVNINFO

$Id: AffymetrixLookup.pm 1248 2009-07-15 11:55:06Z rob $ 

=cut