# $Id: RetrieveMarker.pm 1248 2009-07-15 11:55:06Z rob $

=head1 NAME

HGVbaseG2P::DataImport::RetrieveMarker - Retrieve validated marker from the MarkerDB and set status (used by DataImport::Core)


=head1 SYNOPSIS

	use HGVbaseG2P::DataImport::RetrieveMarker;
	use HGVbaseG2P::Database::Marker;

	#set up retrieve marker object
    my $retrieve_marker =
		  HGVbaseG2P::DataImport::RetrieveMarker->new(
			{ conf_file => $self->config } );
		$retrieve_marker->accession_db( 'dbSNP' );

	#set up marker db and populate db in retrieve_marker
	my $marker_db = HGVbaseG2P::Database::Marker->new(
		{
			conf_file  => $self->config
		}
	);
	$retrieve_marker->db($marker_db);


=head1 DESCRIPTION

This class deals with marker DataImport logic. It returns the marker in BioSeqFeature format along with a status hash.

=head1 SUBROUTINES/METHODS

=cut

package HGVbaseG2P::DataImport::RetrieveMarker;
{

	use strict;
	use warnings;

	# Use OIO inside-out object framework for our OO hierarachy
	use Object::InsideOut qw(HGVbaseG2P::DataImport);
	use Object::InsideOut::Metadata;
	use Bio::SeqFeature::Generic;

	use English qw( -no_match_vars );
	use Data::Dumper;

	# Private variables & constants
	my @db : Field : All(db); #MarkerDB object
	my @accession_db : Field : All(accession_db); #accession DB Name (e.g. dbSNP)

	sub has_config { 0; }

=head2 get_validated_marker

	  Usage      : my $marker = $rmarker->get_validated_marker('rs699');
	  Purpose    : Retrieve validated marker using accession
	  Returns    : Nothing
	  Arguments  : DB accession
	  Throws     :
	  Status     : Public
	  Comments   :

=cut

sub get_validated_marker {
		my ( $self, $accession ) = @_;
		my $status={};

		#retrieve marker from MarkerDB
		my $marker =
		  $self->db->get_marker_by_accession_and_source( $accession,
			$self->accession_db );

		#if no marker present then set n flag and return undef
		if ( !$marker ) {
			$self->log->debug( "Marker with accession $accession ("
				  . $self->accession_db
				  . ") not found" );
				 $status->{n}=1;
			return (undef,$status);
		}

		#if marker is dead set d flag and check revision history to see if it is merged
		if ($marker->status eq "dead") {
			$status->{d}=1;

			# Grab revision history for marker and see if we find a pointer to a current marker
			my $latest_rev = $marker->revisions->search(undef,{ order_by => 'timecreated DESC' })->first;

			#if merged (m flag) then check for strand flip (f flag)
			if ( $latest_rev && $latest_rev->changetype eq 'Merging' ) {
				my $replacedby_marker = $latest_rev->replacedbymarker;

				$status->{m}=1;

				if (defined($latest_rev->strandflipped)) {
					$status->{f}=1;
				}

				#if merged marker is dead then set r flag
				if ($replacedby_marker->status eq "dead") {
					$status->{r}=1;
					$self->log->warn( "Replacement marker ". $replacedby_marker->accession
						  . " marked as DEAD" );
						 return (undef,$status);
				}

				return ($self->_convert_marker_to_seqfeature($replacedby_marker),$status);
			}
			else {
			$self->log->error(
"Got reference to dead marker $accession with no pointer to current marker which replaced it"
				);
				return (undef,$status);
			}
		}

		#convert marker to seqfeature and return
		return ($self->_convert_marker_to_seqfeature($marker), $status)
	}

=head2 _convert_marker_to_seqfeature

	  Usage      : my $marker = $rmarker->_convert_marker_to_seqfeature($marker)
	  Purpose    : Convert HGVbaseG2P::Schema::Marker to Bio::SeqFeature
	  Returns    : Bio::SeqFeature
	  Arguments  : HGVbaseG2P::Schema::Marker object
	  Throws     :
	  Status     : Public
	  Comments   :

=cut

	sub _convert_marker_to_seqfeature {
		my ($self, $marker)=@_;

		#get information required to make seqfeature (e.g. alleles, strand,start,stop,chr)
		my $coords =
		  $marker->markercoords->search( { 'assemblytype' => 'ref_assembly' } )
		  ->first;

		my @alleles = $marker->alleles->all;

		my @allele_seq_list;

		foreach my $allele (@alleles) {
			if ( $allele->status eq "active" ) {
				push @allele_seq_list, $allele->alleleseq;
			}
		}

		my ($start, $stop, $strand, $chr);

		if ( !$coords ) {
			$self->log->error("MarkerCoord not found for ref_assembly all coord data will be NULL");
		}
		else {
			$start = $coords->start;
			$stop = $coords->stop;
			$strand = $coords->strand;
			$chr = $coords->chr;
		}

		#create generic SeqFeature and add chr, alleles and flanks as tags
		#set source to accession and display_name to identifier
		my $feature = Bio::SeqFeature::Generic->new(
			-start => $start,
			-stop => $stop,
			-strand => $strand,
			-source => $marker->accession,
			-display_name => $marker->identifier,
			-tag          => {chr => $chr, alleles=>\@allele_seq_list, upstream30bp => $marker->upstream30bp, downstream30bp => $marker->downstream30bp }
			);
		return $feature;
	}
}
1;

=head1 SEE ALSO

=head1 BUGS AND LIMITATIONS

There are no known bugs in this module.
Please report problems to Rob Free <rcfree@gmail.com>

=head1 CONTACT

This module is part of the HGVbase-G2P project

=head1 AUTHOR

Rob Free <rcfree@gmail.com>


=head1 LICENCE AND COPYRIGHT

Copyright (c) <2009> University of Leicester, Leicester, UK. All rights reserved.

This module is free software; you can redistribute it and/or
modify it under the same terms as Perl itself.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.

=head1 SVNINFO

$Id: RetrieveMarker.pm 1248 2009-07-15 11:55:06Z rob $

=cut