# $Id: Database.pm 1255 2009-07-17 10:51:11Z rcf8 $

=head1 NAME

HGVbaseG2P::Database - Storing to / retrieving from a DBIx::DBStag database


=head1 SYNOPSIS

    use English qw( -no_match_vars );
    use base qw(HGVbaseG2P::Database);
    my $dbh = HGVbaseG2P::Database->new(conf_file => 'path to master config');
    my $node_set = $dbh->selectall_stag(-template => 'hgvbase-marker',
                                        -bind     => {MarkerID => 'HGVM510'});


=head1 DESCRIPTION

This is the base class for the HGVbaseG2P::Database modules.

It provides a simple wrapper to connect to databases in the HGVbaseG2P project.

The following database module are available:
Study - Class::DBIx system
Marker - Class::DBIx system
DB - DBIx::DBStag [todo: extract into separate pipeline]
Browser - DBI connection to Browser database
Mart - BioMart API based connection to HGVmart

When initialising, the HGVbaseG2P::Database module runs the 'init_data_source' method in the child class (if it exists).
This should connect to a database and return the connected 'object'  (e.g. DBI) from this method.

eg.

    sub init_data_source {
        my ($self, $args)=@_;
        my %config = %{$self->config};
        
        # Connect to database using either info from config OR parameters provided as
	    # arguments to the constructor-call. Arguments will override configfile
	    my $m_config=$config{MarkerDB};
          
        # Connect to database using either info from config OR parameters provided as
	    # arguments to the constructor-call. Arguments will override configfile
	    
	    my $dsn = $m_config->{dsn} || $args->{dsn} || $self->log->logcroak("No database DSN provided with the dsn parameter or configfile");
	    my $dbuser = $args->{user} || $m_config->{dbuser} || getlogin();
	    $self->log->info("Connecting to database $dsn as user $dbuser");
	    my $dbpassw = $args->{pass} || $m_config->{dbpassw};
	    	my $dbh = HGVbaseG2P::Schema->connect($dsn,
                               $dbuser,
                               $dbpassw,   
                               {
                                   PrintError => 1, # warn() on errors
                                   RaiseError => 1, # do die on error
                                  # AutoCommit => 0, # does not commit immediately
                               }) or OIO->throw("Could not initialize db-connection. DBI::errstr=".$DBI::errstr);
    		$self->log->info("got database handle ".$dbh);
	    
	    return $dbh;
    }

The db_handle is then accessible from $self->data_source OR $self->dbh.

To connect to the module:

use HGVbaseG2P::Database::Marker;
my $marker_db = HGVbaseG2P::Database::Marker->new({conf_file=>'conf/hgvbase.conf'}); 

TODO: Provide a 'new' method that allows you to specify the module so you can say:
my $marker_db = HGVbaseG2P::Database->new({type=>'Marker', conf_file=>'conf/hgvbase.conf'});

NOTE: These modules will probably be replaced with the HGVbaseG2P::DataAccess modules which will automagically use the remote data access if 
a local database is not available, or a combination of the two depending on the scenario.

=head1 SUBROUTINES/METHODS 

=cut

package HGVbaseG2P::Database; {
    
use strict;
use warnings;

# Use OIO inside-out object framework for our OO hierarachy
use Object::InsideOut qw( DBIx::DBStag );
use Object::InsideOut::Foreign;
use Object::InsideOut::Universal;
use Object::InsideOut::Metadata;

use English qw( -no_match_vars );
use Data::Dumper qw(Dumper);
use FindBin;
use Cwd;
use HGVbaseG2P::Util qw(load_config init_logger );
use Env;


# Private variables & constants
    #my @output_to : Field :All(output_to); #set where all subsequent data should be output (default file name is output.out)
    my @log : Field :All(log); #contains Log4perl Logger
    my @config : Field :All(config); #contains loaded config
    #my @file_settings : Field :All(file_settings); #contains settings related to file output including delimiter and headings flag
    my @prepared : Field :All(prepared);
    my @data_source : Field :All(data_source);
    
    # Initialization arguments
    my %init_args : InitArgs = (
        'conf_file'     => '',
        'log4perl_conf' => '',
        'dsn' => '',
        'user' => '',
        'pass' => '',
    );

    # Initialization method (see OIO docs for details)
    sub _init : Init
    {
        my ( $self, $args ) = @_;

		
        # Load supplied config file, if non provided use default
        my $conf_file = $args->{conf_file} || undef;
 		!$conf_file and die "No config file supplied";
        $self->config({load_config( $conf_file )});
		
		# Initialize logger
        my $log4perl_conf = $args->{log4perl_conf} || $self->config->{log4perl_conf};
        $self->log(init_logger( $log4perl_conf, __PACKAGE__ ));
        
        #set data source
        my $source = $self->init_data_source({
        	dsn=>$args->{dsn},
        	user=>$args->{user},
        	pass=>$args->{pass},
        });

        $self->data_source($source);
        
        #set default file settings
        #$self->file_settings({ headings => 1, delimiter => ',' });
        #$self->prepared({});
    }
	
	#alias for data_source
	sub dbh {
		my ($self)=@_;
		return $self->data_source;
	}
	
#	  sub dbstag_query :Public {
#	    my ($self, $template, $bind_vals, $attrs)=@_;
#	    
#	    #set up DBStag parameters
#	    my @pars = ( -template => $template,
#	        -bind=>$bind_vals);
#	    
#	    #if output to file use HGVbaseG2P::FileWriter
#	    my $output_to = $attrs->{'output_to'} || 'ARRAY';
#	    
#	    if ($attrs->{file} || $output_to eq 'FILE') {
#	    	my $file = $attrs->{'file'} || 'hgvbase.out';
#	    	
#	    	$self->output_to_file({
#	    		file=>$file, 
#	    		data=>$self->selectall_rows(@pars)
#	        });
#	        return;
#	    }
#	    
#	    #if no file return results as an array or DBStag object
#	    $self->log->info("Output results as $output_to");	
#	    if ($output_to eq 'ARRAY') {
#	        return $self->selectall_rows(@pars);
#    	}
#	    elsif ($output_to eq 'STAG') {
#	        return $self->selectall_stag(@pars);  
#	    }
#	}
#	
	sub dbi_select :Public {
        my ($self, $sql, $parameters, $attrs)=@_;
        my $sth;
        $self->log->info("sql:$sql");

        $sth = $self->data_source->prepare($sql);
#       
        $sth->execute(@{$parameters});
        $self->log->info("Executed with parameters:".Dumper($parameters));
        
        #if output to file use HGVbaseG2P::FileWriter
	    my $output_to = $attrs->{'output_to'} || 'hash';
	    
        #if no file return results as an array or DBStag object
        my $result;
        $self->log->info("Output results as $output_to");   
       if ($output_to eq 'hash') {
            while(my $row = $sth->fetchrow_hashref) {
            	push @{$result},$row;
            }
        }
        elsif ($output_to eq 'array') {
            $result = $sth->fetchall_arrayref;
            
        }
        elsif ($output_to eq 'sth') {
            return $sth;  
        }
        $sth->finish;
	
        return $result ? $result : [];
    }
#    
#    sub dbi_select_datatable :Public {
#        my ($self, $sql, $parameters, $attrs)=@_;
#        my $data_table = Data::Table::fromSQL($self->dbh, $sql);
#        return $data_table;
#    }
#    
#    sub dbi_insert :Public {
#        my ($self, $sql, $data, $line_options)=@_;
#        my @fields = keys %{$line_options};
#        my @placeholders=();
#        my $table="precalculated";
#        
#        foreach(@fields) {
#        	push @placeholders,"?";
#        }
#        
#        $sql = "INSERT INTO ".$table."(".join(",",@fields).") VALUES(".join(",",@placeholders).")";
#        my $sth = $self->data_source->prepare($sql);
#
#        foreach my $item(@{$data}) {
#        	my @parameters=();
#        	foreach my $field(@fields) {  
#                push @parameters, $self->get_data($item, $line_options->{$field});
#            }
#        	$sth->execute(@parameters);
#        }
#		$sth->finish;
#    }
#    
#    sub dbi_insert_single :Public {
#        my ($self, $sql, $data, $line_options)=@_;
#        my @fields = keys %{$line_options};
#        my @placeholders=();
#        my $table="precalculated";
#        
#        foreach(@fields) {
#        	push @placeholders,"?";
#        }
#        
#        $sql = "INSERT INTO ".$table."(".join(",",@fields).") VALUES(".join(",",@placeholders).")";
#        my $sth = $self->data_source->prepare($sql);
#
#        foreach my $item(@{$data}) {
#        	my @parameters=();
#        	foreach my $field(@fields) {  
#                push @parameters, $self->get_data($item, $line_options->{$field});
#            }
#        	$sth->execute(@parameters);
#        }
#		$sth->finish;
#    }
#    
#    sub dbi_delete :Public {
#        my ($self, $sql, $parameters)=@_;
#        my $sth = $self->data_source->prepare($sql);
#        $sth->execute(@{$parameters});
#        $sth->finish;
#    }
#	
#	#hack method to be replaced with something using an HGVbaseG2P::FileWriter
#	sub output_to_file {
#		my ($self, $attrs)=@_;
#		my $delimiter = $self->file_settings->{delimiter};
#        my $filename = $attrs->{file};
#        
#        $self->log->info("Writing to file $filename");
#        open SIGOUT, ">", $filename;
#            
#        my @stag_rows = @{$attrs->{data}};
#
#        foreach my $row(@stag_rows) {
#            print SIGOUT join($delimiter,@{$row})."\n";
#        }      
#        $self->log->info("Wrote " . scalar(@stag_rows) ." rows");     
#	}
#    
#    sub get_data {
#        my ($self, $item, $field) = @_;
#        
#        #$self->log->info("Refe:".ref($field));
#        if (ref($field) eq 'ARRAY') {
#        	my $result = '';
#	        foreach my $element(@{$field}) {
#		        
#		        if ($element=~/\$(.*)/) {
#		            #$self->log->info("Field '$1' found in $element element");
#		            #!$item->{$1} and $self->log->info("No element $1 found in ".Dumper($item));
#		            $result.=$item->{$1};
#		        }
#		        else {
#		        	$result.=$element;
#		        	#$self->log->info("Constant field $element found in $element element ($1)");
#		        }
#	        }
#	        return $result;
#	    }
#        else {
#        	#$self->log->info("Constant field for $field");
#        	return $field;
#        }
#        
#    }

=head2 commit

  Usage      : $xmldb->commit();
  Purpose    : Commit db-transaction after one or more insert/update operations
  Returns    : N/A
  Arguments  : none
  Throws     : Dies on errors
  Status     : Public
  Comments   : This method needs to be called after one or more store_file() calls
               to finish the db-operation. Otherwise it will be rolled back.

=cut

sub commit :Public {
    my $self = shift;
    $self->data_source->commit() || $self->log->logconfess("Error committing transaction: $@");
}

=head2 disconnect

  Usage      : $xmldb->disconnect();
  Purpose    : Disconnect from database
  Returns    : N/A
  Arguments  : none
  Throws     : none
  Status     : Public

=cut

sub disconnect :Public {
    my $self = shift;
    $self->log->info("Disconnecting ".ref($self));
    $self->data_source->disconnect() || $self->log->warn("Error disconnecting from database: $@");
}

=head2 rollback

  Usage      : $xmldb->rollback();
  Purpose    : Roll back transaction
  Returns    : N/A
  Arguments  : none
  Throws     : none
  Status     : Public

=cut

sub rollback :Public {
    my $self = shift;
    $self->data_source->dbh->rollback() || $self->log->warn("Error disconnecting from database: $@");
}


}
1;

=head1 SEE ALSO

L<HGVbaseG2P::FileParser>, L<HGVbaseG2P::Database> and related classes

=head1 BUGS AND LIMITATIONS

There are no known bugs in this module.
Please report problems to Rob Free <rcfree@gmail.com>

=head1 CONTACT

This module is part of the HGVbaseG2P project

=head1 AUTHOR

Rob Free <rcfree@gmail.com>

=head1 LICENCE AND COPYRIGHT

Copyright (c) <2007> University of Leicester, Leicester, UK. All rights reserved.

This module is free software; you can redistribute it and/or
modify it under the same terms as Perl itself.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 

=head1 CVSINFO

$Id: Database.pm 1255 2009-07-17 10:51:11Z rcf8 $ 

=cut

