# $Id: Marker.pm 1272 2009-08-21 15:08:35Z rcf8 $

=head1 NAME

HGVbaseG2P::Database::Marker - Query Marker HGVbaseG2P DB using Class::DBIx ORM


=head1 SYNOPSIS

use HGVbaseG2P::Database::Marker;
my $marker_db = HGVbaseG2P::Database::Marker->new( { conf_file => '/conf/hgvbase.conf' } );
my $marker = $marker_db->get_marker_and_refcoords_by_identifier('HGVM1');


=head1 DESCRIPTION

Provides a wrapper for common Marker database access methods. Returns single or arrays of HGVbaseG2P::Schema objects (derived from DBIx::Class) or
a DBIx::Class ResultSet object

=head1 SUBROUTINES/METHODS 

=cut

package HGVbaseG2P::Database::Marker; {
    use FindBin;
use lib "$FindBin::Bin/../../";
    
	use strict;
	use warnings;
	
	# Use OIO inside-out object framework for our OO hierarachy
	use Object::InsideOut qw( HGVbaseG2P::Database );
	use Object::InsideOut::Metadata;
	
	use English qw( -no_match_vars );
	use Data::Dumper;
	use FindBin;
	use Carp qw(croak cluck);
	use HGVbaseG2P::Schema;
	our $AUTOLOAD;
	
	#method common to all 'HGVbaseG2P::Database' modules which initializes the data source (see HGVbaseG2P::Database)
    sub init_data_source {
        my ($self, $args)=@_;
        my %config = %{$self->config};
        
        # Connect to database using either info from config OR parameters provided as
	    # arguments to the constructor-call. Arguments will override configfile
	    my $m_config=$config{MarkerDB};
          
        # Connect to database using either info from config OR parameters provided as
	    # arguments to the constructor-call. Arguments will override configfile
	    
	    my $dsn = $m_config->{dsn} || $args->{dsn} || $self->log->logcroak("No database DSN provided with the dsn parameter or configfile");
	    my $dbuser = $args->{user} || $m_config->{dbuser} || getlogin();
	    $self->log->info("Connecting to database $dsn as user $dbuser");
	    my $dbpassw = $args->{pass} || $m_config->{dbpassw};
	    	my $dbh = HGVbaseG2P::Schema->connect($dsn,
                               $dbuser,
                               $dbpassw,   
                               {
                                   PrintError => 1, # warn() on errors
                                   RaiseError => 1, # do die on error
                                  # AutoCommit => 0, # does not commit immediately
                               }) or OIO->throw("Could not initialize db-connection. DBI::errstr=".$DBI::errstr);
    		$self->log->debug("got database handle ".$dbh);
	    
	    return $dbh;
    }
    
=head2 get_marker_by_accession_and_source

	  Usage      : my $marker = $marker_db->get_marker_by_accession_and_source('HGVM510','dbSNP');
	  Purpose    : Get marker by database accession and source DB
	  Returns    : HGVbaseG2P::Schema::Marker object
	  Arguments  : An HGVbaseG2P Marker identifier;A datasource name
	  Throws     : 
	  Status     : Public
	  Comments   : 
=cut    
    
	sub get_marker_by_accession_and_source {
		my ($self, $accession, $datasource) = @_;
		return $self->dbh->resultset('Marker')->search('accession' => $accession)->single;
	}

=head2 get_limited_marker_by_accession_and_source

	  Usage      : my $marker = $marker_db->get_marker_by_accession_and_source('HGVM510','dbSNP');
	  Purpose    : Get marker by database accession and source DB
	  Returns    : HGVbaseG2P::Schema::Marker object
	  Arguments  : An HGVbaseG2P Marker identifier;A datasource name
	  Throws     : 
	  Status     : Public
	  Comments   : 
=cut    
    
	sub get_limited_marker_by_accession_and_source {
		my ($self, $accession, $datasource) = @_;
		return $self->dbh->resultset('Marker')->search('accession' => $accession, {columns=>['markerid','identifier','accession','status'],page=>1,rows=>1})->single;
	}

=head2 get_limited_marker_by_accession_and_source

	  Usage      : my $marker = $marker_db->get_marker_by_accession_and_source('HGVM510','dbSNP');
	  Purpose    : Get marker by database accession and source DB
	  Returns    : HGVbaseG2P::Schema::Marker object
	  Arguments  : An HGVbaseG2P Marker identifier;A datasource name
	  Throws     : 
	  Status     : Public
	  Comments   : 
=cut    
    
	sub get_limited_markercoord_by_markerid {
		my ($self, $mid) = @_;
		return $self->dbh->resultset('Markercoord')->search({'markerid' => $mid, 'assemblytype'=>"ref_assembly"}, {columns=>['chr','start','stop','strand'],page=>1,rows=>1})->single;
	}
	
	sub get_limited_alleles_by_markerid {
		my ($self, $mid) = @_;
		return $self->dbh->resultset('Allele')->search({'markerid' => $mid}, {columns=>['alleleseq','status']})->all;
	}
	
=head2 get_marker

	  Usage      : my $marker = $marker_db->get_marker(510);
	  Purpose    : Get marker by MarkerID
	  Returns    : HGVbaseG2P::Schema::Marker object
	  Arguments  : MarkerID
	  Throws     : 
	  Status     : Public
	  Comments   : 
=cut    

	sub get_marker {
		my ($self, $markerid) = @_;
		return $self->dbh->resultset('Marker')->search('markerid' => $markerid)->single;
	}

=head2 get_marker_by_identifier

	  Usage      : my $marker = $marker_db->get_marker_by_identifier('HGVM510');
	  Purpose    : Get marker by Marker Identifier
	  Returns    : HGVbaseG2P::Schema::Marker object
	  Arguments  : An HGVbaseG2P Marker identifier
	  Throws     : 
	  Status     : Public
	  Comments   : 
=cut    
	
	sub get_marker_by_identifier {
		my ($self, $identifier) = @_;
		return $self->dbh->resultset('Marker')->search('Identifier' => $identifier)->single;
	}

=head2 get_datasource

	  Usage      : my $ds = $marker_db->get_datasource('dbSNP');
	  Purpose    : Get datasource in MarkerDB using name field
	  Returns    : HGVbaseG2P::Schema::Datasource object 
	  Arguments  : Datasource name
	  Throws     : 
	  Status     : Public
	  Comments   : 

=cut
	sub get_datasource {
		my ($self, $name)=@_;
		my $ds = $self->dbh->resultset("Datasource")->search({ Name => $name })->first;
		return $ds;
	}

=head2 get_marker_by_accession_or_identifier

	  Usage      : my $marker = $marker_db->get_marker_by_accession_or_identifier('rs699','HGVM510');
	  Purpose    : Get marker by source database accession OR marker Identifier
	  Returns    : HGVbaseG2P::Schema::Marker object
	  Arguments  : SourceDB accession; HGVbaseG2P Marker identifier
	  Throws     : 
	  Status     : Public
	  Comments   : 
=cut    

	sub get_marker_by_accession_or_identifier {
		my ($self, $accession, $identifier) = @_;
		return $self->dbh->resultset('Marker')->search({ -or => [
							identifier => $identifier,
							accession => $accession]
						 })->first;
	}

=head2 get_marker_rs_by_accession_or_identifier

	  Usage      : my $marker = $marker_db->get_marker_rs_by_accession_or_identifier('rs699','HGVM510');
	  Purpose    : Get marker resultset by source database accession OR marker Identifier
	  Returns    : Class::DBIx Resultset containing HGVbaseG2P::Schema::Marker objects
	  Arguments  : SourceDB accession; HGVbaseG2P Marker identifier
	  Throws     : 
	  Status     : Public
	  Comments   : 
=cut    
	
	sub get_marker_rs_by_accession_or_identifier {
		my ($self, $accession, $identifier) = @_;
		return $self->dbh->resultset('Marker')->search({ -or => [
							identifier => $identifier,
							accession => $accession]
						 });
	}
	
=head2 get_marker_and_refcoords_by_identifier

	  Usage      : my $marker = $marker_db->get_marker_and_refcoords_by_identifier('HGVM510');
	  Purpose    : Get marker with ReqSeq markercoords by marker Identifier
	  Returns    : HGVbaseG2P::Schema::Marker object
	  Arguments  : HGVbaseG2P Marker identifier
	  Throws     : 
	  Status     : Public
	  Comments   : 
=cut    

	sub get_marker_and_refcoords_by_identifier {
		my ($self, $identifier) = @_;
		my $marker = $self->dbh->resultset('Marker')->search({ 'me.Identifier' => $identifier })->first;
		my $coord = $marker->markercoords->search({'assemblyname' => 'Reference'})->first;
		return ($marker,$coord);
	}
	
	#if method does not exist, attempt to call on the 'dbh'
	sub AUTOLOAD {
	    my ($self, @args) = @_;
	    my $method = $AUTOLOAD;
	    $method =~ s/\A.*:://xms;
	    if($self->dbh->can($method)) {
		return $self->dbh->$method(@args);	
	    }
	    else {
		die "Cannot call method $method on instance of ",ref($self->dbh);
	    }
	}
}
1;

=head1 SEE ALSO

L<HGVbaseG2P::FileParser>, L<HGVbaseG2P::Database> and related classes

=head1 BUGS AND LIMITATIONS

There are no known bugs in this module.
Please report problems to Rob Free <rcfree@gmail.com>

=head1 CONTACT

This module is part of the HGVbase-G2P project

=head1 AUTHOR

Rob Free <rcfree@gmail.com>

=head1 LICENCE AND COPYRIGHT

Copyright (c) <2009> University of Leicester, Leicester, UK. All rights reserved.

This module is free software; you can redistribute it and/or
modify it under the same terms as Perl itself.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 

=head1 CVSINFO

$Id: Marker.pm 1272 2009-08-21 15:08:35Z rcf8 $ 

=cut

