# $Id: Study.pm 1239 2009-07-08 16:21:56Z rcf8 $

=head1 NAME

HGVbaseG2P::Database::Study - Query Study HGVbaseG2P DB using Class::DBIx ORM


=head1 SYNOPSIS

use HGVbaseG2P::Database::Study;
my $study_db = HGVbaseG2P::Database::Study->new( { conf_file => '/conf/hgvbase.conf' } );
my $study = $study_db->get_study_by_identifier('HGVST1');


=head1 DESCRIPTION

Provides a wrapper for common Study database access methods. Returns single or arrays of HGVbaseG2P::Schema objects (derived from DBIx::Class) or
a DBIx::Class ResultSet object

=head1 SUBROUTINES/METHODS 

=cut

package HGVbaseG2P::Database::Study; {

	use strict;
	use warnings;

	# Use OIO inside-out object framework for our OO hierarachy
	use FindBin;

	use Object::InsideOut qw( HGVbaseG2P::Database );
	use Object::InsideOut::Metadata;

	use English qw( -no_match_vars );
	use Data::Dumper;
	
	use Carp qw(croak cluck);
	use HGVbaseG2P::Schema;
	
	#method common to all 'HGVbaseG2P::Database' modules which initializes the data source (see HGVbaseG2P::Database)
    sub init_data_source {
        my ($self, $args)=@_;
        my %config = %{$self->config};

        # Connect to database using either info from config OR parameters provided as
	    # arguments to the constructor-call. Arguments will override configfile
	    my $dsn = $args->{dsn} || $config{StudyDB}->{dsn} ||
		$self->log->logcroak("No database DSN provided with the dsn parameter or configfile");
	    my $dbuser = $args->{user} || $config{StudyDB}->{dbuser} || getlogin();
	    $self->log->info("Connecting to database $dsn as user $dbuser");

	    my $dbpassw = $args->{pass} || $config{StudyDB}->{dbpassw};
	    	my $dbh = HGVbaseG2P::Schema->connect($dsn,
                               $dbuser,
                               $dbpassw,
                               {
                                   PrintError => 1, # warn() on errors
                                   RaiseError => 1, # do die on error
                                   AutoCommit => 1, # does not commit immediately
                               }) or OIO->throw("Could not initialize db-connection. DBI::errstr=".$DBI::errstr);
    		$self->log->info("got database handle ".$dbh);


	    return $dbh;
    }
	
=head2 get_study_by_identifier

	  Usage      : my @studies = $study_db->get_study_by_identifier(['HGVST1','HGVST2']);
	  Purpose    : Get study or studies by identifier or arrayref of identifiers from Study DB
	  Returns    : HGVbaseG2P::Schema::Study object or an array of them depending on the query
	  Arguments  : An HGVbaseG2P Study identifier or arrayref containing multiple identifiers
	  Throws     : 
	  Status     : Public
	  Comments   : 
	  
=cut
	
    sub get_study_by_identifier {
    	my ($self,$identifier)=@_;
    	if (ref($identifier) eq 'ARRAY') {
    		return $self->dbh->resultset("Study")->search({ Identifier => $identifier })->all;
    	}
    	else {
    		return $self->dbh->resultset("Study")->search({ Identifier => $identifier })->single;
    	}
    }

=head2 get_all_studies

	  Usage      : my @studies = $study_db->get_all_studies;
	  Purpose    : Get all studies in the Study DB
	  Returns    : An array of HGVbaseG2P::Schema::Study objects
	  Arguments  : None
	  Throws     : 
	  Status     : Public
	  Comments   : 
	  
=cut
	
    sub get_all_studies {
    	my ($self)=@_;
    	return $self->dbh->resultset("Study")->all;
    }
    
=head2 get_samplepanel_by_identifier

	  Usage      : my $spanel = $study_db->get_samplepanel_by_identifier('HGVSP1');
	  Purpose    : Gets samplepanel by identifier from Study DB
	  Returns    : HGVbaseG2P::Schema::Samplepanel object
	  Arguments  : An HGVbaseG2P Samplepanel identifier
	  Throws     : 
	  Status     : Public
	  Comments   : 
=cut

    sub get_samplepanel_by_identifier {
    	my ($self,$identifier)=@_;
    	return $self->dbh->resultset("Samplepanel")->search({ Identifier => $identifier })->single;
    }

=head2 get_assayedpanel_by_identifier

	  Usage      : my $apanel = $study_db->get_assayedpanel_by_identifier('HGVAP1');
	  Purpose    : Gets assayedpanel by identifier from Study DB
	  Returns    : HGVbaseG2P::Schema::Assayedpanel object
	  Arguments  : An HGVbaseG2P Assayedpanel identifier
	  Throws     : 
	  Status     : Public
	  Comments   : 
=cut

    sub get_assayedpanel_by_identifier {
    	my ($self,$identifier)=@_;
    	return $self->dbh->resultset("Assayedpanel")->search({ Identifier => $identifier })->single;
    }

=head2 get_phenotypemethod_by_identifier

	  Usage      : my $pmethod = $study_db->get_phenotypemethod_by_identifier('HGVPM1');
	  Purpose    : Gets phenotypemethod by identifier from Study DB
	  Returns    : HGVbaseG2P::Schema::Phenotypemethod object
	  Arguments  : An HGVbaseG2P Phenotypemethod identifier
	  Throws     : 
	  Status     : Public
	  Comments   : 
=cut

    sub get_phenotypemethod_by_identifier {
    	my ($self,$identifier)=@_;
    	return $self->dbh->resultset("Phenotypemethod")->search({ Identifier => $identifier })->first;
    }

=head2 get_study

	  Usage      : my $study = $study_db->get_study(3);
	  Purpose    : Gets study by StudyID from Study DB
	  Returns    : HGVbaseG2P::Schema::Study object
	  Arguments  : An HGVbaseG2P StudyID
	  Throws     : 
	  Status     : Public
	  Comments   : 
=cut

    sub get_study {
    	my ($self,$studyid)=@_;
    	return $self->dbh->resultset("Study")->search({ StudyID => $studyid })->first;
    }

=head2 get_assoc_experiments_in_study

	  Usage      : my $study = $study_db->get_study_by_identifier('HGVST1');
	               my $apanel = $study_db->get_assoc_experiments_in_study($study);
	  Purpose    : Gets association experiments from a supplied Study
	  Returns    : Array of HGVbaseG2P::Schema::Experiment objects
	  Arguments  : An HGVbaseG2P::Schema::Study object
	  Throws     : 
	  Status     : Public
	  Comments   : 
=cut

    sub get_assoc_experiments_in_study {
    	my ($self,$study)=@_;
    	return $study->search_related('experiments',{type => {'!='=>'Frequency determination' }})->all;
    }

=head2 get_assoc_experiments_rs_in_study

	  Usage      : my $apanel = $study_db->get_assoc_experiments_rs_by_study_identifier('HGVST1');
	  Purpose    : Gets association experiments for a Study as a DBIx::Class::Resultset
	  Returns    : DBIx::Class::Resultset with HGVbaseG2P::Schema::Experiment objects
	  Arguments  : An HGVbaseG2P Study identifier
	  Throws     : 
	  Status     : Public
	  Comments   : 
=cut

	sub get_assoc_experiments_rs_by_study_identifier {
		my ($self, $identifier)=@_;
		return $self->dbh->resultset("Experiment")->search({'studyid.identifier'=>$identifier, 'type' => {'!=','Frequency determination'} }, {join => 'studyid'});
	}

=head2 get_all_unhidden_studies

	  Usage      : my @studies = $study_db->get_all_unhidden_studies;
	  Purpose    : Get all unhidden (ie. ishidden = null) studies in StudyDB
	  Returns    : Array of HGVbaseG2P::Schema::Study objects 
	  Arguments  : None
	  Throws     : 
	  Status     : Public
	  Comments   : 
=cut

	sub get_all_unhidden_studies {
		my ($self) = @_;
		return $self->dbh->resultset('Study')->search({ishidden => undef })->all;
	}
	
=head2 count_all_unhidden_studies

	  Usage      : my $studies = $study_db->count_all_unhidden_studies;
	  Purpose    : Count all unhidden (ie. ishidden = null) studies in StudyDB
	  Returns    : Scalar containing the number of studies
	  Arguments  : None
	  Throws     : 
	  Status     : Public
	  Comments   : 
=cut

	sub count_all_unhidden_studies {
		my ($self) = @_;
		return $self->dbh->resultset('Study')->search({ishidden => undef })->count;
	}
	
=head2 count_all_available_studies

	  Usage      : my $studies = $study_db->count_all_available_studies;
	  Purpose    : Count all available (ie. isdisplayed = show) studies in StudyDB
	  Returns    : Scalar containing the number of studies
	  Arguments  : None
	  Throws     : 
	  Status     : Public
	  Comments   : 
=cut

	sub count_all_available_studies {
		my ($self) = @_;
		return $self->dbh->resultset('Study')->search({isdisplayed => 'show', ishidden => undef })->count;
	}
	
=head2 count_all_phenotypemethods

	  Usage      : my $pmethods = $study_db->count_all_phenotypemethods;
	  Purpose    : Count all phenotypemethods in StudyDB
	  Returns    : Scalar containing the number of phenotypemethods
	  Arguments  : None
	  Throws     : 
	  Status     : Public
	  Comments   : 
=cut

	sub count_all_phenotypemethods {
		my ($self) = @_;
		return $self->dbh->resultset('Phenotypemethod')->count;
	}

=head2 get_paged_unhidden_studies_rs

	  Usage      : my @studies = $study_db->get_paged_unhidden_studies_rs;
	  Purpose    : Get page of unhidden (ie. ishidden = null) studies in StudyDB
	  Returns    : DBIx::Class::Resultset containing HGVbaseG2P::Schema::Study objects and a Data::Page object
	  Arguments  : None
	  Throws     : 
	  Status     : Public
	  Comments   : 
=cut

	sub get_paged_unhidden_studies_rs {
		my ($self,$page_no, $rows_per_page) = @_;
		return $self->dbh->resultset('Study')->search({ishidden => undef }, {
	      page => $page_no,  # page to return (defaults to 1)
	      rows => $rows_per_page, # number of results per page
	      'order_by'=>'studyid DESC', #order by most recent studies
	    });
	}
	
=head2 get_paged_phenotypemethods_rs

	  Usage      : my @studies = $study_db->get_paged_phenotypemethods_rs;
	  Purpose    : Get page of phenotypemethods in StudyDB
	  Returns    : DBIx::Class::Resultset containing HGVbaseG2P::Schema::Phenotypemethod objects and a Data::Page object
	  Arguments  : None
	  Throws     : 
	  Status     : Public
	  Comments   : 
=cut

	sub get_paged_phenotypemethods_rs {
		my ($self,$page_no, $rows_per_page) = @_;
		return $self->dbh->resultset('Phenotypemethod')->search(undef, {
	      page => $page_no,  # page to return (defaults to 1)
	      rows => $rows_per_page, # number of results per page
	    });
	}
	
=head2 get_paged_phenotypemethods_by_disease_rs

	  Usage      : my @studies = $study_db->get_paged_phenotypemethods_by_disease_rs;
	  Purpose    : Get page of phenotypemethods in StudyDB
	  Returns    : DBIx::Class::Resultset containing HGVbaseG2P::Schema::Phenotypemethod objects and a Data::Page object
	  Arguments  : None
	  Throws     : 
	  Status     : Public
	  Comments   : 
=cut

	sub get_paged_phenotypemethods_by_disease_rs {
		my ($self,$page_no, $rows_per_page, $disease) = @_;
		$self->log->info("disease:$disease");
		
		my @diseases;
		
		if (ref($disease) eq 'ARRAY') {
			$self->log->info("disease is array!");
			@diseases = map { '%'.$_.'%;' } @{$disease};
			$self->log->info("disease:".Dumper(\@diseases));
		}
		else {
			$diseases[0]="%$disease%;";
		}
		
		return $self->dbh->resultset('Phenotypemethod')->search({'diseasecategoryid.name'=>{ '-like' => \@diseases }} , {
	      page => $page_no,  # page to return (defaults to 1)
	      rows => $rows_per_page, # number of results per page,
	      join => { 'phenotypepropertyid'=>'diseasecategoryid' }
	    });
	}
	
=head2 get_experiment_by_identifier

	  Usage      : my $exp = $study_db->get_experiment_by_identifier('HGVE1');
	  Purpose    : Get experiment in StudyDB using identifier
	  Returns    : HGVbaseG2P::Schema::Experiment object 
	  Arguments  : Experiment Identifier
	  Throws     : 
	  Status     : Public
	  Comments   : 
=cut

	sub get_experiment_by_identifier {
		my ($self, $experimentid)=@_;
		my $exp = $self->dbh->resultset("Experiment")->search({ Identifier => $experimentid })->first;
		return $exp;
	}

=head2 get_experiments_with_phenotypemethod_by_identifier

	  Usage      : my @exps = $study_db->experiments_with_phenotypemethod_by_identifier('HGVPM1');
	  Purpose    : Get experiments in StudyDB which have same phenotypemethod identifier
	  Returns    : DBIx::Class Resultset containing HGVbaseG2P::Schema::Experiment objects 
	  Arguments  : Phenotypemethod Identifier
	  Throws     : 
	  Status     : Public
	  Comments   : 

=cut

	sub get_experiments_with_phenotypemethod_by_identifier {
		my ($self, $pmid)=@_;
		my $exp = $self->dbh->resultset("Experiment")->search({ 'phenotypemethodid.identifier' => $pmid }, { join => 'phenotypemethodid' } );
		return $exp;
	}

=head2 get_resultset

	  Usage      : my $rs = $study_db->get_resultset(62);
	  Purpose    : Get resultset in StudyDB using ResultsetID
	  Returns    : HGVbaseG2P::Schema::Resultset object 
	  Arguments  : ResultsetID
	  Throws     : 
	  Status     : Public
	  Comments   : 

=cut

	sub get_resultset {
		my ($self, $resultsetid)=@_;
		my $exp = $self->dbh->resultset("Resultset")->search({ ResultsetID => $resultsetid })->first;
		return $exp;
	}

=head2 get_resultsets

	  Usage      : my @rs = $study_db->get_resultsets([1,2,3,4]);
	  Purpose    : Get several resultsets based on ResultsetID
	  Returns    : Array of HGVbaseG2P::Schema::Study objects 
	  Arguments  : Arrayref of ResultsetIDs
	  Throws     : 
	  Status     : Public
	  Comments   : 
=cut

	sub get_resultsets {
		my ($self, $resultsetids)=@_;
		my @rs = $self->dbh->resultset("Resultset")->search({ ResultsetID => $resultsetids })->all;
		return @rs;
	}

=head2 get_resultset_by_identifier

	  Usage      : my $rs = $study_db->get_resultset(62);
	  Purpose    : Get resultset in StudyDB using ResultsetID
	  Returns    : HGVbaseG2P::Schema::Resultset object 
	  Arguments  : Resultset identifier
	  Throws     : 
	  Status     : Public
	  Comments   : 

=cut

	sub get_resultset_by_identifier {
		my ($self, $resultsetid)=@_;
		my $exp = $self->dbh->resultset("Resultset")->search({ Identifier => $resultsetid })->first;
		return $exp;
	}

=head2 get_resultset_by_name_and_experiment

	  Usage      : my $rs = $study_db->get_resultset_by_name_and_experiment('Unadjusted score',62);
	  Purpose    : Get resultset in StudyDB using resultset name and ExperimentID
	  Returns    : HGVbaseG2P::Schema::Resultset object 
	  Arguments  : Resultset name and ExperimentID
	  Throws     : 
	  Status     : Public
	  Comments   : 

=cut

	sub get_resultset_by_name_and_experiment {
		my ($self, $name, $experimentid)=@_;
		my $exp = $self->dbh->resultset("Resultset")->search({ Name => $name, 'Experimentid' => $experimentid })->first;
		return $exp;
	}

=head2 get_resultset_by_identifier_and_experiment

	  Usage      : my $rs = $study_db->get_resultset_by_name_and_experiment('HGVRS3',62);
	  Purpose    : Get resultset in StudyDB using resultset identifier and ExperimentID
	  Returns    : HGVbaseG2P::Schema::Resultset object 
	  Arguments  : Resultset identifier and ExperimentID
	  Throws     : 
	  Status     : Public
	  Comments   : 

=cut

	sub get_resultset_by_identifier_and_experiment {
		my ($self, $resultsetid, $experimentid)=@_;
		my $exp = $self->dbh->resultset("Resultset")->search({ Identifier => $resultsetid, 'Experimentid' => $experimentid })->first;
		return $exp;
	}

=head2 get_datasource

	  Usage      : my $ds = $study_db->get_datasource('dbSNP');
	  Purpose    : Get datasource in StudyDB using name field
	  Returns    : HGVbaseG2P::Schema::Datasource object 
	  Arguments  : Datasource name
	  Throws     : 
	  Status     : Public
	  Comments   : 

=cut

	sub get_datasource {
		my ($self, $name)=@_;
		my $ds = $self->dbh->resultset("Datasource")->search({ Name => $name })->first;
		return $ds;
	}

=head2 get_panel_by_name_and_study

	  Usage      : my $apanel = $study_db->get_panel_by_name_and_study('CONTROL','HGVST3');
	  Purpose    : Get assayedpanel in StudyDB using name and StudyID
	  Returns    : HGVbaseG2P::Schema::Assayedpanel object 
	  Arguments  : Assayedpanel name and StudyID
	  Throws     : 
	  Status     : Public
	  Comments   : 

=cut

	sub get_panel_by_name_and_study {
		my ($self, $name, $studyid)=@_;
		my $apanels = $self->dbh->resultset("Assayedpanel")->search({ 'Name' => $name, 'Studyid' => $studyid })->first;

		return $apanels;
	}

=head2 get_resultsets_in_study

	  Usage      : my @rs = $study_db->get_resultsets_in_study(5);
	  Purpose    : Get all resultsets 'wrapped' in a study
	  Returns    : Array of HGVbaseG2P::Schema::Resultset objects 
	  Arguments  : StudyID
	  Throws     : 
	  Status     : Public
	  Comments   : 
=cut

	sub get_resultsets_in_study {
		my ($self, $studyid)=@_;
		return $self->dbh->resultset("Resultset")->search({'experimentid.studyid'=>$studyid},{join=>'experimentid'})->all;
	}

=head2 get_significance

	  Usage      : my $sig = $study_db->get_significance(62);
	  Purpose    : Get significance in StudyDB using SignificanceID
	  Returns    : HGVbaseG2P::Schema::Significance object 
	  Arguments  : SignificanceID
	  Throws     : 
	  Status     : Public
	  Comments   : 

=cut

	sub get_significance {
		my ($self, $sigid)=@_;
		return $self->dbh->resultset("Significance")->search({'significanceid'=>$sigid})->single;
	}

=head2 get_frequencyclusters_by_markeridentifier

	  Usage      : my @fc = $study_db->get_frequencyclusters_by_markeridentifier($identifier);
	  Purpose    : Get all frequency clusters for a particular marker
	  Returns    : Array of HGVbaseG2P::Schema::Frequncycluster objects 
	  Arguments  : Marker identifier
	  Throws     : 
	  Status     : Public
	  Comments   : 
=cut

	sub get_frequencyclusters_by_markeridentifier {
		my ($self,$identifier)=@_;
		return $self->dbh->resultset('Frequencycluster')->search({'usedmarkersetid.markeridentifier'=>$identifier} , { join=>'usedmarkersetid' })->all;
	}

=head2 get_significances_above_threshold_for_markeridentifier

	  Usage      : my @sigs = $study_db->get_significances_above_threshold_for_markeridentifier('HGVM510','0.001');
	  Purpose    : Get all frequency clusters for a particular marker above a threshold using the Marker Identifier
	  Returns    : Array of HGVbaseG2P::Schema::Frequencycluster objects 
	  Arguments  : Marker Identifier, P-value threshold
	  Throws     : 
	  Status     : Public
	  Comments   : 
=cut

	sub get_significances_above_threshold_for_markeridentifier {
		my ($self, $identifier, $threshold)=@_;
		return $self->dbh->resultset("Significance")->search(
    	{'usedmarkersetid.markeridentifier' => $identifier, 'unadjustedpvalue' => {'<=',$threshold}},
						  {join => 'usedmarkersetid'} )->all;
	}


	sub get_usedmarkersets_for_markers_above_threshold {
		my ($self, $markers,$threshold)=@_;
		my @marker_identifiers = map { $_->identifier } $markers->all;
		return $self->dbh->resultset("Significance")->search(
    	{'usedmarkersetid.markeridentifier' => \@marker_identifiers, 'unadjustedpvalue' => {'<=',$threshold}},
						  {join => 'usedmarkersetid'} );
	}
	
=head2 get_all_phenotypemethods

	  Usage      : my @studies = $study_db->get_all_phenotypemethods;
	  Purpose    : Get all phenotype methods in StudyDB
	  Returns    : Array of HGVbaseG2P::Schema::Phenotypemethod objects 
	  Arguments  : None
	  Throws     : 
	  Status     : Public
	  Comments   : 
=cut

	sub get_all_phenotypemethods {
		my ($self)=@_;
		return $self->dbh->resultset("Phenotypemethod")->all();
	}
}
1;

=head1 SEE ALSO

L<HGVbaseG2P::FileParser>, L<HGVbaseG2P::Database> and related classes

=head1 BUGS AND LIMITATIONS

There are no known bugs in this module.
Please report problems to Rob Free <rcfree@gmail.com>

=head1 CONTACT

This module is part of the HGVbase-G2P project

=head1 AUTHOR

Rob Free <rcfree@gmail.com>

=head1 LICENCE AND COPYRIGHT

Copyright (c) <2009> University of Leicester, Leicester, UK. All rights reserved.

This module is free software; you can redistribute it and/or
modify it under the same terms as Perl itself.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.

=head1 CVSINFO

$Id: Study.pm 1239 2009-07-08 16:21:56Z rcf8 $

=cut
1;

=head1 SEE ALSO

L<HGVbaseG2P::FileParser>, L<HGVbaseG2P::Database> and related classes

=head1 BUGS AND LIMITATIONS

There are no known bugs in this module.
Please report problems to Rob Free <rcfree@gmail.com>

=head1 CONTACT

This module is part of the HGVbase-G2P project

=head1 AUTHOR

Rob Free <rcfree@gmail.com>

=head1 LICENCE AND COPYRIGHT

Copyright (c) <2009> University of Leicester, Leicester, UK. All rights reserved.

This module is free software; you can redistribute it and/or
modify it under the same terms as Perl itself.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.

=head1 CVSINFO

$Id: Study.pm 1239 2009-07-08 16:21:56Z rcf8 $

=cut

