# $Id: Xapian.pm 1234 2009-07-08 11:44:20Z rcf8 $

=head1 NAME

HGVbaseG2P::Database::Xapian - Query Xapian DB(s) for search criteria


=head1 SYNOPSIS

more to be written


=head1 DESCRIPTION

more to be written

=head1 SUBROUTINES/METHODS 

=cut

package HGVbaseG2P::Database::Xapian;
{

	use strict;
	use warnings;

	# Use OIO inside-out object framework for our OO hierarachy
	use Object::InsideOut qw( HGVbaseG2P::Database );
	use Object::InsideOut::Metadata;

	use English qw( -no_match_vars );
	use Data::Dumper qw(Dumper);
	use FindBin;
	use Carp qw(croak cluck);
	use Data::Pageset;
	use Encode qw/from_to/;

	my @db_of_dbdir : Field : All(db_of_dbdir);
	my @qp : Field : All(qp);
	use Search::Xapian qw/:all/;
	use Search::Xapian::MSet::Tied;

	sub init_data_source {
		my ( $self, $args ) = @_;
		my %config = %{ $self->config };
		$self->db_of_dbdir( {} );
	}

	sub db {
		my ( $self, $dbdir ) = @_;
		my %db_of_dbdir = %{ $self->db_of_dbdir };

		# Open db if we don't have it already opened
		if ( !exists $db_of_dbdir{$dbdir} ) {
			$self->log->debug("Opening database in $dbdir");
			eval {
				$db_of_dbdir{$dbdir} = Search::Xapian::Database->new($dbdir);
			};
			if ($@) {
				$self->log->logdie("Error opening database in $dbdir: $@");
			}
		}
		return $db_of_dbdir{$dbdir};
	}

	sub search {
		my ( $self, $q, $dbname, $page, $page_size ) = @_;

		my $base_config = my $config =
		     $self->config->{'Component'}->{'Controller::Search'}
		  || $self->config->{'Controller::Search'};
		$page      ||= 1;
		$page_size ||= $config->{page_size};

		my $dbdir =
		    $self->config->{'basedir'} . '/'
		  . $config->{index_basedir} . '/'
		  . $dbname;
		$self->log->info("Searching db in $dbdir with query '$q'");
		$self->log->info("page:$page and page_size:$page_size");
		my $db = $self->db($dbdir);
		$db->reopen();
		$self->qp( Search::Xapian::QueryParser->new($db) );
		my $stemmer = Search::Xapian::Stem->new( $config->{language} );
		$self->qp->set_stemmer($stemmer);
		$self->qp->set_default_op(OP_AND);

		$self->qp->add_boolean_prefix( "site",  "H" );
		$self->qp->add_boolean_prefix( "year",  "Y" );
		$self->qp->add_boolean_prefix( "month", "M" );
		$self->qp->add_boolean_prefix( "date",  "D" );
		$self->qp->add_boolean_prefix( "id",    "Q" );
		$self->qp->add_prefix( "title", "T" );

		from_to( $q, 'utf-8', 'iso-8859-1' ) if $config->{utf8_query};
		my $qparser = $self->qp();
		$qparser->set_stemming_strategy(STEM_ALL);
		my $query = $qparser->parse_query( $q,
			FLAG_WILDCARD | FLAG_BOOLEAN | FLAG_LOVEHATE |
			  FLAG_BOOLEAN_ANY_CASE );
		my $enq = $self->db($dbdir)->enquire($query);
		$self->log->info(
			sprintf "Parsed Xapian query is '%s'\n",
			$enq->get_query()->get_description()
		);

		if ( $config->{order_by_date} ) {
			$enq->set_docid_order(ENQ_DESCENDING);
			$enq->set_weighting_scheme( Search::Xapian::BoolWeight->new() );
		}
		my @matches_tied;
		
		my $mset = $enq->get_mset( ( $page - 1 ) * $page_size, $page_size );
		tie( @matches_tied, 'Search::Xapian::MSet::Tied', $mset );
		return undef if @matches_tied == 0;
		
		my $start = ( $page - 1 ) * $page_size;
		my $end = $start + $page_size;
		my $total = scalar(@matches_tied);
		$end > $total and $end = $total;
		
		$self->log->info("start:$start, end:$end, total:$total");
		
		my @matches=();
		foreach my $match (@matches_tied) {
			push @matches, $self->extract_data( $match->get_document );
		}
		
		$self->log->info("matches:".Dumper(\@matches));
		
		my $pager = Data::Page->new;
		$pager->total_entries($mset->get_matches_estimated);
		$pager->entries_per_page($page_size);
		$pager->current_page($page);
		return ( $pager, \@matches );
	}

=item extract_data <item> <query>

    Extract data from a L<Search::Xapian::Document>. Defaults to
    using Storable::thaw.

=cut

	sub extract_data {
		my ( $self, $item ) = @_;
		my $data = $item->get_data;
		return $data;
	}
}
1;

=head1 SEE ALSO

L<HGVbaseG2P::FileParser>, L<HGVbaseG2P::Database> and related classes

=head1 BUGS AND LIMITATIONS

There are no known bugs in this module.
Please report problems to Mummi <gthorisson@gmail.com>

=head1 CONTACT

This module is part of the HGVbase-G2P project

=head1 AUTHOR

Rob Free <rcfree@gmail.com>

=head1 LICENCE AND COPYRIGHT

Copyright (c) <2007> University of Leicester, Leicester, UK. All rights reserved.

This module is free software; you can redistribute it and/or
modify it under the same terms as Perl itself.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 

=head1 CVSINFO

$Id: Xapian.pm 1234 2009-07-08 11:44:20Z rcf8 $ 

=cut

