# $Id: GFF3Loader.pm 1499 2010-06-29 13:03:36Z gt50 $

=head1 NAME

  HGVbaseG2P::GFF3Loader - Customised loading of marker feature data

=head1 SYNOPSIS

    use HGVbaseG2P::GFF3Loader;
    my $loader = HGVbaseG2P::GFF3Loader->new(-store    => $store,
                                             -verbose  => $ARGV{verbose},
                                             -tmpdir   => $tmpdir,
                                             -fast     => 1,
                                            -markerdb => $markerdb,
                                           -index_subfeatures => 0,
      ) or die "Couldn't create GFF3 loader";
    $loader->load(@gff_files);


=head1 DESCRIPTION

    This class extends the BioPerl GFF3 loader with several features required for
    loading of marker feature data into the HGVbaseG2P Marker database.
    [..]

=head1 METHODS

=cut

package HGVbaseG2P::GFF3Loader;

use strict;
use Carp 'croak';
use Bio::DB::GFF::Util::Rearrange;
use Bio::DB::SeqFeature::Store::LoadHelper;
use Data::Dumper;
use CGI qw(unescape);

use base 'Bio::DB::SeqFeature::Store::GFF3Loader';

# Bring in a proper logger mechanism
use HGVbaseG2P::Util qw(init_logger);
use Log::Log4perl::Level;
my $logger = init_logger();
my %stats = ();

sub stats {return %stats;};

sub new {
    my $class = shift;
    my $self  = $class->SUPER::new(@_);

    # Bring in connection to the Marker database and other info passed in from
    # the loader script.
    my ($markerdb, $markertype) = rearrange(['MARKERDB','TYPE'],@_);    
    $self->{markerdb} = $markerdb;
    $self->{markertype} = $markertype;
    $self->verbose and $logger->level($DEBUG);

    # Pre-fetch the dbSNP Hotlink and DataSourice entries, to save time in lookups later.
    # ToDO: make this more generic!! handle non-dbSNP datasources
    $self->{marker_datasource} = $self->{markerdb}->dbh->resultset("Datasource")->search({name => 'dbSNP'})->single;
    $self->{marker_hotlink}    = $self->{markerdb}->dbh->resultset("Hotlink")->search({hotlinklabel => 'dbSNP refSNP'})->single;
    
    
    return $self;
}

sub store_current_feature {
    my $self = shift;

    # Grab the current feature to be stored
    my $ld   = $self->{load_data};
    defined $ld->{CurrentFeature} or return;
    my $feat    = $ld->{CurrentFeature};
    my $feat_label = "$feat | " . $feat->seq_id . ":" . $feat->start ."..".$feat->end ;


    # We also need a bit of metadata from the GFF header
    ## is this possible, BTW?

    #    may need a ::Store derived class and implement unhandled_metadata
      
    
    # Work out the right way to handle this marker, depending whether this is
    # new, changed or other category, and whether we've seen this before during
    # the same import batch job. Remember that the same marker may appear multiple
    # times in the GFFs if it's got multiple mappings to the assembly.

    # need new/unchanged/etc. flag from GFF
    my $markertype = $self->{markertype};
    $stats{Total}++;
    $logger->info("# $stats{Total}: Processing marker feature $feat_label");
    my $marker = $self->{markerdb}->get_marker_by_accession_and_source($feat->name(), $feat->source());
    ## ToDo? put this in method ??

    # If the marker is new, then we need to create a fresh entry in the Marker database.
    # But this might have been done already in this batch job, if the marker appeared 
    # in a GFF file  for a different assembly, or if it has multiple locations to this
    # assembly.
    if($markertype eq 'new') {

        #?? $self->handle_new_marker($marker, $feat);
        
        if($marker) {
            $logger->debug("Found entry in Marker database for 'new' marker feature: ".$marker->identifier."(".$marker->accession.")");
            
            # Abort the entire job if the marker wasn't created earlier in this job; this
            # indicates a problem with the upstream processing job (e.g. incorrect target db).
 
            # ToDo: ensure that date objects are used here
            if($marker->timelasttouched() != '2010-01-11 16:53:44') {
                $logger->logdie("scramble, scramble, we have a problem: marker ts ".$marker->timelasttouched()." > the one we're using");
                die();
            }
            else {
                $logger->debug("Timestamp for $feat is OK, so leaving as-is and adding feature data only");
            }
        } else {                        
            # Pull out the various bits of information from the feature and use this to
            # populate the Marker database entry. Info is held as attribute-value pairs
            # in the Notes field, and we use the standard feature API for this.
            $stats{New}++;
            my $featuretype = $feat->type();
            $featuretype =~ s/\:\w+$//; # clean up the type, dunno why the feature-db API does this

            # ToDo: factor this into subroutine, do the same elsewhere. feature2marker($feat)?
            my $minfo = {accession     => $feat->name(),
                         variationtype => $featuretype,
                         addedfromsourcebuild => 'b130', # ToDo: get from GFF header OR cmdline option
                         timelasttouched      => '2010-01-11 16:53:44'}; # ToDo: get from GFF header!
            ($minfo->{upstream30bp})   = $feat->get_tag_values('upstream30bp');
            ($minfo->{downstream30bp}) = $feat->get_tag_values('downstream30bp');
            ($minfo->{validationcode}) = unescape($feat->get_tag_values('validationcode'));
            ($minfo->{alleleseqs})     = unescape($feat->get_tag_values('alleles'));
            #??  EDA my $new_marker = $datasource->create_related('marker', {}) 
            # 
            $logger->info("Creating new marker for $feat - alleles = " . $minfo->{alleleseqs});
            $logger->is_debug and 
                $logger->debug("Marker info for new $feat: ". Dumper($minfo));

            # Set up FK relations to Hotlink and DataSource
            $minfo->{datasourceid} = $self->{marker_datasource};
            $minfo->{hotlinkid}   = $self->{marker_hotlink};

            my $new_marker = $self->{markerdb}->dbh->resultset("Marker")->create($minfo);
            ## ToDo: RobF's Marker db API needs a method to create a marker
                        
            $new_marker->update();
        }
    }
    # If the marker is updated, we need to update it with the info from the feature
    # file. But only if the marker was not already updated in this batch job.
    elsif($markertype eq 'changed') {

        if($marker) {

            # Check if the marker was already processed in this batch.
            if($marker->timelasttouched() == '2010-01-11 16:53:44') {
                $logger->debug("Timestamp for $feat is OK from earlier in batch, so leaving as-is and adding feature data only");
            } else {

            $stats{Changed}++;            

            ## update all fields as required
            my $featuretype = $feat->type(); 
            $featuretype =~ s/\:\w+$//; # clean up the type, dunno why the feature-db API does this
            my $minfo = {variationtype  => $featuretype,
                         timelasttouched => '2010-01-11 16:53:44'};
            ($minfo->{upstream30bp})   = $feat->get_tag_values('upstream30bp');
            ($minfo->{downstream30bp}) = $feat->get_tag_values('downstream30bp');
            # Handle URL-escaping for some of the fields, as per GFF3 spec.
            ($minfo->{validationcode}) = unescape($feat->get_tag_values('validationcode'));
            ($minfo->{alleleseqs})     = unescape($feat->get_tag_values('alleles'));          
            $logger->is_debug and 
                $logger->debug("Marker info for updated $feat: ". Dumper($minfo));
            $logger->info("Updating marker entry for $feat - alleles = ".$minfo->{alleleseqs});
            $marker->update($minfo);


        
            # Update the revision history for this marker
            foreach my $rev($feat->get_tag_values('revision')) {
                $logger->debug("     revinfo for $feat: $rev");
                my ($replaced_markeraccession, $strandflip, $changetype, $changetrigger, $comment) =
                    split(/\|/, unescape($rev));
                
                my $revinfo = { changetype    => $changetype, # get from revinfo
                                changetrigger => ($changetrigger || ''), # ToDo get from GFF header
                                comment       => ($comment || ''), # ToDo get from revinfo
                };

                $strandflip and $revinfo->{strandflipped} = 'yes';

                # add related tuples in MarkerRevision table
                $logger->is_debug and 
                    $logger->debug("revinfo for $feat: ". Dumper($revinfo));
                my $new_mrev = $marker->create_related('markerrevision_markerids', $revinfo);
                
                # If this is a merge, then there's extra work to do to look up the marker
                # which has replaced this one in the source database.
                if($replaced_markeraccession) {
                    $logger->debug("Got merger of $feat and and a replaced-by accession $replaced_markeraccession");
                    my $replacedbymarker = $self->{markerdb}->get_marker_by_accession_and_source($replaced_markeraccession, $feat->source());
                    $new_mrev->replacedbymarkerid($replacedbymarker);
                }
                $new_mrev->update();

            }
            
            ## what to do about mapping info for existing markers? need decision here.
            # ??? return;
            }
        } else {
            # We need to abort if the marker that's supposed to exist does not turn up in the database query,
            # since like above this indicates a problem upstream in the processing step.
            $logger->error("Marker $feat does NOT exist as it should, BAD!");
        }
        
    }
    # If the marker is unchanged, then just update the timestamp. But only if the 
    # marker was not already created in this batch job.
    elsif($markertype eq 'unchanged') {

        if($marker) {
            $logger->info("No data updates needed for marker $feat, updating timestamp only and new flanks");
            $stats{Unchanged}++;            
            $marker->update({timelasttouched    => '2010-01-11 16:53:44',
                             upstream30bp       => $feat->get_tag_values('upstream30bp'),
                             downstream30bp     => $feat->get_tag_values('downstream30bp')});
            
            # ToDo: still update/replace dbxrefs here?? 
            # ToDo: do this later, AFTER revamping Hotlink/Crossref system. this is too clunky!! 
            if(0){
            foreach my $xref ($feat->get_tag_values('Dbxref')) {
                my ($dbname, $dbid) = split(/:/, $xref);
                $logger->debug("handling Dbxref $xref - creating new MarkerCrossref/Crossref/Hotlink connection");

                # Create the Hotlink if it does not already exist, and create the many-to-many
                # connection via DBIx::Class built-in helper utility
                my $hotlink = $self->{markerdb}->dbh->resultset("Hotlink")->find_or_new({hotlinklabel => $dbname});
                unless ($hotlink->in_storage) {
                    $logger->info("Created new Hotlink for db '$dbname'");
                    $hotlink->insert;
                }
                my $crossref = $marker->add_to_crossrefs({hotlinkid => $hotlink,
                                                          urlid     => $dbid});
                
                # ToDo: link Hotlink to DataSource, so labels become unique within a given dsource? 
                # ToDo: need to handle existing Crossrefs here I think.
                # ooor do this later, once schema + conceptual model have been revamped
            }
            }

            
        } else {
            # Similar to the above, we need to abort if the marker that's supposed to exist 
            # does not turn up in the database query.
            $logger->error("Marker $feat does NOT exist in Marker database as it should, BAD!");
            
        }
    }

    # If the marker is deleted, then we need to set its status to 'dead' and update
    # the timestamp. But only if the marker was not already created in this batch job.
    ## ToDo, maybe in separate application

    
    # Remove the custom tags, to minimize the number of attributes to store in the 
    # feature database.
    $feat->remove_tag('downstream30bp');
    $feat->remove_tag('upstream30bp');
    $feat->remove_tag('revision');            
    
    # Once we're done with our bit, hand control back to the parent class which
    # will proceed to store the feature in the feature database
    $logger->debug("Storing feature $feat with these tags");
     for my $tag ($feat->get_all_tags) {
         for my $value ($feat->get_tag_values($tag)) {
             $logger->debug("   $tag => $value");
         }
     }

        $self->SUPER::store_current_feature();
     
    
}

1;


=head1 AUTHOR

  Gudmundur A. Thorisson <gthorisson@gmail.com>

=head1 BUGS AND LIMITATIONS

There are no known bugs in this module.
Please report problems to Rob Free <rcfree@gmail.com>

=head1 CONTACT

This module is part of the HGVbaseG2P project

=head1 LICENCE AND COPYRIGHT

Copyright (c) <2009> University of Leicester, Leicester, UK. All rights reserved.

This module is free software; you can redistribute it and/or
modify it under the same terms as Perl itself.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 

=head1 CVSINFO

$Id: GFF3Loader.pm 1499 2010-06-29 13:03:36Z gt50 $ 

=cut
