# $Id: Marker.pm 1498 2010-06-29 13:01:15Z gt50 $

=head1 NAME

  HGVbaseG2P::MarkerImport::Marker - Common functionality for marker import modules


=head1 SYNOPSIS

    use English qw( -no_match_vars );
    use HGVbaseG2P::MarkerImport::Marker;
    my $markerImport = HGVbaseG2P::MarkerImport::Marker->new(log4perl_conf => 'log4perl.conf');
    

=head1 DESCRIPTION

  This class implements common functionality required for importing data from various marker datasources,
  such as dbSNP. Subclasses handle datasource-specific parsing and data transformation tasks, while this 
  module handles general tasks such as aligning flanking sequences. 

=head1 METHODS 

=cut

package HGVbaseG2P::MarkerImport::Marker;
{

 use strict;
 use warnings;

 # Use OIO inside-out object framework for our OO hierarachy
 use Object::InsideOut qw( HGVbaseG2P::FileParser );
 use Object::InsideOut::Metadata;

 use List::Compare;
 use English qw( -no_match_vars );
 use Data::Dumper;
 use HGVbaseG2P::Util qw(load_config init_logger open_outfile);
 use POSIX qw(strftime);
 use File::Basename;
 use CGI qw(escape);
 use Data::Stag qw( :all );
 



 # Private variables & constants
my $logger;     # Reference to master logger
my %config;     # Master configuration settings from config-file
my $factory;    # dpAlign Object

 # Non-private fields with accessor methods
my @naive        : Field : Acc(Name => 'naive', Restricted => 1); # Naive mode on/off
my @marker_dbh   : Field : Acc(Name => 'marker_dbh'); # Connection to marker database, if used
my @gffwriters   : Field : Acc(Name => 'gffwriters', Restricted => 1); # Hashref for holding GFF-writers
my @filename     : Field : Acc(Name => 'filename', Restricted => 1); # Name of input marker datafile
my @timecreated  : Field : Acc(Name => 'timecreated'); # Timestamp for this batch job
my @label        : Field : Acc(label); # Label for this batch job
my @source       : Field : Acc(source):Arg('Name' => 'source'); # Name of the source database

my %init_args :InitArgs = ('conf_file'     => '',
			   'log4perl_conf' => '',
			   'naive' 		=> '',
			   'dsn' 		=> '',
			   'user' 		=> '',
			   'passw' 		=> '',
			   'source'     => {Mandatory => 1},			   
			   );

sub _init : Init {
    my ( $self, $args ) = @_;

    # Initialize GFF-writer holder
    $self->gffwriters({});

    # Load master config IF config file is provided
    $args->{conf_file} and %config = load_config($args->{conf_file});
    
    # Initialize logger
    my $log4perl_conf = $args->{log4perl_conf} || $config{log4perl_conf};
    $logger = init_logger( $log4perl_conf, __PACKAGE__ );

    # Check if we're supposed to run in naive mode or not 
    if($args->{naive}) {
        $logger->info("Running in naive mode, NOT connecting to any target database");
        $self->naive(1);
    }
    else {
        $logger->info("Running in normal mode, attempting to connect to target database");
        eval "require HGVbaseG2P::Database::Marker";
        if($@) {
            $logger->ERROR("Sorry, cannot run in full mode without the HGVbaseG2P::Database::Marker library. Please try running again with the -naive option");
            exit 10;
        }
        my $dbh = HGVbaseG2P::Database::Marker->new({ conf_file => $args->{conf_file}});
        $self->marker_dbh($dbh);
    }


    if(!$self->naive()) {
        # Bring in Bioperl libs used for aligning flanking sequences
        eval {
            use Bio::Tools::dpAlign;
            use Bio::Tools::pSW;
            use Bio::AlignIO;
        # Initialize sequence-alignment tool
        $factory = new Bio::Tools::dpAlign(
            -match    => 2,
            -mismatch => -2,
            -gap      => 3,
            -ext      => 2,
            -alg 	 => Bio::Tools::dpAlign::DPALIGN_ENDSFREE_MILLER_MYERS
            );
        };

        if($@) {
            $logger->logdie("ERROR: could not initialise Bio::Tools::dpAlign engine");
        }
    }
    }

=head2 process_file

  Usage      : $markerImport->process_file('/path/to/markerfile.xml','dbSNP b128')
  Purpose    : Main method for processing an input marker file from the source database
  Returns    : undef if non-fatal errors are encountered during run, dies on fatal errors
  Arguments  : Path to input file. Optional trigger label (for tagging marker-revisions).
  Throws     : 
  Status     : Public
  Comments   : 

=cut

sub process_file :Public{
    my ( $self, $filename, $label ) = @_;

    # Set up other bits&things needed during the run
    $label ||= 'none';
    $self->label($label);
    $self->timecreated( strftime "%Y-%m-%d %H:%M:%S", localtime );

  
    my @startTime = localtime(time);
    $logger->info( "Processing markerfile $filename, batch label is set to '$label'");
    $self->filename($filename);
    $self->parse_file( filename => $filename,
                       filetype => 'xml',       #$fileType[1],
                       handler  => $self->event_handler(),
        );
    my @endTime = localtime(time);

    # Clean up: close open output filehandles
    foreach my $assemblyname(keys %{$self->gffwriters()}) {
        foreach my $output(keys %{$self->gffwriters()->{$assemblyname}}) {
            $logger->info( "Closing output GFF-file $assemblyname / $output for input file".$self->filename );
            $self->gffwriters()->{$assemblyname}->{$output}->close();
        }
    }
}


=head2 process_file

  Usage      : my $fh = $self->get_gffwriter($assemblyname, $output)->print($gff3_string);
               $fh->print($gff3_string);
  Purpose    : Retrieve or initialize new GFF3 output file for a certain assembly and certain type
               of output.
  Returns    : An IO::File filehandle object 
  Arguments  : Label for the assembly (e.g. 'reference', 'Celera'). Type of output (e.g. 'changed')
  Throws     : 
  Status     : Public
  Comments   : 

=cut 

sub get_gffwriter :Restricted {
    my ($self, $assemblyname, $output) = @_;

    # Return the requested writer if it's already open, otherwise we need to create one
    my $gffwriter = $self->gffwriters()->{$assemblyname}->{$output};
    return $gffwriter if defined($gffwriter);
    
    # Initialize GFF file containing *all* marker as they are in the source
    my $file_basename = "markers_".$self->source()."_" . basename($self->filename);
    my $file_basename_old = $file_basename;
    $file_basename =~ s/\.(gz|zip)$//;
    $file_basename =~ s/\.\w+$//;
    my $gffwriter_filename = $file_basename.".".$assemblyname.".".$output.".gff";
    $logger->info( "Initializing new GFF-writer $gffwriter_filename");
    $gffwriter = open_outfile($gffwriter_filename);
    $gffwriter->print("##gff-version 3\n");
    $gffwriter->print("##genome-build NCBI B36\n");
    $gffwriter->print("##species http://www.ncbi.nlm.nih.gov/Taxonomy/Browser/wwwtax.cgi?name=Homo+sapiens\n");

    # Print out additional info to GFF header as comments which are later used
    # as directives by the custom GFF3-loader
    $gffwriter->print("# Timestamp=".$self->timecreated."\n");
    $gffwriter->print("# Label=".$self->label."\n");
    $gffwriter->print("# MarkerCategory=".$output."\n");
    $gffwriter->print("# AssemblyName=".$assemblyname."\n");
    
    $self->gffwriters()->{$assemblyname}->{$output} = $gffwriter;
    return $gffwriter;
}


=head2 align_sequences

  Usage      : my ( $alnobj, $aln_strand ) = $self->align_sequences($oldFlankSeq, $newFlankSeq);
               my $score = $alnobj->score();
               [do more stuff with alignment object]
  Purpose    : Align two short sequences
  Returns    : A Bio::SimpleAlign object
               A  1/-1 strand flag depending on whether the sequences align in forward or reverse direction.
  Arguments  : $seq1, $seq2
  Throws     : 
  Status     : Public
  Comments   : 

=cut

sub align_sequences :Public {
    my ( $self, $oldSeq, $newSeq ) = @_;
    chomp($oldSeq);
    chomp($newSeq);
    
    # Create a sequence objects to feed to the alignment-tool.
    $logger->debug("Comparing old seq vs new seq:\n  old: $oldSeq\n  new: $newSeq");
    $oldSeq = new Bio::Seq( -seq      => $oldSeq,
                            -id       => 'oldSeq',
                            -alphabet => 'dna',
                            -format   => 'fasta');
    $newSeq = new Bio::Seq( -seq      => $newSeq,
                            -alphabet => 'dna',
                            -id       => 'newSeq',
                            -format   => 'fasta');
    
    # Perform the alignment both straight-up and in reverse orientation, coz dpAlign
    # will not figure out the best alignment orientation by itself.
    my ($aln_fwd, $aln_rev);
    eval {
        $aln_fwd = $factory->pairwise_alignment( $oldSeq, $newSeq );
        $aln_rev = $factory->pairwise_alignment( $oldSeq, $newSeq->revcom() );
    };
    if($@ || !$aln_fwd || !$aln_rev) {
        $logger->warn("Error in alignment step: $aln_fwd=aln_fwd, aln_rev=$aln_rev, \$@=$@");
        OIO->throw("Could not align flanking sequences: old='".$oldSeq->seq."', new='".$newSeq->seq."'". $@);        
    }
    

    # Figure out which of the two alignments was the better one
    my $score_fwd = $aln_fwd->score();
    my $score_rev = $aln_rev->score();
    my ( $aln2use, $aln_strand );    
    if ( $score_fwd >= $score_rev ) {
        $aln2use    = $aln_fwd;
        $aln_strand = 1;
        $logger->debug("fwd score is higher than or equal to rev score: $score_fwd >= $score_rev");
    } else {
        $aln2use    = $aln_rev;
        $aln_strand = -1;
        $logger->debug("rev score is higher than fwd score:  $score_rev > $score_fwd");
    }

    # Clean up to prevent memory leaks
    undef $oldSeq;
    undef $newSeq;
    undef $aln_fwd;
    undef $aln_rev;

    # Return the alignment object corresponding to the better alignment, plus the strand flag.
    return ( $aln2use, $aln_strand );
}

=head2 pretty_alignment_string

  Usage      : print $self->pretty_alignment_string($alnobj);
  Purpose    : Convert an alignment object created by dpAlign to a human-readable string
  Returns    : Stringified alignment
  Arguments  : Bio::Align object
  Throws     : 
  Status     : Restricted
  Comments   : 

=cut

 sub pretty_alignment_string : Public {
     my ( $self, $aln ) = @_;

     # Create a string filehandle in memory, since we certainly do not want to print this to disk.
     my $alnstring_io = new IO::String;
     print $alnstring_io "Score: " . $aln->score . "\n";    # coz the aln string is not given in clustalw format

     # Set up the formatter (NB we can change to any of the other alignment formats if we want).
     my $alignio = new Bio::AlignIO( -format      => 'clustalw',
                                     -percentages => 1,
                                     -linelength  => 150,
                                     -fh          => $alnstring_io
         );

     # Get a string representation of the alignment object  and return it.
     $alignio->write_aln($aln);
     my $alnstring = ${ $alnstring_io->string_ref };
     $alnstring =~ s/CLUSTA.+\n\n\n//i;
     undef $alignio;
     return $alnstring;
}


=head2 create_markerRevision_node

  Usage      : my $rev = $self->create_markerRevision_node( "AllelesAdded", $self->label );
  Purpose    : Create a MarkerRevision Stag node
  Returns    : MarkerRevision Stag structure
  Arguments  : Category of marker change. Label for this batch job.
  Throws     :
  Status     : public
  Comments   :

=cut

 sub create_markerrevision_node {
     my ( $self, $changeType ) = @_;
     my $markerRevision = stag_nodify(
       [
        MarkerRevision => [
         [ '@' => [ [ op => 'insert' ] ] ],
         [ ChangeType    => $changeType ],
         [ ChangeTrigger => $self->label ],
         [ TimeCreated => $self->timecreated ]
       ]]);

     return $markerRevision;
}


=head2 create_allele_node

  Usage      : my $allele =$self->create_allele_node($alleleseq);
  Purpose    : Create a new Allele Stag node
  Returns    : AlleleNode Stag node
  Arguments  : Allele sequence. Status.
  Throws     :
  Status     : public
  Comments   :

=cut

 sub create_allele_node {
     my ( $self, $alleleSeq) = @_;
     
     my $allele = stag_nodify(
      [Allele => [
         [ AlleleSeq 	  => $alleleSeq ], 
       ]]);        
     return $allele;
}

=head2 create_marker_node

  Usage      : my $marker = $self->create_marker_node($varType,$accession, $validation, $upstream30bp, $downstream30bp);
  Purpose    : Create a new Marker Stag node
  Returns    : MarkerNode
  Arguments  : Variation type. Validation flag. Upstream flanking sequence. Downstream flanking sequence.
  Throws     :
  Status     : public
  Comments   :

=cut

 sub create_marker_node {
     my ( $self, $varType, $localId, $validation, $upstream30bp, $downstream30bp) = @_;
     my $Marker = stag_nodify(
     [
      Marker => [
       [ VariationType   => $varType ],
       [ Status          => "active"],
       [ Source          => $self->source ],
       [ LocalID         => $localId ],
       [ HotlinkID	 => "dbsnp_hotlink"],
       [ Upstream30bp    => $upstream30bp ],
       [ Downstream30bp  => $downstream30bp ],
       [ ValidationCode  => $validation ],
       [ AddedFromSourceBuild => $self->label ],
       [ TimeLastTouched => $self->timecreated ]
      ]]);
  
     $Marker->unset('ValidationCode') if (!defined $validation || $validation eq '');
     $Marker->unset('HotlinkID') if ($self->source ne 'dbSNP');
     $Marker->unset('VariationType') if ( !defined $varType );
     $Marker->unset('Upstream30bp') if ( !defined $upstream30bp);
     $Marker->unset('Downstream30bp') if ( !defined $downstream30bp );
     return $Marker;
}

=head2 compare_variationtype

  Usage      : my $type_is_changed  = $self->compare_variationtype( $marker_DB, $marker_XML);
  Purpose    : To compare the variation type attribute of the markers and see if they are the same.
               Updates the second Marker node with revision information if types are different.
  Returns    : True if the type is NOT the same.
  Arguments  : Two Marker stag node objects
  Throws     : 
  Status     : Restricted
  Comments   : 

=cut

sub compare_variationtype : Restricted {
    my ( $self, $marker_DB, $marker_XML) = @_;
    
    my $oldMarkerType = $marker_DB->variationtype;
    my $newMarkerType = $marker_XML->get("VariationType");
    
    # NB this currently merely checks if the strings are the same, but will possibly
    # require a more intelligent comparison, e.g. based on ontology relationships
    if ( $newMarkerType ne $newMarkerType ) {
        $logger->debug("MarkerType has changed from '$oldMarkerType' to '$newMarkerType'");
        my $markerRevision = $self->create_markerRevision_node( "VariantTypeChange", $self->label );
        $markerRevision->set('Comment', " Old '$oldMarkerType' to new '$newMarkerType'");
        $marker_XML->addkid($markerRevision);
        return 1;
    }
    return 0;
}

=head2 print_marker_gff3

  Usage      : $self->print_marker_gff3($marker_XML, 'unchanged');

  Purpose    : Print marker info as GFF3 line to the specified output file type, for all of
               the assemblies where mapping info is available
  Returns    : 
  Arguments  : Marker node. Type of file ('unchanged', 'changed', 'new' etc.)
  Throws     : 
  Status     : 
  Comments   : 

=cut

sub print_marker_gff3 {
    my ($self, $marker, $output) = @_;

    # ToDo: need a tag/value adding utility method
    ## $tags->{revisioncomment} = ["did this update","did other update"];
    my $marker_accession = $marker->get("LocalID");
    my $marker_attributes = "ID=$marker_accession;Name=$marker_accession";
    $marker_attributes .= ";upstream30bp=" . $marker->get("Upstream30bp");
    $marker_attributes .= ";downstream30bp=" . $marker->get("Downstream30bp");
    if(my $valcode = $marker->get("ValidationCode")) {
        $marker_attributes .= ";validationcode=".escape($valcode);
    }
    if(my @dbxrefs = $marker->get("DbXref")) {
        map {$_ = $_->get("DbAbbr").':'.$_->get("DbID") } @dbxrefs;
        $marker_attributes .= ";Dbxref=" . join(',', @dbxrefs);
    }

    # Encode alleles according to our nomenclature rules and add this to the GFF
    # Reformat each allele-string, including handling longer sequences
    my @alleles = $marker->get("Allele");
    map { $_ = $_->get("AlleleSeq") } @alleles; 
    my @alleles_org;
    my @alleles_shortened;
    foreach my $alleleseq(sort(@alleles)) {
        
        # ToDo: original, large alleleseqs; handling. short vs full? 
        
        # Do some reformatting for very large reported allele sequences
		my $alleleseq_length = length($alleleseq);       
        if($alleleseq =~ /^[ATCGN]+$/ixms and $alleleseq_length > 20) {
		    push(@alleles_shortened, '('. q{"}.unit_label($alleleseq_length).q{ seq"}.')');
        }
        else {
            push(@alleles_shortened, '('.$alleleseq.')');
        }
        push(@alleles_org, '('.$alleleseq.')'); # The full alleleseq encoded
    }   
    
    
    $marker_attributes .= ";alleles="
        #.substr($marker->get("Upstream30bp"), -10, 10)
        #.'('
        .join(':',@alleles_org)
        #. ')'         
        #.substr($marker->get("Downstream30bp"), 0, 10)
        ;
#    $marker_attributes .= ";alleles_shortened=".substr($marker->get("Upstream30bp"), -10, 10)
#        .'('.join(':',@alleles_shortened). ')'
#        .substr($marker->get("Downstream30bp"), 0, 10);

        

    # Add revision information, if any changes need to be catalogued
    if(my @revs = $marker->get("MarkerRevision")) {
        $marker_attributes .= ";revision=";
        # convert Stag structure to string and do URL-escape
        map {$_ = ($_->get("ReplacedByMarker/Marker/LocalID")  || '' )
                 .'|'.($_->get("StrandFlipped") || '' )
                 .'|'. $_->get("ChangeType")
                 .'|'. $_->get("ChangeTrigger")
                 .'|'. ($_->get("Comment")  || '' ) }  @revs;
        map {$_ = escape($_)} @revs;
        $marker_attributes .= join(",", @revs);
    }


    my @mcoords = $marker->get("MarkerCoord");

    # Special case: print markers with no coordinates to special output file
    my $gff3_string_unmapped =  join("\t", 'ChrUn',
                   'dbSNP', 
                   $marker->get("VariationType"),
                   0,
                   0,
                   '.',
                   '.',
                   '.',
                   $marker_attributes)  ."\n";

    $self->get_gffwriter("unmapped", $output)->print($gff3_string_unmapped);
    

    # Otherwise print all genome coordinates to the proper assembly output file
    foreach my $mcoord(@mcoords) {
        
        my $assemblyname = $mcoord->get("AssemblyName"); 
        my $feat_attributes .= ";mapweight=" . $mcoord->get("MapWeight");
        my $gff3_string = join("\t", 'Chr'.$mcoord->get("Chr"),
                   'dbSNP', 
                   $marker->get("VariationType"),
                   $mcoord->get("Start"),
                   $mcoord->get("Stop"),
                   '.',
                   $mcoord->get("Strand"),
                   '.',
                   $marker_attributes . $feat_attributes )  ."\n";

        $logger->debug("Printing GFF to $assemblyname\.$output file: $gff3_string");
        $self->get_gffwriter($assemblyname, $output)->print($gff3_string);


        # Encode alleles according to our nomenclature rules
        #foreach my $allele($marker->get("Allele")) {
        foreach my $allele(()) {
            my $gff3_string = join("\t", 'Chr'.$mcoord->get("Chr"),
                   'dbSNP', 
                   'allele',
                   $mcoord->get("Start"),
                   $mcoord->get("Stop"),
                   '.',
                   $mcoord->get("Strand"),
                   '.',
                   "Parent=". $marker->get("LocalID")) . ";alleleseq=".escape($allele->get("AlleleSeq"))  ."\n";
            #$self->get_gffwriter($assemblyname, $output)->print($gff3_string);
        }
    }    
}

# convert bp into nice Mb/Kb units (borrowed from GBrowse CGI-script!!)
sub unit_label {
    my $value = shift;
    my $unit     =  'bp';
    my $divider  =  1;
    $value /= $divider;
    my $abs = abs($value);
    
    my $label;
    $label = $abs >= 1e9  ? sprintf("%.4g G%s",$value/1e9,$unit)
	: $abs >= 1e6  ? sprintf("%.4g M%s",$value/1e6,$unit)
		: $abs >= 1e3  ? sprintf("%.4g k%s",$value/1e3,$unit)
		: $abs >= 1    ? sprintf("%.4g %s", $value,    $unit)
		: $abs >= 1e-2 ? sprintf("%.4g c%s",$value*100,$unit)
		: $abs >= 1e-3 ? sprintf("%.4g m%s",$value*1e3,$unit)
		: $abs >= 1e-6 ? sprintf("%.4g u%s",$value*1e6,$unit)
		: $abs >= 1e-9 ? sprintf("%.4g n%s",$value*1e9,$unit)
		: sprintf("%.4g p%s",$value*1e12,$unit);
    if (wantarray) {
	return split ' ',$label;
    } else {
	return $label;
    }
}    


=head2 update_legacy_markers

  Usage      : $self->update_legacy_markers($timestamp, $label);
  Purpose    : Update legacy markers. Retrieves all markers from the database which were NOT
               seen in the main marker processing job. These markers are presumed to be deleted
               from the source database, and therefore should be flagged as 'dead' in our database.
  Returns    : 
  Arguments  : Timesetamp. Label for this batch job (should be same as used for the main processing job).
  Throws     : 
  Status     : 
  Comments   : 

=cut

sub update_legacy_markers {
    my ($self, $timestamp, $label) = @_;
    
    $self->label($label);
    my $source = $self->source();
    $self->timecreated( strftime "%Y-%m-%d %H:%M:%S", localtime );
    

    
    # Variables to keep a count of the no of Markers processed
    my $cnt_newLegacyMarker       = 0;
    my $cnt_totalMarkersprocessed = 0;
    my $cnt_deletedMarker         = 0;
    my $cnt_mergedMarker          = 0;

    # Set up and execute the query which finds the legacy markers for this datasource.    
    my $rs = $self->marker_dbh->resultset('Marker')->search({'timelasttouched' => {'<', $timestamp},
                                                             'datasourceid.name' => $source,
                                                            },
                                                            {join => 'datasourceid', #rows => 20000
                                                            });
    
    #$logger->info("Found total ".$rs->count." markers to check for legacy-status");
    
    # Iterate through the result set row by row
    while ( my $marker = $rs->next() ) {
        $cnt_totalMarkersprocessed++;  	
        my $markerlabel = ($marker->identifier || '[no ID]')."($source:".($marker->accession || '[no accession]').")";
        $logger->info("#$cnt_totalMarkersprocessed: legacy marker check for  $markerlabel, timestamp='".$marker->timelasttouched."'");
   
        # 
        if ($marker->status eq 'active') {
            $cnt_newLegacyMarker++;
            $logger->info( "Marker $markerlabel is alive in master db, so we need to flag it as 'dead' now AND also figure out whether it's deleted or merged in source-db");


            # Check revision history for this marker, we need a look at the latest entry
            my @revs_sorted = sort {$b->timecreated->hires_epoch, $a->timecreated->hires_epoch} $marker->revisions();
            if(@revs_sorted == 0) {
                $logger->info("$markerlabel has no revision history, so this is a straight-up deletion and no merger");
                my $revinfo = { changetype    => 'Deletion',
                                changetrigger => $label,
                                comment       => "Marker deleted in $label",
                };
                $logger->is_debug and 
                    $logger->debug("revinfo: ". Dumper($revinfo));
                my $new_mrev = $marker->create_related('markerrevision_markerids', $revinfo);
                $new_mrev->update();
                $marker->update({'timelasttouched' => $timestamp,
                                 'status'          => 'dead'});
                $cnt_deletedMarker++;
            } else {
                $logger->info("$markerlabel has revision history, so must double-check whether this is a merge-event");
                foreach my $rev(@revs_sorted) {
                    my $t = $rev->timecreated;
                    $logger->debug( "$markerlabel epoch_hires='".$t->hires_epoch."', rev=".$rev->changetype."|".$rev->changetrigger ."|". $rev->timecreated."|".$rev->comment);
                }
                my $last_rev = pop(@revs_sorted);
                $logger->debug("$markerlabel type of last marker revision is ".$last_rev->changetype());

            }            
            
            # Get the latest revision entry for this marker. If this is a merged merger, then the
            # last entry will point to a marker that replaced this one.
            #my $marker_revision_set = $self->retrieve_markerRevision_from_DB($r->{AutoMarkerID});
            #$logger->is_debug() and
            #    $logger->debug("MarkerRevision history: ".$marker_revision_set->itext);
            #my $marker_revision     = $marker_revision_set->sget('Marker/MarkerRevision');
            #if(defined($marker_revision) && (my $replacement_marker = $marker_revision->get('ReplacedByMarker/Marker'))) {
            #    $logger->info("$r->{LocalID} is already flagged as dead after merger, replaced by marker ".$replacement_marker->get('LocalID').
            #                  " according to merge-info from source database. No further action needed");
            #    $cnt_mergedMarker++;
            #}
            #else {
            #    $logger->info("Marker $markerlabel merge info found, so this is a new 'deleted' legacy Marker. Adding revision entry to reflect this.");
            #    $cnt_deletedMarker++;
                #my $markerRevision = $self->create_markerRevision_node("Deletion");
                #$marker_updated->addkid($markerRevision);
            #}
        }
        else {
            $logger->info( "Marker $markerlabel already has legacy status, so no need to do anything");
        }
    }

    # Produce a stats summary at the end
    $logger->info("No of markers processed\t $cnt_totalMarkersprocessed ");
    $logger->info("No of new Legacy Markers \t $cnt_newLegacyMarker"); 
    $logger->info("No of new Deleted Legacy Markers \t $cnt_deletedMarker");
    $logger->info("No of new Merged Legacy Markers \t $cnt_mergedMarker");
}

}
1;


=head1 AUTHOR

  Gudmundur A. Thorisson <gthorisson@gmail.com>, Pallavi Sarmah <pallabi2k@gmail.com>

=head1 BUGS AND LIMITATIONS

There are no known bugs in this module.
Please report problems to Rob Free <rcfree@gmail.com>

=head1 CONTACT

This module is part of the HGVbaseG2P project

=head1 LICENCE AND COPYRIGHT

Copyright (c) <2009> University of Leicester, Leicester, UK. All rights reserved.

This module is free software; you can redistribute it and/or
modify it under the same terms as Perl itself.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 

=head1 CVSINFO

$Id: Marker.pm 1498 2010-06-29 13:01:15Z gt50 $ 

=cut
