# $Id: dbSNP.pm 1498 2010-06-29 13:01:15Z gt50 $

=head1 NAME

  HGVbaseG2P::MarkerImport::dbSNP - Marker import functionality specific to the dbSNP datasource

=head1 SYNOPSIS

  [..shouldn't have to use this directly, use via parent class]
    
=head1 DESCRIPTION

  This class implement marker import functionality specific to the dbSNP sequence variation database,
  notably XML-parsing and extraction of the required variant information from the XML nodes.

  Common functionality is inherited from the parent class. 

=head1 METHODS 

=cut

package HGVbaseG2P::MarkerImport::dbSNP; 
{

use strict;
use warnings;

# Use OIO inside-out object framework for our OO hierarachy
use Object::InsideOut qw( HGVbaseG2P::MarkerImport::Marker );
use Object::InsideOut::Metadata;


use English qw( -no_match_vars );
use Carp qw(cluck croak confess);
use Data::Dumper;

#use Devel::Leak::Object;
#use Devel::Leak::Object qw{ GLOBAL_bless };
#$Devel::Leak::Object::TRACKSOURCELINES  = 1;

use Data::Stag qw( :all );
use POSIX qw(strftime);


use HGVbaseG2P::Util qw(load_config init_logger open_outfile);

# Private variables & constants
my $logger;    # Reference to master logger
my %config;    # Master configuration settings from config-file
# Hardcoded (for now) list of linkout-URLs, needed for lookups during XML-parsing.
my %dbxreflabel_of = ('1' => 'RefSNP',
                      '2' => 'SubSNP',
                      '3' => 'LocusLink',
                      '4' => 'dbSTS',
                      '5' => 'PubMed');
# [From dbSNP b130]
#     <BaseURL urlId="1" resourceName="RefSNP" resourceId="snp_id"
#         >http://www.ncbi.nlm.nih.gov/SNP/snp_ref.cgi?rs=</BaseURL>
#     <BaseURL urlId="2" resourceName="SubSNP" resourceId="subsnp_id"
#         >http://www.ncbi.nlm.nih.gov/SNP/snp_retrieve.cgi?subsnp_id=</BaseURL>
#     <BaseURL urlId="3" resourceName="LocusLink" resourceId="locus_id"
#         >http://www.ncbi.nlm.nih.gov/LocusLink/LocRpt.cgi?l=</BaseURL>
#     <BaseURL urlId="4" resourceName="dbSTS" resourceId="uid"
#         >http://www.ncbi.nlm.nih.gov/genome/sts/sts.cgi?uid=</BaseURL>
#     <BaseURL urlId="5" resourceName="PubMed" resourceId="pmid"
#     >http://www.ncbi.nlm.nih.gov/entrez/query.fcgi?cmd=Search&amp;db=PubMed&amp;term=</BaseURL>


 # Non-private fields with accessor methods
my @event_handler  : Field : Acc(Name => 'event_handler', Restricted => 1); # The XML parsing callback

# Variable to track the Markers statistics
my %marker_stats;
my $flag = 0;
my $flag_sf = 0;

# Mapping for dbSNP variation classes to Sequence Ontology terms
my %variationType = ( "snp"    => "SNP",
                      "in-del"         => "indel",
                      "heterozygous"   => "complex_substitution",
                      "microsatellite" => "tandem_repeat",
                      "named-locus"    => "complex_substitution",
                      "no-variation"   => "novar",
                      "mixed"          => "complex_substitution",
                      "multinucleotide-polymorphism" => "MNP",
                      "CopyNumber"                   => "CNV",
                      "Inversion"           => "chromosomal_inversion",
                      "InversionBreakpoint" => "inversion_breakpoint"
    );

sub _init : Init {
    my ( $self, $args ) = @_;

    # Load master config IF config file is provided
    $args->{conf_file} and %config = load_config($args->{conf_file});
    
    # Initialize logger
    my $log4perl_conf = $args->{log4perl_conf} || $config{log4perl_conf};
    $logger = init_logger( $log4perl_conf, __PACKAGE__ );
    
    # Set up Data::Stag inheritance, in OIO-fashion
    # ToDo: change this into a plugin architecture, similar to RobF's if not make this
    #       part of the same data import system. Plugin would then provide A) code
    #       to handle the XML and B) which parsing event(s) to intercept
    my $handler = Data::Stag::BaseHandler->new();
    $handler->trap_h( { Rs => sub { $self->handleRs(@_) }, } );
    $self->event_handler($handler);
}

=head2 parse_genomic_data

  Usage      : $self->parse_genomic_data($node);
  Purpose    : Extract the genomic information of each Marker from an <Rs> node from dbSNP XML
  Returns    : A Data::Stag Marker node 
  Arguments  : A Data::Stag node representing the <Rs> node from dbSNP XML, as constructed
               by the XML-parser
  Throws     : 
  Status     : Restricted
  Comments   : 

=cut

sub parse_genomic_data : Public {
    my ( $self, $r ) = @_;
    my $atts     = $r->get('@'); # '@' holds the attributes of the XML-element
    my $rsid     = "rs" . $atts->get("rsId");
    my $seqType1 = $atts->get("molType");
    my $seqType = $seqType1 eq 'genomic' ? 'G' : $seqType1 eq 'cDNA' ? 'C' : '';
    $logger->warn("SeqType: $rsid\t SeqType\t $seqType1\n") if ( $seqType eq '' );
 
    # Extract validation status flag
    my $validation = $r->get("Validation");
    if (defined($validation) && $validation ne "" ) {
        my $attr       = $validation->get('@');
        if(defined($attr)) {
            my @valKids    = $attr->kids;
            my $valMethods = "";
            my $cnt        = 0;
            $validation = "";
            foreach my $valKid (@valKids) {
                if ( $cnt == 0 ) {
                    $validation .= $valKid->element;
                } else {
                    $validation .= "," . $valKid->element;
                }
                $cnt++;
            }
        }
    }

    # Extract the variation class for this marker and map to a SO term.
    my $varType = $atts->get("snpClass");
    if ( !$variationType{$varType} ) {
        $logger->warn("Unknown variation class: $rsid VariationType='$varType'");
        $marker_stats{UnknownVariationClass}->{$varType}++;
        $varType = '[UNKNOWN]';

    } else {
        $varType = $variationType{$varType};
    }
	
    # Create 30bp upstream and downstream flanking sequence strings by chomping if original
    # flanks are more than 30bp, and padding with N's if less than 30bp.
    my $sequence     = $r->get("Sequence");
    my $upstream30bp = $sequence->get("Seq5") || '';
    my $len          = length($upstream30bp);
    if ( $len > 30 ) {
        $upstream30bp = substr( $upstream30bp, ( $len - 30 ), $len );
    } elsif ( $len < 30 ) {
        $upstream30bp = "N" x ( 30 - $len ) . $upstream30bp;
    }
    my $downstream30bp = $sequence->get("Seq3") || '';    
    $len = length($downstream30bp||'');
    if ( $len > 30 ) {
        $downstream30bp = substr( $downstream30bp, 0, 30 );
    } elsif ( $len < 30 ) {
        $downstream30bp = $downstream30bp . "N" x ( 30 - $len );
    }
    
    # Now create the marker node that will be be populated with the extracted information
    my $markerNode = $self->create_marker_node( $varType, $rsid, $validation, $upstream30bp, $downstream30bp);
    
    # Format the known allele sequences for this marker as we want it in HGVbaseG2P
    my $allele = $sequence->get("Observed");
    my @alleles = split("/", $allele);
    my $first_allele = $alleles[0];

    # Handle short tandem repeat markers
    ## Todo: factor this out into a class method, clunky block of code this.
    if ($first_allele=~/^(\(\w+\)\d+)$/){		# STR ... (CA)17/18/19...
        $first_allele = shift @alleles;
        $logger->debug( "1.$rsid $first_allele");
        my $alleleNode =$self->create_allele_node($first_allele);
        $alleleNode->set( "SeqType", $seqType );
        $markerNode->addkid($alleleNode);
        foreach (@alleles){
            my ($first_allele_seq,$number) = split /\d+/, $first_allele;
            if ($_=~/(\d+)/){
                $logger->debug("1.$rsid $first_allele_seq"."$1");
                my $alleleNode = $self->create_allele_node($first_allele_seq.$1);
                $alleleNode->set( "SeqType", $seqType );
                $markerNode->addkid($alleleNode);
            } else{
                $_ =~s/-/_/;
                $logger->debug("1.$rsid $_");
                my $alleleNode = $self->create_allele_node($_);
                $alleleNode->set( "SeqType", $seqType );
                $markerNode->addkid($alleleNode);
            }
        }
    } else {
        foreach my $com_Allele (@alleles) {
            if ( $com_Allele =~/STR/ ) {
                $com_Allele =~s/\(STR/STR\(/;
                $logger->debug("3.$rsid =>$com_Allele");
                my $alleleNode = $self->create_allele_node( $com_Allele);
                $alleleNode->set( "SeqType", $seqType );
                $markerNode->addkid($alleleNode);	
            } elsif ($com_Allele =~/-/) {
                $com_Allele ='_';
                $logger->debug( "4. $rsid  =>$com_Allele");
                my $alleleNode = $self->create_allele_node($com_Allele);
                $alleleNode->set( "SeqType", $seqType );
                $markerNode->addkid($alleleNode);
            } elsif ( $com_Allele =~ m/(LARGE|NOVARIATION|ALU|INDEL|STR|HETEROZYGOUS|FRAGA|HGF|BID)/i ) {
                if ( $com_Allele =~ m/\(/ ){
                    $com_Allele =~ s/\(/\"/;
                    $com_Allele =~ s/\)/\"/;	
                    $logger->debug ("5. $rsid =>$com_Allele");
                    my $alleleNode = $self->create_allele_node($com_Allele);
                    $alleleNode->set( "SeqType", $seqType );
                    $markerNode->addkid($alleleNode);
                } else {
                    $com_Allele = "\"$com_Allele\"";
                    $logger->debug("6. $rsid $com_Allele");
                    my $alleleNode = $self->create_allele_node($com_Allele);
                    $alleleNode->set( "SeqType", $seqType );
                    $markerNode->addkid($alleleNode);
                }
            } elsif ( $com_Allele =~/\d/ ) {
                $logger->debug( "2.$rsid $com_Allele=>$com_Allele");
                my $alleleNode = $self->create_allele_node($com_Allele);
                $alleleNode->set( "SeqType", $seqType );
                $markerNode->addkid($alleleNode);
            } else {
                $logger->debug("7. $rsid =>$com_Allele ");
                my $alleleNode = $self->create_allele_node($com_Allele);
                $alleleNode->set( "SeqType", $seqType );
                $markerNode->addkid($alleleNode);
            }
        }
    }

    # Extract sequence assembly mapping information
    my @assembly = $r->get("Assembly");
    foreach my $i ( 0 .. $#assembly ) {
        my $atts1 = $assembly[$i]->get("@");
        next unless $atts1->get("current");
        #next unless $atts1->get("reference");        
        my $assemblyname = $atts1->get("groupLabel");

        ## keep this for now; let's take *all* assembly mapping info
        #next if ($assemblyname eq "Celera" || $assemblyname eq "HuRef"); # ToDo: put this in configfile?
                
        #print $assembly[$i]->itext;

        $marker_stats{GenomeAssemblies}->{$assemblyname}++;

        
        my $genomeBuild = $atts1->get("genomeBuild");
        my @genomeBuild = split( "_", $genomeBuild );
        my $component   = $assembly[$i]->get("Component");
        next unless defined($component); # Some of the map-info nodes in the XML don't have actual data
        
        if ($component ne ''){
            my $atts2       = $component->get("@");
            my $maploc      = $component->get("MapLoc");
            my $atts3       = $maploc->get("@");
            my $physMapInt  = $atts3->get("physMapInt");
            my ( $stop, $start, $length );
            
            if ($physMapInt) {
                $start  = $physMapInt + 1;
                $length = abs( $atts3->get('asnTo') - $atts3->get('asnFrom') );
                $stop   = $start + $length;
            } else {
                $start = 0;
                $stop  = 0;
            }
            
            my $loctype1 = $atts3->get("locType");
            #$logger->debug("$rsid coord physMapInt- $physMapInt Len: $length Start:$start Stop:$stop LocType $loctype1");
            my $loctype =
                $loctype1 eq 'exact'       ? '@'
                : $loctype1 eq 'deletion'    ? '^'
                : $loctype1 eq 'insertion'   ? '..'
                : $loctype1 eq 'range-ins'   ? '..'
                : $loctype1 eq 'range-del'   ? '^'
                : $loctype1 eq 'range-exact' ? '@'
                : $loctype1;
            $logger->warn("Span:$rsid\t $loctype1\n") if ( $loctype eq '' );

            #to track any other type of Loctype other than ones mentioned above
            my $strand = $atts3->get("orient") eq 'reverse'? '+' : '-';
            
            my $mapWeight = $assembly[$i]->get("SnpStat")->get("@")->get("mapWeight");
            my $markerCoordNode = stag_nodify(
          [ 
		   MarkerCoord => [ 
		    [ '@' => [ [ op => 'insert' ] ] ],
		    [ Chr        => $atts2->get("chromosome") ],
		    [ Start      => $start ],
		    [ Stop       => $stop ],
		    [ Strand     => $strand ],
		    [ Span       => $loctype ],
		    [ MapWeight  => $mapWeight ],
		    [GenomeBuild => $genomeBuild[0] . "." . $genomeBuild[1]],
		    [ AssemblyType => $atts2->get("groupTerm") ],
		    [ AssemblyName => $assemblyname ]
		   ]]);
            $markerNode->addkid($markerCoordNode);
        } else {
            my $markerCoordNode = stag_nodify(
                [
                   MarkerCoord => [
                    [ '@' => [ [ op => 'insert' ] ] ],
                    [GenomeBuild => $genomeBuild[0] . "." . $genomeBuild[1]],
			    ]]);
            $markerNode->addkid($markerCoordNode);
            last;
        }
        undef $atts1;
    }

    # Extract linkout URLs to various other NCBI resources
    #   sample: <RsLinkout linkValue="3964686" resourceId="1"/>
    my @linkout = $r->get("RsLinkout");
    foreach my $i ( 0 .. $#linkout ) {
        my $linkout_atts = $linkout[$i]->get("@");
        $logger->is_debug and
            $logger->debug("linkout atts=".$linkout_atts->itext);

        # Look up the dbxref label, given the ID provided here.
        my $dbxref_label = $dbxreflabel_of{$linkout_atts->get("resourceId")};
        next if $dbxref_label eq 'RefSNP'; # we know about the primary dbSNP link
        my $dbxref_id    = $linkout_atts->get("linkValue");
        $logger->debug("$rsid DbXref = $dbxref_label:$dbxref_id");
        $markerNode->addkid(stag_nodify([ DbXref => [
                                          ['DbAbbr' => $dbxref_label],
                                          ['DbID' => $dbxref_id],
                                       ]]));
        undef $linkout_atts;
    }

    # Clean up.
    undef $atts;
    undef $r;

    # Return the fully populated marker node.
    return ($markerNode);
}

=head2 compare_alleles

  Usage      : my $alleles_are_changed  = $self->compare_alleles( $marker_DB, $marker_XML );
  Purpose    : Compare the set of allele sequences for the old and new markers
  Returns    : 
  Arguments  : Marker object retrieved from database. New Marker node.
  Throws     : 
  Status     : Restricted
  Comments   : 

=cut

sub compare_alleles : Restricted {
    my ( $self, $marker_DB, $marker_XML) = @_;
    
    # Extract old and New AlleleSeq strings from XML and DB objects. 
    my @oldAlleleSeqs_DB = split(':', $marker_DB->alleleseqs());
    map {s/^\(//; s/\)$//;} @oldAlleleSeqs_DB;
    @oldAlleleSeqs_DB = sort(@oldAlleleSeqs_DB);

#    foreach my $allele($marker_DB->alleles()) {
#        push(@oldAlleleSeqs_DB, $allele->alleleseq);
#    }
    my @newAlleleSeqs 	 = sort($marker_XML->get('Allele/AlleleSeq'));
    my $rsid          	 = $marker_XML->get('LocalID');
    my $returnValue      = 0;
	
    my @oldAlleleSeqs = @oldAlleleSeqs_DB;
    $logger->debug("$rsid Compare Alleles OLD:".join (',', @oldAlleleSeqs_DB)." New:".join (',',@newAlleleSeqs));
	
    # Need to reverse-complement old alleles first if the previous step indicates 
    # a strand-flip in the flanking sequences.
    my $strandFlipped;
    if( defined($marker_XML->get('MarkerRevision/StrandFlipped')) ) {
        map {$_ = reverse($_); $_ =~ tr/ATGCUMKRYWSVBHDXN/TACGUKMYRWSBVDHXN/ } @oldAlleleSeqs;
        @oldAlleleSeqs = sort(@oldAlleleSeqs);
        $logger->debug( "$rsid: comparing old alleles @oldAlleleSeqs (reverse-complemented) with new alleles @newAlleleSeqs" );
        $strandFlipped = 1;
    }
    else {
        $logger->debug( "$rsid: comparing old alleles @oldAlleleSeqs (normal) with new alleles @newAlleleSeqs" );
    }

    # Compare the list of allele sequences from each marker: check if all the alleles are
    # present and whether any have been added or deleted. List::Compare is handy for this.
    my $lc      = List::Compare->new( \@oldAlleleSeqs, \@newAlleleSeqs );
    my @oldonly = $lc->get_unique;
    my @newonly = $lc->get_complement;
    
	# Handling cases where alleles have been flipped
    if ($strandFlipped){
        map{$_ = reverse($_); $_ =~ tr/ATGCUMKRYWSVBHDXN/TACGUKMYRWSBVDHXN/ } @oldonly;
        @oldonly = sort(@oldonly);
        
        # If theres a stand flip and the no. alleles are the same, BUT the allele seqs
        # are not same, then alleles are not correctly reverse-complemented = problem.  		
        if (@oldAlleleSeqs == @newAlleleSeqs) {

            # check for alleles which are same eg A,T allele Reverse+complemented is also A,T
            my $compare = List::Compare->new( \@oldAlleleSeqs, \@oldAlleleSeqs_DB );
            if ($compare->get_unique == 0 && $compare->get_complement == 0){
                $logger->debug("$rsid Alleles are same ");
            }
            if (@oldonly == 0 && @newonly == 0) {
                $logger->debug("$rsid Strand Flip and Allele Seq are also correctly reversed+complemented,Delete the Old Alleles from DB and insert the correct New Ones" );
                $marker_XML->set('MarkerRevision/Comment', "Old alleles ". join('/',@oldAlleleSeqs_DB).' rev-comp to new alleles '.join('/',@newAlleleSeqs));
               	$returnValue = 1;
            } else {
                $marker_stats{manual_curation}->{'IncorrectAlleles'}++;
                $logger->warn( "$rsid - Theres a Strand Flip but Allele Seq are NOT reverse+complement - needs manual curation" );
                my $markerRevision = $self->create_markerrevision_node( "IncorrectAlleles", $self->label );
                $markerRevision->set( 'Comment',' Old alleles '. join( '/', @oldAlleleSeqs_DB )	. 
                                      ' to new alleles ' . join( '/', @newAlleleSeqs ));
                $marker_XML->addkid($markerRevision);
                $returnValue = -1;
            }
        } else {
            if (@oldonly != 0) {
                $marker_stats{Changed}->{'SF+AD'}++;
                $flag = 2;
                $logger->is_info and $logger->info( "$rsid SF + Alleles Deleted: " . join( '/', @oldonly ) );	
                my $markerRevision = $self->create_markerrevision_node( "AllelesDeleted", $self->label );
                $markerRevision->set( 'Comment', "Alleles deleted: ".join('/',@oldonly));
                $marker_XML->addkid($markerRevision);
            }
            if (@newonly != 0) {
                $marker_stats{Changed}->{'SF+AD+AA'}++ if $flag == 2;
                $marker_stats{Changed}->{'SF+AD'}-- if $flag == 2;
                $marker_stats{Changed}->{'SF+AA'}++ if $flag == 0;
                $logger->is_info and $logger->info( "$rsid SF + Alleles Added: " . join( ',', @oldonly ) );	
                my $markerRevision = $self->create_markerrevision_node( "AllelesAdded", $self->label );
                $markerRevision->set( 'Comment', "Alleles added: ".join('/', @newonly) );
                $marker_XML->addkid($markerRevision);
            }
            $logger->debug("$rsid SF--");
            $marker_stats{Changed}->{'StrandFlipped'}--;
            $flag_sf = 0;
            $returnValue = 0;
        }
    } else {
        if ( scalar @oldonly == 0 && scalar @newonly == 0 ) {
            $logger->debug("$rsid No Change in Alleles");
            $returnValue = 1;
        }
        if (@oldonly != 0) {
            $marker_stats{Changed}->{'AllelesDeleted'}++;
            $flag = 1;
			$logger->is_info and $logger->info( "$rsid Alleles Deleted: " . join( ',', @oldonly ) );	
			my $markerRevision = $self->create_markerrevision_node( "AllelesDeleted", $self->label );
            $markerRevision->set( 'Comment', "Alleles deleted: ". join('/',@oldonly));
            $marker_XML->addkid($markerRevision);
			$returnValue = 0;
		}
		if (@newonly != 0) {
			$marker_stats{Changed}->{'AllelesAdded'}++ if $flag == 0;
			$marker_stats{Changed}->{'AD+AA'}++ if $flag == 1;
			$marker_stats{Changed}->{'AllelesDeleted'}-- if $flag == 1;	
			$logger->is_info and $logger->info( "$rsid Alleles Added: " . join( ',', @newonly ) );	
			my $markerRevision = $self->create_markerrevision_node( "AllelesAdded", $self->label );
			$markerRevision->set( 'Comment', "Alleles added: ".join('/',@newonly));
			$marker_XML->addkid($markerRevision);
			$returnValue = 0;
		}		
	}   
    
    # ToDO: remove this crud: we don't need to flag deleted alleles as 'dead' anymore; we just want to 
    # replace old list of alleles w/ new list.
    foreach my $alleleSeq (@oldAlleleSeqs_DB) {
        if (grep {$_ eq $alleleSeq} @newAlleleSeqs) {
            $logger->debug("$rsid $alleleSeq is also present in the new set, not to be deleted");
        } else {
            $logger->debug("$rsid delete $alleleSeq");
            #my $allele = $self->create_allele_node( undef, $alleleSeq, undef, "dead", "update" );
            #$marker_XML->addkid($allele);
        }
    }    
    foreach my $alleleSeq (@newAlleleSeqs) {
        if (grep {$_ eq $alleleSeq} @oldAlleleSeqs_DB) {
            $logger->debug("$rsid $alleleSeq is present in the DB, not to be inserted");
        } else {
            $logger->debug("$rsid insert $alleleSeq");
            #my $allele = $self->create_allele_node($upstream30bp, uc($alleleSeq),$downstream30bp, "active", "insert" );
            #$marker_XML->addkid($allele);
        }
    }
    undef @oldAlleleSeqs_DB;
    undef @newAlleleSeqs;
    undef @oldAlleleSeqs_DB;
    return $returnValue;	
}

=head2 compare_flanks

  Usage      : my $flanksNochange   = $self->compare_flanks( $marker_DB, $marker_XML);
  Purpose    : Compare flanking sequences between old and new marker
  Returns    : 
  Arguments  : Marker object from database. New Marker node from XML.
  Throws     : 
  Status     : Restricted
  Comments   : 

=cut

sub compare_flanks : Restricted {
    my ( $self, $marker_DB, $marker_XML) = @_;

    # Extract upstream & downstream sequences from marker nodes and create a pair of
    # concatenated sequences spanning the marker site.
    my $rsid = $marker_XML->get('LocalID');
    my $oldFlankSeq_DB_u  = $marker_DB->upstream30bp   || q{};
    my $oldFlankSeq_DB_d  = $marker_DB->downstream30bp || q{};
    my $newFlankSeq_XML_u = $marker_XML->get("Upstream30bp");
    my $newFlankSeq_XML_d = $marker_XML->get("Downstream30bp");

    # Check straight away whether we even have flanks to compare in the database. A handful markers
    # do not have flanks, so we  can't do the check for those. Let's assume they are OK.
    if($oldFlankSeq_DB_u eq q{} || $oldFlankSeq_DB_d eq q{}) {
        $logger->warn("$rsid: Upstream and/or downstream flanking seq missing, skipping seq-alignment step.U=$oldFlankSeq_DB_u D=$oldFlankSeq_DB_d");
        $marker_stats{MissingFlankseqInDB}++;
	    return 1;
    }

    
    my $oldFlankSeq_DB  = $oldFlankSeq_DB_u  . $oldFlankSeq_DB_d;
    my $newFlankSeq_XML = $newFlankSeq_XML_u . $newFlankSeq_XML_d;
    
    $logger->debug("$rsid CompareFlanks\nOLD: $oldFlankSeq_DB\nNEW: $newFlankSeq_XML");
    my $newFlankSeq = uc($newFlankSeq_XML);
    my $oldFlankSeq = uc($oldFlankSeq_DB);

    # Remove leading and trailing N's, to simplify alignment-step below
    $newFlankSeq =~ s/\A[N]+//xms;
    $oldFlankSeq =~ s/\A[N]+//xms;
    $newFlankSeq =~ s/[N]+\Z//xms;
    $oldFlankSeq =~ s/[N]+\Z//xms;

    
    # Replacing the IUPAC nucleotides with 'N', as dpAlign cannot handle IUPAC
    $newFlankSeq =~ s/[UMKRYWSVBHDXN-]/N/g;
    $oldFlankSeq =~ s/[UMKRYWSVBHDXN-]/N/g;

    # Check flanking sequence for presence of invalid characters
    if ( ( $newFlankSeq =~ m/[^ATGCN]/i ) || ( $oldFlankSeq =~ m/[^ATGCN]/i ) ){
        $logger->warn("$rsid Flank Seq\tnew:\t$newFlankSeq\tOld:\t$oldFlankSeq");
        $logger->warn("$rsid Flanks have characters other than A T G C N");
        $marker_stats{manual_curation}->{'IncorrectFlanks'}++;
        return -1;
    }
       
              
    # Aligning old and the new sequence
    my ($alnobj, $aln_strand) ;
    eval {
        ($alnobj, $aln_strand) = $self->align_sequences( $oldFlankSeq, $newFlankSeq );
    };
    # Ensure that we catch exceptions  from the seq-alignment step, since dpAlign might
    # have crapped out due to <2 length seq input or some other reason.
    if (my $e = OIO->caught()) {
        $logger->warn("Caught exception from seq-alignment step, so skipping this step:", $e->error); #, "\n", $e->trace->as_string);
        $marker_stats{FailedFlankAlignment}++;
        return 1;
    }
    
    $logger->is_debug and
        $logger->debug("$rsid Alignment string:\n" . $self->pretty_alignment_string($alnobj));
             
    # Check if the score meets the required cutoff. Since all the IUPAC codes were replaced by Ns,
    # we may need to adjust the score to compensate. dpAlign does not know about N's and so will
    # penalize for any nucleotide-to-N mismatch (rather stupid, innit).
    my $score           = $alnobj->score();
    my $perc_identity   = $alnobj->percentage_identity() ;
    my $score_preadjust = $score;
    $marker_stats{'AlnScoreValues'}->{$score}++;
    ## ToDo: move the score threshold parameter to config file, along with the alignment parameters in Marker.pm
    my $score_threshold = 50; 
    if( $score < $score_threshold ) {
        $logger->warn("$rsid Got low score $score < treshold $score_threshold, so adjusting for possible Ns in alignment");
        $marker_stats{'LowAlnScores'}++;
        $logger->warn("$rsid Alignment string:\n" . $self->pretty_alignment_string($alnobj));
                
        # Adjust score upwards by certain amount for each N-vs-nt mismatch we find
        my $alnobj_mismatches = $alnobj->remove_columns(['match','gaps']);
        if($alnobj_mismatches->length > 0) {
            $logger->is_debug and 
                $logger->debug("$rsid sub-aln with mismatches only:\n".$self->pretty_alignment_string($alnobj_mismatches));
            for (my $i = 1; $i <= $alnobj_mismatches->length; $i++) {
                my $alncol = $alnobj_mismatches->slice($i,$i);
                if($alncol->get_seq_by_pos(1) && $alncol->get_seq_by_pos(1) eq 'N' || 
                   $alncol->get_seq_by_pos(2) && $alncol->get_seq_by_pos(2) eq 'N') {
                    $logger->info("$rsid Got N-vs-nt mismatch, will adjust score $score upwards by the 'match' parameter value");
                    $score = $score + 4; # again, this should probably be in a config file - it's the 'match' parameter
                }
            }
            undef $alnobj_mismatches;
        }
        if($score >= $score_threshold) {
            $logger->info("$rsid adjusted score $score now reaches threshold $score_threshold");
            $marker_stats{'LowAlnScoreAdjustedToReachThreshold'}++;	    
        }
    }
    
    # Check if the score (possibly adjusted) meets the required threshold.
    $logger->debug("$rsid Checking score $score against threshold $score_threshold");
    if ( $score < $score_threshold ) {
        $marker_stats{manual_curation}->{'BadFlanks'}++;
        $logger->error("$rsid Full upstream+downstream flank alignment score $score does NOT reach threshold $score_threshold.");
        $logger->error("$rsid Bad alignment string:\n" . $self->pretty_alignment_string($alnobj));
        
        
        # Not good: flank aligment is awful, despite strand flip. Needs curation.
        my $markerRevision = $self->create_markerrevision_node( "BadFlanks", $self->label );
        my $prettyAlnString = $self->pretty_alignment_string($alnobj);
        $markerRevision->set( 'Comment', "Strand $aln_strand \nOverall Alignment $prettyAlnString");
        $marker_XML->addkid($markerRevision);
        $logger->error( "Marker $rsid has severe flank changes:\n$prettyAlnString" );
        undef $alnobj;
        return -1;
    }
    undef $alnobj;    

    if ( $aln_strand == -1 ) {
        $marker_stats{Changed}->{'StrandFlipped'}++;
        $flag_sf = 1;
        # Flanks are different, but a strand flip resolved it so no curation needed
        my $markerRevision = $self->create_markerrevision_node( "StrandFlipped", $self->label );
        $marker_XML->set( 'StrandFlipped', 'Yes' );
        $marker_XML->addkid($markerRevision);
        $logger->info("Marker $rsid has strand-flipped flanks" );
        return 0;
    }
    
    # Finally, if there's no changes found according to the criteria above, BUT the new flanking sequences
    # are a bit different (as in better sequence usually, e.g. replace leading/trailing Ns), then we
    # want to update the flanks to the latest sequences.
    if ( $perc_identity != 100 ) {
        $logger->info("$rsid Flanks for otherwise unchanged marker not 100% identical, so adding newest flanks to the markernode");
        $marker_XML->set('Upstream30bp', $newFlankSeq_XML_u);
        $marker_XML->set('Downstream30bp', $newFlankSeq_XML_d);
    }
    
    return 1; # all is well
}

=head2 handleRs

  Usage      : $self->handleRs($handle,$node); # NB not used directly
  Purpose    : This method is the handler for <Rs> node events emitted by the XML-parser
               It encapsulates the following functionalities:
                 -To parse the genomic data from XML.
                 -To retrieve Marker information from the database.
                 -To compare the old and the new marker.
                 -To write marker data to seperate output files depending on whether
                  differences are found or not.
  Returns    : 
  Arguments  : $handle, $node
  Throws     : 
  Status     : Restricted
  Comments   : 

=cut

# Step D(i) Core Processing of new dbSNP markers in the XML-file

sub handleRs : Restricted {
    my ( $self, $handler, $node ) = @_;
    $marker_stats{Total_Markers}++;


	
    # Grab information from node in XML
    my $marker_XML = $self->parse_genomic_data($node);
    my $rsid       = $marker_XML->get("LocalID");
    $logger->info("#$marker_stats{Total_Markers} dbSNP entry $rsid");
    $logger->is_debug and $logger->debug($marker_XML->itext );


    # First off, we always print every single marker 'fresh' into an 'all' target output file
    $self->print_marker_gff3($marker_XML, 'all');

    # If we are in naive mode, then there is nothing more to do here
    $self->naive() and return;
    
    # If we are NOT in naive mode, we proceed look up this marker by accession in the target database
    my $marker_DB = $self->marker_dbh->get_marker_by_accession_and_source($rsid, $self->source);
    
    # If the Marker is not present in the master Database, then just
    # print out the minimal marker info to file
    if ( !defined $marker_DB ) {
        $logger->info("No marker $rsid found in database, so this is a new marker");
        $self->print_marker_gff3($marker_XML, 'new');
        $marker_stats{New_Markers}++;
        return;    # Don't want to do anything more in this method
    }
    
    # Otherwise we proceed to the old vs new comparison procedure
    $logger->debug("$rsid exists in database, launching old vs new comparison");

    # Look for dummy Markers inserted from RsMerged file
    my $markerID      = $marker_DB->identifier;
    if (!$markerID){
        $logger->info("Marker $rsid found in master database, but its a dummy Marker w/o proper ID, introduced from RsMerged so we print out to new-XML");
        $marker_stats{New_DummyMarkers}++;
        $marker_XML->unset('@');
        $self->print_marker_gff3($marker_XML, 'new');
        return;	
    }

    # If this on the other hand an actual existing marker, we'll need to compare against the new info
    $logger->is_debug() and 
        $logger->debug( "Existing record found in Marker database for dbSNP:$rsid = ". $marker_DB->identifier );

    my $newMarkerType = $marker_XML->get("VariationType");
    my $newValidation = $marker_XML->get("ValidationCode");
    my $upstream30bp  = $marker_XML->sget("Allele/Upstream30bp");
    my $downstream30bp= $marker_XML->sget("Allele/Downstream30bp");
    
    
    # MarkerNode we build during processing. This will become marker-entry ultimately loaded into  the 
    # master database, to update the old marker to match the new marker.
    my $markerIsChanged	    = 0;	# Flag to indicate that at least some changes were found
    my $changeHandlingError = 0;        # Flag to indicate that non-reconcilable changes were found

    # Check if the Marker was a legacy marker (i.e. 'dead' in master database) but now re-appears in
    # the source database and should be re-activated.
    my $status = $marker_DB->status;
    $logger->debug("$rsid - Status -> ".($status ||'[undef]'));    
    if ( $status eq 'dead') {
        $markerIsChanged = 1;    # Set flag indicating marker is changed
        $marker_XML->set( 'Status', 'active'); # The marker is now back in the dbSNP catalogue
        $logger->debug("$rsid|$markerID was a legacy marker, but now has reappeared in the source database. Need to reactivate.");
	
        # Check revision history for this marker, we need a look at the latest entry
        my @revs_sorted = sort {$b->timecreated->hires_epoch, $a->timecreated->hires_epoch} $marker_DB->revisions();
        foreach my $rev(@revs_sorted) {
            my $t = $rev->timecreated;
            $logger->debug( "$rsid epoch_hires='".$t->hires_epoch."', rev=".$rev->changetype."|".$rev->changetrigger ."|". $rev->timecreated."|".$rev->comment);
        }
        my $last_rev = pop(@revs_sorted);
        my $changeType = $last_rev->changetype();
        $logger->debug("type of last marker revision is '$changeType'");

        if ( $changeType eq 'Merging' ) {
            $marker_stats{Changed}->{'UnMerged'}++;
            $logger->debug("   $rsid => UnMerging");
            my $markerRevision = $self->create_markerrevision_node("UnMerging");
            $marker_XML->addkid($markerRevision);
        } elsif ( $changeType eq 'Deletion' ) {
            $marker_stats{Changed}->{'UnDeleted'}++;
            $logger->debug("   $rsid => UnDeletion");
            my $markerRevision = $self->create_markerrevision_node("UnDeletion");
            $marker_XML->addkid($markerRevision);
        } else {
            $logger->logdie("$rsid Legacy changetype '$changeType' is NOT handled, aborting");
        }
    } else {
        $logger->debug("$rsid was NOT a legacy Marker");
    }
    $logger->debug("$rsid Value of isChanged Flag is '$markerIsChanged'");

    
    # Irrespective of the Marker being legacy or not, the full battery of new vs old comparisons is
    # performed, so we just continue on to the rest of the workflow -  Step D(ii).

    # Check if variation type has changed
    ## ToDo: streamline this stuff
    my $type_is_changed   = $self->compare_variationtype( $marker_DB, $marker_XML);
    $markerIsChanged = 1 if $type_is_changed == 1;

    # Check if marker coordinates have changed
    # [ToDo: need some way of indicating whether we count coord-changes as changes or not.]

    # ToDo: check if validation code has changed

    # Marker is found in the Marker table but theres no Allele information, no check 
    # continue to next Marker.
    
    
    my $alleles_DB= $marker_DB->alleles();
    if (!defined $alleles_DB) {
        $logger->error("$rsid - No Allele Information for Marker $rsid, skipping this marker from further tests");
        $marker_stats{NoAlleles}++;
        return;
    }

    # Check for change in flanking sequences
    my $flanksNochange   = $self->compare_flanks( $marker_DB, $marker_XML);
    $markerIsChanged     = 1 if $flanksNochange != 1;
    $changeHandlingError = 1 if $flanksNochange == -1;

    # Check for changes in alleles
    my $allelesNochange  = $self->compare_alleles( $marker_DB, $marker_XML );
    $markerIsChanged     = 1 if $allelesNochange != 1;
    $changeHandlingError = 1 if $allelesNochange == -1;
    $logger->is_debug() and
        $logger->debug( "$rsid Final updated-marker node, after all checks: ", $marker_XML->itext );

    # Marker statistics
    $marker_stats{Changed}->{'VariantTypeChange'}++ if $type_is_changed == 1 && $flag == 0 && $flag_sf == 0;
    $marker_stats{Changed}->{'SF+VTC'}++ if $type_is_changed == 1 && $flag_sf == 1;
#   $marker_stats{Changed}->{'StrandFlipped'}-- if $markerType == 1 && $flag == 0 && $flag_sf == 1;
    $marker_stats{Changed}->{'AA/AD+VTC'}++ if $type_is_changed == 1 && $flag == 1 ;
    $marker_stats{Changed}->{'AA/AD+SF+VTC'}++ if $type_is_changed == 1 && $flag == 2 ;    
    $flag = 0;

    # Direct final marker node to the appropriate destination output file, depending on the
    # outcome of our battery of tests above.
    if ($changeHandlingError) {

        # Changes or errors were encountered that need manual curation to reconcile
        $logger->error("$rsid had at least one non-reconcilable change. Setting aside for manual curation and flagging as 'withheld'");
        $marker_stats{ManualCheckTotal}++;
        $self->print_marker_gff3($marker_XML, 'manualcheck');
        
        # Also put a copy in the 'withheld' file, which gets loaded into the main db
        ## ToDO work out what to do with those: seperate loading file, or just log them?
        #$self->print_marker_gff3($marker_XML, 'withheld');
        
    } elsif ($markerIsChanged) {
        # Changes were encountered, but reconciled without manual intervention
        $logger->info("$rsid had at least one reconcilable change");
        $marker_stats{ChangedTotal}++;
        $self->print_marker_gff3($marker_XML, 'changed');
    } else {
        # No changes were encountered, so no changes needed (but still have to load them to update timestamp).
        $logger->info("$rsid is unchanged");
        $marker_stats{Unchanged}++;
        $self->print_marker_gff3($marker_XML, 'unchanged');
    }

    # Clean up, lest we want to get memory leaks (Perl doesn't necessarily garbage-collect well..).
    undef $marker_XML;
    undef $marker_DB;
    return;
}


# Return stats collector hash.
sub markersProcessed {
    return %marker_stats;
}

}
1;

=head1 AUTHOR

  Gudmundur A. Thorisson <gthorisson@gmail.com>, Pallavi Sarmah <pallabi2k@gmail.com>

=head1 BUGS AND LIMITATIONS

There are no known bugs in this module.
Please report problems to Rob Free <rcfree@gmail.com>

=head1 CONTACT

This module is part of the HGVbaseG2P project

=head1 LICENCE AND COPYRIGHT

Copyright (c) <2009> University of Leicester, Leicester, UK. All rights reserved.

This module is free software; you can redistribute it and/or
modify it under the same terms as Perl itself.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 

=head1 CVSINFO

$Id: dbSNP.pm 1498 2010-06-29 13:01:15Z gt50 $ 

=cut
