=head1 NAME

HGVbaseG2P::PreProcess::LiftOver - functionality for performing and parsing liftover files

=head1 SYNOPSIS

    use HGVbaseG2P::PreProcess::LiftOver;
    

=head1 DESCRIPTION

  This class implements functions required 

  Derived classes such as HGVbaseG2P::PreProcess::LiftOver implement logic specific to Preprocess required.

=head1 SUBROUTINES/METHODS 

=cut


package HGVbaseG2P::PreProcess::LiftOver;
{
	use strict;
	use warnings;

	use FindBin;
	use lib "$FindBin::Bin/../";

	# Use OIO inside-out object framework for our OO hierarachy
	use Object::InsideOut qw(HGVbaseG2P::PreProcess HGVbaseG2P::FileParser);
	use Object::InsideOut::Metadata;
	use Carp qw(cluck croak confess);
	use Data::Dumper;
	use File::Basename;
    use HGVbaseG2P::FileParser;
	use HGVbaseG2P::Util qw(load_config init_logger open_appendfile);

	my @print_liftover_outfile : Field : Acc(print_liftover_outfile) : Arg( 'Name' => 'print_liftover_outfile')
	  ;            # preprocessor object
	  
    my @print_source_liftover_outfile : Field : Acc(print_source_liftover_outfile) : Arg( 'Name' => 'print_source_liftover_outfile')
	  ;            # preprocessor object
	my @print_unmapped_liftover_outfile : Field : Acc(print_unmapped_liftover_outfile) : Arg( 'Name' => 'print_unmapped_liftover_outfile')
	  ;            # preprocessor object


	# Private variables & constants
	my $logger;        # Reference to master logger
	my %config;        # Master configuration settings from config-file
my %mapped;
my %unmapped;


	# Initialization method (see OIO docs for details)
	sub _init : Init {
		my ( $self, $args ) = @_;

		# Initialize logger
		my $log4perl_conf = $args->{log4perl_conf} || $config{log4perl_conf};
		$logger = init_logger( $log4perl_conf, __PACKAGE__ );
	}

# Methods that need to be implemented by subclasses
	sub filetype : Public {
		die "filetype() method needs to be implemented by the derived class!";
	}
	
	# Methods that need to be implemented by subclasses
	sub file_prefix : Public {
		die "file_prefix() method needs to be implemented by the derived class!";
	}
	
	#default separator (tab) for delimited files
	sub field_separator {
		return "\t";
	}

sub create_liftover_file{
	my ($self,$liftoverfile) = @_;
	


#open the output file
my $fh = open_appendfile($liftoverfile);
$self->print_liftover_outfile($fh);
}

sub create_source_liftover_outfile{
	my ($self,$filename) = @_;
	


#open the output for final lifted over output file (new start + rest of file);
my $fh = open_appendfile($filename);
$self->print_source_liftover_outfile($fh);
}

sub create_unmapped_liftover_outfile{
	my ($self,$filename) = @_;
	


#open the output for final lifted over output file (new start + rest of file);
my $fh = open_appendfile($filename);
$self->print_unmapped_liftover_outfile($fh);
}

=head2 run_liftover

  Usage      : $liftover->run_liftover('/path/to/file/to/liftover','/path/to/chainfile')
  Purpose    : 
  Returns    : 
  Arguments  : Path to input file.
  Throws     : 
  Status     : Public
  Comments   : 

=cut

sub run_liftover : Public {
		my ( $self,$infile,$chainfile) = @_;
	
	
#create an output file based on the inputfile

#filter the query file
my ($filename, $directories, $suffix) = fileparse($infile, qr/\.[^.]*/ );

	#output file
my $liftoverfile = $directories.$filename."_"."liftover".".liftover";

$self->create_liftover_file($liftoverfile);

#make the new source out file.
my $new_sourcefile = $directories.$filename."_"."liftover".".txt";

$self->create_source_liftover_outfile($new_sourcefile);

#make unampped new source file
my $new_unmapped_sourcefile = $directories.$filename."_"."unmapped_liftover".".txt";
$self->create_unmapped_liftover_outfile($new_unmapped_sourcefile);


	my $line_count=0;
	

my $handler =
		  $self->filetype eq 'xml' ? $self : sub { 
		  	$self->process_line(@_);
		  	$line_count++;
		    $line_count % 10 == 0 and $logger->info("$line_count entries processed"
		    );	
		};


$self->parse_file(
			filename => $infile,
			filetype => $self->filetype(),
			handler  => $handler,
			field_sep => $self->field_separator
		);

my ($liftover_resultfile,$liftover_unmapped) = $self->liftover($liftoverfile,$chainfile);

#print Dumper ($liftover_resultfile,$liftover_unmapped);

$self->liftover_parse_mapped($liftover_resultfile);
$self->liftover_parse_unmapped($liftover_unmapped);


		
my $reprocess_handler =
		  $self->filetype eq 'xml' ? $self : sub { 
		  	$self->reprocess_line(@_);
		  	$line_count++;
		    $line_count % 10 == 0 and $logger->info("$line_count entries processed"
		    );	
		};


#reparse the file and recreate with the liftover stuff

$self->parse_file(
			filename => $infile,
			filetype => $self->filetype(),
			handler  => $reprocess_handler,
			field_sep => $self->field_separator
		);


$self->print_liftover_outfile->close;
$self->print_source_liftover_outfile->close;
$self->print_unmapped_liftover_outfile->close;

}






	
sub liftover{
	
	my ($self,$infile,$chain_file)=@_;
	
	#print Dumper($chain_file);
my ($filename, $directories, $suffix) = fileparse($infile, qr/\.[^.]*/ );

	my $new_file =$directories.$filename.".mapped";
	my $unmapped = $directories.$filename.".unmapped";
	
	system ("/Users/rob/Documents/tools/liftover/liftOver $infile $chain_file $new_file $unmapped");
	
	return ($new_file,$unmapped);
	
}

#
#
sub liftover_parse_mapped{
	
	my ($self,$mapped_file)=@_;
	
	
	
$self->parse_file(
			filename => $mapped_file,
			filetype => 'delimited',
			handler  => sub {$self->_get_liftover_start_pos(@_),}
			);

}


		

sub _get_liftover_start_pos {
		my ( $self, $line ) = @_;
#chrX    141008946       141008947       rs5907230
		$mapped{ $line->[3] } =  $line->[1];
		
		#print Dumper($line->[3]);
		
	}


sub get_liftover_start_pos{
		my ( $self, $rsid ) = @_;    
		my $pos = $mapped{$rsid};

		if ( !$pos ) {
			$logger->error("no position");
			$self->increment_stat('unmapped_error');
			return undef;
		}
		return $pos;
	}


sub liftover_parse_unmapped{
	
	my ($self,$unmapped_file)=@_;
	
	
	
$self->parse_file(
			filename => $unmapped_file,
			filetype => 'delimited',
			handler  => sub {$self->_get_liftover_unmapped_pos(@_),}
			);

}


#umapped results

sub _get_liftover_unmapped_pos {
		my ( $self, $line ) = @_;
#chrX    141008946       141008947       rs5907230
	return if $line->[0] =~ /^#/;
		$unmapped{ $line->[3] } =  $line->[1];
		
		#print Dumper($line->[3]);
		
	}


sub get_liftover_unmapped_pos{
		my ( $self, $rsid ) = @_;    
		my $pos = $unmapped{$rsid};

		if ( !$pos ) {
			$logger->error("no position");
			$self->increment_stat('ERROR_not_in_mapped_or_unampped');
			return;
		}
		return $pos;
}
		
}    #package
1;

=head1 SEE ALSO



=head1 BUGS AND LIMITATIONS

There are no known bugs in this module.
Please report problems to Mummi <gthorisson@gmail.com>

=head1 CONTACT

This module is part of the HGVbase-G2P project

=head1 AUTHOR

Robert Hastings <rkh7@le.ac.uk>


=head1 LICENCE AND COPYRIGHT

Copyright (c) <2007> University of Leicester, Leicester, UK. All rights reserved.

This module is free software; you can redistribute it and/or
modify it under the same terms as Perl itself.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 

=head1 CVSINFO

$Id: LiftOver.pm 1098 2008-12-05 13:27:01Z rcf8 $ 

=cut





