
=head1 NAME

HGVbaseG2P::PreProcess::Split - functionality for performing and parsing and split files by chr.
gets all chromosomes from input file and splits into seperate files based on the input source

=head1 SYNOPSIS

    use HGVbaseG2P::PreProcess::Split;
    

=head1 DESCRIPTION

  This class implements functions required 

  Derived classes such as HGVbaseG2P::PreProcess::Split::Broad implement logic specific to Preprocess required.

=head1 SUBROUTINES/METHODS 

=cut
package HGVbaseG2P::PreProcess::Split;
{
	use strict;
	use warnings;
	use FindBin;
	use lib "$FindBin::Bin/../";

	# Use OIO inside-out object framework for our OO hierarachy
	use Object::InsideOut qw(HGVbaseG2P::PreProcess HGVbaseG2P::FileParser);
	use Object::InsideOut::Metadata;
	use Carp qw(cluck croak confess);
	use Data::Dumper;
	use File::Basename;
	use HGVbaseG2P::FileParser;
	use HGVbaseG2P::Util qw(load_config init_logger open_outfile);
	my @print_split_outfile : Field : Acc(print_split_outfile) :
	  Arg( 'Name' => 'print_split_outfile');    # preprocessor object for split outfile 
	my @print_split_imputedoutfile : Field : Acc(print_split_imputedoutfile) :
	  Arg( 'Name' => 'print_split_imputedoutfile');    # preprocessor object for split imputed outfiles

	# Private variables & constants
	my $logger;    # Reference to master logger
	my %config;    # Master configuration settings from config-file

	# Initialization method (see OIO docs for details)
	sub _init : Init
	{
		my ( $self, $args ) = @_;

		# Initialize logger
		my $log4perl_conf = $args->{log4perl_conf} || $config{log4perl_conf};
		$logger = init_logger( $log4perl_conf, __PACKAGE__ );
	}

	# Methods that need to be implemented by subclasses
	sub filetype : Public
	{
		die "filetype() method needs to be implemented by the derived class!";
	}

	# Methods that need to be implemented by subclasses
	sub file_prefix : Public
	{
		die
		  "file_prefix() method needs to be implemented by the derived class!";
	}

	#default separator (tab) for delimited files
	sub field_separator
	{
		return "\t";
	}

	sub create_split_file
	{
		my ( $self, $outfile ) = @_;

		#open the output file
		my $fh = open_outfile($outfile);
		$self->print_split_outfile($fh);
	}

	#imputed ooutfile
	sub create_split_imputedfile
	{
		my ( $self, $outfile ) = @_;

		#open the output file
		my $fh = open_outfile($outfile);
		$self->print_split_imputedoutfile($fh);
	}

sub process_file : Public
	{
		my ( $self, $infile ) = @_;

		#filter the query file
		my ( $filename, $directories, $suffix ) =
		  fileparse( $infile, qr/\.[^.]*/ );
		my $line_count = 0;
		my $handler = $self->filetype eq 'xml' ? $self : sub {
			$self->process_line(@_);
#			$line_count++;
#			$line_count % 10 == 0
#			  and $logger->info( "$line_count entries processed" );
		};
		$self->parse_file(
						  filename  => $infile,
						  filetype  => $self->filetype(),
						  handler   => $handler,
						  field_sep => $self->field_separator
		);
		my $hash = $self->return_chr;
		$self->split_files( $infile, $hash );
	}

	#sub to split files
	sub split_files
	{
		my ( $self, $infile, $hash ) = @_;
		my $line_count = 0;
		my $reprocess_line_count= 0;

		#filter the query file
		my ( $filename, $directories, $suffix ) =
		  fileparse( $infile, qr/\.[^.]*/ );
		foreach my $key ( sort keys %$hash )
		{

			#open the outfile
			#make the new source out file.
			my $new_outfile =
			  $directories . $filename . "_" . "chr" . $key . ".txt";
			$self->create_split_file($new_outfile);

			#make the new source out file.
			my $new_imputedoutfile =
			  $directories . $filename . "_" . "chr" . $key . "_imputed.txt";
			$self->create_split_imputedfile($new_imputedoutfile);
			my $handler = $self->filetype eq 'xml' ? $self : sub {
				$self->reprocess_line( @_, $key );
#				$reprocess_line_count++;
#				$reprocess_line_count % 10 == 0
#				  and $logger->info( "$reprocess_line_count entries processed" );
#				 $self->increment_stat("Total");
			};
		
			$self->parse_file(
							  filename  => $infile,
							  filetype  => $self->filetype(),
							  handler   => $handler,
							  field_sep => $self->field_separator
			);
			
		}
			 
	
	
$self->print_split_outfile->close;
$self->print_split_imputedoutfile->close;

	
		
	}    #sub end
}    #package
1;

=head1 SEE ALSO



=head1 BUGS AND LIMITATIONS

There are no known bugs in this module.
Please report problems to Mummi <gthorisson@gmail.com>

=head1 CONTACT

This module is part of the HGVbase-G2P project

=head1 AUTHOR

Robert Hastings <rkh7@le.ac.uk>


=head1 LICENCE AND COPYRIGHT

Copyright (c) <2007> University of Leicester, Leicester, UK. All rights reserved.

This module is free software; you can redistribute it and/or
modify it under the same terms as Perl itself.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 

=head1 CVSINFO

$Id: Split.pm 1098 2008-12-05 13:27:01Z rcf8 $ 

=cut


