# $Id: Search.pm 1235 2009-07-08 13:55:02Z rcf8 $

=head1 NAME

HGVbaseG2P::Database - Storing to / retrieving from a DBIx::DBStag database


=head1 SYNOPSIS

    use English qw( -no_match_vars );
    use HGVbaseG2P::Database;
    my $dbh = HGVbaseG2P::Database->new(conf_file => 'path to master config');
    my $node_set = $dbh->selectall_stag(-template => 'hgvbase-marker',
                                        -bind     => {MarkerID => 'HGVM510'});


=head1 DESCRIPTION

This class is a simple wrapper around different types of searches

=head1 SUBROUTINES/METHODS 

=cut

package HGVbaseG2P::Search; {
    
use strict;
use warnings;

# Use OIO inside-out object framework for our OO hierarachy
use Object::InsideOut;
use Object::InsideOut::Foreign;
use Object::InsideOut::Universal;
use Object::InsideOut::Metadata;
use Text::ParseWords;

use English qw( -no_match_vars );
use Data::Dumper qw(Dumper);
use FindBin;
use Cwd;
use HGVbaseG2P::Util qw(load_config init_logger load_module);
use Env;
my $DEFAULT_PAGE_SIZE = 10;

# Private variables & constants
    my @context : Field :All(context); #Catalyst context
    my @log : Field :All(log); #contains Log4perl Logger
    my @config : Field :All(config); #contains loaded config
	my @page_size : Field :All(page_size);
	my @page : Field :All(page);
	my @qterms : Field :All(qterms);
	
    # Initialization arguments
    my %init_args : InitArgs = (
        'conf_file'     => '',
        'log4perl_conf' => '',
        'dsn' => '',
        'user' => '',
        'pass' => '',
        'query' => '',
    );

    # Initialization method (see OIO docs for details)
    sub _init : Init
    {
        my ( $self, $args ) = @_;

		
        # Load supplied config file, if non provided use default
        my $conf_file = $args->{conf_file} || undef;
        !$conf_file and die "No config file supplied";
        $self->config({load_config( $conf_file )});
		
		# Initialize logger
        my $log4perl_conf = $args->{log4perl_conf} || $self->config->{log4perl_conf};
        $self->log(init_logger( $log4perl_conf, __PACKAGE__ ));

        my $query = $args->{query};
        
        if ($query) {
	        $query =~ s/'/\\'/;
			$self->log->info("query is $query");
			
			my @qterms = grep { !/\A(and|or|\+|\-|near)\Z/xmsi } quotewords('\s+',0,$query);
			$self->log->info("qterms array is:".Dumper(\@qterms));
			
			my $qterms = join(" or ",@qterms);
			$qterms =~ s/'/\\'/;
			$self->log->info("qterms is $qterms");
	        
			$self->qterms($qterms);
	    	$self->log->info("Got qterms from qstring '".$qterms."': ".join('|', @qterms));
        }
    }
    
    sub new_datafeed {
    	my ($self, $content_type,$args)=@_;
    	
		return load_module("HGVbaseG2P::Search::DataFeed",{'conf_file'=>$self->config});
    }
    
    sub new_results {
    	my ($self, $results_module,$args)=@_;
    	return load_module("HGVbaseG2P::Search::Results::$results_module",{'conf_file'=>$self->config});
    }
    
    sub new_aggregator {
    	my ($self)=@_;
    	return load_module("HGVbaseG2P::Search::Aggregator",{'conf_file'=>$self->config});
    }
    
	sub query_xapian {
	    my ($self, $dbname) = @_;
	
	    my $xap = $self->context->model('Xapian')->dbh;
		my $page_size = 10;
		
	    # Send query with supplied query string to text search engine. This may cause an exception
	    # to be thrown (e.g. invalid query syntax), so let's catch that
	    my ($pager,$identifiers) = $xap->search($self->qterms,$dbname, $self->page, $self->page_size);
		
		#grab list of identifiers from Xapian database and then retrieve study object for each
		my $method = "get_".lc($self->get_database_value($dbname,'class'))."_by_identifier";
		my @results;
		foreach my $identifier(@{$identifiers}) {
			push @results, $self->context->model("StudyDB")->$method($identifier);
		}
		
  	  	return ($pager,\@results);
	}
	
	sub get_database_value {
		my ($self, $dbname, $key) = @_;
		$self->log->info("config:".Dumper($self->config));
		$self->config->{'Database'}->{$dbname}->{$key}
	}
	
	sub query_study {
	    my ($self, $dbclass, $qterms) = @_;
	    
	    # We may not have any query terms at all in the string, e.g. 'NOT' 
	    return undef if @$qterms == 0;
	    my $rset = $self->context->model("StudyDB")->resultset($dbclass)->search({ -or => [
								identifier => $self->qterms,
								accession => $self->qterms]
							 });
	    
	    my @results = $rset->all;
	    return ($self->get_pager(\@results), \@results);
	}
	
	sub query_marker {
	    my ($self, $dbclass) = @_;
		$self->log->info("qterms:".Dumper($self->qterms));
		
	    # We may not have any query terms at all in the string, e.g. 'NOT' 
	    return undef if !$self->qterms;
	    
	    my $rset = $self->context->model("MarkerDB")->get_marker_rs_by_accession_or_identifier($self->qterms, $self->qterms);
	    my @results = $rset->all;
	    return ($self->get_pager(\@results), \@results);
	}

	sub get_pager {
		my ($self, $results) = @_;
		
		my $pager = Data::Pageset->new({total_entries    => scalar(@{$results}),
	                                    entries_per_page => $self->page_size || $DEFAULT_PAGE_SIZE,
	                                    current_page     => $self->page || 1});
	    return $pager;
	}

	sub query_mart {
	    my ($self, $c, $db, $qstring) = @_;
	    #my $mart = $c->model('Mart');
	    
	    # 1) Find from search-config which Mart-query to use 
	    # 2) Set the appropriate filter, depending on query ( Idea: make use of attribute:value pairs here also, e.g. start:9999 stop:111000?)
	    # 3) execute query
	    # 4) pass query results to template
	}

	#sub query_seqfeature {
	#    my ($self, $db, $qterms) = @_;
	#    
	#    my $featdb = $self->context->model('SeqFeature');
	#    my $dbconfig = $self->config->{Database}->{$db};
	#    my @ftypes = split(/\s/, $dbconfig->{feature_types});
	#    my (@results, $pager);
	#    foreach my $qterm(@$qterms) {
	#        
	#        # Determine whether this is a plain keyword, or a 'shorthand' chrom-interval
	#        if(my ($chr,$start,$stop) = $qterm =~ /(chr\w+):([0-9\,]+)[\-|\.\.]+([0-9\,]+)\Z/ixms) {
	#            $start =~ s/[\.,]//g;
	#            $stop  =~ s/[\.,]//g;
	#            $c->log->debug("Fetching feature-summary for genomic segment $chr, from $start to $stop");
	#            my $seg = $featdb->segment($chr, $start=>$stop);
	#            push @results, $seg;
	#        }
	#        else { # Otherwise do an alias search 
	#            $c->log->debug("Fetching features by name '$qterm'");
	#            my @feats = $featdb->features(-name    => $qterm,
	#                                          -aliases => 1,
	#                                          -types   => \@ftypes);            
	#            push @results, @feats;
	#            # NOTE TO SELF: do I need handling for >hits per name here, for multiple locations for a marker??
	#        }
	#    }
	#    
	#    my $results;
	#    if(@results > 0) {
	#        $pager = Data::Pageset->new({total_entries    => scalar(@results),
	#                                     entries_per_page => $self->page_size || $DEFAULT_PAGE_SIZE,
	#                                     current_page     => $self->page || 1});
	#        $c->log->debug("complete list of results: ",@results);
	#        if(@results > $pager->entries_per_page()) {
	#            my $index = $pager->entries_per_page() * ($pager->current_page() - 1);
	#            my @results_partial = @results[$index..($index+$pager->entries_per_page()-1)];
	#            $c->log->debug("sliced list of results ($index-".($index+$pager->entries_per_page()-1)."): ",@results_partial);
	#            $results = \@results_partial;
	#        }
	#        else {
	#            $results = \@results;
	#        }
	#    }
	#    return ($pager, $results);
	#
	#
	#}
}
1;

=head1 SEE ALSO

L<HGVbaseG2P::FileParser>, L<HGVbaseG2P::Database> and related classes

=head1 BUGS AND LIMITATIONS

There are no known bugs in this module.
Please report problems to Rob <rcfree@gmail.com>

=head1 CONTACT

This module is part of the HGVbase-G2P project

=head1 AUTHOR

Rob Free <rcfree@gmail.com>

=head1 LICENCE AND COPYRIGHT

Copyright (c) <2009> University of Leicester, Leicester, UK. All rights reserved.

This module is free software; you can redistribute it and/or
modify it under the same terms as Perl itself.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 

=head1 CVSINFO

$Id: Search.pm 1235 2009-07-08 13:55:02Z rcf8 $ 

=cut

