# $Id: Aggregator.pm 1272 2009-08-21 15:08:35Z rcf8 $

=head1 NAME

HGVbaseG2P::Search::Aggregator - Return combined DataFeeds in different formats


=head1 SYNOPSIS

    use HGVbaseG2P::Search;
    my $search = HGVbaseG2P::Search->new({content_type=>'text/xml'});


=head1 DESCRIPTION

This class is a simple wrapper around different types of searches. An instance of the object can be set up and then several searches
performed using the same parameters.

=head1 SUBROUTINES/METHODS 

=cut

package HGVbaseG2P::Search::Aggregator; {
    
use strict;
use warnings;

# Use OIO inside-out object framework for our OO hierarachy
use Object::InsideOut;
use Object::InsideOut::Foreign;
use Object::InsideOut::Universal;
use Object::InsideOut::Metadata;
use Text::ParseWords;

#use English qw( -no_match_vars );
use Data::Dumper qw(Dumper);
use FindBin;
use Cwd;
use XML::Atom::SimpleFeed;
use XML::Simple;

use Env;
my $DEFAULT_PAGE_SIZE = 10;
use JSON::XS;
use URI;
use HGVbaseG2P::Util qw(load_config init_logger);


# Globals
my $json_coder = JSON::XS->new->ascii->pretty->allow_nonref;
$json_coder->space_after(1);

# Private variables & constants
    my @log : Field :All(log); #contains Log4perl Logger
    my @config : Field :All(config); #contains loaded config
	
	my @xml : Field :All(xml); #contains internal XML::Simple object
	my @data_feeds : Field :All(data_feeds);
	my @content_type : Field :All(content_type);
	my @base_server : Field :All(base_server);
	
    # Initialization arguments
    my %init_args : InitArgs = (
        'conf_file'     => '',
        'log4perl_conf' => '',
    );
    
    # Initialization method (see OIO docs for details)
    sub _init : Init
    {
        my ( $self, $args ) = @_;
      	
      	# Load supplied config file, if non provided use default
        my $conf_file = $args->{conf_file} || undef;
        !$conf_file and die "No config file supplied";
        $self->config({load_config( $conf_file )});
        
      	# Initialize logger
        my $log4perl_conf = $args->{log4perl_conf} || $self->config->{log4perl_conf};
        $self->log(init_logger( $log4perl_conf, __PACKAGE__ ));
        
        my $xml = new XML::Simple (NoAttr=>1, RootName=>'result') or die "unable to create XML::Simple";
      	$self->xml($xml);
      	$self->data_feeds([]);
    }
    
    sub output {
    	my ($self) = @_;
    	my $content_type = $self->content_type;
    	$content_type =~ s/\//_/g;
    	my $format_method = "_to_".$content_type;
    	return ($self->$format_method);
    }
    
    sub add_data_feed {
    	my ($self, $data_feed) = @_;
    	push @{$self->data_feeds}, $data_feed;
    }
    
    sub _to_text_xml {
    	my ($self)=@_;
    	
    	my $feed = XML::Atom::SimpleFeed->new(
	    	title=>'HGVbaseG2P search results',
	    	link=>$self->base_server->{uri},
	    	author => 'hgvbaseg2p',
    	);

    	foreach my $df(@{$self->data_feeds} ) {
    		$self->log->info("datafeed:$df");
	    	foreach my $item(@{$df->entries}) {
	    		my $summary="<table style='border-collapse:collapse'>";
	    		foreach my $key(keys %{$item->{summary}}) {
	    			$summary.="<tr><th style='text-align:right'><b>".$key.":</b></th><td>".$item->{summary}->{$key}."</td></tr>";
	    		}
	    		$summary.="</table>";
		    	$feed->add_entry(
					id=>$item->{id}." ",
			    	title=>$item->{title},
			    	summary=>$summary,
			    	link=>$item->{link},
			    	content=>{ type => 'text/xml', content=>$self->xml->XMLout($item->{content}) },
				);
	    	}
    	}
    	return $feed->as_string;
    }
    
    sub _to_text_plain {
    	my ($self)=@_;
    	my $content;
    	$content = join(",", keys %{$self->data_feeds->[0]->entries->[0]})."\n";
    	
    	foreach my $df(@{$self->data_feeds}) {
	    	foreach my $item(@{$df->entries}) {
	    		$content.=join(",", values %{$item})."\n";
	    	}
    	}
    	return $content;
    }
    
    sub _aggregate_feeds {
    	my ($self)=@_;
    	my @items;
    	
    	foreach my $df(@{$self->data_feeds}) {
	    	
	    	foreach my $item(@{$df->entries}) {
	    		push @items, $item;
	    	}
    	}
    	return @items;
    }
    
    sub _to_application_json {
    	my ($self)=@_;
    	my @feed = $self->_aggregate_feeds();
    	$self->log->info("feed:".Dumper($feed[0]));
    	return $json_coder->encode([$self->_aggregate_feeds]);
    }
	
	sub _to_text_html {
		my ($self)=@_;
		
		my @items = @{$self->data_feeds->[0]->entries};
 		
		return (
		{ 
			main_headings => [ keys %{$items[0]} ],
    		content_headings => [ keys %{$items[0]->{content}} ],
			data => \@items
		}, 'search/html.tt2');
	}
}
1;

=head1 SEE ALSO

L<HGVbaseG2P::FileParser>, L<HGVbaseG2P::Database> and related classes

=head1 BUGS AND LIMITATIONS

There are no known bugs in this module.
Please report problems to Rob <rcfree@gmail.com>

=head1 CONTACT

This module is part of the HGVbase-G2P project

=head1 AUTHOR

Rob Free <rcfree@gmail.com>

=head1 LICENCE AND COPYRIGHT

Copyright (c) <2009> University of Leicester, Leicester, UK. All rights reserved.

This module is free software; you can redistribute it and/or
modify it under the same terms as Perl itself.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 

=head1 CVSINFO

$Id: Aggregator.pm 1272 2009-08-21 15:08:35Z rcf8 $ 

=cut

