# $Id: DataFeed.pm 1272 2009-08-21 15:08:35Z rcf8 $

=head1 NAME

HGVbaseG2P::Search::DataFeed - A DataFeed from a single server/file


=head1 SYNOPSIS

    use HGVbaseG2P::Search;
    my $search = HGVbaseG2P::Search::DataFeed->new;


=head1 DESCRIPTION

This class wraps a single DataFeed from a single server. DataFeeds are combined using Aggregator.

=head1 SUBROUTINES/METHODS 

=cut

package HGVbaseG2P::Search::DataFeed; {
    
use strict;
use warnings;

# Use OIO inside-out object framework for our OO hierarachy
use Object::InsideOut;
use Object::InsideOut::Foreign;
use Object::InsideOut::Universal;
use Object::InsideOut::Metadata;
use Text::ParseWords;

use English qw( -no_match_vars );
use Data::Dumper qw(Dumper);
use FindBin;
use Cwd;
use HGVbaseG2P::Util qw(load_config init_logger );
use HGVbaseG2P::Web::Util qw(add_uri_parameters);

use HGVbaseG2P::Search::Aggregator;
use XML::Simple;
use Env;

# Private variables & constants
    my @log : Field :All(log); #contains Log4perl Logger
    my @config : Field :All(config); #contains loaded config
	my @xml : Field :All(xml); #contains internal XML::Simple object
	my @title : Field :All(title);
	my @author : Field :All(author);
	my @server : Field :All(server);
	my @entries : Field :All(entries);

	# Initialization arguments
    my %init_args : InitArgs = (
        'conf_file'     => '',
        'log4perl_conf' => '',
    );
    
    # Initialization method (see OIO docs for details)
    sub _init : Init
    {
        my ( $self, $args ) = @_;
      	
      	# Load supplied config file, if non provided use default
        my $conf_file = $args->{conf_file} || undef;
        !$conf_file and die "No config file supplied";
        $self->config({load_config( $conf_file )});
        
      	# Initialize logger
        my $log4perl_conf = $args->{log4perl_conf} || $self->config->{log4perl_conf};
        $self->log(init_logger( $log4perl_conf, __PACKAGE__ ));
        
      	my $xml = new XML::Simple (NoAttr=>1, RootName=>'result');
      	$self->xml($xml);
      	$self->entries([]);
    }
    
    sub input_from_search {
    	my ($self,$results)=@_;
    	
    	my @new_entries = @{$results};
    	
    	foreach(@new_entries) {
    		$_->{id}=$self->server->{uri}.$_->{id};
    		$_->{link}=$self->server->{uri}.$_->{link}
    	}
    	
    	$self->input(\@new_entries);
    }
    
    
    sub input {
    	my ($self,$results)=@_;
    	
    	my @entries = @{$self->entries};
    	
    	my @new_entries = @{$results};
    	
    	foreach(@new_entries) {
    		$_->{content}->{server}=$self->server->{name};
    	}
    	
    	my @combined_entries = (@entries, @new_entries);
    	
    	$self->entries(\@combined_entries);
    }
    
    sub input_from_rss {
    	my ($self,$uri)=@_;
    	
    	$self->log->info("parse_uri($uri)");
    	my $uri_with_ct = add_uri_parameters($uri,{'content-type'=>'text/xml'});
    	my $feed = XML::Feed->parse(URI->new($uri_with_ct),'Atom') or die "RSS feed $uri not available!";
    	$self->log->info("title:".$feed->title);
    	
    	my @results;
    	foreach my $entry($feed->entries) {
    		my $content_body = $entry->content->body;
    		my $content = $self->xml->XMLin($content_body);
    		my $summary =$entry->summary->body;

    		$summary=~/(<tr>.*)/;
    		my $temp = $1;
    		my @rows = split('</tr>',$temp);
    		my %summary_hash;
    		foreach my $row(@rows) {
    			$row =~ /<b>(.*):<\/b>.*<td>(.*)<\/td>/;
    			$summary_hash{$1}=$2;
    		}
    		
    		my $item={
    			id => $entry->id,
    			title=>$entry->title,
    			link=>$entry->link,
    			content=>$content,
    			summary=>\%summary_hash
    		};

    		push @results,$item;
    	}
    	
    	$self->input(\@results);
    }
    
    sub output_to {
    	my ($self, $content_type)=@_;
    	my $ag = HGVbaseG2P::Search::Aggregator->new({conf_file=>$self->config});
    	$ag->base_server($self->server);
    	$ag->add_data_feed($self);
    	$ag->content_type($content_type);
    	$ag->output;
    }
}
1;

=head1 SEE ALSO

L<HGVbaseG2P::FileParser>, L<HGVbaseG2P::Database> and related classes

=head1 BUGS AND LIMITATIONS

There are no known bugs in this module.
Please report problems to Rob <rcfree@gmail.com>

=head1 CONTACT

This module is part of the HGVbase-G2P project

=head1 AUTHOR

Rob Free <rcfree@gmail.com>

=head1 LICENCE AND COPYRIGHT

Copyright (c) <2009> University of Leicester, Leicester, UK. All rights reserved.

This module is free software; you can redistribute it and/or
modify it under the same terms as Perl itself.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 

=head1 CVSINFO

$Id: DataFeed.pm 1272 2009-08-21 15:08:35Z rcf8 $ 

=cut

