# $Id: Results.pm 1272 2009-08-21 15:08:35Z rcf8 $

=head1 NAME

HGVbaseG2P::Search::Results - Return Results for specific type of search


=head1 SYNOPSIS

    use HGVbaseG2P::Search;
    my $search = HGVbaseG2P::Search::Results->new({content_type=>'text/xml'});


=head1 DESCRIPTION

This class is a simple wrapper around different types of searches. An instance of the object can be set up and then several searches
performed using the same parameters.

=head1 SUBROUTINES/METHODS 

=cut

package HGVbaseG2P::Search::Results; {
    
use strict;
use warnings;

# Use OIO inside-out object framework for our OO hierarachy
use Object::InsideOut;
use Object::InsideOut::Foreign;
use Object::InsideOut::Universal;
use Object::InsideOut::Metadata;
use Text::ParseWords;

use English qw( -no_match_vars );
use Data::Dumper qw(Dumper);
use FindBin;
use Cwd;
use HGVbaseG2P::Util qw(load_config init_logger load_module);
use XML::Atom::SimpleFeed;
use Env;
my $DEFAULT_PAGE_SIZE = 10;

# Private variables & constants
    my @log : Field :All(log); #contains Log4perl Logger
    my @config : Field :All(config); #contains loaded config
    my @params : Field :All(params); #contains parameters - generally loaded from CGI param
	my @page : Field :All(page); #contains parameters - generally loaded from CGI param
	my @page_size : Field :All(page_size); #contains parameters - generally loaded from CGI param
	my @server : Field :All(server_name);
	my @query : Field :All(query);
	my @qterms : Field :All(qterms);
	
    # Initialization arguments
    my %init_args : InitArgs = (
        'conf_file'     => '',
        'log4perl_conf' => '',
    );
    
    # Initialization method (see OIO docs for details)
    sub _init : Init
    {
        my ( $self, $args ) = @_;
      	
      	# Load supplied config file, if non provided use default
        my $conf_file = $args->{conf_file} || undef;
        !$conf_file and die "No config file supplied";
        $self->config({load_config( $conf_file )});
        
      	# Initialize logger
        my $log4perl_conf = $args->{log4perl_conf} || $self->config->{log4perl_conf};
        $self->log(init_logger( $log4perl_conf, __PACKAGE__ ));

    }
    
    sub accepted_parameters {
    	return qw(keywords phenotype diseases threshold);
    }
    
    sub get_db_from_catalyst {
    	my ($self, $c)=@_;
    	my %required_db = %{$self->required_db};
    	foreach my $db(keys %required_db) {
    		my $model = $c->model("$db")->dbh or die "$db not found";
    		my $db_method = $required_db{$db};
    		$self->log->info("Added $db ($model) using $db_method");
    		$self->$db_method($model);
    	}
    }
    
    sub search {
    	my ($self,$item)=@_;
    	die "search method not present";
    }
    
    sub parse_query {
    	my ($self,$query)=@_;
        
        $query =~ s/'/\\'/;
		$self->log->info("query is $query");
		
		my @qterms = grep { !/\A(and|or|\+|\-|near)\Z/xmsi } quotewords('\s+',0,$query);
		
		my @wcterms = map { s/'s//; "$_"; } @qterms;
		$self->log->info("qterms array is:".Dumper(\@wcterms));
		
		my $qterms = join(" or ",@wcterms);
		$qterms =~ s/'/\\'/g;
		$qterms =~ s/\*//g;
		$self->log->info("qterms is $qterms");
        
    	$self->log->info("Got qterms from qstring '".$self->query."': ".join('|', @qterms));
    	return $qterms;
    }
    
    sub param {
    	my ($self, $param) = @_;
    	
    	!$self->params->{$param} and return undef;
    	
    	if (ref($self->params->{$param}) eq 'ARRAY') {
    		$self->log->info("param:$param - ".Dumper($self->params->{$param}));
	    	my @params = @{$self->params->{$param}};
	
	    	if (!defined($params[0]))  {
	    		#$self->log->info("not defined");
	    		return undef;
	    	}
	    	if (length($params[0])==0) {
	    		#$self->log->info("empty value");
	    		return undef;
	    	}
	    	if (scalar(@params==0)) { $self->log->info("nothing in it"); return undef; }
	    	return \@params;
    	}
    	else {
    		if($self->params->{$param} eq '') {
    			return undef;
    		}
    		else {
    			return $self->params->{$param};
    		}
    	}
    }
    
    sub extract_keywords {
    	my ($self,$keywords)=@_;
    	
    	my ($chr, $start, $stop, $type);
    	if ($keywords =~ /(\w+)\D(\d+)..(\d+)/) {
				($chr,$start,$stop) = ($1,$2,$3);
				$chr=~/(chr|Chr)(\w+)/;
				$chr = "$2";
				return ('region', $chr, $start, $stop);
		}
#		elsif ($keywords =~ /[Chr|chr]\w/) {
#			return ('region', $1, $start, $stop);
#		}
		return ('feature');
    }
    
    sub get_feature_region {
    	my ($self,$feature, $featdb) = @_;
    	my %feats;
		my ($chr,$start,$stop);
		#$self->log->info("Fetching features by region '$feature'");
           my @feats = $featdb->dbh->get_features_by_name(-name    => $feature,
                                          -aliases => 1);            
        
    	foreach(@feats) {
	    	my $region_info = $_->display_name."|".$_->ref.":".$_->start."..".$_->stop."|".$_->type;
	    	$feats{$region_info}++;
    	}
    	#$self->log->info("feats:".Dumper(\%feats));
    	my @idea = reverse sort keys %feats;
    	return $self->extract_keywords($idea[0]);
    }
}
1;

=head1 SEE ALSO

L<HGVbaseG2P::FileParser>, L<HGVbaseG2P::Database> and related classes

=head1 BUGS AND LIMITATIONS

There are no known bugs in this module.
Please report problems to Rob <rcfree@gmail.com>

=head1 CONTACT

This module is part of the HGVbase-G2P project

=head1 AUTHOR

Rob Free <rcfree@gmail.com>

=head1 LICENCE AND COPYRIGHT

Copyright (c) <2009> University of Leicester, Leicester, UK. All rights reserved.

This module is free software; you can redistribute it and/or
modify it under the same terms as Perl itself.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 

=head1 CVSINFO

$Id: Results.pm 1272 2009-08-21 15:08:35Z rcf8 $ 

=cut

    