# $Id: phenotypes.pm 1272 2009-08-21 15:08:35Z rcf8 $

=head1 NAME

HGVbaseG2P::Search::Results::phenotypes - Return Results for 'phenotypes' search


=head1 SYNOPSIS

    use HGVbaseG2P::Search;
    my $search = HGVbaseG2P::Search::Results->new({content_type=>'text/xml'});


=head1 DESCRIPTION

This class is a simple wrapper around different types of searches. An instance of the object can be set up and then several searches
performed using the same parameters.

=head1 SUBROUTINES/METHODS 

=cut

package HGVbaseG2P::Search::Results::phenotypes; {
    
use strict;
use warnings;

# Use OIO inside-out object framework for our OO hierarachy
use Object::InsideOut qw(HGVbaseG2P::Search::Results);
use English qw( -no_match_vars );
use Data::Dumper qw(Dumper);
use HGVbaseG2P::Util qw(load_config init_logger );
use Text::ParseWords;
use List::Compare;

	my @study_db : Field :All(study_db); #contains parameters - generally loaded from CGI param
	my @xapian_db : Field :All(xapian_db);
    
    sub accepted_parameters {
    	return qw(q category);
    }
    
    sub required_db {
    	return {
    		'StudyDB'=>'study_db',
    		'Xapian'=>'xapian_db'
    	};
    }
    
    sub search {
    	my ($self)=@_;
    	
		my $keywords = $self->param('q');
		my $diseases = $self->param('category');
		
		my ($data,$pager,$total);
		
		if ($keywords) {
			$self->log->info("keywords found: $keywords");
			($data,$pager,$total) = $self->get_phenotype_data($keywords,$diseases);
		}
		else {
			$self->log->info("no keywords found");
			if ($diseases) {
				$self->log->info("no keywords found but diseases found");
				my $rs = $self->study_db->get_paged_phenotypemethods_by_disease_rs($self->page,$self->page_size, $diseases );
				$data = [ $rs->all ];
				$pager = $rs->pager;
				$total = $self->study_db->count_all_phenotypemethods;
			}
			else {
				$self->log->info("no keywords found");
				my $rs = $self->study_db->get_paged_phenotypemethods_rs($self->page,$self->page_size);
				$data = [ $rs->all ];
				$pager = $rs->pager;
			}
		}
		
		return ($self->filter_results($data),$pager,$total);
    }
    
    sub get_phenotype_data {
    	my ($self, $query, $diseases)=@_;
    	my @data;
    	
	    my $xap = $self->xapian_db;
		my $page_size = 10;
		
		my $qterms = $self->parse_query($query);
		
		# Send query with supplied query string to text search engine. This may cause an exception
	    # to be thrown (e.g. invalid query syntax), so let's catch that
	    my ($pager,$identifiers) = $xap->search($qterms,'phenotypes', $self->page, $self->page_size);
		my $total = $self->study_db->count_all_phenotypemethods;
		
		#grab list of identifiers from Xapian database and then retrieve phenotypemethod object for each
		my $method = "get_phenotypemethod_by_identifier";
		my @results;
		
		$self->log->info("disease_filter:".Dumper($diseases));
		my @d_filter=();
	    if ($diseases) {
	    	if (ref($diseases) ne 'ARRAY') {
	    		push @d_filter,$diseases.";";
	    	}
	    	else {
	    		@d_filter = map { $_.";" } @{$diseases};
	    	}
	    	foreach my $identifier(@{$identifiers}) {
	    		my $phen = $self->study_db->$method($identifier);
	    		my $discat = $phen->phenotypeproperty->diseasecategory;
	    		foreach my $p_disease(@d_filter) {
	    			$self->log->info("p_disease:$p_disease");
	    			
	    			
		    		if ($discat->name =~ m/$p_disease/) {  
		    			$self->log->info("discat:".$discat->name . " eq ".$p_disease);
						push @results, $phen;
						last;
		    		}
		    		else {
		    			$self->log->info("discat:".$discat->name . " ne ".$p_disease);
		    		}
	    		}
	    		$pager->total_entries(scalar(@results));
			}
	    }
	    else {
			foreach my $identifier(@{$identifiers}) {	
				push @results, $self->study_db->$method($identifier);
			}
		}
				
  	  	return (\@results,$pager,$total);
    }
    
    sub filter_results {
    	my ($self, $data)=@_;
    	my @results=();
    	if ($data) {
			my @param_diseases = $self->param('diseases');
    		my $counter=0;
			foreach my $item(@{$data}) {	
				
				
			    my $state;
			    my @resultsets=();
			    foreach my $exp($item->experiments) {
			    	foreach my $rs($exp->resultsets) {
			    		push @resultsets, $rs->resultsetid;
			    	}
			    }
    
			     #check if study is allowed to be added to browser and change image to noentry if so
			    if (!$item->study->isdisplayed) {
			    	$state='noentry';
			    }
			    
			    my $discat = $item->phenotypeproperty->diseasecategory;
			    my @ph_discats;
			    $discat and @ph_discats = split(';',$discat->name);
				
				my $body = {
		        	'identifier' => $item->identifier,
		        	'study' => $item->study->identifier,
		        	'study_name' => $item->study->name,
		        	'diseasecategory' => \@ph_discats,
		        	'name' =>  $item->name,
		        	'description' => $item->description || '',
		        	'state'=>$item->study->isdisplayed,
		        	'resultsets'=>join(',',@resultsets),
		        	'phenotypeproperty' => $item->phenotypeproperty->name,
		        };
				
				my $summary = {
					'Phenotype Identifier'=>$item->identifier,
					'Name'=> $item->name,
					'Description' => substr($item->description,0,200),
					'State'=>$item->study->isdisplayed
				};
				
				push @results, {
					id=>"phenotypemethod/".$item->identifier,
			    	title=>$item->name." (".$item->identifier.")",
			    	summary=>$summary,
			    	link=>"phenotypemethod/".$item->identifier,
			    	content=>$body
				};
				$counter++;
			}
		}
		#$self->log->info("results:".Dumper(\@results));
		return \@results;
    }
}
1;

=head1 SEE ALSO

L<HGVbaseG2P::FileParser>, L<HGVbaseG2P::Database> and related classes

=head1 BUGS AND LIMITATIONS

There are no known bugs in this module.
Please report problems to Rob <rcfree@gmail.com>

=head1 CONTACT

This module is part of the HGVbase-G2P project

=head1 AUTHOR

Rob Free <rcfree@gmail.com>

=head1 LICENCE AND COPYRIGHT

Copyright (c) <2009> University of Leicester, Leicester, UK. All rights reserved.

This module is free software; you can redistribute it and/or
modify it under the same terms as Perl itself.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 

=head1 CVSINFO

$Id: phenotypes.pm 1272 2009-08-21 15:08:35Z rcf8 $ 

=cut

    



    