# $Id: region.pm 1272 2009-08-21 15:08:35Z rcf8 $

=head1 NAME

HGVbaseG2P::Search::Results::region - Return Results for region search


=head1 SYNOPSIS

    use HGVbaseG2P::Search;
    my $search = HGVbaseG2P::Search::Results->new({content_type=>'text/xml'});


=head1 DESCRIPTION

This class is a simple wrapper around different types of searches. An instance of the object can be set up and then several searches
performed using the same parameters.

=head1 SUBROUTINES/METHODS 

=cut

package HGVbaseG2P::Search::Results::region; {
    
use strict;
use warnings;

# Use OIO inside-out object framework for our OO hierarachy
use Object::InsideOut qw(HGVbaseG2P::Search::Results);
use English qw( -no_match_vars );
use Data::Dumper qw(Dumper);
use HGVbaseG2P::Util qw(load_config init_logger );

	my @seqfeature : Field :All(seqfeature); #contains parameters - generally loaded from CGI param
	my @browser_db : Field :All(browser_db);
	my @study_db : Field :All(study_db);
	 	
    # Initialization arguments
    my %init_args : InitArgs = (
        'dsn' => '',
        'user' => '',
        'pass' => '',
    );

    sub accepted_parameters {
    	return qw(keywords phenotype diseases threshold);
    }
    
    sub required_db {
    	return {
    		'SeqFeature'=>'seqfeature',
    		'BrowserDB'=>'browser_db',
    		'StudyDB'=>'study_db'
    	};
    }
    
    sub search {
    	my ($self)=@_;
    	
		my $keywords = $self->param('keywords');
		
		
		my ($type, $chr,$start,$stop);
	
		if ($keywords) {
			($type, $chr,$start,$stop) = $self->extract_keywords($keywords);
		}
		
		if ($type eq 'feature') {
			$self->log->info("type is feature");
			($type,$chr,$start,$stop) = $self->get_feature_region($keywords,$self->seqfeature);
			
		}
		
		my $resultsets = $self->get_resultset_data;
		my $data = $self->get_marker_data($chr,$start,$stop,$resultsets);
		
		$self->filter_results($data,$resultsets);
    }
    
    
    sub get_resultset_data {
    	my ($self, $study_db)=@_;
    	my @resultsets = $self->study_db->dbh->resultset("Resultset")->all;
		my @resultsetids = map { $_->resultsetid } @resultsets;
		my %rs_hash;
		foreach my $rs(@resultsets) {
			$rs_hash{$rs->resultsetid}=$rs;
		}
		return \%rs_hash;
    }
    
    sub get_marker_data {
    	my ($self, $chr, $start, $stop, $resultsets)=@_;
    	my $data;
    	
    	my @resultsetids = keys %{$resultsets};
    	
    	if ($start && $stop) {
	    	$data = $self->browser_db->get_marker_data_by_range_and_separate_resultsets({ 
					threshold=>$self->param('threshold'), 
					resultsetid=>\@resultsetids, 
					chr=>$chr, 
					start=>$start, 
					stop=>$stop
				});
		}
		else {
			$data = $self->browser_db->get_marker_data_by_separate_resultsets({ 
				threshold=>$self->param('threshold'), 
				resultsetid=> \@resultsetids, 
				chr=>$chr, 
				start=>$start, 
				stop=>$stop
			});	
		}
		
		return $data;
    }
    
    sub filter_results {
    	my ($self, $data,$resultsets)=@_;
    	my @results=();
    	if ($data) {
    		my $phenotype = $self->param('phenotype');
			my @diseases = $self->param('diseases');
    		my $counter=0;
			foreach my $r(keys %{$data}) {	
			    my $study_id = $data->{$r}->[0]->{Study_Identifier} || undef;
			    my $resultsetid = $data->{$r}->[0]->{ResultsetID} || undef;
			    next if !$resultsetid;
			    
				my $rs = $resultsets->{$resultsetid} ;
			    my $count = scalar(@{$data->{$r}});

		        my $phen_name = $rs->experiment->phenotypemethod->name;
		        my @dis_cat = split(";",$rs->experiment->phenotypemethod->phenotypeproperty->diseasecategory->name);
		        
				if (defined($phenotype)) {
					next if $phen_name !~ m/$phenotype/i;
				}
				
				if (defined($diseases[0])) {
					my $lc = List::Compare->new(\@dis_cat, \@diseases);
					$lc->get_intersection==0 and next;
				}
				
				my $content = {
		        	'identifier' => $rs->identifier, 
		        	'name' => $rs->name, 
		        	'study'=>$rs->experiment->studyid->identifier, 
		        	'count' => $count,
		        	'phenotype' => $phen_name,
		        	'disease' => \@dis_cat,
		        };	
				
				my $summary = {
					'Study Identifier'=>$rs->experiment->studyid->identifier,
					'Count'=>$count,
					'Server' => $self->uri,
				};
				push @results, {
					id=>$rs->experiment->studyid->identifier,
			    	title=>$rs->experiment->studyid->identifier." has ". $count . " significant markers in ".$rs->identifier,
			    	summary=>$summary,
			    	link=>$self->uri."study/".$rs->experiment->studyid->identifier,
			    	content=>$content
				};
				$counter++;
			}
		}
		return \@results;
    }
    
    
    
    
}
1;

=head1 SEE ALSO

L<HGVbaseG2P::FileParser>, L<HGVbaseG2P::Database> and related classes

=head1 BUGS AND LIMITATIONS

There are no known bugs in this module.
Please report problems to Rob <rcfree@gmail.com>

=head1 CONTACT

This module is part of the HGVbase-G2P project

=head1 AUTHOR

Rob Free <rcfree@gmail.com>

=head1 LICENCE AND COPYRIGHT

Copyright (c) <2009> University of Leicester, Leicester, UK. All rights reserved.

This module is free software; you can redistribute it and/or
modify it under the same terms as Perl itself.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 

=head1 CVSINFO

$Id: region.pm 1272 2009-08-21 15:08:35Z rcf8 $ 

=cut

    