#!perl
package HGVbaseG2P::Test::DataImport::Common;

# $Id: Common.pm 1099 2008-12-05 13:28:04Z rcf8 $

=head1 NAME

  HGVbaseG2Pmart::Test::DataImport::Common - Test summary-level generic data import methods

=head1 SYNOPSIS

	
=head1 DESCRIPTION

  This test class inherits from the XMLdb test class, thereby inheriting a setup
  method which initializes an empty HGVbase-G2P database and connects to it with
  the XMLStore middleware.


=head1 SUBROUTINES/METHODS 

=cut


use strict;
use warnings;

use Data::Dumper;
use Test::More;
use Test::Exception;
use Data::Stag qw( :all );

use base qw(HGVbaseG2P::Test::DataImport);

#It does not matter which import class/source are used here. The methods function the same
sub get_import_class {
	return "HGVbaseG2P::DataImport::Frequency::CGEMS";
}

sub get_source {
	return "CGEMS";
}
sub _01_check_retrieve_markernode :Tests {
    my $self = shift;
    my $dataimport = $self->get_dataimport();

    # Find regular, active marker
    my ($mnode1, $sflip1) = $dataimport->retrieve_current_marker_node('dbSNP', 'rs2837');
    ok defined($mnode1), "Retrieved active marker node rs2837";
    isa_ok $mnode1, "Data::Stag::StagImpl", "marker node is of the proper class";

    # Find marker which we do NOT have any record of at all
    my ($mnode2, $sflip2) = $dataimport->retrieve_current_marker_node('dbSNP', 'foobarfoo');
    ok !defined($mnode2), "Got undef back from request for nonexistent marker";
    
    # Find marker which has been merged with another marker and NOT active any more
    my ($mnode3, $sflip3) = $dataimport->retrieve_current_marker_node('dbSNP', 'rs699');
    ok defined($mnode3), "Retrieved active marker entry for non-current marker, checking if it's the expected marker";
    ok $sflip3, "Marker merger involves a strand flip, so flip flag is set correctly";
    ok $mnode3->get('Accession') eq "rs2795", "Replacement marker has correct LocalID";
    ok $mnode3->get('Identifier') eq "HGVM1984", "Replacement marker has correct MarkerID";
        
}

sub _02_check_get_marker_coord_info {
	my $self = shift;
    my $dataimport = $self->get_dataimport();
    
    #check doesn't die for non-existing marker
	my $markernode = $dataimport->retrieve_current_marker_node('dbSNP', 'rs1234567');
	my ( $db_start, $db_chr, $db_strand ) = $dataimport->get_marker_coordinate_info($markernode);
	ok (!$db_start,"get_marker_coord_info returns correct db_start for non-existing marker");
	ok (!$db_chr,"get_marker_coord_info returns correct db_chr for non-existing marker");
	ok (!$db_strand,"get_marker_coord_info returns correct db_strand for non-existing marker");
	
	 #check existing marker returns correct data
	$markernode = $dataimport->retrieve_current_marker_node('dbSNP', 'rs2837');
	( $db_start, $db_chr, $db_strand ) = $dataimport->get_marker_coordinate_info($markernode);
	ok ($db_start==165144133,"get_marker_coord_info returns correct db_start for existing marker");
	ok ($db_chr eq '1',"get_marker_coord_info returns correct db_chr for existing marker");
	ok ($db_strand==-1,"get_marker_coord_info returns correct db_strand for existing marker");
}

sub _03_check_alleles :Tests {
    my $self = shift;
    my $dataimport = $self->get_dataimport();
    
    
    #check doesn't die for non-existing marker
	my ($markernode, $strand) = $dataimport->retrieve_current_marker_node('dbSNP', 'rs1234567');
	
	
	my $is_allele_same = $dataimport->validate_allele_seq($markernode, 'A','G');
	ok($is_allele_same==0,"validate_allele_seq returns correct value for non-existing marker");
	
	#check existing marker with correct alleles
	($markernode, $strand) = $dataimport->retrieve_current_marker_node('dbSNP', 'rs2837');
	$is_allele_same = $dataimport->validate_allele_seq($markernode, 'A','G');
	ok($is_allele_same==1,"validate_allele_seq returns correct value for correct alleles");
		
	#check existing marker with incorrect alleles
	$is_allele_same = $dataimport->validate_allele_seq($markernode, 'T','C');
	ok($is_allele_same==0,"validate_allele_seq returns correct value for incorrect alleles");
	
	#check existing marker with one correct allele
	$is_allele_same = $dataimport->validate_allele_seq($markernode, 'A','C');
	ok($is_allele_same==0,"validate_allele_seq returns correct value for one correct allele");
}

sub _04_check_chromosome :Tests {
    my $self = shift;
    my $dataimport = $self->get_dataimport();
    
    #check doesn't die for non-existing marker
	my ($markernode, $strand) = $dataimport->retrieve_current_marker_node('dbSNP', 'rs1234567');
	
	my ( $db_start, $db_chr, $db_strand ) = $dataimport->get_marker_coordinate_info($markernode);
	
	my $is_chr_same = $dataimport->validate_chromosome($db_chr, '2','rs1234567');

	ok($is_chr_same==0,"validate_chromosome returns correct value for non-existing marker");
	
	#check existing marker with correct chr
	($markernode, $strand) = $dataimport->retrieve_current_marker_node('dbSNP', 'rs2837');
	
( $db_start, $db_chr, $db_strand ) = $dataimport->get_marker_coordinate_info($markernode);
	$is_chr_same = $dataimport->validate_chromosome($db_chr, '1', 'rs2837');
		
	ok($is_chr_same==1,"validate_chr returns correct value for correct chr");
		
	#check existing marker with incorrect chr
	$is_chr_same = $dataimport->validate_chromosome($db_chr, '2', 'rs2837');
	
	ok($is_chr_same==0,"validate_chr returns correct value for incorrect chr");
}

sub _05_check_marker_position :Tests {
    my $self = shift;
    my $dataimport = $self->get_dataimport();
    
    #check doesn't die for non-existing marker
	my ($markernode, $strand) = $dataimport->retrieve_current_marker_node('dbSNP', 'rs1234567');
	my ( $db_start, $db_chr, $db_strand ) = $dataimport->get_marker_coordinate_info($markernode);
	my $is_pos_same = $dataimport->validate_marker_position($db_start, '1', 'rs2837');
	ok($is_pos_same==0,"validate_marker_pos returns correct value for non-existing marker");
	
	#check existing marker with correct pos
	($markernode, $strand) = $dataimport->retrieve_current_marker_node('dbSNP', 'rs2837');
	( $db_start, $db_chr, $db_strand ) = $dataimport->get_marker_coordinate_info($markernode);
	$is_pos_same = $dataimport->validate_marker_position($db_start, 165144133, 'rs2837');
	ok($is_pos_same==1,"validate_marker_pos returns correct value for correct pos");
    
	#check existing marker with incorrect pos
	$is_pos_same = $dataimport->validate_marker_position($db_start, 2424424, 'rs2837');
	ok($is_pos_same==0,"validate_marker_pos returns correct value for incorrect pos");
}

# Check if we are picking up cases where there has been a marker merge and the new,
# current marker is in reverse strand-orientation relative to the old one.
sub _06_strandflip :Tests {
    my $self = shift;
    my $dataimport = $self->get_dataimport();

    # Fetch current markernode from db
    #my $markerid_old = "";
    #my ($mnode_current, $strandflip) = $dataimport->retrieve_current_marker_node('dbSNP', $markerid_old);

    # Try to validate a marker + associated alleles where the alleles match the old marker.
    # This should throw an error due to alleles not matching ONLY IF the strand-flip is not 
    # being taken into account.
    my $markerid_old = 'rs699';
    my @alleles = qw(C T);
    my ($validation_error, $mnode_current, $db_rsid, $a1, $a2, $strandflip) =
	$dataimport->validate_marker($markerid_old, '1', 120728604, @alleles);
    ok $validation_error == 0, "No validation error from validate_marker() call";
    ok $markerid_old ne $db_rsid, "ID of current marker is *not* the same as old one we started with";
    ok $a1 eq 'G', "Allele1 of current marker is corrent ($a1)";
    ok $a2 eq 'A', "Allele2 of current marker is corrent ($a2)";
    #ok $strandflip == 1, "Old-to-new merge+strandflip found correctly ($strandflip)";
    
 

    # Check for genomic strand doesn't die for non-existing marker
    my ($markernode, $strand) = $dataimport->retrieve_current_marker_node('dbSNP', 'rs1234567');
    my ( $db_start, $db_chr, $db_strand ) = $dataimport->get_marker_coordinate_info($markernode);
    my $is_ori_same = $dataimport->validate_orientation($db_strand, '-1', 'rs1234567');
    ok(!$is_ori_same,"validate_orientation returns correct value for non-existing marker");
    
    #check existing marker with correct pos
    ($markernode, $strand) = $dataimport->retrieve_current_marker_node('dbSNP', 'rs2837');
    ( $db_start, $db_chr, $db_strand ) = $dataimport->get_marker_coordinate_info($markernode);
    $is_ori_same = $dataimport->validate_orientation($db_strand, '-1', 'rs2837');
    ok($is_ori_same==1,"validate_orientation returns correct value for correct orientation");
    
    #check existing marker with incorrect pos
    $is_ori_same = $dataimport->validate_orientation($db_strand, '1', $db_rsid);
    ok($is_ori_same==0,"validate_orientation returns correct value for strand flip");
}

sub _07_check_reverse_complement :Tests {
    my $self = shift;
    my $dataimport = $self->get_dataimport();
    
    #check rev complement for all lower case bases
	my $rev_allele = $dataimport->revcom('a');
	ok($rev_allele eq 't',"reverse complement returns t for a");
	$rev_allele = $dataimport->revcom('t');
	ok($rev_allele eq 'a',"reverse complement returns a for t");
	$rev_allele = $dataimport->revcom('c');
	ok($rev_allele eq 'g',"reverse complement returns g for c");
	$rev_allele = $dataimport->revcom('g');
	ok($rev_allele eq 'c',"reverse complement returns c for g");
	
	#check rev complement for all upper case bases
	$rev_allele = $dataimport->revcom('A');
	ok($rev_allele eq 'T',"reverse complement returns T for A");
	$rev_allele = $dataimport->revcom('T');
	ok($rev_allele eq 'A',"reverse complement returns A for T");
	$rev_allele = $dataimport->revcom('C');
	ok($rev_allele eq 'G',"reverse complement returns C for G");
	$rev_allele = $dataimport->revcom('G');
	ok($rev_allele eq 'C',"reverse complement returns G for C");
}

sub _08_frequency_and_cluster_nodes :Tests {
    my $self = shift;
    my $dataimport = $self->get_dataimport();

    my $fc;
    $fc = $dataimport->create_freqcluster_node('apanel', 99,undef,'rs888');
      #       "Create new freq cluster";
    isa_ok $fc, 'Data::Stag::StagImpl', "freq cluster node is of the proper class";
    ok $fc->name() eq 'FrequencyCluster', "Node name is correct";
    ok $fc->get('NumberOfGenotypedSamples') == 99, "No genotyped samples count is correct";


	my $af;
    lives_ok{ $af = $dataimport->create_allelefreq_node('rs888', 'T', '0.842', 2999); } 
            "Create new allele-freq";
    isa_ok $af, 'Data::Stag::StagImpl',"allele-freq node is of the proper class"; 
    ok $af->name() eq 'AlleleFrequency', "Node name is correct";
    
    lives_ok{ $fc->addchild($af)} "Grafting al freq node onto freqcluster node";
    
    my $gtf;
    lives_ok{ $gtf = $dataimport->create_genotypefreq_node('rs888', '(C)+(T)', '0.842', 2999); } 
            "Create new gt-freq";
    isa_ok $gtf, 'Data::Stag::StagImpl',"gt-freq node is of the proper class"; 
    ok $gtf->name() eq 'GenotypeFrequency', "Node name is correct";
    ok $gtf->get('NumberSamplesWithGenotype') == 2999;
    
    lives_ok{ $fc->addchild($gtf)} "Grafting gt freq node onto freqcluster node";
}

sub _10_usedmarkerset_node :Tests {
    my $self = shift;
    my $dataimport = $self->get_dataimport();

    my $mnode = stag_nodify(
[ Marker => [  
  [ MarkerID => 'HGVM999' ],
  [ Source   => 'dbSNP'   ],
  [ LocalID  => 'rs9999'  ],
]]);
    my $usedms_node;
    lives_ok{ $usedms_node = $dataimport->create_usedmarkerset_node('Foobar',, [$mnode])} 
            "Create new Usedmarkerset node";
    isa_ok $usedms_node, 'Data::Stag::StagImpl',"Usedms is of the proper class"; 
    ok $usedms_node->name() eq 'Usedmarkerset', "Node name is correct";
    ok $usedms_node->get('Name') eq 'Foobar' , "Accession of Usedms is correct";
}

1;

=head1 SEE ALSO

=head1 BUGS AND LIMITATIONS

There are no known bugs in this module.
Please report problems to Gudmundur Thorisson <gthorisson@gmail.com>

=head1 CONTACT

This module is part of the HGVbase-G2P project

=head1 AUTHOR

Gudmundur Thorisson <gthorisson@gmail.com>
Rob Free <rcfree@gmail.com>

=head1 LICENCE AND COPYRIGHT

Copyright (c) <2007> University of Leicester, Leicester, UK. All rights reserved.

This module is free software; you can redistribute it and/or
modify it under the same terms as Perl itself.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 

=head1 CVSINFO

$Id: Common.pm 1099 2008-12-05 13:28:04Z rcf8 $

=cut


