package HGVbaseG2P::Test::DataImport::Marker;

# $Id: Marker.pm 1192 2009-06-18 11:26:43Z rcf8 $

=head1 NAME

  HGVbaseG2P::Test::DataImport::Marker - Test marker import pipeline

=head1 SYNOPSIS

	
=head1 DESCRIPTION

  This test class inherits from the XMLdb test class, thereby inheriting a setup
  method which initializes an empty HGVbase-G2P database and connects to it with
  the XMLStore middleware. 


=head1 SUBROUTINES/METHODS 

=cut


use strict;
use warnings;

use Data::Dumper;
use Test::More;
use Test::Exception;

use base qw(HGVbaseG2P::Test::DataImport);

sub get_import_class {
    return "HGVbaseG2P::DataImport::Marker::dbSNP";
}

sub get_source {
    return "dbSNP";
}


#overrides the DataImport init
sub B_dataimport_init :Test(startup => 5) {
    my $self = shift;

    # Load some test marker data et up the marker-import object
    lives_ok { $self->get_xmldb->store_file('./t/data/markers.xml'); } "Storing XML-file with markers";
    lives_ok { $self->get_xmldb->store_file('./t/data/marker_updates.xml'); } "Storing XML-file with marker updates";
    lives_ok { $self->get_xmldb->commit() } "Committing transaction";

    # Initialize the marker-import object
    my $dataimport;
    my $importclass = $self->get_import_class;
    eval "require $importclass;";
    ok !$@,"DataImport class '$importclass' loaded ok $@";
    
    lives_ok { $dataimport = $importclass->new(dsn       => $self->get_dsn(),
					       user      => $self->get_user(),
					       passw     => $self->get_password(),					       
					       conf_file => './conf/test/hgvbase.conf',
					       log4perl_conf => './conf/log4perl.conf',
					       source => $self->get_source,
					       argv   => {studyid => $self->get_data('studyid'), assayedpanels=>$self->get_data('assayedpanels')}
					       ),
	       } "Create data import object";

    isa_ok($dataimport, 'HGVbaseG2P::DataImport::Marker', 'Marker import object is of the proper class' );
    
    $self->{dataimport} = $dataimport;
    return;
}

sub _01_align_sequences :Tests {
    my $self = shift;

    # Test sequence alignment method
    my $seq1="ATGCTTTTGGGG";
    my $seq2="ATGCTTTTGGGG";
    my ($alignment, $strand);
    my $dataimport = $self->get_dataimport();
    lives_ok { ($alignment, $strand) = $dataimport->align_sequences($seq1, $seq2)
	       } "calling sequence-alignment method";

    # Test pretty-alignment stringification method
    lives_ok { print $dataimport->pretty_alignment_string($alignment) 
	       } "Print pretty-alignment string";
    return;
}

sub _02_process_markerdata :Tests {
    my $self = shift;

    # Test main processing method
    my $dataimport = $self->get_dataimport();    
    lives_ok { $dataimport->process_file('./t/data/chdbsnp.xml', 'b666') 
	       } "Main markerfile processing method";
    my %stats = $dataimport->markersProcessed();
    print Dumper(\%stats);
}


#---------------------------------------------------------------------------
# Verify the contents of each of the output files are as expected (i.e. that
# SNPs with no change end in the 'unchanged' output file, new entries in the 
# 'new' file, and so on).
#---------------------------------------------------------------------------

sub _05_check_new_markerxml :Tests {

    # Markers not found in master database and are therefore new
    my $xml_new;
    lives_ok { $xml_new       = Data::Stag->parse(-file => 'markers_dbSNP_chdbsnp_new.xml'); 
	   } "Parsing new XML-file";
    my @markers_new = $xml_new->get('Marker');
    ok @markers_new == 11, "Correct number of new markers in output XML";
    my $marker_new = shift @markers_new;
    ok defined($marker_new), "Found new marker ".$marker_new->get('LocalID');
    ok $marker_new->get('Allele') > 1, "Marker has more than one allele";
    foreach my $tnode(qw( VariationType 
			  Source 
			  LocalID 
			  TimeLastTouched )) {
	ok defined($marker_new->get($tnode)), "Got data from terminal node $tnode in Marker, as expected";
    }
    ok !$marker_new->get('FooBar'), "Did not find data in non-existent node FooBar, as expected";
    ok $marker_new->get('MarkerCoord'), "New marker has coordinates";
}

sub _06_check_unchanged_markerxml :Tests {

    ## Markers found in the master database but have no change
    my $xml_unchanged;
    lives_ok { $xml_unchanged = Data::Stag->parse(-file => 'markers_dbSNP_chdbsnp_unchanged.xml'); 
	   } "Parsing unchanged XML-file";
    my ($marker_unchanged) = $xml_unchanged->qmatch('Marker', ( LocalID => 'rs920'));
    ok defined($marker_unchanged), "Got unchanged marker node ".$marker_unchanged->get('LocalID');
}

sub _07_check_changed_markerxml :Tests {

    ## Markers found in the master database but have some changes
    my $xml_changed;
    lives_ok { $xml_changed   = Data::Stag->parse(-file => 'markers_dbSNP_chdbsnp_changed.xml'); 
	   } "Parsing changed XML-file";

    # Allele added
    my ($marker_newallele) = $xml_changed->qmatch('Marker', ( LocalID => 'rs8936')); 
    ok defined($marker_newallele), "Got new allele marker node ".$marker_newallele->get('LocalID');
    ok $marker_newallele->get('MarkerRevision/ChangeType') eq 'AllelesAdded', "Correct changetype in mrevision";
    ok $marker_newallele->qmatch('Allele', (AlleleSeq => 'A')), "Correct allele to be added";
    
    # Allele deleted
    my ($marker_deletedallele) = $xml_changed->qmatch('Marker', ( LocalID => 'rs2310700'));
    ok defined($marker_deletedallele), "Got deleted allele marker node ".$marker_deletedallele->get('LocalID');
    ok $marker_deletedallele->get('MarkerRevision/ChangeType') eq 'AllelesDeleted', "Correct changetype in mrevision";
    ok $marker_deletedallele->qmatch('Allele',(AlleleSeq => 'G')), "Correct allele to be deleted";

    # Flanks have been flipped but otherwise no badness
    my ($marker_flankstrandflip) = $xml_changed->qmatch('Marker', ( LocalID => 'rs12187'));
    ok defined($marker_flankstrandflip), "Got flipped-flanks marker node ".$marker_flankstrandflip->get('LocalID');
    ok $marker_flankstrandflip->get('MarkerRevision/ChangeType') eq 'StrandFlipped', "Correct changetype in mrevision";
    # The original alleles need to be flagged as dead
    my ($allele2delete1) = $marker_flankstrandflip->qmatch('Allele', ('AlleleSeq' => 'C'));
    my ($allele2delete2) = $marker_flankstrandflip->qmatch('Allele', ('AlleleSeq' => 'T'));
    ok defined($allele2delete1), "Allele C to delete is defined";
    ok defined($allele2delete2), "Allele T to delete is defined";    
    ok $allele2delete1->get('Status') eq 'dead', "Allele C is flagged as dead";
    ok $allele2delete2->get('Status') eq 'dead', "Allele T is flagged as dead";
    # And the new alleles added
    my ($allele2add1) = $marker_flankstrandflip->qmatch('Allele', ('AlleleSeq' => 'A'));
    my ($allele2add2) = $marker_flankstrandflip->qmatch('Allele', ('AlleleSeq' => 'G'));
    ok defined($allele2add1), "Allele A to delete is defined";
    ok defined($allele2add2), "Allele G to delete is defined";
    ok $allele2add1->get('@')->get('op') eq 'insert', "Allele A has proper insert-op attribute ";
    ok $allele2add2->get('@')->get('op') eq 'insert', "Allele G has proper insert-op attribute ";
    # Flanks are strand-flipped and alleles correctly flipped as well
    my ($marker_strand_and_alleleflip) = $xml_changed->qmatch('Marker', ( LocalID => 'rs4189'));
    'rs4189='.$marker_strand_and_alleleflip->itext;
    ok defined($marker_strand_and_alleleflip), "Got flank+allele flip marker node ";
    ok $marker_strand_and_alleleflip->get('MarkerRevision/ChangeType') eq 'StrandFlipped', "Correct changetype in mrevision";
    # Check that the old allele is flagged as dead
    my ($a2flag) = $marker_strand_and_alleleflip->qmatch('Allele',(AlleleSeq => 'AAGT'));
    ok defined($a2flag), "AAGT to kill is present";
    ok $a2flag->get('Status') eq 'dead', "AAGT allele to kill is flagged as dead";
    # Check that the new allele is added
    my ($a2add) = $marker_strand_and_alleleflip->qmatch('Allele',(AlleleSeq => 'ACTT'));
    ok defined($a2add), "ACTT allele to add is present";
    ok $a2add->get('@')->get('op') eq 'insert', "ACTT allele to add has op=insert ";
    # Check that the existing allele is NOT flagged as dead AND added (current bug being tested for)
    ok !($marker_strand_and_alleleflip->qmatch('Allele',(AlleleSeq => '_'))), "'_' allele NOT present, as expected";
    
    # ToDo Check that rs9554 passed checks, with one all-Ns flank but the other flank good
}

sub _08_check_manualcheck_markerxml :Tests {
    
    ## Markers found in the master database that have non-fixable changes or errors
    my $xml_manualcheck;
    lives_ok { $xml_manualcheck    = Data::Stag->parse(-file => 'markers_dbSNP_chdbsnp_manualCheck.xml'); 
	   } "Parsing bad-entries XML-file";

    # Bad flanking sequences => error
    my ($marker_badflanks) = $xml_manualcheck->qmatch('Marker', ( LocalID => 'rs8896'));
    ok defined($marker_badflanks), "Marker with bad flanks present in the manual-curation file";
    my ($mrev) = $marker_badflanks->qmatch('MarkerRevision', (ChangeType => 'BadFlanks'));
    ok defined($mrev), "Marker with bad flanks has a marker revision for bad flanks";
    warn 'mrevnode='.$mrev->itext;
    ok $mrev->get('ChangeType') eq 'BadFlanks', "Marker with bad flanks has proper marker revision changetype";
    ok $mrev->get('Comment') =~ /Score/, "Marker with bad flanks has strand alignment info in the Comment field";
    
    # Strand-flipped flanks but alleles have not been flipped to match => error
    my ($marker_flanksflipped_stranderror) = $xml_manualcheck->qmatch('Marker', ( LocalID => 'rs121888'));
    ok defined($marker_flanksflipped_stranderror), "Marker with flank-flip + allele strand error present in the manual-curation file";
    warn 'node='.$marker_flanksflipped_stranderror->itext;
    my ($mrev2) = $marker_flanksflipped_stranderror->qmatch('MarkerRevision', (ChangeType => 'IncorrectAlleles'));
    ok defined($mrev), "Marker with flank-flip + allele strand error has a marker revision for bad flanks";
    warn 'mrevnode='.$mrev->itext;
    ok $mrev2->get('ChangeType') eq 'IncorrectAlleles', "Marker with flank-flip + allele strand error has the proper marker revision changetype";
    
    # [add more scenarios here!]

}

sub _09_load_markerxml :Tests {
    my $self = shift;
    
    # Load the XML-files we've got so far  into database, so legacy routine can be run
    my $xmldb = $self->get_xmldb();
    foreach my $f(qw(markers_dbSNP_chdbsnp_new.xml
                     markers_dbSNP_chdbsnp_changed.xml
                     markers_dbSNP_chdbsnp_unchanged.xml
                     markers_dbSNP_chdbsnp_withheld.xml)) {
        lives_ok { $xmldb->storeFile($f); } "Storing XML-file $f";
    }
    lives_ok { $xmldb->commit() } "Committing transaction";
    `mysqladmin flush-threads`;
    `mysqladmin flush-tables`;
    sleep 3;
}

sub _10_process_legacy_markers :Tests {
    my $self = shift;
    
    # Run the legacy-marker routine 
    my $dataimport = $self->get_dataimport();
    my $timestamp = $dataimport->timecreated();
    $timestamp =~ s/\s.+//; 
    lives_ok { $dataimport->update_legacy_markers($timestamp,'b666'); 
       } "call legacy-marker method with datestamp '$timestamp'";    
}


sub _11_check_legacy_markerxml :Tests {
    my $self = shift;

    # Check that the legacy-routine is picking up the right set of markers that are in the old 
    # database, but not in the new build, and should therefore flagged as 'dead'.
    my $xml_deleted;
    lives_ok { $xml_deleted    = Data::Stag->parse(-file => 'markers_dbSNP_deleted.xml'); 
       } "Parsing deleted XML-file";
    my @markers_deleted = $xml_deleted->get('Marker');
    ok @markers_deleted == 4 , "Correct number of deleted, legacy markers";
    my ($marker_merged) = $xml_deleted->qmatch('Marker', ( LocalID => 'rs998877'));
    my $mrev = $marker_merged->get('MarkerRevision');
    ok $mrev->get('ChangeType') eq 'Deletion', "Marker revision Change type is correct";
    #my $replaced_by_marker = $mrev->get('ReplacedByMarker/Marker');
    #ok $replaced_by_marker->get('LocalID') eq 'rs699', "Replaced marker local ID is correct";
}

sub _12_load_markerxml_deleted :Tests {
    my $self = shift;
    
    # Load the XML-file from the legacy-check routine
    my $xmldb = $self->get_xmldb();
    #lives_ok { $xmldb->storeFile('markers_dbSNP_deleted.xml'); } "Storing XML-file with deleted markers";
    #lives_ok { $xmldb->commit() } "Committing transaction";
}

1;

=head1 SEE ALSO

=head1 BUGS AND LIMITATIONS

There are no known bugs in this module.
Please report problems to Gudmundur Thorisson <gthorisson@gmail.com>

=head1 CONTACT

This module is part of the HGVbase-G2P project

=head1 AUTHOR

Gudmundur Thorisson <gthorisson@gmail.com>

=head1 LICENCE AND COPYRIGHT

Copyright (c) <2007> University of Leicester, Leicester, UK. All rights reserved.

This module is free software; you can redistribute it and/or
modify it under the same terms as Perl itself.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 

=head1 CVSINFO

$Id: Marker.pm 1192 2009-06-18 11:26:43Z rcf8 $

=cut


