package HGVbaseG2P::Test::Schema;

# $Id: Schema.pm 1099 2008-12-05 13:28:04Z rcf8 $

=head1 NAME

HGVbaseG2Pmart::Test::ORM - Test object-relational model


=head1 SYNOPSIS

    
=head1 DESCRIPTION

This class inherits from Test::Class and adds features for testing the main relational
database central to the HGVbase-G2P system. The main feature is the startup-method 
which is a common procedure for each of the derived classes that inherit this functionality.

=head1 SUBROUTINES/METHODS 

=cut


use strict;
use warnings;

use Data::Dumper;
use Test::More;
use Test::Exception;
use Env qw($DBI_DSN $DBI_USER $DBI_PASS $DBSTAG_TEMPLATE_DIRS);
use HGVbaseG2P::Util qw(load_config);
use base qw(Test::Class);

BEGIN { use_ok 'HGVbaseG2P::Schema'; }


# Initialize database with schema definition, dropping the test-db if it already exists, and
# connecting to it with the XML middleware.
sub A_db_init :Test(startup => 8) {
    my $self = shift;
    
    my %config=(); 
    
    eval {%config = load_config('./conf/test/hgvbase.conf');};
    die "Unable to load config file:".$@ if $@;
    
    my $dbuser   = $config{dbuser}  || $DBI_USER  || getlogin();
    my $dbpassw  = $config{dbpassw} || $DBI_PASS;

    # Create DSN string if none is provided in test configfile or environment variable
    my $dsn = $config{dsn} || $DBI_DSN;
    my $dbname;
    if(!$dsn) {
        $dbname = 'test999_'.($DBI_USER||getlogin);
        $dsn    = "dbi:mysql:$dbname";
    }
    else {
        ($dbname) = $dsn =~ /^dbi:\w+:([\w=]+)/;
        $dbname =~ s/dbname=//; # handle future DBD::Pg DSN-strings as well
    }
    my ($dbhost) = $dsn =~ /(\w+\.\w+[\.\w]+\.\w+)/;
    my $credentials = " -u ".$dbuser. ($dbhost ? " -h $dbhost -p$dbpassw" : q{});
    
    # Create test database
    my $ddl_file = $config{schema_file};
    ok -f $ddl_file, "provided schema file $ddl_file exists";
    ok system("mysql $credentials -e 'drop database if exists $dbname'") == 0, "Dropping test database '$dbname' if it exists";
    ok system("mysqladmin $credentials create $dbname") == 0, "Creating test database '$dbname'";
    ok system("mysql $credentials $dbname < $ddl_file") == 0, "Loading schema ($ddl_file) into test database '$dbname'";
    $self->{dsn}         = $dsn;
    $self->{user}        = $dbuser;
    $self->{password}    = $dbpassw;
    $self->{config}      = \%config;
    $self->{config_file} ='./conf/test/hgvbase.conf';
    
    # Connect to database via the ORM middleware
    my $db;
    lives_ok {$db = HGVbaseG2P::Schema->connect($dsn)}
              "Connect to database and get schema object";

    ok defined($db), "Database handle object is defined";
    $self->{db} = $db;

    # Connect to database with our DBStag-based middleware
    # NB!!! this is temporary only: until we have a different way of loading data
    # into the database, we have to do it via the XML middleware
    my $xmldb;
    use HGVbaseG2P::XMLStore;
    lives_ok { $xmldb = HGVbaseG2P::XMLStore->new(dsn       => $dsn,
                          user      => $dbuser,
                          pass     => $dbpassw,
                          conf_file => './conf/test/hgvbase.conf',
                          log4perl_conf => $config{log4perl_conf} || './conf/log4perl.conf',
                          stagmap       => $config{dbstag_meta})
           } "Connected to test database '$dbname' as user '$dbuser'";
    ok defined($xmldb), "XML-db object is defined";
    $self->{xmldb} = $xmldb;
    
     # Set environmental variable to point DBStag to the SQL-templates
    $DBSTAG_TEMPLATE_DIRS = $config{sqltemplates} || './sqltemplates';
}

sub get_xmldb {
    my $self = shift;
    return $self->{xmldb};
}

sub db_cleanup :Test(shutdown => 1) {
    my $self = shift;
    lives_ok{ $self->get_db->storage->disconnect() }
              "Disconnecting from database";
}

sub get_db {
    my $self = shift;
    return $self->{db};
}

sub get_dsn {
    my $self = shift;
    return $self->{dsn};
}

sub get_user {
    my $self = shift;
    return $self->{user};
}

sub get_password {
    my $self = shift;
    return $self->{password};
}

sub get_cfg {
    my ($self, $key) = @_;
    return $self->{config}->{$key};
}

1;

=head1 SEE ALSO

=head1 BUGS AND LIMITATIONS

There are no known bugs in this module.
Please report problems to Gudmundur Thorisson <gthorisson@gmail.com>

=head1 CONTACT

This module is part of the HGVbase-G2P project

=head1 AUTHOR

Gudmundur Thorisson <gthorisson@gmail.com>

=head1 LICENCE AND COPYRIGHT

Copyright (c) <2007> University of Leicester, Leicester, UK. All rights reserved.

This module is free software; you can redistribute it and/or
modify it under the same terms as Perl itself.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 

=head1 CVSINFO

$Id: Schema.pm 1099 2008-12-05 13:28:04Z rcf8 $

=cut


