package HGVbaseG2P::Test::SchemaBasicOps;

# $Id: SchemaBasicOps.pm 1153 2009-03-31 14:17:21Z mummi $

=head1 NAME

  HGVbaseG2Pmart::Test::ORMBasicOps - Test object-relational model operations

    
=head1 DESCRIPTION

  The purpose of this test class is merely to check a handful of key object classes
  to ensure that the ORM-layer works in general for retrieving data from and writing data
  to the database. The intention is *not* to exhaustively test every single table class in our schema.

  More advanced examples of how we use the ORM-layer can be found in the various controller
  web-app controller classes etc. These are tested by higher-level test scripts for individual
  web application components and other parts of the system.


=head1 SUBROUTINES/METHODS 

=cut


use strict;
use warnings;

use Data::Dumper;
use Test::More;
use Test::Exception;

use base qw(HGVbaseG2P::Test::Schema);


sub _1_table_structure :Tests {
    my $self = shift;

    # get OO representation of a db-table and inspect it
    my $t = HGVbaseG2P::Schema::Resultset->new();
    ok defined($t), "Table object is defined";
    my @cols = $t->columns();
    ok scalar(@cols) == 11, "Table has proper no. columns";
    my $colinfo = $t->column_info('resultsetid');
    ok $colinfo->{data_type} eq 'INT', "Datatype for this column is correct";
}

sub _2_load_data :Tests {
    my $self = shift;
    my $db = $self->get_db();

    # Storing some XML-file in database
    # NB!!! this is temporary only: until we have a different way of loading data
    # into the database, we have to do it via the XML middleware
    my $xmldb = $self->get_xmldb();
    lives_ok { $xmldb->store_file('./t/data/markers.xml'); } "Storing marker XML-file";
    lives_ok { $xmldb->store_file('./t/data/marker_updates.xml'); } "Storing XML-file with marker revisions";
    lives_ok { $xmldb->store_file('./t/data/study.xml') } "Storing XML-file with basic study-info";
    lives_ok { $xmldb->store_file('./t/data/study_significances.xml') } "Storing XML-file with significance data";
    lives_ok { $xmldb->commit() } "Committing transaction";
}
    

sub _3_retrieve_markers :Tests {
    my $self = shift;
    my $db = $self->get_db();

    # Markers and alleles 
    my $marker;
    my $markerid = 'HGVM510';
    lives_ok {$marker = $db->resultset('Marker')->search('identifier' => $markerid)->single} 
              "Retrieve marker object from database";
    ok defined($marker), "Retrieved marker object is defined";
    ok $marker->identifier eq $markerid, "Marker ID is correct";
    foreach my $method(qw/identifier accession/) {
        can_ok $marker, $method;
    }
    ok $marker->variationtype eq 'SNP', "Marker variation type is correct";
    my @alleles = $marker->alleles();
    ok @alleles == 2, "Marker has correct number of alleles";
    ok defined($marker->upstream30bp),  "Marker has 5' flanking seq: ".$marker->upstream30bp;
    ok defined($alleles[0]->alleleseq),     "Allele 1 has allele seq: ".$alleles[0]->alleleseq;
    ok defined($alleles[1]->alleleseq),     "Allele 2 has allele seq: ".$alleles[1]->alleleseq;
    ok defined($marker->downstream30bp),"Marker has 3' flanking seq: ".$marker->downstream30bp;
    
    # ToDo add something for coords + revisions


    # Fetch Usedmarkerset  entry associated with this marker
    my $usedmset_rs;
    lives_ok {$usedmset_rs = $db->resultset('Usedmarkerset')->search('markeridentifier' => $markerid)}
              "Retrieve markersets with marker ID $markerid";
    
    foreach my $usedmset($usedmset_rs->all()) {
	isa_ok $usedmset, 'HGVbaseG2P::Schema::Usedmarkerset', "Usedmarkerset object is of the proper class";
    }
}

sub _4_retrieve_study :Tests {
    my $self = shift;
    my $db = $self->get_db();
    
    # Fetch particular study to database
    my $study;
    lives_ok {$study = $db->resultset('Study')->search('identifier' => 'HGVSTxxxx')->single} 
              "Retrieve study object from database";

    ok defined($study), "Parent study for this experiment is defined: ".$study->identifier;    
    foreach my $method(qw/identifier name label description accession/) {
        can_ok $study, $method;
    }
    ok defined($study->identifier), "Study identifier is defined: ".$study->identifier;
    ok defined($study->name), "Study name is defined: ".$study->name;

    # Check basic study meta-data, such as submitter, authors and other kinds of contributions
    foreach my $subm($study->submissions()) {
	print "Submission '".$subm->name."', created ".$subm->timecreated."\n";
	foreach my $con($subm->contributions()) {
	    print "   Author: ".$con->researcher->shortname.", author=".($con->isauthor||'no').", submitter=".($con->issubmitter||'no')."\n";
	}
    }
    ## check that we have 2x submissions for this study
    ## check proper no. contributors in one of those
    ## check that for a certain researcher the roles (author, submitter, etc.) are correct


    # Test many-to-many relationships with samplepanel components
    my @used_spanels = $study->usedsamplepanels();
    ok @used_spanels == 4, "Proper number of samplepanels in this study";
    foreach my $sp(@used_spanels) {
        ok $sp, "Samplepanel object defined: ".$sp->name;
	foreach my $method(qw/identifier name description accession/) {
	    can_ok $sp, $method;
	}
	# Check whether study is the one spanel was submitted in or not. Should hold for one particular panel
	if($study->identifier ne $sp->submittedinstudy->identifier) {
	    ok $sp->identifier eq 'HGVSP1', "For spanel HGVSP1, study used-in and study submitted-in are indeed NOT the same";
	}	
    }

    # Test many-to-many & one-to-many relationship with samplepanel
    my $spanel;
    lives_ok {$spanel = $db->resultset('Samplepanel')->search('identifier' => 'HGVSP1')->single} 
              "Retrieve samplepanel object from database";
    my $study_submittedin = $spanel->submittedinstudy();
    ok $study_submittedin->identifier eq 'HGVSTzzzz', "Study that spanel was *submitted in* is correct: ".$study_submittedin->identifier;
    my ($study_usedin) = $spanel->usedinstudies();
    ok $study_usedin->identifier eq 'HGVSTxxxx', "Study that spanel was *used in* is correct: ".$study_usedin->identifier;


    # Test same sort of many-to-many & one-to-many relationships with analysismethod
    my @used_amethds = $study->usedanalysismethods();
    ok @used_amethds == 2, "Proper number of analysismethods in this study";
    foreach my $am(@used_amethds) {
        ok $am, "Analysismethod object defined: ".$am->name;
	foreach my $method(qw/identifier name description accession/) {
	    can_ok $am, $method;
	}
	if($study->identifier ne $am->submittedinstudy->identifier) {
	    ok $am->identifier eq 'HGVAM1248', "For anlmethod HGVAM1248, study used-in and study submitted-in are indeed NOT the same";
	}	
    }

    my $amethd;
    lives_ok {$amethd = $db->resultset('Analysismethod')->search('identifier' => 'HGVAM1248')->single} 
              "Retrieve analysismethod object from database";
    my $study_submittedin2 = $amethd->submittedinstudy();
    ok $study_submittedin2->identifier eq 'HGVSTzzzz', "Study that anlmethod was *submitted in* is correct: ".$study_submittedin2->identifier;
    my ($study_usedin2) = $amethd->usedinstudies();
    ok $study_usedin2->identifier eq 'HGVSTxxxx', "Study that anlmethod was *used in* is correct: ".$study_usedin2->identifier;

    
    # Check simpler, straight-up one-to-many relationships with some other study components
    my @phenmethods = $study->phenotypemethods();
    ok @phenmethods == 3, "Proper number of phenotype methods in study";
    foreach my $pm(@phenmethods) {
        ok $pm, "Phenotypemethod object defined: ".$pm->name;
	foreach my $method(qw/identifier name description/) {
	    can_ok $pm, $method;
	}
    }

    ## ToDo check assayedpanels also
    my @apanels = $study->assayedpanels();
    ok @apanels == 3, "Proper number of assayedpanels in this study";
    foreach my $ap(@apanels) {
        ok $ap, "Assayedpanel object defined: ".$ap->name;
	foreach my $method(qw/identifier name description/) {
	    can_ok $ap, $method;
	}
    }
}

sub _5_retrieve_experiment :Tests {
    my $self = shift;
    my $db = $self->get_db();

    my $exp;
    lives_ok {$exp = $db->resultset('Experiment')->search('identifier' => 'HGVEyyyyy')->single} 
              "Retrieve experiment object from database";
    ok $exp->identifier eq 'HGVEyyyyy', "Experiment ID is correct";
    foreach my $method(qw/identifier name label description accession/) {
        can_ok $exp, $method;
    }

    # Check assayedpanels investigated in this experiment
    my @apanels2 = $exp->usedassayedpanels();
    ok @apanels2 == 3, "Proper number of assayedpanels associated with this experiment";
    isa_ok $apanels2[0], 'HGVbaseG2P::Schema::Assayedpanel', "Assayedpanel object is the right class";

    # Check that summary-level genotype data contained in experiment are OK
    my @gtrsets = $exp->genotypingdetails(); # exp->genotypingresultsets() ???
    my $gtrset = shift @gtrsets;
    print "Got some genotyping results created ".$gtrset->timecreated." done on gt-platform ".$gtrset->genotypingplatform->name."\n";
    my @fclusters = $gtrset->frequencyclusters();
    ok @fclusters == 3, "Proper number of freq clusters found";
    my $fc = shift @fclusters;
    can_ok $fc, 'numberofgenotypedsamples';
    can_ok $fc, 'assayedpanel';

    # Get a resultset from this experiment and check that the G2P results within are OK
    my @rsets = $exp->resultsets(); # exp->statisticalresultsets() ???
    my $rset = shift @rsets;
    ok defined($rset), "Resultset from this experiment is defined";
    foreach my $method(qw/identifier name description accession/) {
	can_ok $rset, $method;
    }
    my @sigs = $rset->significances();
    my $sig = shift(@sigs);
    ok $sig->unadjustedpvalue, "Significance has an unadjusted p-value: ".$sig->unadjustedpvalue;
    my $mset = $sig->usedmarkerset;
    my $marker;
    lives_ok {$marker = $db->resultset('Marker')->search('identifier' => $mset->markeridentifier)->single} 
              "Retrieve marker object from database based on ID in Usedmarkerset";
    ok defined($marker), "Significance linked up with marker properly";
    ok $marker->identifier, "Marker has proper IDs etc.: ".$marker->identifier.'('.$marker->datasource->name.':'.$marker->accession.')';
    my @sig_fcs = $sig->frequencyclusters();
    ok @sig_fcs == 3 ,"Got the right number of freq clusters for this significance";
    foreach my $fc(@sig_fcs) {
        # Check genotype freqs
        foreach my $gf($fc->genotypefrequencies()) {
            print "   ".$gf->genotypecombo." => ".$gf->frequencyasproportion."\n";
        }
        # Check allele freqs
        foreach my $af($fc->allelefrequencies()) {
            print "   ".$af->allelecombo." => ".$af->frequencyasproportion."\n";
        }
    }

    # Coming from a slightly different direction: for the markers tested in an experiment,
    # get the freq and/or assoc data associated with them.
    foreach my $mset($exp->usedmarkersets) {
	print "Getting available data for mset named ".$mset->name."\n";
	print " significances:\n";
	foreach my $sig($mset->significances) {
	    print "   ".$sig->resultset->name.": p = ".$sig->unadjustedpvalue.")\n";
	}
	print " freqs:\n";
	foreach my $fc($mset->frequencyclusters) {
	    print "  ".$fc->genotypingdetails->genotypingplatform->name.":\n";
	    foreach my $gf($fc->genotypefrequencies()) {
		print "    ".$gf->genotypecombo." => ".$gf->frequencyasproportion."\n";
	    }
	    foreach my $af($fc->allelefrequencies()) {
		print "    ".$af->allelecombo." => ".$af->frequencyasproportion."\n";
	    }
	}
    }
}

sub _6_write_objects :Tests {
    my $self = shift;
    my $db = $self->get_db();
    
    # Create new Study object and commit transaction

    # Attempt to retrieve the newly-created study which should work

    # Delete the study 

    # Attempt to retrieve the study which should, well, not work! 


}

1;

=head1 SEE ALSO

=head1 BUGS AND LIMITATIONS

There are no known bugs in this module.
Please report problems to Gudmundur Thorisson <gthorisson@gmail.com>

=head1 CONTACT

This module is part of the HGVbase-G2P project

=head1 AUTHOR

Gudmundur Thorisson <gthorisson@gmail.com>

=head1 LICENCE AND COPYRIGHT

Copyright (c) <2007> University of Leicester, Leicester, UK. All rights reserved.

This module is free software; you can redistribute it and/or
modify it under the same terms as Perl itself.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 

=head1 CVSINFO

$Id: SchemaBasicOps.pm 1153 2009-03-31 14:17:21Z mummi $

=cut


