package HGVbaseG2P::Test::Web::JSON;

# $Id: JSON.pm 1050 2008-10-30 16:45:55Z mummi $

=head1 NAME

  HGVbaseG2Pmart::Test::Web::Pages - Test complete website pages

    
=head1 DESCRIPTION

  This class tests various components of the Catalyst web application from the 
  user interface perspective, i.e. by requesting various pages and checking for
  hyperlinks and other page content ('screen-scraping', essentially).

=head1 SUBROUTINES/METHODS 

=cut


use strict;
use warnings;
use base qw(Test::Class);

use Data::Dumper;
use Test::More;
use Test::Exception;
use Test::WWW::Mechanize::Catalyst 'HGVbaseG2P::Web';
use JSON::XS;

# Set up JSON coder so we can decode the JSON retrieved below
my $json_coder = JSON::XS->new->ascii->pretty->allow_nonref;
$json_coder->space_after(1);



# Initialize user agent which connects to an instance of the web application
sub A_webapp_init :Test(startup => 1) {
    my $self = shift;
    lives_ok{ $self->{ua} = Test::WWW::Mechanize::Catalyst->new} "Initialize user agent";    
}

sub _1_study_list :Tests {
    my $self = shift;
    my $ua = $self->get_ua();

    $ua->get_ok("/study/list", "Get list of available studies as JSON");
    my $json_ref = $self->is_json_format($ua->content());
    #print 'json_ref=',Dumper($json_ref);

    # Proceed to test whether the stuff we expect is in the hash
    
}

sub _2_resultset_list :Tests {
    my $self = shift;
    my $ua = $self->get_ua();

    my $studyid = 'HGVSTxxxx';
    $ua->get_ok("/resultset/list/$studyid", "Get list of resultsets as JSON from study $studyid"); 
    my $json_ref = $self->is_json_format($ua->content());
    #print 'json_ref=',Dumper($json_ref);    

    # Proceed to test whether the stuff we expect is in the hash


}

sub _3_significance_list :Tests {
    my $self = shift;
    my $ua = $self->get_ua();
    
    my $marker_id = 'HGVM510';
    $ua->get_ok("/significance/list/$marker_id", "Get list of significances for marker $marker_id");
    my $json_ref = $self->is_json_format($ua->content());
    #print 'json_ref=',Dumper($json_ref);

    # Proceed to test whether the stuff we expect is in the hash

}


sub _4_study_page :Tests {
    my $self = shift;

}

sub get_ua {
    my $self = shift;
    return $self->{ua};
}

# common methods used by page/tab tests

=head1 SUBROUTINES/METHODS 

=cut

=head2 is_json_format

  Usage      : $ref = is_json_format('/file.json');
  Purpose    : Determines if a URI returns data in JSON format
  Returns    : reference decoded data structure if JSON,  0 for fals if it is not
  Arguments  : JSON-formatted text
  Throws     : 
  Status     : 
  Comments   : 

=cut


sub is_json_format {
    my ($self, $data) = @_;
    my $json_ref;
    lives_ok { $json_ref = $json_coder->decode($data); }
             "JSON data decoded OK and reference to data structure returned";
    ok ref($json_ref) eq 'HASH', "Decoded JSON data is actually a hash";
    return $json_ref;
}

=head2 test_page

  Usage      : test_page($url, $title,$content,$description);
  Purpose    : Tests that a given URL exists and that it matches specific features.
  Returns    : Nothing
  Arguments  : URL to try to get
               Title to compare URL's title with
               Content to compare with URL's content
               Description of the page
  Throws     : 
  Status     : 
  Comments   : Also used to set the current page to the given URL

=cut
	
sub test_page {
    my ($self, $url, $title, $content) = @_;
    my $ua = $self->get_ua();
    $ua->get_ok($url, "Check can get $url");
    is($ua->ct, "text/html", "Check ontent type");
    $title and $ua->title_is($title);
    $content and $ua->content_contains($content);
}

=head2 test_common

  Usage      : &test_common;
  Purpose    : Tests common features of each page in the web-site including
               defined links and the search form which are present in the header.
  Returns    : Nothing
  Arguments  : None
  Throws     : 
  Status     : 
  Comments   : Uses current page selected through WWW::Mechanize

=cut

sub test_common {
    my ($self) = @_;
    $self->test_defined_links({
	'Common "Data" Link'=>'/data',
	'Common "About" Link'=>'/about',
	'Common "Help" Link'=>'/help',
	'Common "Contact" Link'=>'/contact',
			      });
    $self->test_form("searchform","search","submit");
}

=head2 test_full_search

  Usage      : test_full_search;
  Purpose    : Tests the searchform search with all different types and results sizes
  Returns    : Nothing
  Arguments  : None
  Throws     : 
  Status     : 
  Comments   : Uses current page selected through WWW::Mechanize

=cut
sub test_full_search {
    my ($self) = @_;
    my $ua = $self->get_ua;

    $ua->form_name("searchform");
	
    my @types=("Any","panels","Studies","Markers","Phenotypes");
    my @resultsizes= qw(5 10 20 50 100);
    
    foreach my $resultsize(@resultsizes) {
	foreach my $type(@types) {
	    my $response = $ua->submit_form(form_name => 'searchform',			
					    fields => {
						itemsperpage => $resultsize,
						db    => $type
					    }
		);
	    ok($response->is_success,"Search form submitted ok (db=$type, size=$resultsize)");
	}
    }
}

=head2 test_form

  Usage      : test_form($formname,$expected_action,[$submit_name]);
  Purpose    : Tests features of a specific form.
  Returns    : Nothing
  Arguments  : Form name attribute in HTML page
               Expected action (without '/')
               Submit button name [optional]
  Throws     : 
  Status     : 
  Comments   : Uses current page selected through WWW::Mechanize

=cut

sub test_form {
	my ($self, $formname,$expected_action,$submit_name) = @_;
	my $ua = $self->get_ua;
	
	my @action = ();
	my $found = 0;
	$ua->form_name($formname);
	
	#check actions are the same
	@action = split('/',$ua->current_form->action);
	ok($action[$#action] eq $expected_action,"Search form action is correct ($formname)"); 
	
	#check submit is successful
	my $response;
	if ($submit_name) {
		$response = $ua->click_button(name=>$submit_name);
	}
	else {
		$response = $ua->submit;
	}
	ok($response->is_success,"Blank search form submitted ok ($formname)");
}

=head2 test_defined_links

  Usage      : test_defined_links($links);
  Purpose    : Tests that a list of defined links to URLs are present
  Returns    : Nothing
  Arguments  : A hashref containing defined links.
                 The values are descriptive
                 The keys are the URLs to test.
  Throws     : 
  Status     : 
  Comments   : Uses current page selected through WWW::Mechanize

=cut

sub test_defined_links {
    my ($self, $links) = @_;
    my $ua = $self->get_ua;

    # loop through each link, look for it in the page and if present check page exists
    foreach my $key(keys %{$links}) {
	my $value = $links->{$key};
	my $link = $ua->find_link(url=> $value);
	ok($link,"Defined link found in page ".$ua->uri . " ($key)");
      SKIP: {
	  skip "Defined link not found",1 if !$link;
	  $ua->links_ok($link->url,"Defined link returns correct status ($key):".$link->url);
	}
    }
}


1;

=head1 SEE ALSO

=head1 BUGS AND LIMITATIONS

There are no known bugs in this module.
Please report problems to Gudmundur Thorisson <gthorisson@gmail.com>

=head1 CONTACT

This module is part of the HGVbase-G2P project

=head1 AUTHOR

Gudmundur Thorisson <gthorisson@gmail.com>

=head1 LICENCE AND COPYRIGHT

Copyright (c) <2007> University of Leicester, Leicester, UK. All rights reserved.

This module is free software; you can redistribute it and/or
modify it under the same terms as Perl itself.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 

=head1 CVSINFO

$Id: JSON.pm 1050 2008-10-30 16:45:55Z mummi $

=cut


