# $Id: Util.pm 1272 2009-08-21 15:08:35Z rcf8 $

=head1 NAME

HGVbaseG2P::Util - Collection of utility modules for the HGVbase-G2P system


=head1 SYNOPSIS

    use HGVbaseG2P::Util qw(open_infile);

    # Get filehandle on regular or compressed files
    my $fh = open_infile('/tmp/foo/bar.txt');
    my $fh = open_infile('/tmp/foo/bar.txt.gz');
    
    # Load configuration file into hash
    my %config = load_config('./conf/hgvbase.conf');
    

=head1 DESCRIPTION

This module contains several utility modules that are used in several parts
of the HGVbase-G2P system, such as initializing the logging subsystem and
reading configuration file. No subroutines are exported by default. 

=head1 SUBROUTINES/METHODS 

=cut

package HGVbaseG2P::Util;
{

	use strict;
	use warnings;
	use English qw( -no_match_vars );

	# Use OIO inside-out object framework for this non-OO package as well
	use Object::InsideOut qw( Exporter );
	use Data::Dumper;
	use Config::General;
	use Log::Log4perl qw( :easy );
	use IO::Uncompress::AnyUncompress qw(anyuncompress $AnyUncompressError);
	use Carp qw(croak cluck confess carp);

	BEGIN {

		# Export no subroutines by default, only this list on demand
		our @EXPORT_OK = qw( load_config
		  init_logger
		  open_infile
		  open_outfile
		  open_appendfile
		  load_module
		);
	}

=head2 load_config

  Usage      : my %config = load_config('./conf/hgvbase.conf');
  Purpose    : Load configuration info from file into hash via Config::General
  Returns    : Multi-level hash 
  Arguments  : Path to the configuration file
  Throws     : N/A
  Status     : Public
  Comments   : See Config::General for details on the structure of config-files, 
               as well as conf/hgvbase.conf for the master HGVbase-G2P config.

=cut

	sub load_config {
		my ($config_file, $nointerp) = @_;
		#return hash if provided instead of config file
		if ( ref($config_file) eq 'HASH' ) {
			DEBUG "Using hash instead of config file";
			return %{$config_file};
		}
		-f $config_file
		  or LOGDIE "Provided configfile-path $config_file is invalid";
		my $config = new Config::General(
			-ConfigFile      => $config_file,
			-InterPolateVars => ($nointerp ? 0 : 1)
		);
		return $config->getall();
	}

=head2 init_logger

  Usage      : $logger = init_logger('./conf/log4perl.conf', 'MyCategory');
               $logger = init_logger();
  Purpose    : Initialize a Log::Log4Perl logger, as part of a logging framework
               (via a configuration file), or a generic logger (no config file).
  Returns    : Log::Log4perl::Logger instance.
  Arguments  : Path to the configuration file (optional)
               Category for the logger (optional)
  Throws     : N/A
  Status     : Public

=cut

	sub init_logger {
		my ( $log4perl_config, $category ) = @_;

  # Initalize logging framework if not already done, and get reference to logger
		my $logger;
		if ( $log4perl_config && -f $log4perl_config && $category ) {
			Log::Log4perl->init_once($log4perl_config);
			$logger = Log::Log4perl->get_logger($category);
			$logger->trace(
"Logger initialized for $category, using configuration in $log4perl_config"
			);
		}
		else {
			Log::Log4perl->easy_init(
				{
					level  => $INFO,
					layout => "%F{1} %L:%p> %m%n",
					file   => 'STDERR'
				}
			);
			$logger = Log::Log4perl->get_logger();
			$logger->debug("Generic logger initialized");
		}
		return $logger;
	}

=head2 open_infile

  Usage      : $fh = open_infile('test1.csv');
               $fh_gzipped = open_infile('test2.csv.gz') 
               print $fh "foobar\n";
  Purpose    : Open file for reading, decompressing on the fly if needed
  Returns    : IO::Handle compatible object
  Arguments  : Path to the file to open. Can be .gz, .zip or .bz2 which
               will prompt the usage of an appropriate decompression-module
  Throws     : N/A
  Status     : Public
  Comments   : Relies on IO::Uncompress::AnyUncompress for decompressing a 
               variety of compression formats 

=cut

	sub open_infile {
		my ($filename) = @_;

		# Decompress on the fly if appropriate and open IO handle to return
		my $fh;
		if ( $filename =~ /\.(gz|Z|zip)\Z/xms ) {
			$fh = IO::Uncompress::AnyUncompress->new($filename)
			  || die "Uncompress failed: $AnyUncompressError";
		}
		else {
			$fh = IO::File->new();
			$fh->open( $filename, '<' )
			  || die "can't open file '$filename': " . $!;
		}
		return $fh;
	}

=head2 open_outfile

  Usage      : $fh = open_outfile('testout.csv');
               $fh_gzipped = open_outfile('testout.csv.gz') 
               print $fh "foobar\n";
  Purpose    : Open file for writing, compressing on the fly if needed
  Returns    : IO::Handle compatible object
  Arguments  : Path to the file to open. Can be .gz, .zip or .bz2 which
               will prompt the usage of an appropriate compression-module
  Throws     : N/A
  Status     : Public
  Comments   : Relies on the IO::Compress::*  family of modules for compressing,
               so if you need to e.g. make a .bz2 file you need to have the
               IO::Compress::Bzip CPAN-module installed 

=cut

	sub open_outfile {
		my ($filename) = @_;

		# Set up handle to either plain file or compressed stream into file
		my $fh;
		my %compress_class_of = (
			gz  => 'Gzip',
			zip => 'Zip',
			Z   => 'Zip',
			bz2 => 'Bzip2'
		);
		if ( my ($suffix) = $filename =~ /\.((gz|Z|zip|bz)\d*)\Z/xms ) {
			my $compress_class = 'IO::Compress::' . $compress_class_of{$suffix};
			eval "require $compress_class";
			$fh = $compress_class->new($filename)
			  || die "Compress failed: $! $@ ($filename)";
		}
		else {
			$fh = IO::File->new();
			$fh->open( $filename, '>' )
			  || die "can't open file $filename: " . $!;
		}
		return $fh;
	}

	sub open_appendfile {
		my ($filename) = @_;

		# Set up handle to either plain file or compressed stream into file
		my $fh;
		my %compress_class_of = (
			gz  => 'Gzip',
			zip => 'Zip',
			Z   => 'Zip',
			bz2 => 'Bzip2'
		);
		if ( my ($suffix) = $filename =~ /\.((gz|Z|zip|bz)\d*)\Z/xms ) {
			my $compress_class = 'IO::Compress::' . $compress_class_of{$suffix};
			eval "require $compress_class";
			$fh = $compress_class->new($filename)
			  || die "Compress failed: $! $@";
		}
		else {
			$fh = IO::File->new();
			$fh->open( $filename, '>>' )
			  || die "can't open file $filename: " . $!;
		}
		return $fh;
	}

	sub load_module {
		my ( $module, $args ) = @_;
		eval "require $module";
		confess "$@" if $@;

		return $module->new($args);
	}

}
1;

=head1 SEE ALSO

L<HGVbaseG2P::FileParser>, L<HGVbaseG2P::Database> and related classes

=head1 BUGS AND LIMITATIONS

There are no known bugs in this module.
Please report problems to Mummi <gthorisson@gmail.com>

=head1 CONTACT

This module is part of the HGVbase-G2P project

=head1 AUTHOR

Mummi <gthorisson@gmail.com>

=head1 LICENCE AND COPYRIGHT

Copyright (c) <2007> University of Leicester, Leicester, UK. All rights reserved.

This module is free software; you can redistribute it and/or
modify it under the same terms as Perl itself.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 

=head1 CVSINFO

$Id: Util.pm 1272 2009-08-21 15:08:35Z rcf8 $ 

=cut

