# $Id: Marker.pm 1258 2009-08-03 11:13:08Z rcf8 $

package HGVbaseG2P::Web::Controller::Marker;

=head1 NAME

  HGVbaseG2P::Web::Controller::Marker - Catalyst Controller for Markers

=head1 DESCRIPTION

  This class handles retrieving & transforming data for a single marker, this includes frequency and association data.
  Data is then passed to the respective template.

=head1 METHODS

=cut

use strict;
use warnings;
use base qw(HGVbaseG2P::Web::Controller::Root);

use Data::Stag qw( :all );
use Data::Dumper qw(Dumper);

=head2 default

Retrieves marker by identifier and places this in the stash.

=cut

sub default : Regex('^marker/(HGV\w+)$') {
    my ( $self, $c) = @_;
    my $id = $c->req->captures->[0];

    # Retrieve requested marker from the database
    $c->log->debug("Creating marker report for $id");
    my $marker = $c->model("MarkerDB")->get_marker_by_identifier($id);
    $c->stash->{markerid} = $id;
    $c->stash->{template} = 'marker/view.tt2';
    $c->stash->{marker}   = $marker || return;
    $c->stash->{threshold} = $c->req->param('threshold') || '3';
    return;
}

=head2 results

Retrieves association results for a specific marker and threshold. Processes the results to determine the associated
 studies, experiments and resultsets and then makes these available to the template.

=cut

sub results : Regex('^marker/results/(\w+)$') {
	my ( $self, $c) = @_;
    my $id = $c->req->captures->[0];
    my $threshold = $c->req->param('threshold');
    my $pvalue_threshold = 10 ** (-int($threshold));
	
	#retrieve G2P association results where this marker is tested which pass the threshold
    my $marker = $c->model("MarkerDB")->get_marker_by_identifier($id);
    $threshold eq 'ZERO' and $threshold='0';
    my @sigs = $c->model("StudyDB")->get_significances_above_threshold_for_markeridentifier($marker->identifier,$pvalue_threshold);
    
    my $studies={};
    
    #loop through each significance and work out which study, experiment and resultset they are present in
    if(@sigs > 0) {
    	foreach my $sig (@sigs) {
	        my $resultset = $sig->resultsetid;
	        my $resultsetid = $resultset->resultsetid;                 
	        my $study = $resultset->experimentid->studyid;
	        my $studyid = $study->studyid;
	        my $experiment = $resultset->experiment;
	        my $experimentid = $experiment->experimentid;
	        
	        $studies->{$studyid}->{'study'}=$study;
	        $studies->{$studyid}->{$experimentid}->{'experiment'}=$experiment;
	        $studies->{$studyid}->{$experimentid}->{$resultsetid}=$sig;
    	}

    	my @studies = map { $studies->{$_} } keys %{$studies};
    	
    	$c->stash->{studies}=\@studies;  
    }
    $c->stash->{template} = 'marker/results.tt2';
}

=head2 frequencies

Retrieves frequency data for a specific marker. This is converted into a format which can be used by the template to present stacked
frequency data.

=cut

sub frequencies : Regex('^marker/frequencies/(\w+)$') {
	my ( $self, $c) = @_;
    my $id = $c->req->captures->[0];
    
	# Retrieve G2P frequencies where this marker is tested. For efficiency, we
    my @freqclusters = $c->model("StudyDB")->get_frequencyclusters_by_markeridentifier($id);
    $c->log->info("freqclusters for ".$id.": ".scalar(@freqclusters));
    my $studies={};
	my %reported_alleles   = ();
    my %reported_genotypes = ();
    
    #loop through frequencies calculating which study and experiment they belong to and the alleles/genotypes
    if(scalar(@freqclusters) > 0) {
    	foreach my $fc (@freqclusters) {
	        my $experiment = $fc->usedmarkersetid->experiment;
	        my $experimentid = $experiment->experimentid;
	        my $study = $experiment->studyid;
	        my $studyid = $study->studyid;
	        
	        $studies->{$studyid}->{'study'}=$study;
	        $studies->{$studyid}->{$experimentid}->{'experiment'}=$experiment;
	        $studies->{$studyid}->{$experimentid}->{$fc->frequencyclusterid}=$fc;
			
			#determine reported alleles/genotypes
			foreach my $af($fc->allelefrequencies) {
			    $reported_alleles{$af->allelecombo} = 1;
			}
			foreach my $gf($fc->genotypefrequencies) {
			    $reported_genotypes{$gf->genotypecombo} = 1;
			}	
    	}
	    my @reported_alleles   = keys(%reported_alleles);
	    my @reported_genotypes = keys(%reported_genotypes);
	    $c->stash->{reported_alleles}   = \@reported_alleles;
	    $c->stash->{reported_genotypes} = \@reported_genotypes;

    	my @studies = map { $studies->{$_} } keys %{$studies};
    	
    	$c->stash->{studies}=\@studies;  
    }
    $c->stash->{template} = 'marker/frequencies.tt2';
}


=head1 AUTHOR

  Gudmundur A. Thorisson <gthorisson@gmail.com>
  Rob Free <rcfree@gmail.com>

=head1 LICENSE

This library is free software, you can redistribute it and/or modify
it under the same terms as Perl itself.

=cut

1;
