# $Id: Root.pm 1258 2009-08-03 11:13:08Z rcf8 $

package HGVbaseG2P::Web::Controller::Root;


=head1 NAME

HGVbaseG2P::Web::Controller::Root - Root Catalyst Controller

=head1 DESCRIPTION

This is the base controller for the HGVbase web application. It sets up a session if it has not been populated.
It also contains methods called by child classes to retrieve and collate various IDs

=head1 METHODS

=cut

use strict;
use warnings;
use base 'Catalyst::Controller';
use Data::Dumper qw(Dumper);
use Bio::Graphics;
use HGVbaseG2P::Web::Util qw(add_uri_parameters);

#
# Sets the actions in this controller to be registered with no prefix
# so they function identically to actions created in MyApp.pm
__PACKAGE__->config->{namespace} = '';

my $home =  $ENV{HGVBASEG2PWEB_HOME} ;

=head2 begin

Begin is run prior to default. Here core variables are set up for use by templates.
The most important is initialisation of the session if it has not been populated already.
There is also a hack for IE6 (which does not play nicely with these sessions for some reason).

Also set up are base_ values for resultsets, studies and phenotypes
and a GUID which can be used to prevent file caching in browsers.

=cut

sub begin : Private {
    my ($self, $c) = @_;
    $c->log->info("session id". $c->get_session_id);
    if ( $c->session_expires==0 && $c->session_delete_reason eq 'session expired') {
    	
    	$c->stash->{template}="info/session.tt2";
    	$c->stash->{expired}=1;
    	$c->stash->{reason}=$c->session_delete_reason();
    	$c->log->info("session created!");
    	$self->init_session($c);
    	return;
    }
    
    if (!$c->session->{populated}) {
    	$c->log->info("session not populated!");
    	$self->init_session($c);
    }
    
    $c->log->info("get resultsets and studies");
    $c->stash->{base_resultsetids}=$self->get_resultsets($c);
    $c->stash->{base_studies}=$self->get_studies($c);

    $c->stash->{base_phenotypes}=$self->get_phenotypes($c,$c->stash->{base_resultsetids});
    $c->stash->{guid}=time;
}

=head2 default

Default behaviour of the root controller is to just display a template 
corresponding to the URL path in the request. This is more or less the
same as static HTML-file handling, except provides full template functionality.

=cut

sub default : Private {
    my ( $self, $c, @path ) = @_;
    
    if ($c->stash->{expired}) {
    	$c->stash->{template}="info/session.tt2";
    	return;
    }
    	
    my $path = join('/', @path);
    $c->log->info("path='$path'");
    @path == 0 and $c->res->redirect('index');
    
    # Serve template directly if available
    $c->stash->{template} = @path > 0 ?  $path.'.tt2' : 'index.tt2'; # serve /index if nothing specified
   	
}

=head2 get_studies

Returns study identifier list based on session OR if IE6 from the request parameters (ie. querystring)

=cut

sub get_studies {
	my ($self, $c)=@_;
	my $b = $c->req->browser;
	if ($b->browser_string eq 'MSIE' &&  $b->version<7) {
		my $studyids = $c->req->param('studies');
		$c->log->info("Browser is MSIE 6 or less with study: $studyids");
		my @studies = split(",",$studyids);
		my %studies = map { $_ => 1; } @studies;
		$c->log->info("studies from params:".Dumper(\%studies));
		return \%studies;	
	}
	else {
		return $c->session->{studies};
	}
}

=head2 get_resultsets

Returns ResultsetID list based on session OR if IE6 from the request parameters (ie. querystring)

=cut

sub get_resultsets {
	my ($self, $c)=@_;
	my $b = $c->req->browser;
	if ($b->browser_string eq 'MSIE' &&  $b->version<7) {
		my $rsids = $c->req->param('resultsetids');
		$c->log->info("Browser is MSIE 6 or less with rs: $rsids");
		my @resultsets = split(",",$rsids);
		$c->log->info("resultsets from params:".Dumper(\@resultsets));
		return \@resultsets;	
	}
	else {
		return $c->session->{resultsetid};
	}
}

=head2 get_phenotypes

Returns phenotype identifier list by looking up each resultset (in session OR if IE6 from the request parameters (ie. querystring))

=cut

sub get_phenotypes {
	my ($self, $c,$resultsets)=@_;
	
	my %phenotype_rs;
	if (defined($resultsets) and scalar(@{$resultsets})>0) {
		foreach my $rsid(@{$resultsets}) {
			my $resultset = $c->model("StudyDB")->get_resultset($rsid);
		    #$c->log->info("phenotype identifier for rs:$rsid is ".$resultset->experiment->phenotypemethodid->identifier);
		    $resultset and $phenotype_rs{$resultset->experiment->phenotypemethodid->identifier}++;
		}
    }
    return \%phenotype_rs;
}

=head2 init_session

Initialises session with default values

=cut

sub init_session {
	my ($self, $c)=@_;
	$c->log->info("init_session");

	my $config = $c->config->{HGVBrowser};
	 my @colors = ();
	    my $panel = Bio::Graphics::Panel->new;
	    #print "<pre>".Dumper($config)."</pre>";   
	    my @bgcolors = split(' ', $config->{bgcolors});
	    for (my $counter=0; $counter<8; $counter++) {
	    	my @rgb = $panel->color_name_to_rgb($bgcolors[$counter]);
	    	push @colors, "rgb(".join(",",@rgb).")";
	    }
	    $c->log->info("colors:".Dumper(\@colors));
    	$c->session->{colors}=\@colors;
    	$c->session->{settings}=$config->{Settings};
    	$c->session->{tracks}=$config->{Tracks};
    	$c->session->{resultsetid}=[];
    	$c->session->{g_threshold}=3;
    	$c->session->{r_threshold}=3;
    	$c->session->{chrs_to_display}=['ALL'];
    	$c->session->{populated}=1;
    	my ($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst)=localtime(time);

    	$c->session->{timestamp}=sprintf("%4d-%02d-%02d %02d:%02d:%02d\n",$year+1900,$mon+1,$mday,$hour,$min,$sec);
}

=head2 collate_studies

Converts selected studies (ie. those in the session) into a study identifier based hash containing resultset ID hash (ie. hash of hashes).
Returns a this hash and a joined list of resultset IDs

=cut

sub collate_studies {
	my ($self, $c)=@_;
	
	my $studies;
	my $sess_rsid = $self->get_resultsets($c);
	if ($sess_rsid and scalar(@{$sess_rsid})>0) {
    	
    	#get resultset studies so can provide number of resultsets selected
	    my %study_rs;
	    
	    $c->log->info("resultset ids in session:".Dumper($sess_rsid));
	    if (defined($sess_rsid) and scalar(@{$sess_rsid})>0) {
	    	my @resultsets = $c->model("StudyDB")->get_resultsets($sess_rsid);
		    foreach my $rs(@resultsets) {
		    	my $studyid=$rs->experiment->studyid->studyid;
		    	$c->log->debug("resultset id:".$rs->resultsetid." has studyid:".$rs->experiment->studyid->studyid);
	    		$study_rs{$studyid}->{$rs->resultsetid}=1;
	    	}
	    }
	    foreach(sort {$a <=> $b } keys %study_rs ) {
			my $count = scalar($c->model("StudyDB")->get_resultsets_in_study($_));
			$c->log->debug("resultsets in study $_:".$count);
	    	my $study = $c->model("StudyDB")->get_study($_);
	    	push @{$studies}, {data=>$study, rs_added=>$study_rs{$_}, count=>$count};
	        $c->log->debug("study added:".$study->identifier);
	        $c->log->debug("rs added:".Dumper($study_rs{$_}));
	    }
    }
    my $resultsets = $sess_rsid ? join(",",@{$sess_rsid}) : "";
    return ($studies,$resultsets);
}

=head2 collate_chrs

Maps selected chromosomes from session into a hash with value 1  -> used by template

=cut

sub collate_chrs {
	my ($self, $c)=@_;
    
    if ($c->session->{chrs_to_display}) {
    	if ($c->session->{'chrs_to_display'}->[0] eq 'ALL') {
    		$c->log->info("set to all");
    		return 'ALL';
    	}
    	else {
	    	my %chrs = map { $_ => 1 } @{$c->session->{'chrs_to_display'}};
	    	$c->log->info("mapping chrs");
	    	$c->log->info(Dumper(\%chrs));
	    	return \%chrs;
    	}
    }
    else {
    	return 'ALL';
    }
}

sub do_search {
	my ($self, $type, $c)=@_;
	my $path = $c->req->uri;
	my $base = $c->req->base;
    my $content_type = $c->req->param('content-type') || 'text/html';
	my $page = $c->req->param('page') || 1;
	my $page_size = $c->req->param('page_size') || 20;
	my @server_error = ();
	
	$path =~ s/$base//;
	$path =~ s/content-type=\w//;
	
	my $search = HGVbaseG2P::Search->new({conf_file => $c->config});
	
	my $ag = $search->new_aggregator;

	#local data
	my $local_df = $search->new_datafeed;
	my $local_server = {
		'uri' => $base,
		'name' => 'central',
		'description' => 'HGVbaseG2P Central Server',
		'version' => '2.0'
	};
	
	$local_df->server($local_server);
	
	my $results = $search->new_results($type);
	$results->get_db_from_catalyst($c);
	$results->params($c->req->params);
	$results->page($page);
	$results->page_size($page_size);
	
	$c->log->info("populate from local");
	
	my ($data,$pager,$total) = $results->search($c);
	$local_df->input_from_search($data);
	$ag->add_data_feed($local_df);
	
	$ag->content_type($content_type);
	$ag->base_server($local_server);
	
	$content_type ne 'text/html' and $c->res->content_type($content_type);
	
	
	my ($output,$template)=$ag->output;

	if ($template) {
		my $search_stash = {
			content=>$output,
			server_error=>\@server_error,
			#ext_servers=>($c->config->{central_server} ? scalar(@servers) : 0),
			rss_link=>add_uri_parameters($c->req->uri,{'content-type'=>'text/xml'}),
			text_link=>add_uri_parameters($c->req->uri,{'content-type'=>'text/plain'}),
			json_link=>add_uri_parameters($c->req->uri,{'content-type'=>'application/json'}),
			pager=>$pager,
			total=>$total,
			dbconfig=>$c->config->{'Controller::Search'}->{'Database'}->{$type}
		};
		$c->stash->{search}=$search_stash;
		return $template;
	}
	else {
		$c->res->body($ag->output);
		return undef;
	}
}
	
=head2 end

Attempt to render a view, if needed.

=cut 

sub end : ActionClass('RenderView') {
}

=head1 AUTHOR

  Gudmundur A. Thorisson <gthorisson@gmail.com>
  Rob Free <rcfree@gmail.com>

=head1 LICENSE

This library is free software, you can redistribute it and/or modify
it under the same terms as Perl itself.

=cut

1;
