# $Id: Significance.pm 1258 2009-08-03 11:13:08Z rcf8 $

package HGVbaseG2P::Web::Controller::Significance;


=head1 NAME

  HGVbaseG2P::Web::Controller::Significance - Catalyst Controller for Significance data

=head1 DESCRIPTION

  This class deals with obtaining significance results from the database and transforming them as 
  needed before handing to the appropriate template (or in the case of odds ratios generating an image)

=head1 METHODS

=cut

use strict;
use warnings;
use HGVbaseG2P::Browser::Graphics;
use base qw(HGVbaseG2P::Web::Controller::Root);
use Data::Dumper qw(Dumper);
use HGVbaseG2P::Browser::Util qw(inv_neglog);

=head2 or

Returns a stemp-plot containing the odds ratio as a PNG content type

=cut

sub or :Regex('^significance/or$') {
	my ( $self, $c, $id ) = @_;
	$c->response->content_type('image/png');
	my $graphics = HGVbaseG2P::Browser::Graphics->new({conf_file=>$c->config});
	my $or = $graphics->odds_ratio({
		lower_ci=>$c->req->param('lower_ci'),
		upper_ci=>$c->req->param('upper_ci'),
		odds_ratio=>$c->req->param('odds_ratio')
	});
	$c->log->info("odds ratio:".$c->req->param('odds_ratio'));
	
	$c->response->body($or->png);
}

=head2 view

Retrieves data for one or more significances (passed in through 'id' parameter as CSV) and puts it in the stash

=cut

sub view :Regex('^significance/view$') {
    my ( $self, $c ) = @_;
    
    
    
    my @signifs;
	my %studies;
	my %resultsets;
	my %sigs;
	
	#get id, marker and threshold from params/session
	my $id = $c->req->param('id');
	my $marker_identifier = $c->req->param('marker');
	my $threshold = $c->req->param('threshold') || $c->session->{r_threshold};
	
	
	#if ids supplied - split significances and loop through each significance getting each from the database
	if ($id) {
		%sigs = map { $_ => $c->model("StudyDB")->get_significance($_) } split(/\,/,$id);
	}
	
	#if marker_identifier supplied - retrieve all markers within the resultsets and above the threshold
	if ($marker_identifier) {
		my $pval_threshold = inv_neglog($threshold);
		my @rsids = $c->req->param('resultsets');
		my @id_sigs = $c->model('StudyDB')->dbh->dbh->resultset('Significance')->search({'usedmarkersetid.markeridentifier'=> $marker_identifier, 'unadjustedpvalue'=>{'<=',$pval_threshold},'resultsetid'=>\@rsids},{join=>'usedmarkersetid'})->all;
	    %sigs = map { $_->significanceid => $_ } @id_sigs;
	}
	
	#use the first sig to get marker info
	my @ids = keys %sigs;
	my $marker_sig = $sigs{$ids[0]};
	
	!$marker_identifier and $marker_identifier = $marker_sig->usedmarkersetid->markeridentifier;
	
	my $marker = $c->model("MarkerDB")->get_marker_and_refcoords_by_identifier($marker_identifier);
	
	$c->stash->{marker}=$marker;
	$c->stash->{template} = 'significance/view.tt2';
	
	foreach my $id(keys %sigs) {
		my $sig = $sigs{$id};
		my %single_signif=();
		
		#build up resultset name for significance
		my $resultsetname = $sig->resultset->experiment->study->label."_".$sig->resultset->experiment->label."_".$sig->resultset->label;
		if (length($resultsetname)>30) {
			$resultsetname = substr($resultsetname,0,30)."..";
		}
		
		#set significance fields from experiment,resultset,phenotype etc.
		$single_signif{is_displayed} = $sig->resultset->experiment->study->isdisplayed eq 'show' ? 1 : 0;
		$single_signif{title}=$sig->resultset->experiment->study->name;
		$single_signif{resultset}=$sig->resultset->name;
		$single_signif{studyid}=$sig->resultset->experiment->study->identifier;
		my $phenotype = $sig->resultset->experiment->phenotypemethodid;
		$single_signif{phenotype}=$phenotype ? $phenotype->phenotypepropertyid->name : 'Not defined';
		$single_signif{fulllabel}=$resultsetname;
		$single_signif{sig}=$sig;
		
		#set study and resultset hash flags
		$studies{$single_signif{studyid}}=1;
		$resultsets{$sig->resultset->resultsetid}=1;
	
		my %apanels_seen=();
		
		#get frequencyclusters attached to significance
		my @fclusters = $sig->frequencyclusters;
		
		if (scalar(@fclusters)>0) {
			#for each frequency cluster add the panel name to a hash
			foreach my $apanel(@fclusters) {
				$apanels_seen{$apanel->assayedpanel->name} = $apanel->assayedpanel->name;
			}
			
			
			my $gtlabels      = [];
			my $gtlabels_seen = {};
			
			#loop through genotypefrequencies and figure out which genotypes are present, to print table column headers
			foreach my $gt($fclusters[0]->genotypefrequencies) {
				
				my $gtlabel = strip($gt->genotypecombo);
				if (!$gtlabels_seen->{$gtlabel}) {
				    push @{$gtlabels}, $gtlabel;
				    $gtlabels_seen->{$gtlabel} = 1;
				}
			}
			my @gttemp = keys %{$gtlabels_seen};
			$single_signif{gtlabels}=[sort @gttemp]; #sort genotypes
			
			
			my $alabels      = [];
			my $alabels_seen = {};
			
			#loop through allelefrequencies and figure out which alleles are present, to print table column headers
			foreach my $al($fclusters[0]->allelefrequencies) {
				
				my $alabel = strip($al->allelecombo);
				if (!$alabels_seen->{$alabel}) {
				    push @{$alabels}, $alabel;
				    $alabels_seen->{$alabel} = 1;
				}
			}
			my @altemp = keys %{$alabels_seen};
			$single_signif{alabels}=[sort @altemp];
			
			#loop through each cluster and frequencies within to add these to a hash (based on allele/genotype)
			my %freqclusters=();
			foreach my $fc(@fclusters) {
				my $panelname = $fc->assayedpanel->name;
				my %panelinfo = ();
	
				my %alfreqs=();
				my %gtfreqs=();
				foreach my $alfreq ($fc->allelefrequencies) {
					$alfreqs{strip($alfreq->allelecombo)}=$alfreq->frequencyasproportion;
				}
				foreach my $gtfreq ($fc->genotypefrequencies) {
					$gtfreqs{strip($gtfreq->genotypecombo)}=$gtfreq->frequencyasproportion;
				}
				$panelinfo{allelefrequencies}=\%alfreqs;
				$panelinfo{genotypefrequencies}=\%gtfreqs;
				$panelinfo{numberofgenotypedsamples}=$fc->numberofgenotypedsamples;
				$panelinfo{pvaluehwe}=$fc->pvaluehwe;
				$freqclusters{$panelname}=\%panelinfo;
			}
			
			#add the derived frequency info to this significance
			$single_signif{frequencyinfo}=\%freqclusters;
		}
#		$single_signif{odds_ratio}=2.4;
#		$single_signif{lower_ci}=1.3;
#	 	$single_signif{upper_ci}=6.7;
		push @signifs, \%single_signif;
	}
	
    $c->stash->{signifs}=\@signifs;
    $c->stash->{total_studies}=scalar(keys %studies);
    $c->stash->{total_resultsets}=scalar(keys %resultsets);
}

=head2 strip

Pull out the allele or genotype from the allelecombo/genotypecombo field (remove the flanks)

=cut

sub strip {
	my ($text)=@_;
	my $temp = substr($text,0,length($text)-10);
	return substr($temp,10);
}

1;

=head1 AUTHOR

  Gudmundur A. Thorisson <gthorisson@gmail.com>
  Rob Free <rcfree@gmail.com>
  
=head1 LICENSE

This library is free software, you can redistribute it and/or modify
it under the same terms as Perl itself.

=cut
