package HGVbaseG2P::Web::Controller::Study;

# $Id: Study.pm 1239 2009-07-08 16:21:56Z rcf8 $

=head1 NAME

HGVbaseG2P::Web::Controller::Study - Catalyst Controller for a study

=head1 DESCRIPTION

  This class pulls out various pieces of data for a particular Study or 
  list of studies and either hands off to a TT template or prints out 
  directly to the browser (e.g. in JSON format).

=head1 METHODS

=cut

use strict;
use warnings;
use base qw(HGVbaseG2P::Web::Controller::Root);

use JSON::XS;
use Data::Dumper qw(Dumper);
use LWP::UserAgent;

# Globals
my $json_coder = JSON::XS->new->ascii->pretty->allow_nonref;
$json_coder->space_after(1);

# Merely redirects to the new, shorter report URL-format 
sub view : Local {
    my ( $self, $c ) = @_;
    my $url = $c->req->uri->as_string;
    $url =~ s|(\w+)/view|$1|;
    $c->response->redirect($url);
}

=head2 default

  Usage      : /study/HGVSTxxx;
  Purpose    : Business logic related to displaying a study report
  Returns    : N/A
  Arguments  : HGVSTxx study identifier 
  Throws     : N/A
  Status     : Public
  Comments   : 

=cut

sub default :Regex('^study/(HGV\w+)$') {
    my ( $self, $c) = @_;
    my $id = $c->req->captures->[0];
    my $study = $c->model("StudyDB")->get_study_by_identifier($id);
	
    if(!$study) {
        $c->stash->{notfound_type} = "Study";
        $c->stash->{notfound_id}=$id;
        $c->stash->{template} = 'notfound.tt2';
        return;
    }
    
    $c->stash->{studyid} = $id;
    $c->stash->{template} = 'study/view.tt2';
    my $browser_action;
    if ($study->isdisplayed) {
    	$browser_action = $c->session->{studies}->{$id} ? 'remove' : 'add';
    }
    else {
    	$browser_action = 'noentry';
    }
    
    $c->stash->{browser_action}=$browser_action;
    $c->stash->{resultsetid}=$c->session->{resultsetid} ? join(",",@{$c->session->{resultsetid}}) : "";
    
    my @resultsets=();
    foreach my $exp($study->experiments) {
    	foreach my $rs($exp->resultsets) {
    		push @resultsets, $rs->resultsetid;
    	}
    }
    $c->stash->{study_resultsets}=join(",",@resultsets);
    
    my @assoc_exps = $c->model("StudyDB")->get_assoc_experiments_in_study($study);
    
    $c->stash->{study} = $study || return;
    $c->stash->{assoc_exps}=\@assoc_exps;
    return; 
}

=head2 default

  Usage      : /study/list
  Purpose    : Retrieve all available study records and print out as JSON
  Returns    : N/A
  Arguments  : none
  Throws     : N/A
  Status     : Public
  Comments   : 

=cut

sub list :Regex('^study/list$') {
    my ( $self, $c ) = @_;
	
    # Retrieve all unhidden studies from the database
    my @studies = $c->model("StudyDB")->get_all_unhidden_studies;
    my %json = ();
    
    #get resultset studies so can provide number of resultsets selected
    my $study_hash = $c->stash->{base_studies};

    foreach my $study(@studies) {	
		if (!$study->ishidden) {
#		my @exp_idlist = ();
#		foreach my $exp($study->experiments()) {
#		    push(@{$json{items}}, $exp->to_jsonhash());	    
#		}
		
		push(@{$json{items}}, $study->to_jsonhash(($study_hash->{$study->identifier} ? 'Remove From Browser' : 'Add To Browser')));
	    }
    }
    
    $c->response->body($json_coder->encode(\%json));
    return;
}


sub studies :Regex('^studies$') {
	my ($self, $c) = @_;
	if ($self->do_search('studies',$c)) {
		my $avail = $c->model("StudyDB")->count_all_available_studies;
		$c->log->info("avail:$avail");
		$c->stash->{search}->{avail}=$avail;
		$c->stash->{template}='studies.tt2';
		
	}
}

sub resultsets :Regex('^study/resultsets/(HGV\w+)$') {
    my ( $self, $c ) = @_;
	my $id = $c->req->captures->[0];
	
    # Retrieve case-control experiments for study from the database
    my $experiments = $c->model("StudyDB")->get_assoc_experiments_rs_by_study_identifier($id);
    
  	if ($experiments->count==1 && $experiments->single->resultsets->count==1) {
  		$c->stash->{single}=1;
  	}
  	
    $c->stash->{experiments}=$experiments;
    
    $c->stash->{template}="study/resultsets.tt2";
    return;
}

sub experiment :Regex('^study/experiment/(HGV\w+)$') {
    my ( $self, $c ) = @_;
	my $id = $c->req->captures->[0];
	
    # Retrieve case-control experiments for study from the database
    my $experiment = $c->model("StudyDB")->get_experiment_by_identifier($id);
	
	$c->stash->{template}='study/experiment.tt2';
    $c->stash->{experiment}=$experiment;
    $c->stash->{marker_count}=$experiment->usedmarkersets->count;
    return;
}

sub add_all :Regex('^study/add_all$') {
	my ($self, $c)=@_;
	my @studies = $c->model("StudyDB")->get_all_unhidden_studies;

    if (@studies) {
    		$c->session->{studies}={map { $_->identifier => 1; } @studies};
    }
    
    my @resultsetids=();
    foreach my $study(@studies) {
    	foreach my $exp($study->experiments) {
    		foreach my $rs($exp->resultsets) {
    			push @resultsetids, $rs->resultsetid;
    		}
    	}
    }
    
    if (@resultsetids) { 
    	$c->session->{resultsetid}=\@resultsetids;
    }
    
    $c->stash->{template}='studies.tt2';
}
=head1 AUTHOR

  Gudmundur A. Thorisson <gthorisson@gmail.com>>
  Rob Free <rcfree@gmail.com>
  
=head1 LICENSE

This library is free software, you can redistribute it and/or modify
it under the same terms as Perl itself.

=cut

1;
