# $Id: XMLStore.pm 1272 2009-08-21 15:08:35Z rcf8 $

=head1 NAME

HGVbaseG2P::XMLStore - Storing XML-data to a DBIx::DBStag database


=head1 SYNOPSIS

    use English qw( -no_match_vars );
    use HGVbaseG2P::XMLStore;
    my $xmldb = HGVbaseG2P::XMLStore->new(conf_file => 'path to master config');
    eval {
       $xmldb->store_file($xmlfile);
    };
    # catch exceptions
    if($EVAL_ERROR) {
        ...
    }


=head1 DESCRIPTION

This class implements a handler for a stream of XML-events which will attempt to 
store them in a relational database. It will also generate those events via its
inherited parsing-capabilities.
  In addition to inherited generic DBStag functionality from Database.pm, this class has several
methods for handling XML-content specific to HGVbase-G2P. This mostly involves
<Study> and <Experiment> container-elements and ways of storing these bit-by-bit
and not all in one big chunk. See code for details.

NB Objects of this class can also do send regular non-DBStag queries to the
database, and so taking advantage of both the master configuration and the
shared SQL-templates in perl/sqltemplates/.

=head1 SUBROUTINES/METHODS 

=cut

package HGVbaseG2P::XMLStore; {
    
use strict;
use warnings;

# Use OIO inside-out object framework for our OO hierarachy
use Object::InsideOut qw( HGVbaseG2P::FileParser 
			  HGVbaseG2P::Database::Stag 
			  Data::Stag::BaseHandler );
use Object::InsideOut::Metadata;

use English qw( -no_match_vars );
use Time::HiRes qw( time );
use Number::Format qw(format_number);
use Digest::MD5 qw( md5_hex );
use Data::Dumper;
use FindBin;
use Cwd;
use Data::Stag qw( :all );
use HGVbaseG2P::Util qw(load_config init_logger );

### Private variables & constants
my $logger;                     # Reference to master logger
my %stats;                      # Place to put various stats in
my %config;                     # Master configuration settings from config-file
my $current_studyid;            # Global ID of the Study currently being processed
my $current_experimentid;       # Global ID of the Experiment currently being processed

### Variables & constants handled by OIO

# The default depth at which we want to start storing elements
my @depth2store :Field :Acc(Name => 'depth2store', Restricted => 1);

# Flag indicating an error during db-storing operation
my @store_error : Field :Acc(Name => 'store_error', Restricted => 1);

### Initialization arguments handled by OIO


# Initialization routine (see Object::InsideOut POD)
sub _init :Init {
    my ($self, $args) = @_;

    # Load master config IF config file is provided
    $args->{conf_file} and %config = load_config($args->{conf_file});

    # Initialize logger
    my $log4perl_conf = $args->{log4perl_conf} || $config{log4perl_conf};
    $logger = init_logger($log4perl_conf, __PACKAGE__);
		
    # Inherit from foreign (non-OIO) objects, special tricks needed here to
    # install the proper SAX-even handler callbacks :!
    $logger->info("creates basehandler");
    my $handler = Data::Stag::BaseHandler->new();
    $handler->trap_h({Allele         => sub { $self->handle_allele(@_)         },
		      StudyInfo      => sub { $self->handle_studyinfo(@_)      },
		      Experiment     => sub { $self->handle_experiment(@_)     },
		      ExperimentInfo => sub { $self->handle_experimentinfo(@_) },
		  });
    *Data::Stag::BaseHandler::catch_end = sub {$self->catch_end(@_)};
    $self->inherit($handler);
    $logger->info("inherits basehandler");
    
    # Set mapconf, so DBIx::DBStag knows about some cryptic FK-relations. See 
    # http://search.cpan.org/~cmungall/DBIx-DBStag-0.09/DBIx/DBStag.pm#mapping
    my $stagmap_file;
    if($config{dbstag_meta}) {
        $stagmap_file  = "$FindBin::Bin/../".$config{dbstag_meta};
    }
    elsif($args->{stagmap}) {
        $stagmap_file = $args->{stagmap};
    }
    if($stagmap_file) {
        -f $stagmap_file or $logger->logcroak("Provided dbstag_meta file $stagmap_file is not valid");
        $logger->info("Setting stagmap-file to $stagmap_file");
        $self->data_source->mapconf($stagmap_file);
    }

    # Set the depth at which want to start storing stuff from the XML. Nomally
    # this is 1, but when things are nested under <Study> or <Experiment>, this
    # needs special care (see below)
    $self->depth2store(1);
    $logger->info("set depth to store");
}


=head2 store_file

  Usage      : $xmldb->store_file('markers.xml');
  Purpose    : Store data in XML-file to compatible relational database
  Returns    : N/A
  Arguments  : Path to the XML-file to parse
  Throws     : Dies on errors
  Status     : Public
  Comments   : Storing will only be attempted for XML-elements which
               match the database tables and FK-relations between them.

=cut

sub store_file :Public {
    my ($self, $filename) = @_;
    
    # Parse file and handle the XML-stream with self
    #$logger->debug("Storing XML-file $filename");
    my $stime = time();
    
    # ensure current study ID and experiment ID are reset for each initialisation (important for tests which use one instance of XMLStore
	$current_studyid=undef;
	$current_experimentid=undef;
    
    $logger->info("store file");
    %stats = (); # reset stats
    $self->parse_file(filename => $filename,
		     filetype => 'xml',
		     handler  => $self);

    # Print out some stats
    my $totaltime = time() - $stime;
    $stats{totaltime} = $totaltime;
    $stats{'Performance'} = format_number(($stats{totalstored}||0)/$totaltime, 1).' nodes/s';
    $logger->warn("STATS:\n",Dumper(\%stats),"\n");
    
    # Die if any store-errors were encountered 
    if($self->store_error()) {
	$logger->logdie("Got error(s) during load, so throwing exception right here");
    }
    
    return 1; # Or else return true if everything went smoothly
}


# Internal methods for handling project-specific XML-elements

sub handle_studyinfo :Restricted {
    my ($self, $handler, $node) = @_;
    $self->log->info("storing studyinfo");
    
    # Create a new Study node with the metadata
    my $study_node = stag_nodify([Study => []]);
    
    foreach my $n($node->children()) {
	$study_node->addchild($n);
    }
    
    $current_studyid = $node->get('Identifier') || die "Need Identifier column for the Study container";
    $study_node->set(q{@} => [[id=>$current_studyid]]);
    $logger->info("Storing new Study node w/ id=$current_studyid, currently at depth ".$self->depth);
    # !!TODO: need exception handling here, like in catch_end
    $self->data_source->storenode($study_node);
    $stats{stored}->{'Study'}++;
    $self->depth2store($self->depth);
    return;
}

sub handle_experiment :Restricted {
    my ($self, $handler, $node) = @_;
    $logger->info("in handle_experiment(), depth=".$self->depth.", decreasing depth coz we're closing <Experiment>");
    $self->depth2store($self->depth);
    $current_experimentid = undef;
    if(my $atts = $node->get(q{@})) {
	$node->set('StudyID', $current_studyid);
	$logger->debug("Got experiment for lookup, ",$node->itext);
	# !!TODO: need exception handling here, like in catch_end
	$self->data_source->storenode($node) || $logger->logconfess($EVAL_ERROR);
    }
    return;
}

sub handle_experimentinfo :Restricted {
    my ($self, $handler, $node) = @_;

    # Create new Experiment node
    my $exp_node = stag_nodify([Experiment => [[StudyID => $current_studyid]]]);
    foreach my $n($node->children()) {
	$exp_node->addchild($n);
    }
    $current_experimentid = 'exp_global';
    $exp_node->set(q{@} => [[id=>$current_experimentid]]);
    $logger->debug("Storing new Experiment node w/ id=$current_experimentid, currently at depth ".$self->depth);
    # !!TODO: need exception handling here, like in catch_end
    $self->data_source->storenode($exp_node, {assigned_node_h=>{'Type'=>1}});
    $stats{stored}->{'Experiment'}++;
    $self->depth2store($self->depth);
    return;
}

## Other db-specific handling of elements being imported

# Create digest from allele-seq before storing
sub handle_allele :Restricted {
    my ($self, $handler, $node) = @_;
    my $aseq_digest = md5_hex($node->get('AlleleSeq'));
    $logger->is_debug and 
	#$logger->debug("Creating digest $aseq_digest from allele '",$node->get('AlleleSeq'),"'");
    $node->add('AlleleSeqDigest', $aseq_digest);
    return $node;
}

# Handler for any elements not specifically handled above. Basically just store
# the node in the database if it's at a certain level in the tree and DBIx::DBStag
# will take care of the rest
sub catch_end :Restricted {
    my ($self, $handler, $event, $node) = @_;
    #$logger->debug("checking if I should store element '$event' at depth ".$self->depth);
    if ($event !~ /\A(Experiment|ExperimentInfo|StudyInfo|Study)\Z/xms && $self->depth == $self->depth2store) {
	$stats{total2store}++;
	#$logger->debug("Current depth is ",$self->depth, ", depth2store=".$self->depth2store.", storing node #$stats{total2store} <".$node->name.'>');
	    $stats{total2store} % 100 == 0 and $logger->info("Total $stats{total2store} nodes stored");

	# Switch statement, sort of, to figure out which global ID to set before storing
	defined($current_experimentid)  ? $node->set('ExperimentID', $current_experimentid)
      : defined($current_studyid)       ? $node->set('StudyID', $current_studyid)
      : undef
      ;

	eval {
	    $self->data_source->storenode($node);
	};
	if($EVAL_ERROR) {
	    $logger->error("could not store node #$stats{total2store}: $@\n", $node->xml());
	    $stats{errorstore}++;
	    $self->store_error(1);
 	}
	else {
	    $stats{totalstored}++;
	    $stats{stored}->{$event}++;
	}
	return;
    }
    else {return $node;}
}

}
1;

=head1 SEE ALSO

L<HGVbaseG2P::FileParser>, L<HGVbaseG2P::Database> and related classes

=head1 BUGS AND LIMITATIONS

There are no known bugs in this module.
Please report problems to Mummi <gthorisson@gmail.com>

=head1 CONTACT

This module is part of the HGVbase-G2P project

=head1 AUTHOR

Mummi <gthorisson@gmail.com>

=head1 LICENCE AND COPYRIGHT

Copyright (c) <2007> University of Leicester, Leicester, UK. All rights reserved.

This module is free software; you can redistribute it and/or
modify it under the same terms as Perl itself.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 

=head1 CVSINFO

$Id: XMLStore.pm 1272 2009-08-21 15:08:35Z rcf8 $ 

=cut

