use strict;
use warnings;
use FindBin;
use lib "$FindBin::Bin/../lib";
use HGVbaseG2P::Browser::Core;
use HGVbaseG2P::Browser::Util qw(chr_list chr_length split_data_by_field);
use Log::Log4perl qw(:easy);
Log::Log4perl->easy_init($DEBUG);

use Data::Dumper qw(Dumper);
use Getopt::Euclid qw( :minimal_keys );

#instantiate browser db
my $bin_size = $ARGV{binsize} || 3000000;
my $short_binsize = substr( $bin_size, 0, 1 );
my $studyid = $ARGV{study};
INFO "Study:$studyid";

#use browser methods to generate data -> all logic (region and genome views) in one place then
#don't use autoconnect option, pass browser db object in here
my $db = HGVbaseG2P::Database::Browser->new( { conf_file => '../conf/hgvbase.conf' });

my $browser =
  HGVbaseG2P::Browser::Core->new( { conf_file => '../conf/hgvbase.conf' } );
  $browser->db($db);
my $resultsets = $browser->db->get_resultsets_in_study( { studyid => $studyid } );
INFO Dumper($resultsets);
  
$browser->major_bin($bin_size);

#loop through result sets returned
foreach my $rs ( @{$resultsets} ) {

	my $rs_id = $rs->{ResultsetID};
	INFO "Precalculating data for resultset: $rs_id";

	INFO "Removing any previous data for resultset and binsize $bin_size";

	$browser->resultsetid($rs_id);
	$db->delete_marker_binned_by_resultset( { resultsetid => $rs_id, binsize => $short_binsize } );

	#get data via browser
	#$browser->chr(undef);
	
	my $data = $db->get_marker_data_by_resultset({resultsetid => $rs_id});
	my $split_data = split_data_by_field($data,'Chr',1);

	foreach my $chr ( keys %{chr_list()}) {
		my %chr_data=();
    
    	foreach my $item(@{$split_data->{$chr}}) {
    		my $start = $item->{'Start'};
    		!$chr_data{$start} and $chr_data{$start}=[];
    		$chr_data{$start}->[0]=$item;	
    	}
		
		$browser->chr($chr);
		$browser->seg_start(1);
		$browser->seg_stop(chr_length($chr));
		$browser->round_positions;
		
		my ($binned_data)=$browser->calculate_binned_data( { bin_size => $bin_size, data => \%chr_data });
		
		INFO("Calculating data for Chr:$chr (length:".chr_length($chr));
		#INFO( scalar( @{ $chr_data } ) . " rows retrieved" );
		
		#retrieve bins with no P-values in at all and add to hash
		
		
		#loop through each threshold level 
		foreach my $threshold (qw(ZERO 1 2 3 4 5 6 7 8)) {
			$browser->threshold($threshold);
			print "threshold $threshold\n";
			#calculate significance counts using browser object
			
			
			my ($max_and_counts, $sig_markers, $max_pvalue, $max_count) = $browser->calculate_max_and_sig_counts( { data => $binned_data });
			
			#then store data in browser db
			my @precalc_data = ();
			my $sth          = $browser->get_db->dbh->prepare(
"INSERT INTO marker_binned (Reference, Source, Type, Name, Start, Stop, Score, ResultsetID,Threshold, BinSize) 
				VALUES (?,?,?,?,?,?,?,?,?,?)"
			);
#			print Dumper($max_and_counts);
#			exit;
			foreach my $item ( @{$max_and_counts} ) {
				my @pars=();
				if ( $item->{bin_counts} ne '-1'  ) {
					@pars = (
						$chr,       'ensembl',
						'sig',      "sig$chr",
						$item->{bin_start}, $item->{bin_stop},
						$item->{bin_counts}, $rs_id,
						$threshold, $short_binsize
					);
				}
				else {
					@pars = (
						$chr,       'ensembl',
						'sig',      "sig$chr",
						$item->{bin_start}, $item->{bin_stop},
						'-1', $rs_id,
						$threshold, $short_binsize
					);
					INFO( "Bin marked as empty: " . $item->{bin_start} );
				}
				$sth->execute(@pars);
				#print join(",",@pars)."\n";
			}

		}
	}
}
$browser->db->disconnect;
1;

__END__


=head1 NAME

  - Precalculate bins

=head1 VERSION

  0.1

=head1 DESCRIPTION

  This script generates binned versions of P value significances passing each threshold level (from -log 0 to 8). The results can be output
  to the BrowserDB (default) or to a file. Data for a single study ID can be specified, while the default is all studies.
 
  $Id

=head1 USAGE

  [ignored, usage line is built by Getopt::Euclid from interface-description below]

=head1 OPTIONS

=over

=item -v[erbose]

  Set logging to verbose (Log4perl level of 'DEBUG')
  
=item -c[onfigfile] <file>
  
  Mart generating config file to use

=for Euclid:
        file.type:    readable
        file.default: '../conf/hgvbase.conf'

=item -st[udy] <studyid>

  Only calculate bins for this study

=item -o[utfile] <file>

  Output results to CSV file and not Browser DB

=item -b[insize] <bin>

  Size of bin to generate binned data for

=item --version

=item --usage

=item --help

=item --man

  Print the usual program information

