#!perl

# $Id: studyToBrowserDB.pl 1273 2009-08-21 15:12:40Z rcf8 $

# create Browser marker data table and binned data from Study/Marker database

use strict;
use warnings;
use FindBin;
use lib "$FindBin::Bin/../lib/";
use HGVbaseG2P::Database::Study;
use HGVbaseG2P::Database::Marker;
use HGVbaseG2P::Browser::Core;
use HGVbaseG2P::Util qw(load_config);
use HGVbaseG2P::Browser::Util qw(neglog chr_list chr_length split_data_by_field);
use HGVbaseG2P::Browser::Core;

use Log::Log4perl qw(:easy);
Log::Log4perl->easy_init($INFO);

use Data::Dumper qw(Dumper);

# NB this module here handles commandline-option processing, according
# to the POD documention below the code.
use Getopt::Euclid qw( :minimal_keys );

my $conf_file  = $ARGV{c};

my %config     = load_config($conf_file);
my $SUMMARY_NO = 100000;

#instantiate browser db
!$ARGV{binsize} and $ARGV{binsize}='3000000';
my @bin_sizes = split(',',$ARGV{binsize});

my $studyid       = $ARGV{study};
my $rsid          = $ARGV{rs};

my $study_db   = HGVbaseG2P::Database::Study->new(
	{ conf_file=>\%config }
);

my $marker_db   = HGVbaseG2P::Database::Marker->new(
	{ conf_file=>\%config }
);
my $browser_db =
  HGVbaseG2P::Database::Browser->new( { conf_file => \%config } );

my $browser =
  HGVbaseG2P::Browser::Core->new( { conf_file => \%config } );
$browser->db($browser_db);


INFO("Initialised marker_data tables");
foreach my $chr ( keys %{ chr_list() } ) {
	$browser_db->create_marker_data_table( { chr => $chr } );
}
INFO("Initialised resultset_data table");
$browser_db->create_resultset_data_table();
INFO("Initialised marker_binned table");
$browser_db->create_marker_binned_table();

my %prepped_sth = ();
LOGDIE "No study ID or ID range supplied" if !$studyid;

my @study_range = split("-",$studyid);

my @studies;
if (scalar(@study_range)>1) {
	INFO "Using a range of studies:".$study_range[0]." to ".$study_range[1];
	my @temp = ($study_range[0]..$study_range[1]);
	my @study_ids = map { "HGVST".$_ } @temp;
	@studies       = $study_db->get_study_by_identifier(\@study_ids);
}
else {
	INFO "Using a single study:".$studyid;
	@studies       = ($study_db->get_study_by_identifier($studyid));
}

INFO scalar(@studies)." studies match criteria";

foreach my $study(@studies) {
	INFO "Processing study ".$study->identifier."\n";
	foreach my $exp ( $study->experiments->all ) {
		foreach my $rs ( $exp->resultsets->all ) {
			$ARGV{m} and copy_marker_data($study,$exp,$rs);
			$ARGV{p} and precalculate_bins($rs);
		}
	}
}
$browser->db->disconnect;

sub copy_marker_data {
	my ($study,$exp,$rs) = @_;
	
	foreach my $chr ( keys %{ chr_list() } ) {
	$browser_db->delete_marker_data_by_resultset_and_chr(
		{ resultsetid => $rs->resultsetid, chr => $chr } );
	}
	$browser_db->delete_resultset_data_by_resultset(
		{ resultsetid => $rs->resultsetid } );

	my $rs_record = {
		StudyID               => $study->studyid,
		Study_Identifier      => $study->identifier,
		Study_Name            => $study->name,
		Study_Label           => $study->label,
		IsDisplayed           => $study->isdisplayed,
		ExperimentID          => $exp->experimentid,
		Experiment_Identifier => $exp->identifier,
		Experiment_Label      => $exp->label,
		ResultsetID           => $rs->resultsetid,
		Resultset_Identifier  => $rs->identifier,
		Resultset_Label       => $rs->label,
		Resultset_Name        => $rs->name,
	};

	get_insert_rs( $browser_db, $rs_record )->execute( values %{$rs_record} );

	my $sigs;
	
	if ($ARGV{l}) {
		$sigs = $rs->significances->search(undef, { rows => 1000, order_by => 'unadjustedpvalue DESC'});
	}
	else {
		$sigs = $rs->significances;
	}
	
    INFO "Resultset ".$rs->identifier." has ".$sigs->count." significances\n";
	my $line_count = 0;
	while ( my $sig = $sigs->next ) {
		my $ums         = $sig->usedmarkerset;
		my $identifier = $ums->markeridentifier;
		my $marker = $marker_db->get_marker_by_identifier( $identifier );

		if ($marker) {
			my $coord =
			  $marker->markercoords->search(
				{ 'assemblytype' => 'ref_assembly' } )->first;
			if ($coord && $coord->chr ne 'MT') {
				my $record = {
					StudyID          => $study->studyid,
					ExperimentID     => $exp->experimentid,
					ResultsetID      => $rs->resultsetid,
					AnalysisMethodID => $rs->analysismethodid->analysismethodid,
					Identifier_Study => $study->identifier,
					SignificanceID   => $sig->significanceid,
					UnadjustedPValue => $sig->unadjustedpvalue,
					NegLogPValue     =>
					  sprintf( "%.3f", neglog( $sig->unadjustedpvalue ) ),
					MarkerID          => $marker->markerid,
					Marker_Identifier => $marker->identifier,
					Marker_Accession  => $marker->accession,
					DataSourceID      => $marker->datasourceid->datasourceid,
					VariationType     => $marker->variationtype,
					Chr               => $coord->chr,
					Start             => $coord->start,
					Stop              => $coord->stop,
					Strand            => $coord->strand
				};
				my $sth = get_insert_sig( $browser_db, $record, $coord->chr );
				$sth->execute( values %{$record} );
				$line_count++;
				$line_count % $SUMMARY_NO == 0
				  and print(
"$line_count marker data records copied\n"
				  );
			}
		}
		else {
			ERROR "Marker "
			  . $identifier
			  . " not found\n";
		}
	}

}

sub precalculate_bins {
	my ($rs) = @_;
	my $rs_id = $rs->resultsetid;
	TRACE "Precalculating data for resultset: $rs_id";

	
	$browser->resultsetid( [$rs_id] );
	
	

	#get data via browser
	
	foreach my $bin_size(@bin_sizes) {
		my $short_binsize = substr( $bin_size, 0, 1 );
		TRACE "Removing any previous data for resultset and binsize $bin_size";
		$browser_db->delete_marker_binned_by_resultset(
		{ resultsetid => $rs_id, binsize => $short_binsize } );
		$browser->_settings->{"lowres_binsize"}=$bin_size;
		
		foreach my $chr ( keys %{ chr_list() } ) {
			$browser->chr($chr);
			$browser->round_start(undef);
			$browser->round_stop(undef);
	
			my $data;
			$data = $browser->get_marker_data;

			$browser->seg_start(1);
			$browser->seg_stop( chr_length($chr) );
			$browser->round_positions;
	
			#$browser->chr(undef);
	
			my ($binned_data) = $browser->calculate_binned_data(
				{
					bin_size => $bin_size,
					data     => $data
				}
			);
	
			TRACE( "Calculating data for Chr:$chr (length:" . chr_length($chr) );

			#retrieve bins with no P-values in at all and add to hash
	
			#loop through each threshold level
			foreach my $threshold (qw(ZERO 1 2 3 4 5 6 7 8)) {
				$browser->threshold($threshold);
	
				#print "threshold $threshold\n";
				#calculate significance counts using browser object
				my ( $max_and_counts, $sig_markers, $max_pvalue, $max_count ) =
				  $browser->calculate_max_and_sig_counts(
					{ data => $binned_data } );
	
				#then store data in browser db
				my @precalc_data = ();
				my $sth          = $browser->get_db->dbh->prepare(
	"INSERT INTO marker_binned (Reference, Source, Type, Name, Start, Stop, Score, ResultsetID,Threshold, BinSize) 
					VALUES (?,?,?,?,?,?,?,?,?,?)"
				);
	
				foreach my $item ( @{$max_and_counts} ) {
					my @pars = ();
					if ( $item->{bin_counts} ne '-1' ) {
						@pars = (
							$chr,                'ensembl',
							'sig',               "sig$chr",
							$item->{bin_start},  $item->{bin_stop},
							$item->{bin_counts}, $rs_id,
							$threshold,          $short_binsize
						);
						#INFO( "Bin NOT marked as empty: " . $item->{bin_start} );
					}
					else {
						@pars = (
							$chr,               'ensembl',
							'sig',              "sig$chr",
							$item->{bin_start}, $item->{bin_stop},
							'-1',               $rs_id,
							$threshold,         $short_binsize
						);
						#INFO( "Bin marked as empty: " . $item->{bin_start} );
					}
					$sth->execute(@pars);
	
					#print join(",",@pars)."\n";
				}
	
			}
		}
	}
}

sub get_insert_sig {
	my ( $db, $record, $chr ) = @_;
	my $sth = $prepped_sth{$chr};
	if ( !$sth ) {
		my $length   = scalar( keys %{$record} );
		my @fields   = keys %{$record};
		my $pholders = "?," x $length;
		$pholders = substr( $pholders, 0, length($pholders) - 1 );
		my $sql =
		    "INSERT INTO marker_data_chr" . $chr . " ("
		  . join( ",", @fields )
		  . ") VALUES("
		  . $pholders . ")";
		$sth = $browser_db->dbh->prepare($sql);
		$prepped_sth{$chr} = $sth;
	}
	return $sth;
}

sub get_insert_rs {
	my ( $db, $record ) = @_;
	my $length   = scalar( keys %{$record} );
	my @fields   = keys %{$record};
	my $pholders = "?," x $length;
	$pholders = substr( $pholders, 0, length($pholders) - 1 );
	my $sql =
	    "INSERT INTO resultset_data ("
	  . join( ",", @fields )
	  . ") VALUES($pholders)";
	my $sth = $browser_db->dbh->prepare($sql);
	return $sth;
}

__END__


=head1 NAME

  - Create BrowserDB from StudyDB for all or a subset of studies

=head1 VERSION

  0.1

=head1 DESCRIPTION

  This script creates or updates the Browser DB
 
  $Id

=head1 USAGE

  [ignored, usage line is built by Getopt::Euclid from interface-description below]

=head1 OPTIONS

=over

=item -v[erbose]

  Set logging to verbose (Log4perl level of 'DEBUG')

=item -s[tudy] <study>

  Study identifier for study
  
=item -c[onfigfile] <file>
  
  HGVbaseG2P config file to use

=for Euclid:
        file.type:    readable
        file.default: 'conf/hgvbase.conf'
         
=item -m[arker]
  
  Copy sig data to browser DB
  
=item -p[recalculate]

  Precalculate binned data in browser DB

=item -b[insize] <bin>

  Size of bins to generate binned data for (comma separated for multiple binsizes)

=item -l[imit] <limit>

  Limit to x most significant markers

=item --version

=item --usage

=item --help

=item --man

  Print the usual program information

=back

=head1 AUTHOR

  Rob Free <rcfree@gmail.com>>

=head1 BUGS

  None known, but please report any to the author
    
=cut
